@@ -26,7 +26,8 @@ strncasecmp (const char *s1, const char *s2, size_t n)
 
   for (index = 0; index < n; index++)
     {
-      if (tolower (s1[index]) != tolower (s2[index]))
+      if (tolower ((unsigned char) s1[index])
+          != tolower ((unsigned char) s2[index]))
 	return (((unsigned const char *)s1)[index] 
 		- ((unsigned const char *)s2)[index]);
       if (s1[index] == 0)
