@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -20,18 +20,19 @@
 #include <config.h>
 #include <assert.h>
 #include <stdlib.h>
-#include "common.h"
+#include "alloc.h"
 #include "approx.h"
+#include "command.h"
 #include "error.h"
-#include "misc.h"
-#include "lexer.h"
-#include "var.h"
 #include "file-handle.h"
+#include "lexer.h"
+#include "misc.h"
 #include "settings.h"
+#include "sfm.h"
 #include "sort.h"
 #include "stats.h"
-#include "sfm.h"
 #include "str.h"
+#include "var.h"
 #include "vfm.h"
 #include "vfmP.h"
 
@@ -40,17 +41,16 @@
 #include "debug-print.h"
 
 /* Specifies how to make an aggregate variable. */
-typedef struct agr_var agr_var;
 struct agr_var
   {
-    agr_var *next;		/* Next in list. */
+    struct agr_var *next;		/* Next in list. */
 
     /* Collected during parsing. */
-    variable *src;		/* Source variable. */
-    variable *dest;		/* Target variable. */
+    struct variable *src;	/* Source variable. */
+    struct variable *dest;	/* Target variable. */
     int function;		/* Function. */
     int include_missing;	/* 1=Include user-missing values. */
-    value arg[2];		/* Arguments. */
+    union value arg[2];		/* Arguments. */
 
     /* Accumulated during AGGREGATE execution. */
     double dbl[3];
@@ -72,17 +72,16 @@ enum
   };
 
 /* Attributes of an aggregation function. */
-typedef struct
+struct agr_func
   {
-    const char *name;	/* Aggregation function name. */
-    int n_args;		/* Number of arguments. */
-    int alpha_type;	/* When given ALPHA arguments, output type. */
-    fmt_spec format;	/* Format specification if alpha_type != ALPHA. */
-  }
-agr_func;
+    const char *name;		/* Aggregation function name. */
+    int n_args;			/* Number of arguments. */
+    int alpha_type;		/* When given ALPHA arguments, output type. */
+    struct fmt_spec format;	/* Format spec if alpha_type != ALPHA. */
+  };
 
 /* Attributes of aggregation functions. */
-static agr_func agr_func_tab[] = 
+static struct agr_func agr_func_tab[] = 
   {
     {"<NONE>",  0, -1,      {0, 0, 0}},
     {"SUM",     0, -1,      {FMT_F, 8, 2}},
@@ -123,33 +122,33 @@ enum
 static int missing;
 
 /* Aggregate variables. */
-static agr_var *agr_first, *agr_next;
+static struct agr_var *agr_first, *agr_next;
 
 /* Aggregate dictionary. */
-static dictionary *agr_dict;
+static struct dictionary *agr_dict;
 
 /* Number of cases passed through aggregation. */
 static int case_count;
 
 /* Last values of the break variables. */
-static value *prev_case;
+static union value *prev_case;
 
 /* Buffers for use by the 10x transformation. */
 static flt64 *buf64_1xx;
-static ccase *buf_1xx;
+static struct ccase *buf_1xx;
 
 static void initialize_aggregate_info (void);
 
 /* Prototypes. */
 static int parse_aggregate_functions (void);
 static void free_aggregate_functions (void);
-static int aggregate_single_case (ccase *input, ccase *output);
+static int aggregate_single_case (struct ccase *input, struct ccase *output);
 static int create_sysfile (void);
 
-static int agr_00x_trns_proc (trns_header *, ccase *);
+static int agr_00x_trns_proc (struct trns_header *, struct ccase *);
 static void agr_00x_end_func (void);
-static int agr_10x_trns_proc (trns_header *, ccase *);
-static void agr_10x_trns_free (trns_header *);
+static int agr_10x_trns_proc (struct trns_header *, struct ccase *);
+static void agr_10x_trns_free (struct trns_header *);
 static void agr_10x_end_func (void);
 static int agr_11x_func (void);
 
@@ -176,25 +175,26 @@ cmd_aggregate (void)
   
   agr_dict = new_dictionary (1);
   
-  match_id (AGGREGATE);
+  lex_match_id ("AGGREGATE");
 
   /* Read most of the subcommands. */
   for (;;)
     {
-      match_tok('/');
+      lex_match('/');
       
-      if (match_id (OUTFILE))
+      if (lex_match_id ("OUTFILE"))
 	{
 	  if (seen & 1)
 	    {
 	      free (v_sort);
 	      free_dictionary (agr_dict);
-	      return msg (SE, _("OUTFILE specified multiple times."));
+	      msg (SE, _("OUTFILE specified multiple times."));
+	      return CMD_FAILURE;
 	    }
 	  seen |= 1;
 	      
-	  match_tok ('=');
-	  if (match_tok ('*'))
+	  lex_match ('=');
+	  if (lex_match ('*'))
 	    outfile = NULL;
 	  else 
 	    {
@@ -203,40 +203,42 @@ cmd_aggregate (void)
 		{
 		  free (v_sort);
 		  free_dictionary (agr_dict);
-		  return 0;
+		  return CMD_FAILURE;
 		}
 	    }
 	}
-      else if (match_id (MISSING))
+      else if (lex_match_id ("MISSING"))
 	{
-	  match_tok ('=');
-	  if (!match_id (COLUMNWISE))
+	  lex_match ('=');
+	  if (!lex_match_id ("COLUMNWISE"))
 	    {
 	      free (v_sort);
 	      free_dictionary (agr_dict);
-	      return syntax_error (_("while expecting COLUMNWISE"));
+	      lex_error (_("while expecting COLUMNWISE"));
+	      return CMD_FAILURE;
 	    }
 	  missing = COLUMNWISE;
 	}
-      else if (match_id (DOCUMENT))
+      else if (lex_match_id ("DOCUMENT"))
 	seen |= 2;
-      else if (match_id (PRESORTED))
+      else if (lex_match_id ("PRESORTED"))
 	seen |= 4;
-      else if (match_id (BREAK))
+      else if (lex_match_id ("BREAK"))
 	{
 	  if (seen & 8)
 	    {
 	      free (v_sort);
 	      free_dictionary (agr_dict);
-	      return msg (SE, _("BREAK specified multiple times."));
+	      msg (SE, _("BREAK specified multiple times."));
+	      return CMD_FAILURE;
 	    }
 	  seen |= 8;
 
-	  match_tok ('=');
+	  lex_match ('=');
 	  if (!parse_sort_variables ())
 	    {
 	      free_dictionary (agr_dict);
-	      return 0;
+	      return CMD_FAILURE;
 	    }
 	  
 	  {
@@ -244,10 +246,10 @@ cmd_aggregate (void)
 	    
 	    for (i = 0; i < nv_sort; i++)
 	      {
-		variable *v;
+		struct variable *v;
 	      
 		v = dup_variable (agr_dict, v_sort[i], v_sort[i]->name);
-		assert (v);
+		assert (v != NULL);
 	      }
 	  }
 	}
@@ -263,7 +265,7 @@ cmd_aggregate (void)
     {
       free_aggregate_functions ();
       free (v_sort);
-      return 0;
+      return CMD_FAILURE;
     }
 
   /* Delete documents. */
@@ -339,7 +341,7 @@ cmd_aggregate (void)
       case 0:
       case0:
 	{
-	  trns_header *t = xmalloc (sizeof (trns_header));
+	  struct trns_header *t = xmalloc (sizeof *t);
 	  t->proc = agr_00x_trns_proc;
 	  t->free = NULL;
 	  add_transformation (t);
@@ -361,7 +363,7 @@ cmd_aggregate (void)
 	    goto lossage;
 	  
 	  {
-	    trns_header *t = xmalloc (sizeof (trns_header));
+	    struct trns_header *t = xmalloc (sizeof *t);
 	    t->proc = agr_10x_trns_proc;
 	    t->free = agr_10x_trns_free;
 	    add_transformation (t);
@@ -381,7 +383,7 @@ cmd_aggregate (void)
 	read_sort_output (agr_11x_func);
 	
 	{
-	  ccase *save_temp_case = temp_case;
+	  struct ccase *save_temp_case = temp_case;
 	  temp_case = NULL;
 	  agr_11x_func ();
 	  temp_case = save_temp_case;
@@ -402,7 +404,7 @@ cmd_aggregate (void)
   free_aggregate_functions ();
   free (prev_case);
   
-  return 1;
+  return CMD_SUCCESS;
 
 lossage:
   /* Clean up. */
@@ -410,7 +412,7 @@ lossage:
   free_aggregate_functions ();
   free (prev_case);
 
-  return 0;
+  return CMD_FAILURE;
 }
 
 /* Create a system file for use in aggregation to an external file,
@@ -418,7 +420,7 @@ lossage:
 static int
 create_sysfile (void)
 {
-  sfm_write_info w;
+  struct sfm_write_info w;
   w.h = outfile;
   w.dict = agr_dict;
   w.compress = set_scompression;
@@ -430,8 +432,8 @@ create_sysfile (void)
       return 0;
     }
     
-  buf64_1xx = xmalloc (sizeof (flt64) * w.case_size);
-  buf_1xx = xmalloc (sizeof (ccase) + sizeof (value) * (agr_dict->nval - 1));
+  buf64_1xx = xmalloc (sizeof *buf64_1xx * w.case_size);
+  buf_1xx = xmalloc (sizeof (struct ccase) + sizeof (union value) * (agr_dict->nval - 1));
 
   return 1;
 }
@@ -453,12 +455,12 @@ parse_aggregate_functions (void)
       int n_dest;
 
       int include_missing;
-      agr_func *function;
+      struct agr_func *function;
       int func_index;
 
-      value arg[2];
+      union value arg[2];
 
-      variable **src;
+      struct variable **src;
       int n_src;
 
       int i;
@@ -472,7 +474,7 @@ parse_aggregate_functions (void)
       arg[1].c = NULL;
 
       /* Parse the list of target variables. */
-      while (!match_tok ('='))
+      while (!lex_match ('='))
 	{
 	  int n_dest_prev = n_dest;
 	  
@@ -483,47 +485,46 @@ parse_aggregate_functions (void)
 	  {
 	    int j;
 
-	    dest_label = xrealloc (dest_label, sizeof (char *) * n_dest);
+	    dest_label = xrealloc (dest_label, sizeof *dest_label * n_dest);
 	    for (j = n_dest_prev; j < n_dest; j++)
 	      dest_label[j] = NULL;
 	  }
 	  
-	  if (token == STRING)
+	  if (token == T_STRING)
 	    {
-	      if (strlen (tokstr) > 120)
-		tokstr[120] = 0;
-	      dest_label[n_dest - 1] = xstrdup (tokstr);
-	      get_token ();
+	      ds_truncate (&tokstr, 120);
+	      dest_label[n_dest - 1] = xstrdup (ds_value (&tokstr));
+	      lex_get ();
 	    }
 	}
 
       /* Get the name of the aggregation function. */
-      if (token != ID)
+      if (token != T_ID)
 	{
-	  syntax_error (_("expecting aggregation function"));
+	  lex_error (_("expecting aggregation function"));
 	  goto lossage;
 	}
 
       include_missing = 0;
-      if (tokstr[strlen(tokstr) - 1] == '.')
+      if (tokid[strlen (tokid) - 1] == '.')
 	{
 	  include_missing = 1;
-	  tokstr[strlen(tokstr) - 1] = 0;
+	  tokid[strlen (tokid) - 1] = 0;
 	}
       
       for (function = agr_func_tab; function->name; function++)
-	if (streq (function->name, tokstr))
+	if (!strcmp (function->name, tokid))
 	  break;
       if (NULL == function->name)
 	{
-	  msg (SE, _("Unknown aggregation function %s."), tokstr);
+	  msg (SE, _("Unknown aggregation function %s."), tokid);
 	  goto lossage;
 	}
       func_index = function - agr_func_tab;
-      get_token ();
+      lex_get ();
 
       /* Check for leading lparen. */
-      if (!match_tok ('('))
+      if (!lex_match ('('))
 	{
 	  if (func_index == N)
 	    func_index = N_NO_VARS;
@@ -531,7 +532,7 @@ parse_aggregate_functions (void)
 	    func_index = NU_NO_VARS;
 	  else
 	    {
-	      syntax_error (_("expecting `('"));
+	      lex_error (_("expecting `('"));
 	      goto lossage;
 	    }
 	} else {
@@ -555,13 +556,13 @@ parse_aggregate_functions (void)
 	      {
 		int type;
 	    
-		match_tok (',');
-		if (token == STRING)
+		lex_match (',');
+		if (token == T_STRING)
 		  {
-		    arg[i].c = xstrdup (tokstr);
+		    arg[i].c = xstrdup (ds_value (&tokstr));
 		    type = ALPHA;
 		  }
-		else if (token == NUM)
+		else if (token == T_NUM)
 		  {
 		    arg[i].f = tokval;
 		    type = NUMERIC;
@@ -570,7 +571,7 @@ parse_aggregate_functions (void)
 		    goto lossage;
 		  }
 	    
-		get_token ();
+		lex_get ();
 
 		if (type != src[0]->type)
 		  {
@@ -582,9 +583,9 @@ parse_aggregate_functions (void)
 	      }
 
 	  /* Trailing rparen. */
-	  if (!match_tok(')'))
+	  if (!lex_match(')'))
 	    {
-	      syntax_error (_("expecting `)'"));
+	      lex_error (_("expecting `)'"));
 	      goto lossage;
 	    }
 	  
@@ -607,7 +608,7 @@ parse_aggregate_functions (void)
          variables. */
       for (i = 0; i < n_dest; i++)
 	{
-	  agr_var *v = xmalloc (sizeof (agr_var));
+	  struct agr_var *v = xmalloc (sizeof *v);
 
 	  /* Add variable to chain. */
 	  if (agr_first)
@@ -619,7 +620,7 @@ parse_aggregate_functions (void)
 	  /* Create the target variable in the aggregate
              dictionary. */
 	  {
-	    variable *destvar;
+	    struct variable *destvar;
 	    
 	    agr_next->function = func_index;
 
@@ -655,7 +656,7 @@ parse_aggregate_functions (void)
 			&& (func_index == N || func_index == N_NO_VARS
 			    || func_index == NU || func_index == NU_NO_VARS))
 		      {
-			fmt_spec f = {FMT_F, 8, 2};
+			struct fmt_spec f = {FMT_F, 8, 2};
 		      
 			destvar->print = destvar->write = f;
 		      }
@@ -690,7 +691,7 @@ parse_aggregate_functions (void)
 	  
 	  agr_next->include_missing = include_missing;
 
-	  if (agr_next->src)
+	  if (agr_next->src != NULL)
 	    {
 	      int j;
 
@@ -703,7 +704,7 @@ parse_aggregate_functions (void)
 	    }
 	}
       
-      if (src && src[0]->type == ALPHA)
+      if (src != NULL && src[0]->type == ALPHA)
 	for (i = 0; i < function->n_args; i++)
 	  {
 	    free (arg[i].c);
@@ -714,12 +715,13 @@ parse_aggregate_functions (void)
       free (dest);
       free (dest_label);
 
-      if (!match_tok ('/'))
+      if (!lex_match ('/'))
 	{
 	  if (token == '.')
 	    return 1;
-	  else
-	    return syntax_error ("expecting end of command");
+
+	  lex_error ("expecting end of command");
+	  return 0;
 	}
       continue;
       
@@ -749,7 +751,7 @@ parse_aggregate_functions (void)
 static void
 free_aggregate_functions (void)
 {
-  agr_var *iter, *next;
+  struct agr_var *iter, *next;
 
   if (agr_dict)
     free_dictionary (agr_dict);
@@ -773,8 +775,8 @@ free_aggregate_functions (void)
 
 /* Execution. */
 
-static void accumulate_aggregate_info (ccase *input);
-static void dump_aggregate_info (ccase *output);
+static void accumulate_aggregate_info (struct ccase *input);
+static void dump_aggregate_info (struct ccase *output);
 
 /* Processes a single case INPUT for aggregation.  If output is
    warranted, it is written to case OUTPUT, which may be (but need not
@@ -783,7 +785,7 @@ static void dump_aggregate_info (ccase *output);
 /* The code in this function has an eerie similarity to
    vfm.c:SPLIT_FILE_procfunc()... */
 static int
-aggregate_single_case (ccase *input, ccase *output)
+aggregate_single_case (struct ccase *input, struct ccase *output)
 {
   /* The first case always begins a new break group.  We also need to
      preserve the values of the case for later comparison. */
@@ -798,16 +800,16 @@ aggregate_single_case (ccase *input, ccase *output)
 	  n_elem += v_sort[i]->nv;
       }
       
-      prev_case = xmalloc (sizeof (value) * n_elem);
+      prev_case = xmalloc (sizeof *prev_case * n_elem);
 
       /* Copy INPUT into prev_case. */
       {
-	value *iter = prev_case;
+	union value *iter = prev_case;
 	int i;
 
 	for (i = 0; i < nv_sort; i++)
 	  {
-	    variable *v = v_sort[i];
+	    struct variable *v = v_sort[i];
 	    
 	    if (v->type == NUMERIC)
 	      (iter++)->f = input->data[v->fv].f;
@@ -827,12 +829,12 @@ aggregate_single_case (ccase *input, ccase *output)
   /* Compare the value of each break variable to the values on the
      previous case. */
   {
-    value *iter = prev_case;
+    union value *iter = prev_case;
     int i;
     
     for (i = 0; i < nv_sort; i++)
       {
-	variable *v = v_sort[i];
+	struct variable *v = v_sort[i];
       
 	switch (v->type)
 	  {
@@ -866,12 +868,12 @@ not_equal:
 
   /* Copy INPUT into prev_case. */
   {
-    value *iter = prev_case;
+    union value *iter = prev_case;
     int i;
 
     for (i = 0; i < nv_sort; i++)
       {
-	variable *v = v_sort[i];
+	struct variable *v = v_sort[i];
 	    
 	if (v->type == NUMERIC)
 	  (iter++)->f = input->data[v->fv].f;
@@ -888,16 +890,16 @@ not_equal:
 
 /* Accumulates aggregation data from the case INPUT. */
 static void 
-accumulate_aggregate_info (ccase *input)
+accumulate_aggregate_info (struct ccase *input)
 {
-  agr_var *iter;
+  struct agr_var *iter;
 
 #define WEIGHT (input->data[default_dict.weight_index].f)
 
   for (iter = agr_first; iter; iter = iter->next)
     if (iter->src)
       {
-	value *v = &input->data[iter->src->fv];
+	union value *v = &input->data[iter->src->fv];
 
 	if ((!iter->include_missing && is_missing (v, iter->src))
 	    || (iter->include_missing && iter->src->type == NUMERIC
@@ -1154,7 +1156,7 @@ accumulate_aggregate_info (ccase *input)
    more of the break variables.  Make an output record from the
    accumulated statistics in the OUTPUT case. */
 static void 
-dump_aggregate_info (ccase *output)
+dump_aggregate_info (struct ccase *output)
 {
   debug_printf (("(dumping "));
   
@@ -1168,15 +1170,15 @@ dump_aggregate_info (ccase *output)
 	n_elem += v_sort[i]->nv;
     }
     debug_printf (("n_elem=%d:", n_elem));
-    memcpy (output->data, prev_case, sizeof (value) * n_elem);
+    memcpy (output->data, prev_case, sizeof (union value) * n_elem);
   }
   
   {
-    agr_var *i;
+    struct agr_var *i;
   
     for (i = agr_first; i; i = i->next)
       {
-	value *v = &output->data[i->dest->fv];
+	union value *v = &output->data[i->dest->fv];
 
 	debug_printf ((" %d,%d", i->dest->fv, i->dest->nv));
 
@@ -1320,7 +1322,7 @@ dump_aggregate_info (ccase *output)
 static void
 initialize_aggregate_info (void)
 {
-  agr_var *iter;
+  struct agr_var *iter;
 
   for (iter = agr_first; iter; iter = iter->next)
     {
@@ -1352,7 +1354,7 @@ initialize_aggregate_info (void)
 /* Aggregate each case as it comes through.  Cases which aren't needed
    are dropped. */
 static int
-agr_00x_trns_proc (unused trns_header *h, ccase *c)
+agr_00x_trns_proc (struct trns_header *h unused, struct ccase *c)
 {
   int code = aggregate_single_case (c, compaction_case);
   debug_printf (("%d ", code));
@@ -1384,7 +1386,7 @@ write_case_to_sfm (void)
 
   for (i = 0; i < agr_dict->nvar; i++)
     {
-      variable *v = agr_dict->var[i];
+      struct variable *v = agr_dict->var[i];
       
       if (v->type == NUMERIC)
 	{
@@ -1409,7 +1411,7 @@ write_case_to_sfm (void)
 /* Aggregate the current case and output it if we passed a
    breakpoint. */
 static int
-agr_10x_trns_proc (unused trns_header *h, ccase *c)
+agr_10x_trns_proc (struct trns_header *h unused, struct ccase *c)
 {
   int code = aggregate_single_case (c, buf_1xx);
 
@@ -1421,7 +1423,7 @@ agr_10x_trns_proc (unused trns_header *h, ccase *c)
 
 /* Close the system file now that we're done with it.  */
 static void
-agr_10x_trns_free (unused trns_header *h)
+agr_10x_trns_free (struct trns_header *h unused)
 {
   fh_close_handle (outfile);
 }
@@ -1490,11 +1492,11 @@ debug_print (int flags)
   }
   
   {
-    agr_var *iter;
+    struct agr_var *iter;
     
     for (iter = agr_first; iter; iter = iter->next)
       {
-	agr_func *f = &agr_func_tab[iter->function & FUNC];
+	struct agr_func *f = &agr_func_tab[iter->function & FUNC];
 	
 	printf (" /%s", iter->dest->name);
 	if (iter->dest->label)
