@@ -1,6 +1,5 @@
 /* libavl - manipulates AVL trees.
-   Copyright (C) 1998 Free Software Foundation, Inc.
-   Written by Ben Pfaff <blp@gnu.org>.
+   Copyright (C) 1998-9, 2000 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -15,22 +14,26 @@
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA. */
+   02111-1307, USA.
 
-/* This is file avl.c in libavl, version 1.1.0. */
+   The author may be contacted at <pfaffben@pilot.msu.edu> on the
+   Internet, or as Ben Pfaff, 12167 Airport Rd, DeWitt MI 48820, USA
+   through more mundane means. */
+
+/* This is file avl.c in libavl. */
 
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
 #if PSPP
-#include "common.h"
-#include "arena.h"
+#include "pool.h"
+#define HAVE_XMALLOC 1
 #endif
 #if SELF_TEST 
-#include <stdio.h>
 #include <limits.h>
 #include <time.h>
 #endif
+#include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 #include "avl.h"
@@ -47,42 +50,49 @@
 #endif
 #endif
 
-#if !PSPP
+#ifdef HAVE_XMALLOC
+void *xmalloc (size_t);
+#else /* !HAVE_XMALLOC */
 /* Allocates SIZE bytes of space using malloc().  Aborts if out of
    memory. */
 static void *
 xmalloc (size_t size)
 {
   void *vp;
+
   if (size == 0)
     return NULL;
   vp = malloc (size);
+
   assert (vp != NULL);
   if (vp == NULL)
-    abort ();
+    {
+      fprintf (stderr, "virtual memory exhausted\n");
+      exit (EXIT_FAILURE);
+    }
   return vp;
 }
-#endif
+#endif /* !HAVE_XMALLOC */
 
-/* Creates an AVL tree in arena OWNER (which can be NULL).  The arena
-   is owned by the caller, not by the AVL tree.  CMP is a order
-   function for the data to be stored in the tree.  PARAM is arbitrary
-   data that becomes an argument to the comparison function. */
+/* Creates an AVL tree in POOL (which can be NULL).  POOL is owned by
+   the caller, not by the AVL tree.  CMP is a order function for the
+   data to be stored in the tree.  PARAM is arbitrary data that
+   becomes an argument to the comparison function. */
 avl_tree *
-avl_create (MAYBE_ARENA avl_comparison_func cmp, void *param)
+avl_create (MAYBE_POOL avl_comparison_func cmp, void *param)
 {
   avl_tree *tree;
 
-  assert (cmp);
+  assert (cmp != NULL);
 #if PSPP
-  if (owner)
-    tree = arena_alloc (owner, sizeof (avl_tree));
+  if (pool)
+    tree = pool_alloc (pool, sizeof *tree);
   else
 #endif
-    tree = xmalloc (sizeof (avl_tree));
+    tree = xmalloc (sizeof *tree);
 
 #if PSPP
-  tree->owner = owner;
+  tree->pool = pool;
 #endif
   tree->root.link[0] = NULL;
   tree->root.link[1] = NULL; 
@@ -96,18 +106,18 @@ avl_create (MAYBE_ARENA avl_comparison_func cmp, void *param)
 /* Destroy tree TREE.  Function FREE_FUNC is called for every node in
    the tree as it is destroyed.  
 
-   No effect if the tree has an arena owner and free_func is NULL.
-   The caller owns the arena and must destroy it itself.
+   No effect if the tree has an pool owner and free_func is NULL.
+   The caller owns the pool and must destroy it itself.
 
    Do not attempt to reuse the tree after it has been freed.  Create a
    new one.  */
 void
 avl_destroy (avl_tree *tree, avl_node_func free_func)
 {
-  assert (tree);
+  assert (tree != NULL);
   
 #if PSPP
-  if (free_func || tree->owner == NULL)
+  if (free_func || tree->pool == NULL)
 #endif
     {
       /* Uses Knuth's Algorithm 2.3.1T as modified in exercise 13
@@ -115,7 +125,7 @@ avl_destroy (avl_tree *tree, avl_node_func free_func)
       
       /* T1. */
       avl_node *an[AVL_MAX_HEIGHT];	/* Stack A: nodes. */
-      unsigned long ab = 0;		/* Stack A: bits. */
+      char ab[AVL_MAX_HEIGHT];		/* Stack A: bits. */
       int ap = 0;			/* Stack A: height. */
       avl_node *p = tree->root.link[0];
 
@@ -125,7 +135,7 @@ avl_destroy (avl_tree *tree, avl_node_func free_func)
 	  while (p != NULL)
 	    {
 	      /* T3. */
-	      ab &= ~(1ul << ap);
+	      ab[ap] = 0;
 	      an[ap++] = p;
 	      p = p->link[0];
 	    }
@@ -137,9 +147,9 @@ avl_destroy (avl_tree *tree, avl_node_func free_func)
 		goto done;
 
 	      p = an[--ap];
-	      if ((ab & (1ul << ap)) == 0)
+	      if (ab[ap] == 0)
 		{
-		  ab |= (1ul << ap++);
+		  ab[ap++] = 1;
 		  p = p->link[1];
 		  break;
 		}
@@ -147,7 +157,7 @@ avl_destroy (avl_tree *tree, avl_node_func free_func)
 	      if (free_func)
 		free_func (p->data, tree->param);
 #if PSPP
-	      if (tree->owner == NULL)
+	      if (tree->pool == NULL)
 #endif
 		free (p);
 	    }
@@ -156,7 +166,7 @@ avl_destroy (avl_tree *tree, avl_node_func free_func)
 
  done:
 #if PSPP
-  if (tree->owner == NULL)
+  if (tree->pool == NULL)
 #endif
     free (tree);
 }
@@ -172,18 +182,18 @@ avl_free (avl_tree *tree)
 int
 avl_count (const avl_tree *tree)
 {
-  assert (tree);
+  assert (tree != NULL);
   return tree->count;
 }
 
-/* Allocates room for a new avl_node in arena OWNER, or using
-   xmalloc() if OWNER is NULL. */
+/* Allocates room for a new avl_node in POOL, or using xmalloc() if
+   POOL is NULL. */
 #if PSPP
 static inline avl_node *
-new_node (arena **owner)
+new_node (struct pool *pool)
 {
-  if (owner != NULL)
-    return arena_alloc (owner, sizeof (avl_node));
+  if (pool != NULL)
+    return pool_alloc (pool, sizeof (avl_node));
   else
     return xmalloc (sizeof (avl_node));
 }
@@ -194,17 +204,17 @@ new_node (void)
   return xmalloc (sizeof (avl_node));
 }
 
-#define new_node(owner)				\
+#define new_node(POOL)				\
 	new_node ()
 #endif
 
-/* Copy the contents of TREE to a new tree in arena OWNER.  If COPY is
+/* Copy the contents of TREE to a new tree in POOL.  If COPY is
    non-NULL, then each data item is passed to function COPY, and the
    return values are inserted into the new tree; otherwise, the items
    are copied verbatim from the old tree to the new tree.  Returns the
    new tree. */
 avl_tree *
-avl_copy (MAYBE_ARENA const avl_tree *tree, avl_copy_func copy)
+avl_copy (MAYBE_POOL const avl_tree *tree, avl_copy_func copy)
 {
   /* This is a combination of Knuth's Algorithm 2.3.1C (copying a
      binary tree) and Algorithm 2.3.1T as modified by exercise 12
@@ -222,9 +232,9 @@ avl_copy (MAYBE_ARENA const avl_tree *tree, avl_copy_func copy)
   avl_node **qp = qa;		/* Stack QA: stack pointer. */
   avl_node *q;
   
-  assert (tree);
+  assert (tree != NULL);
 #if PSPP
-  new_tree = avl_create (owner, tree->cmp, tree->param);
+  new_tree = avl_create (pool, tree->cmp, tree->param);
 #else
   new_tree = avl_create (tree->cmp, tree->param);
 #endif
@@ -236,7 +246,7 @@ avl_copy (MAYBE_ARENA const avl_tree *tree, avl_copy_func copy)
       /* C4. */
       if (p->link[0] != NULL)
 	{
-	  avl_node *r = new_node (owner);
+	  avl_node *r = new_node (pool);
 	  r->link[0] = r->link[1] = NULL;
 	  q->link[0] = r;
 	}
@@ -276,7 +286,7 @@ avl_copy (MAYBE_ARENA const avl_tree *tree, avl_copy_func copy)
       /* C2. */
       if (p->link[1])
 	{
-	  avl_node *r = new_node (owner);
+	  avl_node *r = new_node (pool);
 	  r->link[0] = r->link[1] = NULL;
 	  q->link[1] = r;
 	}
@@ -385,16 +395,18 @@ avl_probe (avl_tree *tree, void *item)
      random insertions.  */
 
   /* A1. */
-  avl_node *t = &tree->root;
+  avl_node *t;
   avl_node *s, *p, *q, *r;
-  assert (tree);
+  
+  assert (tree != NULL);
+  t = &tree->root;
   s = p = t->link[0];
 
   if (s == NULL)
     {
       tree->count++;
       assert (tree->count == 1);
-      q = t->link[0] = new_node (tree->owner);
+      q = t->link[0] = new_node (tree->pool);
       q->data = item;
       q->link[0] = q->link[1] = NULL;
       q->bal = 0;
@@ -413,7 +425,7 @@ avl_probe (avl_tree *tree, void *item)
 	  q = p->link[0];
 	  if (q == NULL)
 	    {
-	      p->link[0] = q = new_node (tree->owner);
+	      p->link[0] = q = new_node (tree->pool);
 	      break;
 	    }
 	}
@@ -424,7 +436,7 @@ avl_probe (avl_tree *tree, void *item)
 	  q = p->link[1];
 	  if (q == NULL)
 	    {
-	      p->link[1] = q = new_node (tree->owner);
+	      p->link[1] = q = new_node (tree->pool);
 	      break;
 	    }
 	}
@@ -557,12 +569,11 @@ avl_find (const avl_tree *tree, const void *item)
 {
   const avl_node *p;
 
-  assert (tree);
+  assert (tree != NULL);
   for (p = tree->root.link[0]; p; )
     {
       int diff = tree->cmp (item, p->data, tree->param);
 
-      /* A3. */
       if (diff < 0)
 	p = p->link[0];
       else if (diff > 0)
@@ -590,34 +601,41 @@ avl_delete (avl_tree *tree, const void *item)
   /* D1. */
   avl_node *pa[AVL_MAX_HEIGHT];		/* Stack P: Nodes. */
   char a[AVL_MAX_HEIGHT];		/* Stack P: Bits. */
-  int k = 0;				/* Stack P: Pointer. */
+  int k = 1;				/* Stack P: Pointer. */
   
   avl_node **q;
   avl_node *p;
 
-  assert (tree);
+  assert (tree != NULL);
 
-  a[k] = 0;
-  pa[k++] = &tree->root;
+  a[0] = 0;
+  pa[0] = &tree->root;
   p = tree->root.link[0];
   for (;;)
     {
       /* D2. */
-      int diff = tree->cmp (item, p->data, tree->param);
+      int diff;
 
+      if (p == NULL)
+	return NULL;
+
+      diff = tree->cmp (item, p->data, tree->param);
       if (diff == 0)
 	break;
 
       /* D3, D4. */
       pa[k] = p;
       if (diff < 0)
-	p = p->link[0], a[k] = 0;
+	{
+	  p = p->link[0];
+	  a[k] = 0;
+	}
       else if (diff > 0)
-	p = p->link[1], a[k] = 1;
+	{
+	  p = p->link[1];
+	  a[k] = 1;
+	}
       k++;
-
-      if (p == NULL)
-	return NULL;
     }
   tree->count--;
   
@@ -647,13 +665,12 @@ avl_delete (avl_tree *tree, const void *item)
 	{
 	  /* D7. */
 	  avl_node *s = r->link[0];
-	  int l = k;
-	  l = k++;
+	  int l = k++;
 
 	  a[k] = 0;
 	  pa[k++] = r;
 	    
-	    /* D8. */
+	  /* D8. */
 	  while (s->link[0] != NULL)
 	    {
 	      r = s;
@@ -674,7 +691,7 @@ avl_delete (avl_tree *tree, const void *item)
     }
 
 #if PSPP
-  if (tree->owner == NULL)
+  if (tree->pool == NULL)
 #endif
     free (p);
 
@@ -701,7 +718,8 @@ avl_delete (avl_tree *tree, const void *item)
 	  assert (s->bal == +1);
 	  r = s->link[1];
 
-	  if (r == NULL || r->bal == 0)
+	  assert (r != NULL);
+	  if (r->bal == 0)
 	    {
 	      /* D11. */
 	      s->link[1] = r->link[0];
@@ -734,7 +752,7 @@ avl_delete (avl_tree *tree, const void *item)
 	      else
 		{
 		  assert (p->bal == -1);
-		  s->bal = 0, r->bal = 1;
+		  s->bal = 0, r->bal = +1;
 		}
 	      p->bal = 0;
 	      pa[k - 1]->link[(int) a[k - 1]] = p;
@@ -756,7 +774,7 @@ avl_delete (avl_tree *tree, const void *item)
 	      break;
 	    }
 
-	  assert (s->bal = -1);
+	  assert (s->bal == -1);
 	  r = s->link[0];
 
 	  if (r == NULL || r->bal == 0)
@@ -790,7 +808,7 @@ avl_delete (avl_tree *tree, const void *item)
 		s->bal = r->bal = 0;
 	      else
 		{
-		  assert (p->bal = 1);
+		  assert (p->bal == 1);
 		  s->bal = 0, r->bal = -1;
 		}
 	      p->bal = 0;
@@ -809,7 +827,7 @@ avl_insert (avl_tree *tree, void *item)
 {
   void **p;
   
-  assert (tree);
+  assert (tree != NULL);
   
   p = avl_probe (tree, item);
   return (*p == item) ? NULL : *p;
@@ -824,7 +842,7 @@ avl_replace (avl_tree *tree, void *item)
 {
   void **p;
 
-  assert (tree);
+  assert (tree != NULL);
   
   p = avl_probe (tree, item);
   if (*p == item)
@@ -843,7 +861,7 @@ void *
 (avl_force_delete) (avl_tree *tree, void *item)
 {
   void *found = avl_delete (tree, item);
-  assert (found);
+  assert (found != NULL);
   return found;
 }
 
@@ -878,14 +896,14 @@ print_structure (avl_node *node, int level)
 
 /* Compare two integers A and B and return a strcmp()-type result. */
 int
-compare_ints (const void *a, const void *b, unused void *param)
+compare_ints (const void *a, const void *b, void *param unused)
 {
   return ((int) a) - ((int) b);
 }
 
 /* Print the value of integer A. */
 void
-print_int (void *a, unused void *param)
+print_int (void *a, void *param unused)
 {
   printf (" %d", (int) a);
 }
@@ -903,7 +921,8 @@ print_contents (avl_tree *tree)
    root of the tree, PARENT should be INT_MIN, otherwise it should be
    the parent node value.  DIR is the direction that the current node
    is linked from the parent: -1 for left child, +1 for right child;
-   it is not used if PARENT is INT_MIN. */
+   it is not used if PARENT is INT_MIN.  Returns the height of the
+   tree rooted at NODE. */
 int
 recurse_tree (avl_node *node, int *count, int parent, int dir)
 {
@@ -915,17 +934,28 @@ recurse_tree (avl_node *node, int *count, int parent, int dir)
       (*count)++;
 
       if (nr - nl != node->bal)
-	printf (" Node %d is unbalanced: right height=%d, left height=%d, "
-		"difference=%d, but balance factor=%d.\n", d, nr, nl, nr - nl, node->bal), done = 1;
+	{
+	  printf (" Node %d is unbalanced: right height=%d, left height=%d, "
+		"difference=%d, but balance factor=%d.\n",
+		  d, nr, nl, nr - nl, node->bal);
+	  done = 1;
+	}
+      
       if (parent != INT_MIN)
 	{
 	  assert (dir == -1 || dir == +1);
 	  if (dir == -1 && d > parent)
-	    printf (" Node %d is smaller than its left child %d.\n",
-		    parent, d), done = 1;
+	    {
+	      printf (" Node %d is smaller than its left child %d.\n",
+		      parent, d);
+	      done = 1;
+	    }
 	  else if (dir == +1 && d < parent)
-	    printf (" Node %d is larger than its right child %d.\n",
-		    parent, d), done = 1;
+	    {
+	      printf (" Node %d is larger than its right child %d.\n",
+		      parent, d);
+	      done = 1;
+	    }
 	}
       assert (node->bal >= -1 && node->bal <= 1);
       return 1 + (nl > nr ? nl : nr);
@@ -940,7 +970,11 @@ verify_tree (avl_tree *tree)
   int count = 0;
   recurse_tree (tree->root.link[0], &count, INT_MIN, 0);
   if (count != tree->count)
-    printf (" Tree has %d nodes, but tree count is %d.\n", count, tree->count), done = 1;
+    {
+      printf (" Tree has %d nodes, but tree count is %d.\n",
+	      count, tree->count);
+      done = 1;
+    }
   if (done)
     abort ();
 }
@@ -1018,23 +1052,29 @@ compare_trees (avl_node *a, avl_node *b)
    places in order to be able to see what's really going on.  Also,
    memory debuggers like Checker or Purify are very handy. */
 #define TREE_SIZE 1024
+#define N_ITERATIONS 16
 int
 main (int argc, char **argv)
 {
   int array[TREE_SIZE];
   int seed;
+  int iteration;
   
   if (argc == 2)
     seed = atoi (argv[1]);
   else
     seed = time (0) * 257 % 32768;
 
-  for (;;)
+  fputs ("Testing avl...\n", stdout);
+  
+  for (iteration = 1; iteration <= N_ITERATIONS; iteration++)
     {
       avl_tree *tree;
       int i;
       
-      printf ("Random number seed: %d\n", seed);
+      printf ("Iteration %4d/%4d: seed=%5d", iteration, N_ITERATIONS, seed);
+      fflush (stdout);
+      
       srand (seed++);
 
       for (i = 0; i < TREE_SIZE; i++)
@@ -1061,10 +1101,11 @@ main (int argc, char **argv)
 
 	  if (i % 128 == 0)
 	    {
-	      fputc ('.', stdout);
+	      putchar ('.');
 	      fflush (stdout);
 	    }
 	}
+      fputs (" good.\n", stdout);
 
       avl_destroy (tree, NULL);
     }
