@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -23,20 +23,19 @@
 /* Vectors. */
 
 /* A vector of longs. */
-typedef struct long_vec
+struct long_vec
   {
     long *vec;			/* Contents. */
     int n;			/* Number of elements. */
     int m;			/* Number of elements room is allocated for. */
-  }
-long_vec;
+  };
 
 struct variable;
 
-void vec_init (long_vec *);
-void vec_clear (long_vec *);
-void vec_insert (long_vec *, long);
-void vec_delete (long_vec *, long a, long b);
+void vec_init (struct long_vec *);
+void vec_clear (struct long_vec *);
+void vec_insert (struct long_vec *, long);
+void vec_delete (struct long_vec *, long a, long b);
 void devector (const struct variable *);
 void envector (const struct variable *);
 
