@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -30,7 +30,20 @@ enum
     STATE_ERROR			/* Invalid state transition. */
   };
 
+/* Command return values. */
+enum
+  {
+    CMD_FAILURE = 0x1000,	/* Command not executed. */
+    CMD_SUCCESS,		/* Command successfully parsed and executed. */
+    CMD_PART_SUCCESS_MAYBE,	/* Command may have been partially executed. */
+    CMD_PART_SUCCESS,		/* Command fully executed up to error. */
+    CMD_TRAILING_GARBAGE,	/* Command followed by garbage. */
+  };
+
 extern int pgm_state;
-extern char *cur_proc;
+extern const char *cur_proc;
+
+void cmd_init (void);
+int cmd_parse (void);
 
 #endif /* !command_h */
