@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -37,30 +37,27 @@ char *alloca ();
 
 #include <assert.h>
 #include <ctype.h>
-#include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
+#include "alloc.h"
+#include "command.h"
 #include "error.h"
-#include "getline.h"
-#include "misc.h"
-#include "lexer.h"
-#include "lexerP.h"
-#include "str.h"
-#include "var.h"
-#include "dfm.h"
 #include "file-handle.h"
-#include "vfm.h"
 #include "filename.h"
+#include "getline.h"
+#include "lexer.h"
+#include "misc.h"
+#include "str.h"
+#include "vfm.h"
 
 #undef DEBUGGING
 /*#define DEBUGGING 1*/
 #include "debug-print.h"
 
 /* file_handle extension structure. */
-typedef struct
+struct dfm_fhuser_ext
   {
-    file_ext file;		/* Associated file. */
-    int ln;			/* Line number of record just read. */
+    struct file_ext file;	/* Associated file. */
 
     char *line;			/* Current line, not null-terminated. */
     size_t len;			/* Length of line. */
@@ -70,35 +67,34 @@ typedef struct
     size_t size;		/* Number of bytes allocated for line. */
     int advance;		/* Nonzero=dfm_get_record() reads a new
 				   record; otherwise returns current record. */
-  }
-dfm_fhuser_ext;
+  };
 
 /* These are defined at the end of this file. */
-static fh_ext_class dfm_r_class;
-static fh_ext_class dfm_w_class;
+static struct fh_ext_class dfm_r_class;
+static struct fh_ext_class dfm_w_class;
 
-static void read_record (file_handle *h);
+static void read_record (struct file_handle *h);
 
 /* Internal (low level). */
 
 /* Closes the file handle H which was opened by open_file_r() or
    open_file_w(). */
 static void
-dfm_close (file_handle *h)
+dfm_close (struct file_handle *h)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  struct dfm_fhuser_ext *ext = h->ext;
 
   /* Skip any remaining data on the inline file. */
   if (h == inline_file)
     while (ext->line != NULL)
       read_record (h);
       
-  verbose_msg (2, _("%s: Closing data-file handle %s."),
-	       fh_handle_filename (h), fh_handle_name (h));
+  msg (VM (2), _("%s: Closing data-file handle %s."),
+       fh_handle_filename (h), fh_handle_name (h));
   assert (h->class == &dfm_r_class || h->class == &dfm_w_class);
   if (ext->file.file)
     {
-      close_file_ext (&ext->file);
+      fn_close_ext (&ext->file);
       free (ext->file.filename);
       ext->file.filename = NULL;
     }
@@ -108,18 +104,17 @@ dfm_close (file_handle *h)
 
 /* Initializes EXT properly as an inline data file. */
 static void
-open_inline_file (dfm_fhuser_ext *ext)
+open_inline_file (struct dfm_fhuser_ext *ext)
 {
   /* We want to indicate that the file is open, that we are not at
      eof, and that another line needs to be read in.  */
 #if __CHECKER__
-  memset (&ext->file, 0, sizeof (ext->file));
+  memset (&ext->file, 0, sizeof ext->file);
 #endif
   ext->file.file = NULL;
-  ext->ln = 0;
   ext->line = xmalloc (128);
 #if !PRODUCTION
-  strcpy (ext->line, _("HAHAHAHA! <<This message indicates a bug.>>"));
+  strcpy (ext->line, _("<<Bug in dfm.c>>"));
 #endif
   ext->len = strlen (ext->line);
   ext->ptr = ext->line;
@@ -129,26 +124,29 @@ open_inline_file (dfm_fhuser_ext *ext)
 
 /* Opens a file handle for reading as a data file. */
 static int
-open_file_r (file_handle *h)
+open_file_r (struct file_handle *h)
 {
-  dfm_fhuser_ext ext;
-  
+  struct dfm_fhuser_ext ext;
+
+  h->where.line_number = 0;
   ext.file.file = NULL;
-  ext.ln = 0;
   ext.line = NULL;
   ext.len = 0;
   ext.ptr = NULL;
   ext.size = 0;
   ext.advance = 0;
 
-  verbose_msg (1, _("%s: Opening data-file handle %s for reading."),
-	       fh_handle_filename (h), fh_handle_name (h));
+  msg (VM (1), _("%s: Opening data-file handle %s for reading."),
+       fh_handle_filename (h), fh_handle_name (h));
   
-  assert (h);
+  assert (h != NULL);
   if (h == inline_file)
     {
       char *s;
 
+      /* WTF can't this just be done with tokens?
+	 Is this really a special case?
+	 FIXME! */
       do
 	{
 	  char *cp;
@@ -156,25 +154,25 @@ open_file_r (file_handle *h)
 	  if (!getl_read_line ())
 	    {
 	      msg (SE, _("BEGIN DATA expected."));
-	      failure ();
+	      err_failure ();
 	    }
 
 	  /* Skip leading whitespace, separate out first word, so that
 	     S points to a single word reduced to lowercase. */
-	  s = getl_buf;
+	  s = ds_value (&getl_buf);
 	  while (isspace ((unsigned char) *s))
 	    s++;
 	  for (cp = s; isalpha ((unsigned char) *cp); cp++)
 	    *cp = tolower ((unsigned char) (*cp));
-	  *cp = '\0';
+	  ds_truncate (&getl_buf, cp - s);
 	}
       while (*s == '\0');
 
-      if (!id_match ("begin", s))
+      if (!lex_id_match_len ("begin", 5, s, strcspn (s, " \t\r\v\n")))
 	{
 	  msg (SE, _("BEGIN DATA expected."));
-	  cond_fail ();
-	  preprocess_line ();
+	  err_cond_fail ();
+	  lex_preprocess_line ();
 	  return 0;
 	}
       getl_prompt = GETL_PRPT_DATA;
@@ -190,44 +188,45 @@ open_file_r (file_handle *h)
       ext.file.param = NULL;
       ext.file.postopen = NULL;
       ext.file.preclose = NULL;
-      if (!open_file_ext (&ext.file))
+      if (!fn_open_ext (&ext.file))
 	{
 	  msg (ME, _("An error occurred while opening \"%s\" for reading "
 	       "as a data file: %s."), h->fn, strerror (errno));
-	  cond_fail ();
+	  err_cond_fail ();
 	  return 0;
 	}
     }
 
   h->class = &dfm_r_class;
-  h->ext = xmalloc (sizeof (dfm_fhuser_ext));
-  memcpy (h->ext, &ext, sizeof (dfm_fhuser_ext));
+  h->ext = xmalloc (sizeof (struct dfm_fhuser_ext));
+  memcpy (h->ext, &ext, sizeof (struct dfm_fhuser_ext));
 
   return 1;
 }
 
 /* Opens a file handle for writing as a data file. */
 static int
-open_file_w (file_handle *h)
+open_file_w (struct file_handle *h)
 {
-  dfm_fhuser_ext ext;
+  struct dfm_fhuser_ext ext;
   
   ext.file.file = NULL;
-  ext.ln = 0;
   ext.line = NULL;
   ext.len = 0;
   ext.ptr = NULL;
   ext.size = 0;
   ext.advance = 0;
 
-  verbose_msg (1, _("%s: Opening data-file handle %s for writing."),
-	       fh_handle_filename (h), fh_handle_name (h));
+  h->where.line_number = 0;
+
+  msg (VM (1), _("%s: Opening data-file handle %s for writing."),
+       fh_handle_filename (h), fh_handle_name (h));
   
-  assert (h);
+  assert (h != NULL);
   if (h == inline_file)
     {
       msg (ME, _("Cannot open the inline file for writing."));
-      cond_fail ();
+      err_cond_fail ();
       return 0;
     }
 
@@ -239,17 +238,17 @@ open_file_w (file_handle *h)
   ext.file.postopen = NULL;
   ext.file.preclose = NULL;
       
-  if (!open_file_ext (&ext.file))
+  if (!fn_open_ext (&ext.file))
     {
       msg (ME, _("An error occurred while opening \"%s\" for writing "
 	   "as a data file: %s."), h->fn, strerror (errno));
-      cond_fail ();
+      err_cond_fail ();
       return 0;
     }
 
   h->class = &dfm_w_class;
-  h->ext = xmalloc (sizeof (dfm_fhuser_ext));
-  memcpy (h->ext, &ext, sizeof (dfm_fhuser_ext));
+  h->ext = xmalloc (sizeof (struct dfm_fhuser_ext));
+  memcpy (h->ext, &ext, sizeof (struct dfm_fhuser_ext));
 
   return 1;
 }
@@ -288,9 +287,9 @@ count_tabs (char *s, size_t len)
 /* Converts all the tabs in H->EXT->LINE to an equivalent number of
    spaces, if necessary. */
 static void
-tabs_to_spaces (file_handle *h)
+tabs_to_spaces (struct file_handle *h)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  struct dfm_fhuser_ext *ext = h->ext;
   
   char *first_tab;		/* Location of first tab (if any). */
   char *second_tab;		/* Location of second tab (if any). */
@@ -378,9 +377,9 @@ tabs_to_spaces (file_handle *h)
    of the line.  The line is not null-terminated.  If an error occurs
    or end-of-file is encountered, H->EXT->LINE is set to NULL. */
 static void
-read_record (file_handle *h)
+read_record (struct file_handle *h)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  struct dfm_fhuser_ext *ext = h->ext;
 
   if (h == inline_file)
     {
@@ -391,25 +390,21 @@ read_record (file_handle *h)
 	       "a missing or misformatted END DATA command.  "
 	       "END DATA must appear by itself on a single line "
 	       "with exactly one space between words."));
-	  failure ();
+	  err_failure ();
 	}
 
-      ext->ln++;
+      h->where.line_number++;
 
-      if (!memchr (getl_buf, '\0', 8))
+      if (ds_length (&getl_buf) >= 8
+	  && !strncasecmp (ds_value (&getl_buf), "end data", 8))
 	{
-	  if (!strncasecmp (getl_buf, "end data", 8))
-	    {
-	      extern void set_prog (char *);	/* Exported by lexer.c. */
-
-	      set_prog (strchr (getl_buf, 0));
-	      goto eof;
-	    }
+	  lex_set_prog (ds_value (&getl_buf) + ds_length (&getl_buf));
+	  goto eof;
 	}
 
-      ext->len = strlen (getl_buf);
+      ext->len = ds_length (&getl_buf);
       force_line_buffer_expansion ();
-      strcpy (ext->line, getl_buf);
+      strcpy (ext->line, ds_value (&getl_buf));
     }
   else
     {
@@ -425,7 +420,7 @@ read_record (file_handle *h)
 		{
 		  msg (ME, _("Error reading file %s: %s."),
 		       fh_handle_name (h), strerror (errno));
-		  cond_fail ();
+		  err_cond_fail ();
 		}
 	      goto eof;
 	    }
@@ -452,14 +447,14 @@ read_record (file_handle *h)
 	      else
 		goto eof;
 
-	      cond_fail ();
+	      err_cond_fail ();
 	      goto eof;
 	    }
 	}
       else
 	assert (0);
 
-      cust_ln = ++ext->ln;
+      h->where.line_number++;
     }
 
   /* Strip trailing whitespace, I forget why.  But there's a good
@@ -493,7 +488,7 @@ eof:
    Returns NULL at end of file.  Calls fail() on attempt to read past
    end of file.  */
 char *
-dfm_get_record (file_handle *h, int *len)
+dfm_get_record (struct file_handle *h, int *len)
 {
   if (h->class == NULL)
     {
@@ -509,7 +504,7 @@ dfm_get_record (file_handle *h, int *len)
     }
   else
     {
-      dfm_fhuser_ext *ext = h->ext;
+      struct dfm_fhuser_ext *ext = h->ext;
 
       if (ext->advance)
 	{
@@ -525,7 +520,7 @@ dfm_get_record (file_handle *h, int *len)
     }
 
   {
-    dfm_fhuser_ext *ext = h->ext;
+    struct dfm_fhuser_ext *ext = h->ext;
 
     if (ext)
       {
@@ -541,7 +536,7 @@ dfm_get_record (file_handle *h, int *len)
 lossage:
   /* Come here on reading beyond eof or reading from a file already
      open for something else. */
-  cond_fail ();
+  err_cond_fail ();
 
   return NULL;
 }
@@ -549,9 +544,9 @@ lossage:
 /* Causes dfm_get_record() to read in the next record the next time it
    is executed on file HANDLE. */
 void
-dfm_fwd_record (file_handle *h)
+dfm_fwd_record (struct file_handle *h)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  struct dfm_fhuser_ext *ext = h->ext;
 
   assert (h->class == &dfm_r_class);
   ext->advance = 1;
@@ -562,9 +557,9 @@ dfm_fwd_record (file_handle *h)
    COLUMN, as with dfm_set_record but based on a column number instead
    of a character pointer. */
 void
-dfm_bkwd_record (file_handle *h, int column)
+dfm_bkwd_record (struct file_handle *h, int column)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  struct dfm_fhuser_ext *ext = h->ext;
 
   assert (h->class == &dfm_r_class);
   ext->advance = 0;
@@ -575,9 +570,9 @@ dfm_bkwd_record (file_handle *h, int column)
    somewhere in the line last returned by dfm_get_record().  Used by
    DATA LIST FREE to strip the leading portion off the current line.  */
 void
-dfm_set_record (file_handle *h, char *new_line)
+dfm_set_record (struct file_handle *h, char *new_line)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  struct dfm_fhuser_ext *ext = h->ext;
 
   assert (h->class == &dfm_r_class);
   ext->ptr = new_line;
@@ -587,9 +582,9 @@ dfm_set_record (file_handle *h, char *new_line)
    HANDLE is set.  Unless dfm_set_record() or dfm_bkwd_record() have
    been called, this is 0. */
 int
-dfm_get_cur_col (file_handle *h)
+dfm_get_cur_col (struct file_handle *h)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  struct dfm_fhuser_ext *ext = h->ext;
 
   assert (h->class == &dfm_r_class);
   return ext->ptr - ext->line;
@@ -599,7 +594,7 @@ dfm_get_cur_col (file_handle *h)
    HANDLE.  REC is not null-terminated.  Returns nonzero on success,
    zero on failure. */
 int
-dfm_put_record (file_handle *h, const char *rec, size_t len)
+dfm_put_record (struct file_handle *h, const char *rec, size_t len)
 {
   char *ptr;
   size_t amt;
@@ -613,7 +608,7 @@ dfm_put_record (file_handle *h, const char *rec, size_t len)
     {
       msg (ME, _("Cannot write to file %s already opened for %s."),
 	   fh_handle_name (h), gettext (h->class->name));
-      cond_fail ();
+      err_cond_fail ();
       return 0;
     }
 
@@ -628,13 +623,16 @@ dfm_put_record (file_handle *h, const char *rec, size_t len)
       memset (&ptr[len], ch, amt - len);
     }
   else
-    ptr = (char *) rec, amt = len;
+    {
+      ptr = (char *) rec;
+      amt = len;
+    }
 
-  if (1 != fwrite (ptr, amt, 1, ((dfm_fhuser_ext *) h->ext)->file.file))
+  if (1 != fwrite (ptr, amt, 1, ((struct dfm_fhuser_ext *) h->ext)->file.file))
     {
       msg (ME, _("Error writing file %s: %s."), fh_handle_name (h),
 	   strerror (errno));
-      cond_fail ();
+      err_cond_fail ();
       return 0;
     }
 
@@ -644,20 +642,20 @@ dfm_put_record (file_handle *h, const char *rec, size_t len)
   return 1;
 }
 
-/* Pushes the filename and line number on the custom fn/ln stack. */
+/* Pushes the filename and line number on the fn/ln stack. */
 void
-dfm_push_cust (file_handle *h)
+dfm_push (struct file_handle *h)
 {
-  dfm_fhuser_ext *ext = h->ext;
+  if (h != inline_file)
+    err_push_file_locator (&h->where);
+}
 
-  push_cust (h->fn);
-  if (h->class == NULL)
-    cust_ln = 0;
-  else
-    {
-      assert (h->class == &dfm_r_class);
-      cust_ln = ext->ln;
-    }
+/* Pops the filename and line number from the fn/ln stack. */
+void
+dfm_pop (struct file_handle *h)
+{
+  if (h != inline_file)
+    err_pop_file_locator (&h->where);
 }
 
 /* BEGIN DATA...END DATA procedure. */
@@ -666,7 +664,7 @@ dfm_push_cust (file_handle *h)
 int
 cmd_begin_data (void)
 {
-  dfm_fhuser_ext *ext;
+  struct dfm_fhuser_ext *ext;
 
   /* FIXME: figure out the *exact* conditions, not these really
      lenient conditions. */
@@ -677,14 +675,14 @@ cmd_begin_data (void)
     {
       msg (SE, _("This command is not valid here since the current "
 	   "input program does not access the inline file."));
-      cond_fail ();
-      return 1;
+      err_cond_fail ();
+      return CMD_FAILURE;
     }
 
   /* Initialize inline_file. */
-  verbose_msg (1, _("inline file: Opening for reading."));
+  msg (VM (1), _("inline file: Opening for reading."));
   inline_file->class = &dfm_r_class;
-  inline_file->ext = xmalloc (sizeof (dfm_fhuser_ext));
+  inline_file->ext = xmalloc (sizeof (struct dfm_fhuser_ext));
   open_inline_file (inline_file->ext);
 
   /* We don't actually read from the inline file.  The input procedure
@@ -702,17 +700,17 @@ cmd_begin_data (void)
     }
   assert (inline_file->ext == NULL);
 
-  return 1;
+  return CMD_SUCCESS;
 }
 
-static fh_ext_class dfm_r_class =
+static struct fh_ext_class dfm_r_class =
 {
   1,
   N_("reading as a data file"),
   dfm_close,
 };
 
-static fh_ext_class dfm_w_class =
+static struct fh_ext_class dfm_w_class =
 {
   2,
   N_("writing as a data file"),
