@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -50,19 +50,21 @@ char *alloca ();
 #include <assert.h>
 #include <math.h>
 #include <errno.h>
+#include <stdio.h>
 #include "approx.h"
-#include "common.h"
+#include "data-in.h"
 #include "error.h"
 #include "expr.h"
 #include "exprP.h"
 #include "julcal/julcal.h"
-#include "misc.h"
+#include "magic.h"
+#include "random.h"
 #include "stats.h"
 #include "str.h"
 #include "var.h"
 #include "vector.h"
 #include "vfm.h"
-#include "vfmP.h" /* Need access to vfm_sink_info.ncases for $CASENUM. */
+#include "vfmP.h"
 
 /* FIXME: This could be even more efficient if we caught SYSMIS when
    it first reared its ugly head, then threw it into an entirely new
@@ -91,36 +93,36 @@ char *alloca ();
 #define CHECK_STRING_SPACE(X)						\
 	do								\
           {								\
-	    if(str_stk + X >= str_end)					\
+	    if (str_stk + X >= str_end)					\
 	      {								\
 		e->str_size += 1024;					\
 		e->str_stk = xrealloc (e->str_stk, e->str_size);	\
 		str_end = e->str_stk + e->str_size - 1;			\
 	      }								\
 	  }								\
-	while(0)
+	while (0)
      
 #define ALLOC_STRING_SPACE(X)			\
 	(str_stk += X + 1, str_stk - X - 1)
 #endif /* !PAGED_STACK */
 
 double
-evaluate_expression (expression * e, ccase * c, value * v)
+expr_evaluate (struct expression *e, struct ccase *c, union value *v)
 {
-  operator *op = e->op;
-  double *dbl = e->dbl;
+  unsigned char *op = e->op;
+  double *dbl = e->num;
   unsigned char *str = e->str;
 #if !PAGED_STACK
   unsigned char *str_stk = e->str_stk;
   unsigned char *str_end = e->str_stk + e->str_size - 1;
 #endif
-  variable **vars = e->vars;
+  struct variable **vars = e->var;
   int i, j;
 
   /* Stack pointer. */
-  value *sp = e->stack;
+  union value *sp = e->stack;
 
-  while (1)
+  for (;;)
     {
       switch (*op++)
 	{
@@ -306,27 +308,33 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  /* String operators. */
 	case OP_STRING_EQ:
 	  sp--;
-	  sp->f = cmp_str (&sp->c[1], sp->c[0], &sp[1].c[1], sp[1].c[0]) == 0;
+	  sp[0].f = st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				    &sp[1].c[1], sp[1].c[0]) == 0;
 	  break;
 	case OP_STRING_GE:
 	  sp--;
-	  sp->f = cmp_str (&sp->c[1], sp->c[0], &sp[1].c[1], sp[1].c[0]) >= 0;
+	  sp[0].f = st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				    &sp[1].c[1], sp[1].c[0]) >= 0;
 	  break;
 	case OP_STRING_GT:
 	  sp--;
-	  sp->f = cmp_str (&sp->c[1], sp->c[0], &sp[1].c[1], sp[1].c[0]) > 0;
+	  sp[0].f = st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				    &sp[1].c[1], sp[1].c[0]) > 0;
 	  break;
 	case OP_STRING_LE:
 	  sp--;
-	  sp->f = cmp_str (&sp->c[1], sp->c[0], &sp[1].c[1], sp[1].c[0]) <= 0;
+	  sp[0].f = st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				    &sp[1].c[1], sp[1].c[0]) <= 0;
 	  break;
 	case OP_STRING_LT:
 	  sp--;
-	  sp->f = cmp_str (&sp->c[1], sp->c[0], &sp[1].c[1], sp[1].c[0]) < 0;
+	  sp[0].f = st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				    &sp[1].c[1], sp[1].c[0]) < 0;
 	  break;
 	case OP_STRING_NE:
 	  sp--;
-	  sp->f = cmp_str (&sp->c[1], sp->c[0], &sp[1].c[1], sp[1].c[0]) != 0;
+	  sp[0].f = st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				    &sp[1].c[1], sp[1].c[0]) != 0;
 	  break;
 
 	  /* Unary functions. */
@@ -462,7 +470,8 @@ evaluate_expression (expression * e, ccase * c, value * v)
 
 	    sp -= n_args - 1;
 	    for (i = 1; i <= n_args; i++)
-	      if (!cmp_str (&sp[0].c[1], sp[0].c[0], &sp[i].c[1], sp[i].c[0]))
+	      if (!st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				   &sp[i].c[1], sp[i].c[0]))
 		{
 		  sp->f = 1.0;
 		  goto main_loop;
@@ -603,9 +612,10 @@ evaluate_expression (expression * e, ccase * c, value * v)
 
 	    sp -= n_args - 1;
 	    for (i = 1; i <= n_args; i += 2)
-	      if (cmp_str (&sp[0].c[1], sp[0].c[0], &sp[i].c[1], sp[i].c[0]) >= 0
-		  && cmp_str (&sp[0].c[1], sp[0].c[0],
-			      &sp[i + 1].c[1], sp[i + 1].c[0]) <= 0)
+	      if (st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				  &sp[i].c[1], sp[i].c[0]) >= 0
+		  && st_compare_pad (&sp[0].c[1], sp[0].c[0],
+				     &sp[i + 1].c[1], sp[i + 1].c[0]) <= 0)
 		{
 		  sp->f = 1.0;
 		  goto main_loop;
@@ -617,8 +627,9 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  {
 	    int n_args = *op++;
 	    int nv = 0;
-	    double sum[2] =
-	    {0.0, 0.0};
+	    double sum[2];
+
+	    sum[0] = sum[1] = 0.0;
 
 	    sp -= n_args - 1;
 	    for (i = 0; i < n_args; i++)
@@ -657,8 +668,9 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  {
 	    int n_args = *op++;
 	    int nv = 0;
-	    double sum[2] =
-	    {0.0, 0.0};
+	    double sum[2];
+
+	    sum[0] = sum[1] = 0.0;
 
 	    sp -= n_args - 1;
 	    for (i = 0; i < n_args; i++)
@@ -752,11 +764,15 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  break;
 	case OP_XDATE_JDAY:
 	  if (sp->f != SYSMIS)
-	    sp->f = unjul (sp->f)->tm_yday + 1;
+	    sp->f = 86400. * julian_to_jday (sp->f / 86400.);
 	  break;
 	case OP_XDATE_MDAY:
 	  if (sp->f != SYSMIS)
-	    sp->f = unjul (sp->f)->tm_mday;
+	    {
+	      int day;
+	      julian_to_calendar (sp->f / 86400., NULL, NULL, &day);
+	      sp->f = day;
+	    }
 	  break;
 	case OP_XDATE_MINUTE:
 	  if (sp->f != SYSMIS)
@@ -764,11 +780,19 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  break;
 	case OP_XDATE_MONTH:
 	  if (sp->f != SYSMIS)
-	    sp->f = unjul (sp->f)->tm_mon + 1;
+	    {
+	      int month;
+	      julian_to_calendar (sp->f / 86400., NULL, &month, NULL);
+	      sp->f = month;
+	    }
 	  break;
 	case OP_XDATE_QUARTER:
 	  if (sp->f != SYSMIS)
-	    sp->f = (unjul (sp->f)->tm_mon - 1.) / 3. + 1.;
+	    {
+	      int month;
+	      julian_to_calendar (sp->f / 86400., NULL, &month, NULL);
+	      sp->f = (month - 1) / 3 + 1;
+	    }
 	  break;
 	case OP_XDATE_SECOND:
 	  if (sp->f != SYSMIS)
@@ -784,15 +808,19 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  break;
 	case OP_XDATE_WEEK:
 	  if (sp->f != SYSMIS)
-	    sp->f = unjul (sp->f)->tm_yday / 7. + 1.;
+	    sp->f = (julian_to_jday (sp->f / 86400.) - 1) / 7 + 1;
 	  break;
 	case OP_XDATE_WKDAY:
 	  if (sp->f != SYSMIS)
-	    sp->f = unjul (sp->f)->tm_wday + 1.;
+	    sp->f = julian_to_wday (sp->f / 86400.);
 	  break;
 	case OP_XDATE_YEAR:
 	  if (sp->f != SYSMIS)
-	    sp->f = unjul (sp->f)->tm_year + 1900.;
+	    {
+	      int year;
+	      julian_to_calendar (sp->f / 86400., &year, NULL, NULL);
+	      sp->f = year;
+	    }
 	  break;
 
 	  /* String functions. */
@@ -1048,30 +1076,36 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  break;
 	case OP_NUMBER:
 	  {
-	    static fmt_spec f =
-	    {FMT_F, 0, 0};
+	    struct data_in di;
 
-	    f.w = sp->c[0];
-	    fmt_parse_ignore_error = 1;
-	    parse_string_as_format (&sp->c[1], sp->c[0], &f, 1, sp);
-	    fmt_parse_ignore_error = 0;
+	    di.s = &sp->c[1];
+	    di.e = &sp->c[1] + sp->c[0];
+	    di.v = sp;
+	    di.flags = DI_IGNORE_ERROR;
+	    di.f1 = 1;
+	    di.format.type = FMT_F;
+	    di.format.w = sp->c[0];
+	    di.format.d = 0;
+	    data_in (&di);
 	  }
 	  break;
 	case OP_NUMBER_OPT:
 	  {
-	    fmt_spec f;
-	    f.type = *op++;
-	    f.w = *op++;
-	    f.d = *op++;
-
-	    fmt_parse_ignore_error = 1;
-	    parse_string_as_format (&sp->c[1], sp->c[0], &f, 1, sp);
-	    fmt_parse_ignore_error = 0;
+	    struct data_in di;
+	    di.s = &sp->c[1];
+	    di.e = &sp->c[1] + sp->c[0];
+	    di.v = sp;
+	    di.flags = DI_IGNORE_ERROR;
+	    di.f1 = 1;
+	    di.format.type = *op++;
+	    di.format.w = *op++;
+	    di.format.d = *op++;
+	    data_in (&di);
 	  }
 	  break;
 	case OP_STRING:
 	  {
-	    fmt_spec f;
+	    struct fmt_spec f;
 	    unsigned char *dest;
 
 	    f.type = *op++;
@@ -1082,7 +1116,7 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	    dest = ALLOC_STRING_SPACE (f.w);
 	    dest[0] = f.w;
 
-	    convert_format_to_string (&dest[1], &f, sp);
+	    data_out (&dest[1], &f, sp);
 	    sp->c = dest;
 	  }
 	  break;
@@ -1144,9 +1178,9 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  else if (sp->f != SYSMIS)
 	    {
 	      msg (SE, _("A number being treated as a Boolean in an "
-		   "expression was found to have a value other than "
-		   "0 (false), 1 (true), or the system-missing value.  "
-		   "The result was forced to 0."));
+			 "expression was found to have a value other than "
+			 "0 (false), 1 (true), or the system-missing value.  "
+			 "The result was forced to 0."));
 	      sp->f = 0.0;
 	    }
 	  break;
@@ -1182,17 +1216,17 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	case OP_VEC_ELEM_NUM:
 	  {
 	    int rindx = sp[0].f + EPSILON;
-	    vector *v = &vec[*op++];
+	    struct vector *v = &vec[*op++];
 
 	    if (sp[0].f == SYSMIS || rindx < 1 || rindx > v->nv)
 	      {
 		if (sp[0].f == SYSMIS)
 		  msg (SE, _("SYSMIS is not a valid index value for vector "
-		       "%s.  The result will be set to SYSMIS."),
+			     "%s.  The result will be set to SYSMIS."),
 		       v->name);
 		else
 		  msg (SE, _("%g is not a valid index value for vector %s.  "
-		       "The result will be set to SYSMIS."),
+			     "The result will be set to SYSMIS."),
 		       sp[0].f, v->name);
 		sp->f = SYSMIS;
 		break;
@@ -1203,18 +1237,19 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	case OP_VEC_ELEM_STR:
 	  {
 	    int rindx = sp[0].f + EPSILON;
-	    vector *vect = &vec[*op++];
-	    variable *v;
+	    struct vector *vect = &vec[*op++];
+	    struct variable *v;
 
 	    if (sp[0].f == SYSMIS || rindx < 1 || rindx > vect->nv)
 	      {
 		if (sp[0].f == SYSMIS)
 		  msg (SE, _("SYSMIS is not a valid index value for vector "
-		       "%s.  The result will be set to the empty "
-		       "string."), vect->name);
+			     "%s.  The result will be set to the empty "
+			     "string."),
+		       vect->name);
 		else
 		  msg (SE, _("%g is not a valid index value for vector %s.  "
-		       "The result will be set to the empty string."),
+			     "The result will be set to the empty string."),
 		       sp[0].f, vect->name);
 		CHECK_STRING_SPACE (0);
 		sp->c = ALLOC_STRING_SPACE (0);
@@ -1263,7 +1298,7 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  break;
 	case OP_NUM_LAG:
 	  {
-	    ccase *c = lagged_case (*op++);
+	    struct ccase *c = lagged_case (*op++);
 
 	    sp++;
 	    if (c == NULL)
@@ -1279,7 +1314,7 @@ evaluate_expression (expression * e, ccase * c, value * v)
 	  }
 	case OP_STR_LAG:
 	  {
-	    ccase *c = lagged_case (*op++);
+	    struct ccase *c = lagged_case (*op++);
 	    int width = (*vars)->width;
 
 	    sp++;
@@ -1318,7 +1353,7 @@ evaluate_expression (expression * e, ccase * c, value * v)
 
 #if __CHECKER__
 	  /* This case prevents Checker from choking. */
-	case (operator) 42000:
+	case 42000:
 	  assert (0);
 #endif
 
@@ -1349,8 +1384,8 @@ finished:
       assert (v);
 
 #if PAGED_STACK
-      memcpy (e->str_stk, sp->c, sp->c[0] + 1);
-      v->c = e->str_stk;
+      memcpy (e->str_stack, sp->c, sp->c[0] + 1);
+      v->c = e->str_stack;
 #else
       v->c = sp->c;
 #endif
