@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -23,16 +23,17 @@
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
+#include "alloc.h"
 #include "approx.h"
-#include "common.h"
+#include "data-in.h"
 #include "error.h"
 #include "expr.h"
+#include "exprP.h"
 #include "julcal/julcal.h"
 #include "misc.h"
 #include "stats.h"
-#include "var.h"
-#include "exprP.h"
 #include "str.h"
+#include "var.h"
 
 /*
    Expression "optimizer"
@@ -56,11 +57,11 @@
    is here could already be considered overkill.
  */
 
-static nonterm_node *evaluate_tree (nonterm_node * n);
-static nonterm_node *optimize_tree (nonterm_node * n);
+static struct nonterm_node *evaluate_tree (struct nonterm_node * n);
+static struct nonterm_node *optimize_tree (struct nonterm_node * n);
 
-nonterm_node *
-optimize_expression (nonterm_node * n)
+struct nonterm_node *
+optimize_expression (struct nonterm_node * n)
 {
   int i;
 
@@ -77,11 +78,11 @@ optimize_expression (nonterm_node * n)
   /* Start by optimizing all the children. */
   for (i = 0; i < n->n; i++)
     {
-      n->arg[i] = ((any_node *)
-		   optimize_expression ((nonterm_node *) n->arg[i]));
+      n->arg[i] = ((union any_node *)
+		   optimize_expression ((struct nonterm_node *) n->arg[i]));
       if (n->arg[i]->type == OP_NUM_CON)
 	{
-	  if (n->arg[i]->nmc.value == SYSMIS)
+	  if (n->arg[i]->num_con.value == SYSMIS)
 	    sysmis++;
 	}
       else if (n->arg[i]->type != OP_STR_CON)
@@ -92,11 +93,11 @@ optimize_expression (nonterm_node * n)
     /* Just about any operation produces SYSMIS when given any SYSMIS
        arguments. */
     {
-      num_con_node *num = xmalloc (sizeof (num_con_node));
-      free_node ((any_node *) n);
+      struct num_con_node *num = xmalloc (sizeof *num);
+      free_node ((union any_node *) n);
       num->type = OP_NUM_CON;
       num->value = SYSMIS;
-      n = (nonterm_node *) num;
+      n = (struct nonterm_node *) num;
     }
   else if (!nonconst)
     /* If all the children of this node are constants, then there are
@@ -109,25 +110,25 @@ optimize_expression (nonterm_node * n)
   return n;
 }
 
-static nonterm_node *repl_num_con (nonterm_node *, double);
-static nonterm_node *force_repl_num_con (nonterm_node *, double);
-static nonterm_node *repl_str_con (nonterm_node *, char *, int);
+static struct nonterm_node *repl_num_con (struct nonterm_node *, double);
+static struct nonterm_node *force_repl_num_con (struct nonterm_node *, double);
+static struct nonterm_node *repl_str_con (struct nonterm_node *, char *, int);
 
-#define n0 n->arg[0]->nmc.value
-#define n1 n->arg[1]->nmc.value
-#define n2 n->arg[2]->nmc.value
+#define n0 n->arg[0]->num_con.value
+#define n1 n->arg[1]->num_con.value
+#define n2 n->arg[2]->num_con.value
 
-#define s0 n->arg[0]->stc.s
-#define s0l n->arg[0]->stc.len
-#define s1 n->arg[1]->stc.s
-#define s1l n->arg[1]->stc.len
-#define s2 n->arg[2]->stc.s
-#define s2l n->arg[2]->stc.len
-#define s(X) n->arg[X]->stc.s
-#define sl(X) n->arg[X]->stc.len
+#define s0 n->arg[0]->str_con.s
+#define s0l n->arg[0]->str_con.len
+#define s1 n->arg[1]->str_con.s
+#define s1l n->arg[1]->str_con.len
+#define s2 n->arg[2]->str_con.s
+#define s2l n->arg[2]->str_con.len
+#define s(X) n->arg[X]->str_con.s
+#define sl(X) n->arg[X]->str_con.len
 
-static nonterm_node *
-optimize_tree (nonterm_node * n)
+static struct nonterm_node *
+optimize_tree (struct nonterm_node * n)
 {
   int i;
 
@@ -144,7 +145,7 @@ optimize_tree (nonterm_node * n)
       int nvar = 0;
 
       /* New node. */
-      nonterm_node *m;
+      struct nonterm_node *m;
 
       /* Argument copying counter. */
       int c;
@@ -155,12 +156,12 @@ optimize_tree (nonterm_node * n)
       for (i = 0; i < n->n; i++)
 	if (n->arg[i]->type == OP_NUM_CON)
 	  {
-	    if (n->arg[i]->nmc.value != SYSMIS)
+	    if (n->arg[i]->num_con.value != SYSMIS)
 	      {
 		if (n->type == OP_MUL)
-		  cval *= n->arg[i]->nmc.value;
+		  cval *= n->arg[i]->num_con.value;
 		else
-		  cval += n->arg[i]->nmc.value;
+		  cval += n->arg[i]->num_con.value;
 	      }
 	    else
 	      sysmis++;
@@ -188,15 +189,16 @@ optimize_tree (nonterm_node * n)
 	     terms, replace with the nonconstant term. */
 	  for (i = 0; i < n->n; i++)
 	    if (n->arg[i]->type != OP_NUM_CON)
-	      m = (nonterm_node *) n->arg[i];
+	      m = (struct nonterm_node *) n->arg[i];
 	    else
 	      free_node (n->arg[i]);
 	}
       else
 	{
 	  /* Otherwise consolidate all the nonconstant terms. */
-	  m = xmalloc (sizeof (nonterm_node)
-		+ (nvar + approx_ne (cval, def) - 1) * sizeof (any_node *));
+	  m = xmalloc (sizeof (struct nonterm_node)
+		       + ((nvar + approx_ne (cval, def) - 1)
+			  * sizeof (union any_node *)));
 	  for (i = c = 0; i < n->n; i++)
 	    if (n->arg[i]->type != OP_NUM_CON)
 	      m->arg[c++] = n->arg[i];
@@ -205,9 +207,9 @@ optimize_tree (nonterm_node * n)
 
 	  if (approx_ne (cval, def))
 	    {
-	      m->arg[c] = xmalloc (sizeof (num_con_node));
-	      m->arg[c]->nmc.type = OP_NUM_CON;
-	      m->arg[c]->nmc.value = cval;
+	      m->arg[c] = xmalloc (sizeof (struct num_con_node));
+	      m->arg[c]->num_con.type = OP_NUM_CON;
+	      m->arg[c]->num_con.value = cval;
 	      c++;
 	    }
 
@@ -223,7 +225,7 @@ optimize_tree (nonterm_node * n)
 	{
 	  if (approx_eq (n1, 1.0))
 	    {
-	      nonterm_node *m = (nonterm_node *) n->arg[0];
+	      struct nonterm_node *m = (struct nonterm_node *) n->arg[0];
 
 	      free_node (n->arg[1]);
 	      free (n);
@@ -231,7 +233,7 @@ optimize_tree (nonterm_node * n)
 	    }
 	  else if (approx_eq (n1, 2.0))
 	    {
-	      n = xrealloc (n, sizeof (nonterm_node));
+	      n = xrealloc (n, sizeof (struct nonterm_node));
 	      n->type = OP_SQUARE;
 	      n->n = 1;
 	    }
@@ -260,12 +262,12 @@ str_search (char *haystack, int haystack_len, char *needle, int needle_len)
 static inline int
 str_rsearch (char *haystack, int haystack_len, char *needle, int needle_len)
 {
-  char *p = memrmem (haystack, haystack_len, needle, needle_len);
+  char *p = mm_find_reverse (haystack, haystack_len, needle, needle_len);
   return p ? p - haystack + 1 : 0;
 }
 
-static nonterm_node *
-evaluate_tree (nonterm_node * n)
+static struct nonterm_node *
+evaluate_tree (struct nonterm_node * n)
 {
   static char *strbuf;
   int add;
@@ -273,7 +275,7 @@ evaluate_tree (nonterm_node * n)
   int i;
 
   if (!strbuf)
-    strbuf = xmalloc (sizeof (char[256]));
+    strbuf = xmalloc (256);
   errno = 0;
 
   switch (n->type)
@@ -334,22 +336,22 @@ evaluate_tree (nonterm_node * n)
 
       /* String operators. */
     case OP_STRING_EQ:
-      rnc (cmp_str (s0, s0l, s1, s1l) == 0);
+      rnc (st_compare_pad (s0, s0l, s1, s1l) == 0);
       break;
     case OP_STRING_GE:
-      rnc (cmp_str (s0, s0l, s1, s1l) >= 0);
+      rnc (st_compare_pad (s0, s0l, s1, s1l) >= 0);
       break;
     case OP_STRING_GT:
-      rnc (cmp_str (s0, s0l, s1, s1l) > 0);
+      rnc (st_compare_pad (s0, s0l, s1, s1l) > 0);
       break;
     case OP_STRING_LE:
-      rnc (cmp_str (s0, s0l, s1, s1l) <= 0);
+      rnc (st_compare_pad (s0, s0l, s1, s1l) <= 0);
       break;
     case OP_STRING_LT:
-      rnc (cmp_str (s0, s0l, s1, s1l) < 0);
+      rnc (st_compare_pad (s0, s0l, s1, s1l) < 0);
       break;
     case OP_STRING_NE:
-      rnc (cmp_str (s0, s0l, s1, s1l) != 0);
+      rnc (st_compare_pad (s0, s0l, s1, s1l) != 0);
       break;
 
       /* Unary functions. */
@@ -410,7 +412,7 @@ evaluate_tree (nonterm_node * n)
 
 	  for (i = 1; i < n->n; i++)
 	    {
-	      ni = n->arg[i]->nmc.value;
+	      ni = n->arg[i]->num_con.value;
 	      if (approx_eq (n0, ni))
 		{
 		  frnc (1.0);
@@ -425,8 +427,8 @@ evaluate_tree (nonterm_node * n)
       break;
     case OP_ANY_STRING:
       for (i = 1; i < n->n; i++)
-	if (!cmp_str (n->arg[0]->stc.s, n->arg[0]->stc.len,
-		      n->arg[i]->stc.s, n->arg[i]->stc.len))
+	if (!st_compare_pad (n->arg[0]->str_con.s, n->arg[0]->str_con.len,
+			     n->arg[i]->str_con.s, n->arg[i]->str_con.len))
 	  {
 	    frnc (1.0);
 	    goto any_string_done;
@@ -454,7 +456,7 @@ evaluate_tree (nonterm_node * n)
 
 	for (i = 0; i < n->n; i++)
 	  {
-	    ni = n->arg[i]->nmc.value;
+	    ni = n->arg[i]->num_con.value;
 	    if (ni != SYSMIS)
 	      {
 		nv++;
@@ -511,8 +513,8 @@ evaluate_tree (nonterm_node * n)
 
 	  for (i = 1; i < n->n; i += 2)
 	    {
-	      min = n->arg[i]->nmc.value;
-	      max = n->arg[i + 1]->nmc.value;
+	      min = n->arg[i]->num_con.value;
+	      max = n->arg[i + 1]->num_con.value;
 	      if (min == SYSMIS || max == SYSMIS)
 		continue;
 	      sysmis = 0;
@@ -528,10 +530,11 @@ evaluate_tree (nonterm_node * n)
       break;
     case OP_RANGE_STRING:
       for (i = 1; i < n->n; i += 2)
-	if (cmp_str (n->arg[0]->stc.s, n->arg[0]->stc.len,
-		     n->arg[i]->stc.s, n->arg[i]->stc.len) >= 0
-	    && cmp_str (n->arg[0]->stc.s, n->arg[0]->stc.len,
-			n->arg[i + 1]->stc.s, n->arg[i + 1]->stc.len) <= 0)
+	if (st_compare_pad (n->arg[0]->str_con.s, n->arg[0]->str_con.len,
+			    n->arg[i]->str_con.s, n->arg[i]->str_con.len) >= 0
+	    && st_compare_pad (n->arg[0]->str_con.s, n->arg[0]->str_con.len,
+			       n->arg[i + 1]->str_con.s,
+			       n->arg[i + 1]->str_con.len) <= 0)
 	  {
 	    frnc (1.0);
 	    goto range_str_done;
@@ -585,19 +588,31 @@ evaluate_tree (nonterm_node * n)
       rnc (fmod (floor (n0 / 60. / 60.), 24.));
       break;
     case OP_XDATE_JDAY:
-      rnc (unjul (n0)->tm_yday + 1);
+      rnc (julian_to_jday (n0 / 86400.));
       break;
     case OP_XDATE_MDAY:
-      rnc (unjul (n0)->tm_mday);
+      {
+	int day;
+	julian_to_calendar (n0 / 86400., NULL, NULL, &day);
+	rnc (day);
+      }
       break;
     case OP_XDATE_MINUTE:
       rnc (fmod (floor (n0 / 60.), 60.));
       break;
     case OP_XDATE_MONTH:
-      rnc (unjul (n0)->tm_mon + 1);
+      {
+	int month;
+	julian_to_calendar (n0 / 86400., NULL, &month, NULL);
+	rnc (month);
+      }
       break;
     case OP_XDATE_QUARTER:
-      rnc ((unjul (n0)->tm_mon - 1) / 3 + 1);
+      {
+	int month;
+	julian_to_calendar (n0 / 86400., NULL, &month, NULL);
+	rnc ((month - 1) / 3 + 1);
+      }
       break;
     case OP_XDATE_SECOND:
       rnc (fmod (n0, 60.));
@@ -609,13 +624,17 @@ evaluate_tree (nonterm_node * n)
       rnc (n0 - floor (n0 / 60. / 60. / 24.) * 60. * 60. * 24.);
       break;
     case OP_XDATE_WEEK:
-      rnc (unjul (n0)->tm_yday / 7 + 1);
+      rnc ((julian_to_jday (n0) - 1) / 7 + 1);
       break;
     case OP_XDATE_WKDAY:
-      rnc (unjul (n0)->tm_wday + 1);
+      rnc (julian_to_wday (n0));
       break;
     case OP_XDATE_YEAR:
-      rnc (unjul (n0)->tm_year + 1900);
+      {
+	int year;
+	julian_to_calendar (n0 / 86400., &year, NULL, NULL);
+	rnc (year);
+      }
       break;
 
       /* String functions. */
@@ -780,39 +799,42 @@ evaluate_tree (nonterm_node * n)
     case OP_NUMBER:
     case OP_NUMBER_OPT:
       {
-	value v;
-#if __GNUC__ && !__STRICT_ANSI__
-	fmt_spec f = {FMT_F, s0l, 0};
-#else
-	fmt_spec f;
-	f.type = FMT_F;
-	f.w = s0l;
-	f.d = 0;
-#endif
+	union value v;
+	struct data_in di;
+
+	di.s = s0;
+	di.e = s0 + s0l;
+	di.v = &v;
+	di.flags = DI_IGNORE_ERROR;
+	di.f1 = 1;
 
 	if (n->type == OP_NUMBER_OPT)
-	  f.type = (int) n->arg[1], f.w = (int) n->arg[2], f.d = (int) n->arg[3];
-
-	fmt_parse_ignore_error = 1;
-	parse_string_as_format (s0, s0l, &f, 1, &v);
-	fmt_parse_ignore_error = 0;
+	  {
+	    di.format.type = (int) n->arg[1];
+	    di.format.w = (int) n->arg[2];
+	    di.format.d = (int) n->arg[3];
+	  }
+	else
+	  {
+	    di.format.type = FMT_F;
+	    di.format.w = s0l;
+	    di.format.d = 0;
+	  }
+	
+	data_in (&di);
 	frnc (v.f);
       }
       break;
     case OP_STRING:
       {
-	value v;
-#if __GNUC__ && !__STRICT_ANSI__
-	fmt_spec f = {(int) n->arg[1], (int) n->arg[2], (int) n->arg[3]};
-#else
-	fmt_spec f;
+	union value v;
+	struct fmt_spec f;
 	f.type = (int) n->arg[1];
 	f.w = (int) n->arg[2];
 	f.d = (int) n->arg[3];
-#endif
 	v.f = n0;
 
-	convert_format_to_string (strbuf, &f, &v);
+	data_out (strbuf, &f, &v);
 	n = repl_str_con (n, strbuf, f.w);
       }
       break;
@@ -888,15 +910,15 @@ evaluate_tree (nonterm_node * n)
 #undef rnc
 #undef frnc
 
-static nonterm_node *
-repl_num_con (nonterm_node * n, double d)
+static struct nonterm_node *
+repl_num_con (struct nonterm_node * n, double d)
 {
   int i;
   if (!finite (d) || errno)
     d = SYSMIS;
   else
     for (i = 0; i < n->n; i++)
-      if (n->arg[i]->type == OP_NUM_CON && n->arg[i]->nmc.value == SYSMIS)
+      if (n->arg[i]->type == OP_NUM_CON && n->arg[i]->num_con.value == SYSMIS)
 	{
 	  d = SYSMIS;
 	  break;
@@ -904,33 +926,33 @@ repl_num_con (nonterm_node * n, double d)
   return force_repl_num_con (n, d);
 }
 
-static nonterm_node *
-force_repl_num_con (nonterm_node * n, double d)
+static struct nonterm_node *
+force_repl_num_con (struct nonterm_node * n, double d)
 {
-  num_con_node *num;
+  struct num_con_node *num;
 
   if (!finite (d) || errno)
     d = SYSMIS;
-  free_node ((any_node *) n);
-  num = xmalloc (sizeof (num_con_node));
+  free_node ((union any_node *) n);
+  num = xmalloc (sizeof *num);
   num->type = OP_NUM_CON;
   num->value = d;
-  return (nonterm_node *) num;
+  return (struct nonterm_node *) num;
 }
 
-static nonterm_node *
-repl_str_con (nonterm_node * n, char *s, int len)
+static struct nonterm_node *
+repl_str_con (struct nonterm_node * n, char *s, int len)
 {
-  str_con_node *str;
+  struct str_con_node *str;
 
   /* The ordering here is important since the source string may be
      part of a subnode of n. */
-  str = xmalloc (sizeof (str_con_node) + (len - 1) * sizeof (char));
+  str = xmalloc (sizeof *str + len - 1);
   str->type = OP_STR_CON;
   str->len = len;
   memcpy (str->s, s, len);
-  free_node ((any_node *) n);
-  return (nonterm_node *) str;
+  free_node ((union any_node *) n);
+  return (struct nonterm_node *) str;
 }
 
 /* Returns the number of days since 10 Oct 1582 for the date
@@ -939,8 +961,6 @@ repl_str_con (nonterm_node * n, char *s, int len)
 double
 yrmoda (double year, double month, double day)
 {
-  struct tm t;
-
   if (year == SYSMIS || month == SYSMIS || day == SYSMIS)
     return SYSMIS;
 
@@ -957,30 +977,27 @@ yrmoda (double year, double month, double day)
       || (month < -1 || month > 13)
       || (day < -1 || day > 32))
     return SYSMIS;
-  t.tm_year = year - 1900;
-  t.tm_mon = month - 1;
-  t.tm_mday = day;
-  return juldnj (&t, JUL_ROME) - julian_offset;
+  return calendar_to_julian (year, month, day);
 }
 
 /* Expression dumper. */
 
-static expression *e;
+static struct expression *e;
 static int nop, mop;
 static int ndbl, mdbl;
 static int nstr, mstr;
 static int nvars, mvars;
 
-static void dump_node (any_node * n);
-static void emit (operator);
+static void dump_node (union any_node * n);
+static void emit (int);
 static void emit_num_con (double);
 static void emit_str_con (char *, int);
-static void emit_var (variable *);
+static void emit_var (struct variable *);
 
 void
-dump_expression (any_node * n, expression * expr)
+dump_expression (union any_node * n, struct expression * expr)
 {
-  operator *o;
+  unsigned char *o;
 
   int height = 0;
 
@@ -988,9 +1005,9 @@ dump_expression (any_node * n, expression * expr)
 
   e = expr;
   e->op = NULL;
-  e->dbl = NULL;
+  e->num = NULL;
   e->str = NULL;
-  e->vars = NULL;
+  e->var = NULL;
   nop = mop = 0;
   ndbl = mdbl = 0;
   nstr = mstr = 0;
@@ -1015,27 +1032,27 @@ dump_expression (any_node * n, expression * expr)
      block.  If only all the world were a VAX... */
   max_height++;
 
-  e->stack = xmalloc (max_height * sizeof (value));
+  e->stack = xmalloc (max_height * sizeof *e->stack);
 
 #if PAGED_STACK
-  e->str_stk = e->type == EX_STRING ? xmalloc (256) : NULL;
+  e->str_stack = e->type == EX_STRING ? xmalloc (256) : NULL;
 #else
-  e->str_stk = xmalloc (256);
+  e->str_stack = xmalloc (256);
   e->str_size = 256;
 #endif
 }
 
 static void
-dump_node (any_node * n)
+dump_node (union any_node * n)
 {
   if (n->type == OP_AND || n->type == OP_OR)
     {
       int i;
 
-      dump_node (n->nnt.arg[0]);
-      for (i = 1; i < n->nnt.n; i++)
+      dump_node (n->nonterm.arg[0]);
+      for (i = 1; i < n->nonterm.n; i++)
 	{
-	  dump_node (n->nnt.arg[i]);
+	  dump_node (n->nonterm.arg[i]);
 	  emit (n->type);
 	}
       return;
@@ -1043,27 +1060,27 @@ dump_node (any_node * n)
   else if (n->type < OP_TERMINAL)
     {
       int i;
-      for (i = 0; i < n->nnt.n; i++)
-	dump_node (n->nnt.arg[i]);
+      for (i = 0; i < n->nonterm.n; i++)
+	dump_node (n->nonterm.arg[i]);
       emit (n->type);
       if (ops[n->type].flags & OP_VAR_ARGS)
-	emit (n->nnt.n);
+	emit (n->nonterm.n);
       if (ops[n->type].flags & OP_MIN_ARGS)
-	emit ((int) n->nnt.arg[n->nnt.n]);
+	emit ((int) n->nonterm.arg[n->nonterm.n]);
       if (ops[n->type].flags & OP_FMT_SPEC)
 	{
-	  emit ((int) n->nnt.arg[n->nnt.n]);
-	  emit ((int) n->nnt.arg[n->nnt.n + 1]);
-	  emit ((int) n->nnt.arg[n->nnt.n + 2]);
+	  emit ((int) n->nonterm.arg[n->nonterm.n]);
+	  emit ((int) n->nonterm.arg[n->nonterm.n + 1]);
+	  emit ((int) n->nonterm.arg[n->nonterm.n + 2]);
 	}
       return;
     }
 
   emit (n->type);
   if (n->type == OP_NUM_CON)
-    emit_num_con (n->nmc.value);
+    emit_num_con (n->num_con.value);
   else if (n->type == OP_STR_CON)
-    emit_str_con (n->stc.s, n->stc.len);
+    emit_str_con (n->str_con.s, n->str_con.len);
   else if (n->type == OP_NUM_VAR || n->type == OP_STR_VAR
 	   || n->type == OP_STR_MIS)
     emit_var (n->var.v);
@@ -1073,18 +1090,18 @@ dump_node (any_node * n)
       emit (n->lag.lag);
     }
   else if (n->type == OP_NUM_SYS || n->type == OP_NUM_VAL)
-    emit (n->sys.v->fv);
-  else if (n->type != OP_CASENUM)
-    assert (0);
+    emit (n->var.v->fv);
+  else
+    assert (n->type == OP_CASENUM);
 }
 
 static void
-emit (operator op)
+emit (int op)
 {
   if (nop >= mop)
     {
       mop += 16;
-      e->op = xrealloc (e->op, mop * sizeof (operator));
+      e->op = xrealloc (e->op, mop * sizeof *e->op);
     }
   e->op[nop++] = op;
 }
@@ -1095,9 +1112,9 @@ emit_num_con (double dbl)
   if (ndbl >= mdbl)
     {
       mdbl += 16;
-      e->dbl = xrealloc (e->dbl, mdbl * sizeof (double));
+      e->num = xrealloc (e->num, mdbl * sizeof *e->num);
     }
-  e->dbl[ndbl++] = dbl;
+  e->num[ndbl++] = dbl;
 }
 
 static void
@@ -1106,7 +1123,7 @@ emit_str_con (char *str, int len)
   if (nstr + len + 1 > mstr)
     {
       mstr += 256;
-      e->str = xrealloc (e->str, mstr * sizeof (char));
+      e->str = xrealloc (e->str, mstr);
     }
   e->str[nstr++] = len;
   memcpy (&e->str[nstr], str, len);
@@ -1114,12 +1131,12 @@ emit_str_con (char *str, int len)
 }
 
 static void
-emit_var (variable * v)
+emit_var (struct variable * v)
 {
   if (nvars >= mvars)
     {
       mvars += 16;
-      e->vars = xrealloc (e->vars, mvars * sizeof (variable *));
+      e->var = xrealloc (e->var, mvars * sizeof *e->var);
     }
-  e->vars[nvars++] = v;
+  e->var[nvars++] = v;
 }
