@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -39,11 +39,11 @@ char *alloca ();
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
-#include "common.h"
+#include "alloc.h"
 #include "error.h"
-#include "str.h"
 #include "filename.h"
 #include "settings.h"
+#include "str.h"
 #include "version.h"
 
 #undef DEBUGGING
@@ -70,283 +70,290 @@ char *alloca ();
 #if __DJGPP__
 #include <sys/stat.h>
 #endif
+
+/* Initialization. */
 
 const char *config_path;
 
 void
-init_filename (void)
+fn_init (void)
 {
-  config_path = getenv_default ("STAT_CONFIG_PATH", default_config_path);
+  config_path = fn_getenv_default ("STAT_CONFIG_PATH", default_config_path);
 }
-
-/* Allocates room for at least AMT + *M_OUT characters in *OUTPUT and
-   sets *END and *OP appropriately. */
-static void
-expand_line (char **output, char **end, char **op, int *m_out, int amt)
-{
-  int n_out = *op - *output;
-
-  amt += *m_out;
-  do
-    *m_out *= 2;
-  while (*m_out < amt);
-  *output = xrealloc (*output, *m_out);
-  *end = &(*output)[*m_out];
-  *op = &(*output)[n_out];
-}
-
-#define EXPAND_LINE(AMT)					\
-	do							\
-	  {							\
-	    if (op + AMT >= end)				\
-	      expand_line (&output, &end, &op, &m_out, AMT);	\
-	  }							\
-	while (0)
+
+/* Functions for performing operations on filenames. */
 
 /* Substitutes $variables as defined by GETENV into INPUT and returns
    a copy of the resultant string.  Supports $var and ${var} syntaxes;
    $$ substitutes as $. */
 char *
-interp_vars (const char *input, const char *(*getenv) (const char *))
+fn_interp_vars (const char *input, const char *(*getenv) (const char *))
 {
-  char *output, *end;
-  const char *ip;
-  char *op;
-  int m_out = 1024;
+  struct string output;
 
   if (NULL == strchr (input, '$'))
     return xstrdup (input);
-  output = xmalloc (m_out);
-  end = &output[m_out];
-  ip = input;
-  op = output;
-  while (*ip)
-    if (*ip != '$')
-      {
-	EXPAND_LINE (1);
-	*op++ = *ip++;
-	continue;
-      }
-    else
-      {
-	ip++;
-	if (*ip == '$')
-	  {
-	    EXPAND_LINE (1);
-	    *op++ = *ip++;
-	    continue;
-	  }
-	if (*ip == '{')
-	  {
-	    char *sop = op;
-	    const char *val;
 
-	    ip++;
-	    while (*ip && *ip != '}')
-	      {
-		EXPAND_LINE (1);
-		*op++ = *ip++;
-	      }
-	    *op = 0;
+  ds_init (NULL, &output, strlen (input));
 
-	    val = getenv (sop);
-	    op = sop;
-	    if (val)
-	      {
-		EXPAND_LINE (strlen (val));
-		op = stpcpy (op, val);
-	      }
-	    if (*ip == '}')
-	      ip++;
+  for (;;)
+    switch (*input)
+      {
+      case '\0':
+	return ds_value (&output);
+	
+      case '$':
+	input++;
+
+	if (*input == '$')
+	  {
+	    ds_putchar (&output, '$');
+	    input++;
 	  }
 	else
 	  {
-	    char *sop = op;
-	    const char *val;
+	    int stop;
+	    int start;
+	    const char *value;
 
-	    if (isalpha ((unsigned char) *ip))
-	      while (*ip && isalpha ((unsigned char) *ip))
-		{
-		  EXPAND_LINE (1);
-		  *op++ = *ip++;
-		}
-	    else if (*ip)
-	      {
-		EXPAND_LINE (1);
-		*op++ = *ip++;
-	      }
-	    *op = 0;
+	    start = ds_length (&output);
 
-	    val = getenv (sop);
-	    op = sop;
-	    if (val)
+	    if (*input == '(')
 	      {
-		EXPAND_LINE (strlen (val));
-		op = stpcpy (op, val);
+		stop = ')';
+		input++;
 	      }
+	    else if (*input == '{')
+	      {
+		stop = '}';
+		input++;
+	      }
+	    else
+	      stop = 0;
+
+	    while (*input && *input != stop
+		   && (stop || isalpha ((unsigned char) *input)))
+	      ds_putchar (&output, *input++);
+	    
+	    value = getenv (ds_value (&output) + start);
+	    ds_truncate (&output, start);
+	    ds_concat (&output, value);
+
+	    if (stop && *input == stop)
+	      input++;
 	  }
-      }
-  *op++ = 0;
-  return xrealloc (output, op - output);
-}
 
-/* Returns the current working directory, as a malloc()'d string.
-   From libc.info. */
-char *
-gnu_getcwd (void)
-{
-  int size = 100;
-  char *buffer = (char *) xmalloc (size);
-     
-  while (1)
-    {
-      char *value = getcwd (buffer, size);
-      if (value != 0)
-	return buffer;
-      size *= 2;
-      free (buffer);
-      buffer = (char *) xmalloc (size);
-    }
+      default:
+	ds_putchar (&output, *input++);
+      }
 }
 
 #if unix
 /* Expands csh tilde notation from the path INPUT into a malloc()'d
    returned string. */
 char *
-tilde_expand (const char *input)
+fn_tilde_expand (const char *input)
 {
-  char *output, *end;
   const char *ip;
-  char *op;
-  int m_out = 1024;
+  struct string output;
 
   if (NULL == strchr (input, '~'))
     return xstrdup (input);
-  output = xmalloc (m_out);
-  end = &output[m_out];
+  ds_init (NULL, &output, strlen (input));
+
   ip = input;
-  op = output;
-  for (ip = input; *ip;)
-    {
-      if (*ip != '~')
-	{
-	  EXPAND_LINE (1);
-	  *op++ = *ip++;
-	}
-      else if (ip == input || ip[-1] == PATH_DELIMITER)
-	{
-	  char *cp;
 
-	  ip++;
-	  cp = strchr (ip, DIR_SEPARATOR);
-	  if (!cp)
-	    cp = strchr (ip, 0);
-	  if (cp > ip)
-	    {
-	      struct passwd *pwd;
-	      char username[9];
+  for (ip = input; *ip; )
+    if (*ip != '~' || (ip != input && ip[-1] != PATH_DELIMITER))
+      ds_putchar (&output, *ip++);
+    else
+      {
+	static const char stop_set[3] = {DIR_SEPARATOR, PATH_DELIMITER, 0};
+	const char *cp;
+	
+	ip++;
 
-	      strncpy (username, ip, cp - ip + 1);
-	      username[8] = 0;
-	      pwd = getpwnam (username);
+	cp = ip + strcspn (ip, stop_set);
 
-	      if (!pwd || !pwd->pw_dir)
-		{
-		  EXPAND_LINE (1);
-		  *op++ = *ip++;
-		}
-	      else
-		{
-		  EXPAND_LINE (strlen (pwd->pw_dir));
-		  op = stpcpy (op, pwd->pw_dir);
-		}
-	    }
-	  else
-	    {
-	      const char *home = blp_getenv ("HOME");
-	      if (!home)
-		{
-		  EXPAND_LINE (1);
-		  *op++ = *ip++;
-		}
-	      else
-		{
-		  EXPAND_LINE (strlen (home));
-		  op = stpcpy (op, home);
-		}
-	    }
-	  ip = cp;
-	}
-      else
-	{
-	  EXPAND_LINE (1);
-	  *op++ = *ip++;
-	}
-    }
-  *op++ = 0;
-  return xrealloc (output, op - output);
+	if (cp > ip)
+	  {
+	    struct passwd *pwd;
+	    char username[9];
+
+	    strncpy (username, ip, cp - ip + 1);
+	    username[8] = 0;
+	    pwd = getpwnam (username);
+
+	    if (!pwd || !pwd->pw_dir)
+	      ds_putchar (&output, *ip++);
+	    else
+	      ds_concat (&output, pwd->pw_dir);
+	  }
+	else
+	  {
+	    const char *home = fn_getenv ("HOME");
+	    if (!home)
+	      ds_putchar (&output, *ip++);
+	    else
+	      ds_concat (&output, home);
+	  }
+
+	ip = cp;
+      }
+
+  return ds_value (&output);
 }
 #else /* !unix */
 char *
-tilde_expand (char *input)
+fn_tilde_expand (char *input)
 {
   return xstrdup (input);
 }
 #endif /* !unix */
 
-/* normalize_filename(): This very OS-dependent routine canonicalizes
+/* Searches for a configuration file with name NAME in the path given
+   by PATH, which is tilde- and environment-interpolated.  Directories
+   in PATH are delimited by PATH_DELIMITER, defined in <pref.h>.
+   Returns the malloc'd full name of the first file found, or NULL if
+   none is found.
+
+   If PREPEND is non-NULL, then it is prepended to each filename;
+   i.e., it looks like PREPEND/PATH_COMPONENT/NAME.  This is not done
+   with absolute directories in the path. */
+#if unix || __MSDOS__ || __WIN32__
+char *
+fn_search_path (const char *basename, const char *path, const char *prepend)
+{
+  char *subst_path;
+  struct string filename;
+  const char *bp;
+
+  if (fn_absolute_p (basename))
+    return fn_tilde_expand (basename);
+  
+  {
+    char *temp = fn_interp_vars (path, fn_getenv);
+    bp = subst_path = fn_tilde_expand (temp);
+    free (temp);
+  }
+
+  msg (VM (4), _("Searching for `%s'..."), basename);
+  ds_init (NULL, &filename, 64);
+
+  for (;;)
+    {
+      const char *ep;
+      if (0 == *bp)
+	{
+	  msg (VM (4), _("Search unsuccessful!"));
+	  ds_destroy (&filename);
+	  free (subst_path);
+	  return NULL;
+	}
+
+      for (ep = bp; *ep && *ep != PATH_DELIMITER; ep++)
+	;
+
+      /* Paste together PREPEND/PATH/BASENAME. */
+      ds_clear (&filename);
+      if (prepend && !fn_absolute_p (bp))
+	{
+	  ds_concat (&filename, prepend);
+	  ds_putchar (&filename, DIR_SEPARATOR);
+	}
+      ds_concat_buffer (&filename, bp, ep - bp);
+      if (ep - bp
+	  && ds_value (&filename)[ds_length (&filename) - 1] != DIR_SEPARATOR)
+	ds_putchar (&filename, DIR_SEPARATOR);
+      ds_concat (&filename, basename);
+      
+      msg (VM (5), " - %s", ds_value (&filename));
+      if (fn_exists_p (ds_value (&filename)))
+	{
+	  msg (VM (4), _("Found `%s'."), ds_value (&filename));
+	  free (subst_path);
+	  return ds_value (&filename);
+	}
+
+      if (0 == *ep)
+	{
+	  msg (VM (4), _("Search unsuccessful!"));
+	  free (subst_path);
+	  ds_destroy (&filename);
+	  return NULL;
+	}
+      bp = ep + 1;
+    }
+}
+#else /* not unix, msdog, lose32 */
+char *
+fn_search_path (const char *basename, const char *path, const char *prepend)
+{
+  size_t size = strlen (path) + 1 + strlen (basename) + 1;
+  char *string;
+  char *cp;
+  
+  if (prepend)
+    size += strlen (prepend) + 1;
+  string = xmalloc (size);
+  
+  cp = string;
+  if (prepend)
+    {
+      cp = stpcpy (cp, prepend);
+      *cp++ = DIR_SEPARATOR;
+    }
+  cp = stpcpy (cp, path);
+  *cp++ = DIR_SEPARATOR;
+  strcpy (cp, basename);
+
+  return string;
+}
+#endif /* not unix, msdog, lose32 */
+
+/* Prepends directory DIR to filename FILE and returns a malloc()'d
+   copy of it. */
+char *
+fn_prepend_dir (const char *file, const char *dir)
+{
+  char *temp;
+  char *cp;
+  
+  if (fn_absolute_p (file))
+    return xstrdup (file);
+
+  temp = xmalloc (strlen (file) + 1 + strlen (dir) + 1);
+  cp = stpcpy (temp, dir);
+  if (cp != temp && cp[-1] != DIR_SEPARATOR)
+    *cp++ = DIR_SEPARATOR;
+  cp = stpcpy (cp, file);
+
+  return temp;
+}
+
+/* fn_normalize(): This very OS-dependent routine canonicalizes
    filename FN1.  The filename should not need to be the name of an
    existing file.  Returns a malloc()'d copy of the canonical name.
    This function must always succeed; if it needs to bail out then it
    should return xstrdup(FN1).  */
 #if unix
-/* Stolen from libc.info but heavily modified, this is a wrapper
-   around readlink() that allows for arbitrary filename length. */
 char *
-readlink_malloc (const char *filename)
-{
-  int size = 128;
-
-  while (1)
-    {
-      char *buffer = (char *) xmalloc (size);
-      int nchars;
-#if __CHECKER__
-      memset (buffer, 0, size);
-#endif
-      nchars = readlink (filename, buffer, size);
-      if (nchars == -1)
-	{
-	  free (buffer);
-	  return NULL;
-	}
-      if (nchars < size - 1)
-	{
-	  buffer[nchars] = 0;
-	  return buffer;
-	}
-      free (buffer);
-      size *= 2;
-    }
-}
-
-char *
-normalize_filename (const char *filename)
+fn_normalize (const char *filename)
 {
   const char *src;
   char *fn1, *fn2, *dest;
   int maxlen;
 
-  if (is_special_filename (filename))
+  if (fn_special_p (filename))
     return xstrdup (filename);
   
-  fn1 = tilde_expand (filename);
+  fn1 = fn_tilde_expand (filename);
 
   /* Follow symbolic links. */
-  while (1)
+  for (;;)
     {
       fn2 = fn1;
-      fn1 = readlink_malloc (fn1);
+      fn1 = fn_readlink (fn1);
       if (!fn1)
 	{
 	  fn1 = fn2;
@@ -396,7 +403,7 @@ normalize_filename (const char *filename)
 	*dest++ = DIR_SEPARATOR;
     }
 
-  while (1)
+  for (;;)
     {
       int c, f;
 
@@ -447,7 +454,7 @@ normalize_filename (const char *filename)
 }
 #elif __WIN32__
 char *
-normalize_filename (char *fn1)
+fn_normalize (const char *fn1)
 {
   DWORD len;
   DWORD success;
@@ -478,7 +485,7 @@ normalize_filename (char *fn1)
 }
 #elif __BORLANDC__
 char *
-normalize_filename (char *fn1)
+fn_normalize (const char *fn1)
 {
   char *fn2 = _fullpath (NULL, fn1, 0);
   if (fn2)
@@ -492,7 +499,7 @@ normalize_filename (char *fn1)
 }
 #elif __DJGPP__
 char *
-normalize_filename (char *fn1)
+fn_normalize (const char *fn1)
 {
   char *fn2 = xmalloc (1024);
   _fixpath (fn1, fn2);
@@ -501,44 +508,73 @@ normalize_filename (char *fn1)
 }
 #else /* not Lose32, Unix, or DJGPP */
 char *
-normalize_filename (char *fn)
+fn_normalize (const char *fn)
 {
   return xstrdup (fn);
 }
 #endif /* not Lose32, Unix, or DJGPP */
 
-/* Simulates $VER and $ARCH environment variables. */
-const char *
-blp_getenv (const char *s)
+/* Returns the directory part of FILENAME, as a malloc()'d
+   string. */
+char *
+fn_dirname (const char *filename)
 {
-  if (streq (s, "VER"))
-    return getenv_default ("STAT_VER", bare_version);
-  else if (streq (s, "ARCH"))
-    return getenv_default ("STAT_ARCH", host_system);
+  const char *p;
+  char *s;
+  size_t len;
+
+  len = strlen (filename);
+  if (len == 1 && filename[0] == '/')
+    p = filename + 1;
+  else if (len && filename[len - 1] == DIR_SEPARATOR)
+    p = mm_find_reverse (filename, len - 1, filename + len - 1, 1);
   else
-    return getenv (s);
+    p = strrchr (filename, DIR_SEPARATOR);
+  if (p == NULL)
+    p = filename;
+
+  s = xmalloc (p - filename + 1);
+  memcpy (s, filename, p - filename);
+  s[p - filename] = 0;
+
+  return s;
 }
 
-/* Returns nonzero if file with name NAME exists. */
-int
-file_exists (const char *name)
+/* Returns the basename part of FILENAME as a malloc()'d string. */
+#if 0
+char *
+fn_basename (const char *filename)
 {
-#if unix
-  struct stat temp;
-
-  return stat (name, &temp) == 0;
-#else
-  FILE *f = fopen (name, "r");
-  if (!f)
-    return 0;
-  fclose (f);
-  return 1;
-#endif
+  /* Not used, not implemented. */
+  abort ();
 }
+#endif
+
+/* Returns the current working directory, as a malloc()'d string.
+   From libc.info. */
+char *
+fn_get_cwd (void)
+{
+  int size = 100;
+  char *buffer = xmalloc (size);
+     
+  for (;;)
+    {
+      char *value = getcwd (buffer, size);
+      if (value != 0)
+	return buffer;
+
+      size *= 2;
+      free (buffer);
+      buffer = xmalloc (size);
+    }
+}
+
+/* Find out information about files. */
 
 /* Returns nonzero iff NAME specifies an absolute filename. */
 int
-absolute_filename_p (const char *name)
+fn_absolute_p (const char *name)
 {
 #if unix
   if (name[0] == '/'
@@ -557,131 +593,98 @@ absolute_filename_p (const char *name)
   return 0;
 }
   
-/* Searches for a configuration file with name NAME in the path given
-   by PATH, which is tilde- and environment-interpolated.  Directories
-   in PATH are delimited by PATH_DELIMITER, defined in <pref.h>.
-   Returns the malloc'd full name of the first file found, or NULL if
-   none is found.
-
-   If PREPEND is non-NULL, then it is prepended to each filename;
-   i.e., it looks like PREPEND/PATH_COMPONENT/NAME.  This is not done
-   with absolute directories in the path. */
-#if unix || __MSDOS__ || __WIN32__
-char *
-search_path (const char *name, const char *path, const char *prepend)
+/* Returns 1 if the filename specified is a virtual file that doesn't
+   really exist on disk, 0 if it's a real filename. */
+int
+fn_special_p (const char *filename)
 {
-  char *subst_path;
+  if (!strcmp (filename, "-") || !strcmp (filename, "stdin")
+      || !strcmp (filename, "stdout") || !strcmp (filename, "stderr")
+#if unix
+      || filename[0] == '|'
+      || (*filename && filename[strlen (filename) - 1] == '|')
+#endif
+      )
+    return 1;
 
-  char *fn = NULL;
-  int fn_size = 0;
-  int fn_len = 0;
+  return 0;
+}
 
-  char *bp, *ep;
+/* Returns nonzero if file with name NAME exists. */
+int
+fn_exists_p (const char *name)
+{
+#if unix
+  struct stat temp;
 
-  int boilerplate_len;
+  return stat (name, &temp) == 0;
+#else
+  FILE *f = fopen (name, "r");
+  if (!f)
+    return 0;
+  fclose (f);
+  return 1;
+#endif
+}
 
-  boilerplate_len = 1 + strlen (name);
-  if (prepend)
-    boilerplate_len += 1 + strlen (prepend);
+#if unix
+/* Stolen from libc.info but heavily modified, this is a wrapper
+   around readlink() that allows for arbitrary filename length. */
+char *
+fn_readlink (const char *filename)
+{
+  int size = 128;
 
-  if (absolute_filename_p (name))
-    return tilde_expand (name);
-  
-  {
-    char *temp = interp_vars (path, blp_getenv);
-    bp = subst_path = tilde_expand (temp);
-    free (temp);
-  }
-
-  verbose_msg (4, _("Searching for `%s'..."), name);
-
-  while (1)
+  for (;;)
     {
-      if (0 == *bp)
+      char *buffer = xmalloc (size);
+      int nchars  = readlink (filename, buffer, size);
+      if (nchars == -1)
 	{
-	  verbose_msg (4, _("Search unsuccessful!"));
-	  free (fn);
-	  free (subst_path);
+	  free (buffer);
 	  return NULL;
 	}
 
-      for (ep = bp; *ep && *ep != PATH_DELIMITER; ep++)
-	;
-
-      fn_len = (ep - bp) + boilerplate_len;
-      if (fn_len > fn_size)
+      if (nchars < size - 1)
 	{
-	  fn = xrealloc (fn, fn_len + 1);
-	  fn_size = fn_len;
+	  buffer[nchars] = 0;
+	  return buffer;
 	}
-
-      /* Paste together PREPEND/PATH/NAME. */
-      {
-	char *cp = fn;
-
-	if (prepend && !absolute_filename_p (bp))
-	  {
-	    cp = stpcpy (cp, prepend);
-	    *cp++ = DIR_SEPARATOR;
-	  }
-	memcpy (cp, bp, ep - bp);
-	cp += ep - bp;
-	if (ep - bp && cp[-1] != DIR_SEPARATOR)
-	  *cp++ = DIR_SEPARATOR;
-	strcpy (cp, name);
-      }
-      
-      verbose_msg (5, " - %s", fn);
-      if (file_exists (fn))
-	{
-	  verbose_msg (4, _("Found `%s'."), fn);
-	  free (subst_path);
-	  return fn;
-	}
-
-      if (0 == *ep)
-	{
-	  verbose_msg (4, _("Search unsuccessful!"));
-	  free (subst_path);
-	  free (fn);
-	  return NULL;
-	}
-      bp = ep + 1;
+      free (buffer);
+      size *= 2;
     }
 }
-#else /* not unix, msdog, lose32 */
+#else /* Not UNIX. */
 char *
-search_path (char *name, char *path, char *prepend)
+fn_readlink (const char *filename)
 {
-  size_t size = strlen (path) + 1 + strlen (name) + 1;
-  char *string;
-  char *cp;
-  
-  if (prepend)
-    size += strlen (prepend) + 1;
-  string = xmalloc (size);
-  
-  cp = string;
-  if (prepend)
-    {
-      cp = stpcpy (cp, prepend);
-      *cp++ = DIR_SEPARATOR;
-    }
-  cp = stpcpy (cp, path);
-  *cp++ = DIR_SEPARATOR;
-  strcpy (cp, name);
+  return NULL;
+}
+#endif /* Not UNIX. */
+
+/* Environment variables. */
 
-  return string;
+/* Simulates $VER and $ARCH environment variables. */
+const char *
+fn_getenv (const char *s)
+{
+  if (!strcmp (s, "VER"))
+    return fn_getenv_default ("STAT_VER", bare_version);
+  else if (!strcmp (s, "ARCH"))
+    return fn_getenv_default ("STAT_ARCH", host_system);
+  else
+    return getenv (s);
 }
-#endif /* not unix, msdog, lose32 */
 
 /* Returns getenv(KEY) if that's non-NULL; else returns DEF. */
 const char *
-getenv_default (const char *key, const char *def)
+fn_getenv_default (const char *key, const char *def)
 {
   const char *value = getenv (key);
   return value ? value : def;
 }
+
+/* Basic file handling. */
 
 /* Used for giving an error message on a set_safer security
    violation. */
@@ -703,15 +706,15 @@ safety_violation (const char *fn)
    NULL on failure.  If NULL is returned then errno is set to a
    sensible value.  */
 FILE *
-open_file (const char *fn, const char *mode)
+fn_open (const char *fn, const char *mode)
 {
   assert (mode[0] == 'r' || mode[0] == 'w');
 
-  if (mode[0] == 'r' && (streq (fn, "stdin") || streq (fn, "-"))) 
+  if (mode[0] == 'r' && (!strcmp (fn, "stdin") || !strcmp (fn, "-"))) 
     return stdin;
-  else if (mode[0] == 'w' && (streq (fn, "stdout") || streq (fn, "-")))
+  else if (mode[0] == 'w' && (!strcmp (fn, "stdout") || !strcmp (fn, "-")))
     return stdout;
-  else if (mode[0] == 'w' && streq (fn, "stderr"))
+  else if (mode[0] == 'w' && !strcmp (fn, "stderr"))
     return stderr;
   
 #if unix
@@ -719,6 +722,7 @@ open_file (const char *fn, const char *mode)
     {
       if (set_safer)
 	return safety_violation (fn);
+
       return popen (&fn[1], mode);
     }
   else if (*fn && fn[strlen (fn) - 1] == '|')
@@ -743,33 +747,40 @@ open_file (const char *fn, const char *mode)
 #endif
     {
       FILE *f = fopen (fn, mode);
+
       if (f && mode[0] == 'w')
 	setvbuf (f, NULL, _IOLBF, 0);
+
       return f;
     }
 }
 
-/* Counterpart to open_file that closes file F with name FN; returns 0
+/* Counterpart to fn_open that closes file F with name FN; returns 0
    on success, EOF on failure.  If EOF is returned, errno is set to a
    sensible value. */
 int
-close_file (FILE *f, const char *fn)
+fn_close (const char *fn, FILE *f)
 {
-  if (streq (fn, "-"))
+  if (!strcmp (fn, "-"))
     return 0;
 #if unix
   else if (fn[0] == '|' || (*fn && fn[strlen (fn) - 1] == '|'))
-    return pclose (f), 0;
+    {
+      pclose (f);
+      return 0;
+    }
 #endif
   else
     return fclose (f);
 }
+
+/* More extensive file handling. */
 
-/* File open routine that extends open_file().  Opens or reopens a
+/* File open routine that extends fn_open().  Opens or reopens a
    file according to the contents of file_ext F.  Returns nonzero on
    success.  If 0 is returned, errno is set to a sensible value. */
 int
-open_file_ext (file_ext * f)
+fn_open_ext (struct file_ext *f)
 {
   char *p;
 
@@ -791,43 +802,51 @@ open_file_ext (file_ext * f)
 	    if (f->preclose (f) == 0)
 	      error = errno;
 
-	  if (EOF == close_file (f->file, f->filename) || error)
+	  if (EOF == fn_close (f->filename, f->file) || error)
 	    {
 	      f->file = NULL;
 	      local_free (s);
+
 	      if (error)
 		errno = error;
+
 	      return 0;
 	    }
+
 	  f->file = NULL;
 	}
 
-      f->file = open_file (s, f->mode);
+      f->file = fn_open (s, f->mode);
       local_free (s);
 
       if (f->file && f->postopen)
 	if (f->postopen (f) == 0)
 	  {
 	    int error = errno;
-	    close_file (f->file, f->filename);
+	    fn_close (f->filename, f->file);
 	    errno = error;
+
 	    return 0;
 	  }
+
       return (f->file != NULL);
     }
   else if (f->file)
     return 1;
   else
     {
-      f->file = open_file (f->filename, f->mode);
+      f->file = fn_open (f->filename, f->mode);
+
       if (f->file && f->postopen)
 	if (f->postopen (f) == 0)
 	  {
 	    int error = errno;
-	    close_file (f->file, f->filename);
+	    fn_close (f->filename, f->file);
 	    errno = error;
+
 	    return 0;
 	  }
+
       return (f->file != NULL);
     }
 }
@@ -836,7 +855,7 @@ open_file_ext (file_ext * f)
    Return nonzero on success.  If zero is returned, errno is set to a
    sensible value. */
 int
-close_file_ext (file_ext * f)
+fn_close_ext (struct file_ext *f)
 {
   if (f->file)
     {
@@ -846,70 +865,17 @@ close_file_ext (file_ext * f)
 	if (f->preclose (f) == 0)
 	  error = errno;
 
-      if (EOF == close_file (f->file, f->filename) || error)
+      if (EOF == fn_close (f->filename, f->file) || error)
 	{
 	  f->file = NULL;
+
 	  if (error)
 	    errno = error;
+
 	  return 0;
 	}
+
       f->file = NULL;
     }
   return 1;
 }
-
-/* Returns the directory part of the file FILE, as a malloc()'d
-   string. */
-char *
-blp_dirname (const char *file)
-{
-  const char *p = strrchr (file, DIR_SEPARATOR);
-  char *s;
-  
-  if (p == NULL)
-    p = file;
-  s = xmalloc (p - file + 1);
-  strncpy (s, file, p - file);
-  s[p - file] = 0;
-
-  return s;
-}
-
-/* Prepends directory DIR to filename FILE and returns a malloc()'d
-   copy of it. */
-char *
-prepend_dir (const char *file, const char *dir)
-{
-  char *temp;
-  char *cp;
-  
-  if (absolute_filename_p (file))
-    return xstrdup (file);
-
-  temp = xmalloc (strlen (file) + 1 + strlen (dir) + 1);
-  cp = stpcpy (temp, dir);
-  if (cp != temp && cp[-1] != DIR_SEPARATOR)
-    *cp++ = DIR_SEPARATOR;
-  cp = stpcpy (cp, file);
-
-  return temp;
-}
-
-/* Returns 1 if the filename specified is a virtual file that doesn't
-   really exist on disk, 0 if it's a real filename. */
-int
-is_special_filename (const char *filename)
-{
-  if (streq (filename, "-") || streq (filename, "stdin")
-      || streq (filename, "stdout") || streq (filename, "stderr")
-#if unix
-      || filename[0] == '|'
-      || (*filename && filename[strlen (filename) - 1] == '|')
-#endif
-      )
-    return 1;
-
-  return 0;
-}
-
-
