@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -20,17 +20,17 @@
 #include <config.h>
 #include <assert.h>
 #include <stdlib.h>
-#include "common.h"
+#include "alloc.h"
 #include "avl.h"
 #include "command.h"
 #include "error.h"
 #include "file-handle.h"
+#include "lexer.h"
 #include "misc.h"
-#include "settings.h"
 #include "pfm.h"
+#include "settings.h"
 #include "sfm.h"
 #include "str.h"
-#include "lexer.h"
 #include "var.h"
 #include "vfm.h"
 #include "vfmP.h"
@@ -40,15 +40,14 @@
 #include "debug-print.h"
 
 /* XSAVE transformation (and related SAVE, EXPORT procedures). */
-typedef struct
+struct save_trns
   {
-    trns_header h;
-    file_handle *f;		/* Associated system file. */
+    struct trns_header h;
+    struct file_handle *f;	/* Associated system file. */
     int nvar;			/* Number of variables. */
     int *var;			/* Indices of variables. */
     flt64 *case_buf;		/* Case transfer buffer. */
-  }
-save_trns;
+  };
 
 /* Options bits set by trim_dictionary(). */
 #define GTSV_OPT_COMPRESSED	001	/* Compression; (X)SAVE only. */
@@ -57,45 +56,45 @@ save_trns;
 #define GTSV_OPT_NONE		0
 
 /* The file being read by the input program. */
-static file_handle *get_file;
+static struct file_handle *get_file;
 
 /* The transformation being used by the SAVE procedure. */
-static save_trns *trns;
+static struct save_trns *trns;
 
-static int trim_dictionary (dictionary * dict, int *options);
-static int save_write_case_func (ccase *);
-static int save_trns_proc (any_trns *, ccase *);
-static void save_trns_free (any_trns *);
+static int trim_dictionary (struct dictionary * dict, int *options);
+static int save_write_case_func (struct ccase *);
+static int save_trns_proc (struct trns_header *, struct ccase *);
+static void save_trns_free (struct trns_header *);
 
 #if DEBUGGING
-void dump_dict_variables (dictionary *);
+void dump_dict_variables (struct dictionary *);
 #endif
 
 /* Parses the GET command. */
 int
 cmd_get (void)
 {
-  file_handle *handle;
-  dictionary *dict;
+  struct file_handle *handle;
+  struct dictionary *dict;
   int options = GTSV_OPT_NONE;
 
   int i;
   int nval;
 
-  match_id (GET);
+  lex_match_id ("GET");
   discard_variables ();
 
-  match_tok ('/');
-  if (match_id (FILE))
-    match_tok ('=');
+  lex_match ('/');
+  if (lex_match_id ("FILE"))
+    lex_match ('=');
 
   handle = fh_parse_file_handle ();
   if (handle == NULL)
-    return 0;
+    return CMD_FAILURE;
 
   dict = sfm_read_dictionary (handle, NULL);
   if (dict == NULL)
-    return 0;
+    return CMD_FAILURE;
 
 #if DEBUGGING
   dump_dict_variables (dict);
@@ -103,7 +102,7 @@ cmd_get (void)
   if (0 == trim_dictionary (dict, &options))
     {
       fh_close_handle (handle);
-      return 0;
+      return CMD_FAILURE;
     }
 #if DEBUGGING
   dump_dict_variables (dict);
@@ -114,7 +113,7 @@ cmd_get (void)
   nval = 0;
   for (i = 0; i < dict->nvar; i++)
     {
-      variable *v = dict->var[i];
+      struct variable *v = dict->var[i];
 
       v->fv = nval;
       nval += v->nv;
@@ -126,7 +125,7 @@ cmd_get (void)
   printf (_("GET translation table from file to memory:\n"));
   for (i = 0; i < dict->nvar; i++)
     {
-      variable *v = dict->var[i];
+      struct variable *v = dict->var[i];
 
       printf (_("  %8s from %3d,%3d to %3d,%3d\n"), v->name,
 	      v->get.fv, v->get.nv, v->fv, v->nv);
@@ -138,7 +137,7 @@ cmd_get (void)
   vfm_source = &get_source;
   get_file = handle;
 
-  return 1;
+  return CMD_SUCCESS;
 }
 
 /* Parses the SAVE (for X==0) and XSAVE (for X==1) commands.  */
@@ -148,24 +147,24 @@ cmd_get (void)
 int
 cmd_save_internal (int x)
 {
-  file_handle *handle;
-  dictionary *dict;
+  struct file_handle *handle;
+  struct dictionary *dict;
   int options = GTSV_OPT_SAVE;
 
-  save_trns *t;
-  sfm_write_info inf;
+  struct save_trns *t;
+  struct sfm_write_info inf;
 
   int i;
 
-  match_id (SAVE);
+  lex_match_id ("SAVE");
 
-  match_tok ('/');
-  if (match_id (OUTFILE))
-    match_tok ('=');
+  lex_match ('/');
+  if (lex_match_id ("OUTFILE"))
+    lex_match ('=');
 
   handle = fh_parse_file_handle ();
   if (handle == NULL)
-    return 0;
+    return CMD_FAILURE;
 
   dict = save_dictionary ();
 #if DEBUGGING
@@ -176,7 +175,7 @@ cmd_save_internal (int x)
   if (0 == trim_dictionary (dict, &options))
     {
       fh_close_handle (handle);
-      return 0;
+      return CMD_FAILURE;
     }
 
 #if DEBUGGING
@@ -191,32 +190,32 @@ cmd_save_internal (int x)
     {
       free_dictionary (dict);
       fh_close_handle (handle);
-      return 0;
+      return CMD_FAILURE;
     }
 
   /* Fill in transformation structure. */
-  t = trns = xmalloc (sizeof (save_trns));
+  t = trns = xmalloc (sizeof *t);
   t->h.proc = save_trns_proc;
   t->h.free = save_trns_free;
   t->f = handle;
   t->nvar = dict->nvar;
-  t->var = xmalloc (sizeof (int) * dict->nvar);
+  t->var = xmalloc (sizeof *t->var * dict->nvar);
   for (i = 0; i < dict->nvar; i++)
     t->var[i] = dict->var[i]->foo;
-  t->case_buf = xmalloc (sizeof (flt64) * inf.case_size);
+  t->case_buf = xmalloc (sizeof *t->case_buf * inf.case_size);
   free_dictionary (dict);
 
   if (x == 0)
     /* SAVE. */
     {
       procedure (NULL, save_write_case_func, NULL);
-      save_trns_free ((any_trns *) t);
+      save_trns_free ((struct trns_header *) t);
     }
   else
     /* XSAVE. */
-    add_transformation ((any_trns *) t);
+    add_transformation ((struct trns_header *) t);
 
-  return 1;
+  return CMD_SUCCESS;
 }
 
 /* Parses and performs the SAVE procedure. */
@@ -234,21 +233,21 @@ cmd_xsave (void)
 }
 
 static int
-save_write_case_func (ccase * c)
+save_write_case_func (struct ccase * c)
 {
-  save_trns_proc ((any_trns *) trns, c);
+  save_trns_proc ((struct trns_header *) trns, c);
   return 1;
 }
 
 static int
-save_trns_proc (unused any_trns * t, ccase * c)
+save_trns_proc (struct trns_header * t unused, struct ccase * c)
 {
   flt64 *p = trns->case_buf;
   int i;
 
   for (i = 0; i < trns->nvar; i++)
     {
-      variable *v = default_dict.var[trns->var[i]];
+      struct variable *v = default_dict.var[trns->var[i]];
       if (v->type == NUMERIC)
 	{
 	  double src = c->data[v->fv].f;
@@ -261,8 +260,8 @@ save_trns_proc (unused any_trns * t, ccase * c)
 	{
 	  memcpy (p, c->data[v->fv].s, v->width);
 	  memset (&((char *) p)[v->width], ' ',
-		  REM_RND_UP (v->width, sizeof (flt64)));
-	  p += DIV_RND_UP (v->width, sizeof (flt64));
+		  REM_RND_UP (v->width, sizeof *p));
+	  p += DIV_RND_UP (v->width, sizeof *p);
 	}
     }
 
@@ -271,9 +270,9 @@ save_trns_proc (unused any_trns * t, ccase * c)
 }
 
 static void
-save_trns_free (any_trns *_t)
+save_trns_free (struct trns_header *pt)
 {
-  save_trns *t = (save_trns *) _t;
+  struct save_trns *t = (struct save_trns *) pt;
 
   fh_close_handle (t->f);
   free (t->var);
@@ -285,7 +284,7 @@ save_trns_free (any_trns *_t)
    variables must have consecutive indices.  The variables are cleared
    and freed. */
 static void
-dict_delete_run (dictionary *dict, int first, int nv)
+dict_delete_run (struct dictionary *dict, int first, int nv)
 {
   int i;
 
@@ -302,7 +301,7 @@ dict_delete_run (dictionary *dict, int first, int nv)
   dict->nvar -= nv;
 }
 
-static int rename_variables (dictionary * dict);
+static int rename_variables (struct dictionary * dict);
 
 /* The GET and SAVE commands have a common structure after the
    FILE/OUTFILE subcommand.  This function parses this structure and
@@ -311,7 +310,7 @@ static int rename_variables (dictionary * dict);
    GTSV_OPT_COMPRESSED bit. */
 /* FIXME: IN, FIRST, LAST, MAP. */
 static int
-trim_dictionary (dictionary *dict, int *options)
+trim_dictionary (struct dictionary *dict, int *options)
 {
   if (set_scompression)
     *options |= GTSV_OPT_COMPRESSED;
@@ -338,19 +337,19 @@ trim_dictionary (dictionary *dict, int *options)
 	}
     }
   
-  while ((*options & GTSV_OPT_MATCH_FILES) || match_tok ('/'))
+  while ((*options & GTSV_OPT_MATCH_FILES) || lex_match ('/'))
     {
-      if (!(*options & GTSV_OPT_MATCH_FILES) && match_id (COMPRESSED))
+      if (!(*options & GTSV_OPT_MATCH_FILES) && lex_match_id ("COMPRESSED"))
 	*options |= GTSV_OPT_COMPRESSED;
-      else if (!(*options & GTSV_OPT_MATCH_FILES) && match_id (UNCOMPRESSED))
+      else if (!(*options & GTSV_OPT_MATCH_FILES) && lex_match_id ("UNCOMPRESSED"))
 	*options &= ~GTSV_OPT_COMPRESSED;
-      else if (match_id (DROP))
+      else if (lex_match_id ("DROP"))
 	{
-	  variable **v;
+	  struct variable **v;
 	  int nv;
 	  int i;
 
-	  match_tok ('=');
+	  lex_match ('=');
 	  if (!parse_variables (dict, &v, &nv, PV_NONE))
 	    return 0;
 
@@ -369,12 +368,12 @@ trim_dictionary (dictionary *dict, int *options)
 	      i = j;
 	    }
 	}
-      else if (match_id (KEEP))
+      else if (lex_match_id ("KEEP"))
 	{
-	  variable **v;
+	  struct variable **v;
 	  int nv;
 
-	  match_tok ('=');
+	  lex_match ('=');
 	  if (!parse_variables (dict, &v, &nv, PV_NONE))
 	    return 0;
 
@@ -388,7 +387,7 @@ trim_dictionary (dictionary *dict, int *options)
 		int i2 = v[i1]->index;
 
 		/* Swap variables with indices i1 and i2. */
-		variable *t = dict->var[i1];
+		struct variable *t = dict->var[i1];
 		dict->var[i1] = dict->var[i2];
 		dict->var[i2] = t;
 		dict->var[i1]->index = i1;
@@ -408,16 +407,19 @@ trim_dictionary (dictionary *dict, int *options)
 		free (dict->var[i]);
 	      }
 	  }
-	  dict->var = xrealloc (dict->var, sizeof (variable *) * nv);
+	  dict->var = xrealloc (dict->var, sizeof *dict->var * nv);
 	  dict->nvar = nv;
 	}
-      else if (match_id (RENAME))
+      else if (lex_match_id ("RENAME"))
 	{
 	  if (!rename_variables (dict))
 	    return 0;
 	}
       else
-	return syntax_error (_("while expecting a valid subcommand"));
+	{
+	  lex_error (_("while expecting a valid subcommand"));
+	  return 0;
+	}
 
       if (dict->nvar == 0)
 	{
@@ -430,44 +432,53 @@ trim_dictionary (dictionary *dict, int *options)
     }
 
   if (token != '.')
-    return msg (SE, _("Command terminator expected."));
+    {
+      lex_error (_("expecting end of command"));
+      return 0;
+    }
+  
   return 1;
 }
 
 /* Parses and performs the RENAME subcommand of GET and SAVE. */
 static int
-rename_variables (dictionary * dict)
+rename_variables (struct dictionary * dict)
 {
   int i;
 
   int success = 0;
 
-  variable **v;
+  struct variable **v;
   char **new_names;
   int nv, nn;
 
   int group;
 
-  match_tok ('=');
+  lex_match ('=');
   if (token != '(')
     {
-      variable *v;
+      struct variable *v;
 
       v = parse_dict_variable (dict);
       if (v == NULL)
 	return 0;
-      force_match ('=');
-      force_id ();
-      if (!strncmp (tokstr, v->name, 8))
+      if (!lex_force_match ('=')
+	  || !lex_force_id ())
+	return 0;
+      if (!strncmp (tokid, v->name, 8))
 	return 1;
-      if (is_dict_varname (dict, tokstr))
-	return msg (SE, _("Cannot rename %s as %s because there already exists "
-		    "a variable named %s.  To rename variables with "
-		    "overlapping names, use a single RENAME subcommand such as "
-		    "\"/RENAME (A=B)(B=C)(C=A)\", or equivalently, \"/RENAME "
-		    "(A B C=B C A)\"."), v->name, tokstr, tokstr);
-      rename_variable (dict, v, tokstr);
-      get_token ();
+      if (is_dict_varname (dict, tokid))
+	{
+	  msg (SE, _("Cannot rename %s as %s because there already exists "
+		     "a variable named %s.  To rename variables with "
+		     "overlapping names, use a single RENAME subcommand "
+		     "such as \"/RENAME (A=B)(B=C)(C=A)\", or equivalently, "
+		     "\"/RENAME (A B C=B C A)\"."), v->name, tokid, tokid);
+	  return 0;
+	}
+      
+      rename_variable (dict, v, tokid);
+      lex_get ();
       return 1;
     }
 
@@ -475,13 +486,13 @@ rename_variables (dictionary * dict)
   v = NULL;
   new_names = 0;
   group = 1;
-  while (match_tok ('('))
+  while (lex_match ('('))
     {
       int old_nv = nv;
 
       if (!parse_variables (dict, &v, &nv, PV_NO_DUPLICATE | PV_APPEND))
 	goto lossage;
-      if (!match_tok ('='))
+      if (!lex_match ('='))
 	{
 	  msg (SE, _("`=' expected after variable list."));
 	  goto lossage;
@@ -496,7 +507,8 @@ rename_variables (dictionary * dict)
 	       nv - old_nv, nn - old_nv, group);
 	  goto lossage;
 	}
-      force_match (')');
+      if (!lex_force_match (')'))
+	goto lossage;
       group++;
     }
 
@@ -526,7 +538,7 @@ lossage:
 
 #if DEBUGGING
 void
-dump_dict_variables (dictionary * dict)
+dump_dict_variables (struct dictionary * dict)
 {
   int i;
 
@@ -557,7 +569,7 @@ get_source_read (void)
   get_source_destroy_source ();
 }
 
-case_stream get_source =
+struct case_stream get_source =
   {
     NULL,
     get_source_read,
@@ -583,53 +595,52 @@ enum
   };
 
 /* One of the files on MATCH FILES. */
-typedef struct mtf_file
+struct mtf_file
   {
     struct mtf_file *next, *prev;
 				/* Next, previous in the list of files. */
     struct mtf_file *next_min;	/* Next in the chain of minimums. */
     
     int type;			/* One of MTF_*. */
-    variable **by;		/* List of BY variables for this file. */
-    file_handle *handle;	/* File handle for the file. */
-    dictionary *dict;		/* Dictionary from system file. */
+    struct variable **by;	/* List of BY variables for this file. */
+    struct file_handle *handle;	/* File handle for the file. */
+    struct dictionary *dict;	/* Dictionary from system file. */
     char in[9];			/* Name of the variable from IN=. */
     char first[9], last[9];	/* Name of the variables from FIRST=, LAST=. */
-    value *input;		/* Input record. */
-  }
-mtf_file;
+    union value *input;		/* Input record. */
+  };
 
 /* All the files mentioned on FILE= or TABLE=. */
-static mtf_file *mtf_head, *mtf_tail;
+static struct mtf_file *mtf_head, *mtf_tail;
 
 /* Variables on the BY subcommand. */
-static variable **mtf_by;
+static struct variable **mtf_by;
 static int mtf_n_by;
 
 /* Master dictionary. */
-static dictionary *mtf_master;
+static struct dictionary *mtf_master;
 
 static void mtf_free (void);
-static void mtf_free_file (mtf_file *file);
-static int mtf_merge_dictionary (mtf_file *f);
-static void mtf_delete_file_in_place (mtf_file **file);
+static void mtf_free_file (struct mtf_file *file);
+static int mtf_merge_dictionary (struct mtf_file *f);
+static void mtf_delete_file_in_place (struct mtf_file **file);
 
 static void mtf_read_nonactive_records (void);
 static void mtf_processing_finish (void);
-static int mtf_processing (ccase *);
+static int mtf_processing (struct ccase *);
 
-static char *var_type_description (variable *);
+static char *var_type_description (struct variable *);
 
 /* Parse and execute the MATCH FILES command. */
 int
 cmd_match_files (void)
 {
-  mtf_file *first_table = NULL;
+  struct mtf_file *first_table = NULL;
   
   int seen = 0;
   
-  match_id (MATCH);
-  match_id (FILES);
+  lex_match_id ("MATCH");
+  lex_match_id ("FILES");
 
   mtf_head = mtf_tail = NULL;
   mtf_by = NULL;
@@ -639,9 +650,9 @@ cmd_match_files (void)
   
   do
     {
-      match_tok ('/');
+      lex_match ('/');
 
-      if (match_tok (BY))
+      if (lex_match (T_BY))
 	{
 	  if (seen & 1)
 	    {
@@ -650,28 +661,28 @@ cmd_match_files (void)
 	    }
 	  seen |= 1;
 	      
-	  match_tok ('=');
+	  lex_match ('=');
 	  if (!parse_variables (mtf_master, &mtf_by, &mtf_n_by,
 				PV_NO_DUPLICATE | PV_NO_SCRATCH))
 	    goto lossage;
 	}
-      else if (token != ID)
+      else if (token != T_ID)
 	{
-	  syntax_error (NULL);
+	  lex_error (NULL);
 	  goto lossage;
 	}
-      else if (id_match ("FILE", tokstr) || id_match ("TABLE", tokstr))
+      else if (lex_id_match ("FILE", tokid) || lex_id_match ("TABLE", tokid))
 	{
-	  mtf_file *file = xmalloc (sizeof (mtf_file));
+	  struct mtf_file *file = xmalloc (sizeof *file);
 
 	  file->in[0] = file->first[0] = file->last[0] = '\0';
 	  file->dict = NULL;
 	  file->by = NULL;
 	  file->input = NULL;
 
-	  if (match_id (FILE))
+	  if (lex_match_id ("FILE"))
 	    file->type = MTF_FILE;
-	  else if (match_id (TABLE))
+	  else if (lex_match_id ("TABLE"))
 	    {
 	      file->type = MTF_TABLE;
 	      seen |= 4;
@@ -704,9 +715,9 @@ cmd_match_files (void)
 	      first_table->prev = file;
 	    }
 	  
-	  match_tok ('=');
+	  lex_match ('=');
 	  
-	  if (match_tok ('*'))
+	  if (lex_match ('*'))
 	    {
 	      file->handle = NULL;
 
@@ -744,8 +755,9 @@ cmd_match_files (void)
 	  if (!mtf_merge_dictionary (file))
 	    goto lossage;
 	}
-      else if (id_match ("IN", tokstr) || id_match ("FIRST", tokstr)
-	       || id_match ("LAST", tokstr))
+      else if (lex_id_match ("IN", tokid)
+	       || lex_id_match ("FIRST", tokid)
+	       || lex_id_match ("LAST", tokid))
 	{
 	  const char *sbc;
 	  char *name;
@@ -757,17 +769,17 @@ cmd_match_files (void)
 	      goto lossage;
 	    }
 
-	  if (match_id (IN))
+	  if (lex_match_id ("IN"))
 	    {
 	      name = mtf_tail->in;
 	      sbc = "IN";
 	    }
-	  else if (match_id (FIRST))
+	  else if (lex_match_id ("FIRST"))
 	    {
 	      name = mtf_tail->first;
 	      sbc = "FIRST";
 	    }
-	  else if (match_id (LAST))
+	  else if (lex_match_id ("LAST"))
 	    {
 	      name = mtf_tail->last;
 	      sbc = "LAST";
@@ -775,10 +787,10 @@ cmd_match_files (void)
 	  else
 	    assert (0);
 
-	  match_tok ('=');
-	  if (token != ID)
+	  lex_match ('=');
+	  if (token != T_ID)
 	    {
-	      syntax_error (NULL);
+	      lex_error (NULL);
 	      goto lossage;
 	    }
 
@@ -789,8 +801,8 @@ cmd_match_files (void)
 		   sbc);
 	      goto lossage;
 	    }
-	  strcpy (name, tokstr);
-	  get_token ();
+	  strcpy (name, tokid);
+	  lex_get ();
 
 	  if (!create_variable (mtf_master, name, NUMERIC, 0))
 	    {
@@ -800,8 +812,9 @@ cmd_match_files (void)
 	      goto lossage;
 	    }
 	}
-      else if (id_match ("RENAME", tokstr) || id_match ("KEEP", tokstr)
-	       || id_match ("DROP", tokstr))
+      else if (lex_id_match ("RENAME", tokid)
+	       || lex_id_match ("KEEP", tokid)
+	       || lex_id_match ("DROP", tokid))
 	{
 	  int options = GTSV_OPT_MATCH_FILES;
 	  
@@ -815,13 +828,13 @@ cmd_match_files (void)
 	  if (!trim_dictionary (mtf_tail->dict, &options))
 	    goto lossage;
 	}
-      else if (match_id (MAP))
+      else if (lex_match_id ("MAP"))
 	{
 	  /* FIXME. */
 	}
       else
 	{
-	  syntax_error (NULL);
+	  lex_error (NULL);
 	  goto lossage;
 	}
     }
@@ -839,13 +852,13 @@ cmd_match_files (void)
 
   if (seen & 1)
     {
-      mtf_file *iter;
+      struct mtf_file *iter;
 
       for (iter = mtf_head; iter; iter = iter->next)
 	{
 	  int i;
 	  
-	  iter->by = xmalloc (sizeof (variable *) * mtf_n_by);
+	  iter->by = xmalloc (sizeof *iter->by * mtf_n_by);
 
 	  for (i = 0; i < mtf_n_by; i++)
 	    {
@@ -864,7 +877,7 @@ cmd_match_files (void)
 #if DEBUGGING
   {
     /* From sfm-read.c. */
-    extern void dump_dictionary (dictionary *);
+    extern void dump_dictionary (struct dictionary *);
 
     dump_dictionary (mtf_master);
   }
@@ -918,11 +931,11 @@ cmd_match_files (void)
   mtf_master = NULL;
   
   mtf_free ();
-  return 1;
+  return CMD_SUCCESS;
   
 lossage:
   mtf_free ();
-  return 0;
+  return CMD_FAILURE;
 }
 
 /* Repeats 2...8 an arbitrary number of times. */
@@ -931,7 +944,7 @@ mtf_processing_finish (void)
 {
   /* Find the active file and delete it. */
   {
-    mtf_file *iter;
+    struct mtf_file *iter;
     
     for (iter = mtf_head; iter; iter = iter->next)
       if (iter->handle == NULL)
@@ -949,7 +962,7 @@ mtf_processing_finish (void)
 /* Return a string in a static buffer describing V's variable type and
    width. */
 static char *
-var_type_description (variable *v)
+var_type_description (struct variable *v)
 {
   static char buf[2][32];
   static int x = 0;
@@ -970,7 +983,7 @@ var_type_description (variable *v)
 
 /* Free FILE and associated data. */
 static void
-mtf_free_file (mtf_file *file)
+mtf_free_file (struct mtf_file *file)
 {
   fh_close_handle (file->handle);
   if (file->dict && file->dict != &default_dict)
@@ -985,7 +998,7 @@ mtf_free_file (mtf_file *file)
 static void
 mtf_free (void)
 {
-  mtf_file *iter, *next;
+  struct mtf_file *iter, *next;
 
   for (iter = mtf_head; iter; iter = next)
     {
@@ -1002,9 +1015,9 @@ mtf_free (void)
 /* Remove *FILE from the mtf_file chain.  Make *FILE point to the next
    file in the chain, or to NULL if was the last in the chain. */
 static void
-mtf_delete_file_in_place (mtf_file **file)
+mtf_delete_file_in_place (struct mtf_file **file)
 {
-  mtf_file *f = *file;
+  struct mtf_file *f = *file;
 
   if (f->prev)
     f->prev->next = f->next;
@@ -1021,7 +1034,7 @@ mtf_delete_file_in_place (mtf_file **file)
 
     for (i = 0; i < f->dict->nvar; i++)
       {
-	variable *v = f->dict->var[i];
+	struct variable *v = f->dict->var[i];
 	  
 	if (v->type == NUMERIC)
 	  compaction_case->data[v->p.mtf.master->fv].f = SYSMIS;
@@ -1038,14 +1051,14 @@ mtf_delete_file_in_place (mtf_file **file)
 static void
 mtf_read_nonactive_records (void)
 {
-  mtf_file *iter;
+  struct mtf_file *iter;
 
   for (iter = mtf_head; iter; )
     {
       if (iter->handle)
 	{
 	  assert (iter->input == NULL);
-	  iter->input = xmalloc (sizeof (value) * iter->dict->nval);
+	  iter->input = xmalloc (sizeof *iter->input * iter->dict->nval);
 	  
 	  if (!sfm_read_case (iter->handle, iter->input, iter->dict))
 	    mtf_delete_file_in_place (&iter);
@@ -1063,7 +1076,7 @@ mtf_read_nonactive_records (void)
 /* Compare the BY variables for files A and B; return -1 if A < B, 0
    if A == B, 1 if A > B. */
 static inline int
-mtf_compare_BY_values (mtf_file *a, mtf_file *b)
+mtf_compare_BY_values (struct mtf_file *a, struct mtf_file *b)
 {
   int i;
   
@@ -1105,16 +1118,16 @@ static int mtf_seq_no = 0;
 
 /* Perform one iteration of steps 3...7 above. */
 static int
-mtf_processing (unused ccase *c)
+mtf_processing (struct ccase *c unused)
 {
   /* List of files with minimum BY values. */
-  mtf_file *min_head, *min_tail;
+  struct mtf_file *min_head, *min_tail;
 
   /* List of files with non-minimum BY values. */
-  mtf_file *max_head, *max_tail;
+  struct mtf_file *max_head, *max_tail;
 
   /* Iterator. */
-  mtf_file *iter;
+  struct mtf_file *iter;
 
   for (;;)
     {
@@ -1172,7 +1185,7 @@ mtf_processing (unused ccase *c)
 	 from the TABLE input record into the output record. */
       while (iter)
 	{
-	  mtf_file *next = iter->next;
+	  struct mtf_file *next = iter->next;
 	  
 	  assert (iter->type == MTF_TABLE);
       
@@ -1220,7 +1233,7 @@ mtf_processing (unused ccase *c)
 
 	  for (i = 0; i < iter->dict->nvar; i++)
 	    {
-	      variable *v = iter->dict->var[i];
+	      struct variable *v = iter->dict->var[i];
 	  
 	      if (v->p.mtf.master->foo == mtf_seq_no)
 		continue;
@@ -1258,7 +1271,7 @@ mtf_processing (unused ccase *c)
 
 	  for (i = 0; i < iter->dict->nvar; i++)
 	    {
-	      variable *v = iter->dict->var[i];
+	      struct variable *v = iter->dict->var[i];
 	  
 	      if (v->p.mtf.master->foo == mtf_seq_no)
 		continue;
@@ -1297,7 +1310,7 @@ mtf_processing (unused ccase *c)
 	 files. */
       for (iter = min_head; iter && iter->type == MTF_FILE; )
 	{
-	  mtf_file *next = iter->next_min;
+	  struct mtf_file *next = iter->next_min;
 	  
 	  if (iter->handle)
 	    {
@@ -1320,10 +1333,10 @@ mtf_processing (unused ccase *c)
 /* Merge the dictionary for file F into the master dictionary
    mtf_master. */
 static int
-mtf_merge_dictionary (mtf_file *f)
+mtf_merge_dictionary (struct mtf_file *f)
 {
-  dictionary *const m = mtf_master;
-  dictionary *d = f->dict;
+  struct dictionary *const m = mtf_master;
+  struct dictionary *d = f->dict;
       
   if (d->label && m->label == NULL)
     m->label = xstrdup (d->label);
@@ -1343,8 +1356,8 @@ mtf_merge_dictionary (mtf_file *f)
     d->nval = 0;
     for (i = 0; i < d->nvar; i++)
       {
-	variable *dv = d->var[i];
-	variable *mv = find_dict_variable (m, dv->name);
+	struct variable *dv = d->var[i];
+	struct variable *mv = find_dict_variable (m, dv->name);
 
 	dv->fv = d->nval;
 	d->nval += dv->nv;
@@ -1390,49 +1403,55 @@ mtf_merge_dictionary (mtf_file *f)
 int
 cmd_import (void)
 {
-  file_handle *handle = NULL;
-  dictionary *dict;
+  struct file_handle *handle = NULL;
+  struct dictionary *dict;
   int options = GTSV_OPT_NONE;
   int type;
 
   int i;
   int nval;
 
-  match_id (IMPORT);
+  lex_match_id ("IMPORT");
 
   for (;;)
     {
-      match_tok ('/');
+      lex_match ('/');
       
-      if (match_id (FILE) || token == STRING)
+      if (lex_match_id ("FILE") || token == T_STRING)
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 
 	  handle = fh_parse_file_handle ();
 	  if (handle == NULL)
-	    return 0;
+	    return CMD_FAILURE;
 	}
-      else if (match_id (TYPE))
+      else if (lex_match_id ("TYPE"))
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 
-	  if (match_id (COMM))
+	  if (lex_match_id ("COMM"))
 	    type = PFM_COMM;
-	  else if (match_id (TAPE))
+	  else if (lex_match_id ("TAPE"))
 	    type = PFM_TAPE;
 	  else
-	    return syntax_error (_("expecting COMM or TAPE"));
+	    {
+	      lex_error (_("expecting COMM or TAPE"));
+	      return CMD_FAILURE;
+	    }
 	}
       else break;
     }
-  if (!match_tok ('/') && token != '.')
-    return syntax_error (_("expecting end of command"));
+  if (!lex_match ('/') && token != '.')
+    {
+      lex_error (NULL);
+      return CMD_FAILURE;
+    }
 
   discard_variables ();
 
   dict = pfm_read_dictionary (handle, NULL);
   if (dict == NULL)
-    return 0;
+    return CMD_FAILURE;
 
 #if DEBUGGING
   dump_dict_variables (dict);
@@ -1440,7 +1459,7 @@ cmd_import (void)
   if (0 == trim_dictionary (dict, &options))
     {
       fh_close_handle (handle);
-      return 0;
+      return CMD_FAILURE;
     }
 #if DEBUGGING
   dump_dict_variables (dict);
@@ -1451,7 +1470,7 @@ cmd_import (void)
   nval = 0;
   for (i = 0; i < dict->nvar; i++)
     {
-      variable *v = dict->var[i];
+      struct variable *v = dict->var[i];
 
       v->fv = nval;
       nval += v->nv;
@@ -1463,7 +1482,7 @@ cmd_import (void)
   printf (_("IMPORT translation table from file to memory:\n"));
   for (i = 0; i < dict->nvar; i++)
     {
-      variable *v = dict->var[i];
+      struct variable *v = dict->var[i];
 
       printf (_("  %8s from %3d,%3d to %3d,%3d\n"), v->name,
 	      v->get.fv, v->get.nv, v->fv, v->nv);
@@ -1475,7 +1494,7 @@ cmd_import (void)
   vfm_source = &import_source;
   get_file = handle;
 
-  return 1;
+  return CMD_SUCCESS;
 }
 
 /* Reads all the cases from the data file and passes them to
@@ -1489,7 +1508,7 @@ import_source_read (void)
   get_source_destroy_source ();
 }
 
-case_stream import_source =
+struct case_stream import_source =
   {
     NULL,
     import_source_read,
@@ -1500,30 +1519,30 @@ case_stream import_source =
     "IMPORT",
   };
 
-static int export_write_case_func (ccase *c);
+static int export_write_case_func (struct ccase *c);
      
 /* Parses the EXPORT command.  */
 /* FIXME: same as cmd_save_internal(). */
 int
 cmd_export (void)
 {
-  file_handle *handle;
-  dictionary *dict;
+  struct file_handle *handle;
+  struct dictionary *dict;
   int options = GTSV_OPT_SAVE;
 
-  save_trns *t;
+  struct save_trns *t;
 
   int i;
 
-  match_id (EXPORT);
+  lex_match_id ("EXPORT");
 
-  match_tok ('/');
-  if (match_id (OUTFILE))
-    match_tok ('=');
+  lex_match ('/');
+  if (lex_match_id ("OUTFILE"))
+    lex_match ('=');
 
   handle = fh_parse_file_handle ();
   if (handle == NULL)
-    return 0;
+    return CMD_FAILURE;
 
   dict = save_dictionary ();
 #if DEBUGGING
@@ -1534,7 +1553,7 @@ cmd_export (void)
   if (0 == trim_dictionary (dict, &options))
     {
       fh_close_handle (handle);
-      return 0;
+      return CMD_FAILURE;
     }
 
 #if DEBUGGING
@@ -1546,36 +1565,36 @@ cmd_export (void)
     {
       free_dictionary (dict);
       fh_close_handle (handle);
-      return 0;
+      return CMD_FAILURE;
     }
 
   /* Fill in transformation structure. */
-  t = trns = xmalloc (sizeof (save_trns));
+  t = trns = xmalloc (sizeof *t);
   t->h.proc = save_trns_proc;
   t->h.free = save_trns_free;
   t->f = handle;
   t->nvar = dict->nvar;
-  t->var = xmalloc (sizeof (int) * dict->nvar);
+  t->var = xmalloc (sizeof *t->var * dict->nvar);
   for (i = 0; i < dict->nvar; i++)
     t->var[i] = dict->var[i]->foo;
-  t->case_buf = xmalloc (sizeof (value) * dict->nvar);
+  t->case_buf = xmalloc (sizeof *t->case_buf * dict->nvar);
   free_dictionary (dict);
 
   procedure (NULL, export_write_case_func, NULL);
-  save_trns_free ((any_trns *) t);
+  save_trns_free ((struct trns_header *) t);
 
-  return 1;
+  return CMD_SUCCESS;
 }
 
 static int
-export_write_case_func (ccase *c)
+export_write_case_func (struct ccase *c)
 {
-  value *p = (value *) trns->case_buf;
+  union value *p = (union value *) trns->case_buf;
   int i;
 
   for (i = 0; i < trns->nvar; i++)
     {
-      variable *v = default_dict.var[trns->var[i]];
+      struct variable *v = default_dict.var[trns->var[i]];
 
       if (v->type == NUMERIC)
 	*p++ = c->data[v->fv];
@@ -1586,6 +1605,6 @@ export_write_case_func (ccase *c)
   printf (".");
   fflush (stdout);
   
-  pfm_write_case (trns->f, (value *) trns->case_buf);
+  pfm_write_case (trns->f, (union value *) trns->case_buf);
   return 1;
 }
