@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -22,42 +22,66 @@
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
-#include "common.h"
+#include "alloc.h"
 #include "error.h"
 #include "filename.h"
 #include "getline.h"
+#include "lexer.h"
 #include "settings.h"
-#include "misc.h"
+#include "str.h"
+#include "tab.h"
 #include "var.h"
 #include "version.h"
-#include "str.h"
-#include "lexer.h"
-#include "lexerP.h"
 
 /* Global variables. */
-getl_script *getl_head;
-getl_script *getl_tail;
+struct string getl_buf;
+struct getl_script *getl_head;
+struct getl_script *getl_tail;
 int getl_interactive;
 int getl_welcomed;
 int getl_mode;
-char *getl_buf;
-int getl_buf_len;
-size_t getl_buf_size;
 int getl_prompt;
-char *getl_history;
-char *getl_include_path;
+
+#if HAVE_LIBHISTORY
+static char *history_file;
+
+#if HAVE_READLINE_HISTORY_H
+#include <readline/history.h>
+#else /* no readline/history.h */
+extern void add_history (char *);
+extern void using_history (void);
+extern int read_history (char *);
+extern void stifle_history (int);
+extern int write_history (char *);
+#endif /* no readline/history.h */
+#endif /* -lhistory */
+
+static struct string getl_include_path;
 
 /* Number of levels of DO REPEAT structures we're nested inside.  If
    this is greater than zero then DO REPEAT macro substitutions are
    performed. */
 static int DO_REPEAT_level;
 
+static int read_console (void);
+
 /* Initialize getline. */
 void
 getl_initialize (void)
 {
-  getl_include_path = xstrdup (getenv_default ("STAT_INCLUDE_PATH",
-					       include_path));
+  ds_create (NULL, &getl_include_path,
+	     fn_getenv_default ("STAT_INCLUDE_PATH", include_path));
+  ds_init (NULL, &getl_buf, 256);
+}
+
+/* Close getline. */
+void
+getl_uninitialize (void)
+{
+#if HAVE_LIBHISTORY && unix
+  if (history_file)
+    write_history (history_file);
+#endif
 }
 
 /* Returns a string that represents the directory that the syntax file
@@ -67,35 +91,24 @@ getl_initialize (void)
 char *
 getl_get_current_directory (void)
 {
-  if (getl_head)
-    return blp_dirname (getl_head->fn);
-  else
-    return gnu_getcwd ();
+  return getl_head ? fn_dirname (getl_head->fn) : fn_get_cwd ();
 }
 
 /* Delete everything from the include path. */
 void
 getl_clear_include_path (void)
 {
-  free (getl_include_path);
-  getl_include_path = NULL;
+  ds_clear (&getl_include_path);
 }
 
 /* Add to the include path. */
 void
 getl_add_include_dir (const char *path)
 {
-  if (getl_include_path)
-    {
-      int len = strlen (getl_include_path);
+  if (ds_length (&getl_include_path))
+    ds_putchar (&getl_include_path, PATH_DELIMITER);
 
-      getl_include_path = xrealloc (getl_include_path,
-				    len + 1 + strlen (path) + 1);
-      getl_include_path[len] = PATH_DELIMITER;
-      strcpy (&getl_include_path[len + 1], path);
-    }
-  else
-    getl_include_path = xstrdup (path);
+  ds_concat (&getl_include_path, path);
 }
 
 /* Adds FN to the tail end of the list of script files to execute.
@@ -106,9 +119,9 @@ getl_add_include_dir (const char *path)
 void
 getl_add_file (const char *fn, int separate, int where)
 {
-  getl_script *n = xmalloc (sizeof (getl_script));
+  struct getl_script *n = xmalloc (sizeof *n);
 
-  assert (fn);
+  assert (fn != NULL);
   n->next = NULL;
   if (getl_tail == NULL)
     getl_head = getl_tail = n;
@@ -133,15 +146,15 @@ getl_add_file (const char *fn, int separate, int where)
 void
 getl_include (const char *fn)
 {
-  getl_script *n;
+  struct getl_script *n;
   char *real_fn;
 
-  assert (getl_include_path);
   {
     char *cur_dir = getl_get_current_directory ();
-    real_fn = search_path (fn, getl_include_path, cur_dir);
+    real_fn = fn_search_path (fn, ds_value (&getl_include_path), cur_dir);
     free (cur_dir);
   }
+
   if (!real_fn)
     {
       msg (SE, _("Can't find `%s' in include file search path."), fn);
@@ -155,7 +168,7 @@ getl_include (const char *fn)
     }
   else
     {
-      n = xmalloc (sizeof (getl_script));
+      n = xmalloc (sizeof *n);
       n->included_from = getl_head;
       getl_head = getl_head->includes = n;
       n->includes = NULL;
@@ -171,7 +184,7 @@ getl_include (const char *fn)
 /* Add the virtual file FILE to the list of files to be processed.
    The first_line field in FILE must already have been initialized. */
 void 
-getl_add_virtual_file (getl_script *file)
+getl_add_virtual_file (struct getl_script *file)
 {
   if (getl_tail == NULL)
     getl_head = getl_tail = file;
@@ -184,7 +197,7 @@ getl_add_virtual_file (getl_script *file)
   file->separate = 0;
   file->f = NULL;
   file->cur_line = NULL;
-  file->remaining_loops = 2;
+  file->remaining_loops = 1;
   file->loop_index = -1;
   file->macros = NULL;
 }
@@ -194,7 +207,7 @@ getl_add_virtual_file (getl_script *file)
    loop_index, and macros fields in FILE must already have been
    initialized. */
 void
-getl_add_DO_REPEAT_file (getl_script *file)
+getl_add_DO_REPEAT_file (struct getl_script *file)
 {
   /* getl_head == NULL can't happen. */
   assert (getl_head);
@@ -216,17 +229,14 @@ void
 welcome (void)
 {
   getl_welcomed = 1;
-  puts (_("PSPP is free software and you are welcome to distribute "
-	"copies of it"));
-  puts (_("under certain conditions; type \"show copying.\" to see "
-	"the conditions."));
-  puts (_("There is ABSOLUTELY NO WARRANTY for PSPP; type \"show "
-	"warranty.\" for details."));
+  fputs ("PSPP is free software and you are welcome to distribute copies of"
+	 "it\nunder certain conditions; type \"show copying.\" to see the "
+	 "conditions.\nThere is ABSOLUTELY NO WARRANTY for PSPP; type \"show "
+	 "warranty.\" for details.\n", stdout);
   puts (stat_version);
 }
 
 /* Reads a single line from the user's terminal. */
-static int read_console (void);
 
 /* From repeat.c. */
 extern void perform_DO_REPEAT_substitutions (void);
@@ -234,19 +244,18 @@ extern void perform_DO_REPEAT_substitutions (void);
 /* Reads a single line from the line buffer associated with getl_head.
    Returns 1 if a line was successfully read or 0 if no more lines are
    available. */
-int
+static int
 handle_line_buffer (void)
 {
-  getl_script *s = getl_head;
+  struct getl_script *s = getl_head;
 
   /* Check that we're not all done. */
   do
     {
       if (s->cur_line == NULL)
 	{
-	  s->remaining_loops--;
 	  s->loop_index++;
-	  if (s->remaining_loops == 0)
+	  if (s->remaining_loops-- == 0)
 	    return 0;
 	  s->cur_line = s->first_line;
 	}
@@ -261,15 +270,7 @@ handle_line_buffer (void)
     }
   while (s->cur_line == NULL);
 
-  /* Check that the line buffer is big enough. */
-  if ((size_t) s->cur_line->len + 1 > getl_buf_size)
-    {
-      getl_buf_size = s->cur_line->len * 2;
-      getl_buf = xrealloc (getl_buf, getl_buf_size);
-    }
-  
-  memcpy (getl_buf, s->cur_line->line, s->cur_line->len + 1);
-  getl_buf_len = s->cur_line->len;
+  ds_concat_buffer (&getl_buf, s->cur_line->line, s->cur_line->len);
 
   /* Advance pointers. */
   s->cur_line = s->cur_line->next;
@@ -289,8 +290,9 @@ getl_read_line (void)
   
   while (getl_head)
     {
-      getl_script *s = getl_head;
+      struct getl_script *s = getl_head;
 
+      ds_clear (&getl_buf);
       if (s->separate)
 	return 0;
 
@@ -302,13 +304,16 @@ getl_read_line (void)
 	      continue;
 	    }
 	  perform_DO_REPEAT_substitutions ();
+	  if (getl_head->print)
+	    tab_output_text (TAB_LEFT | TAT_FIX | TAT_PRINTF, "+%s",
+			     ds_value (&getl_buf));
 	  return 1;
 	}
       
-      if (NULL == s->f)
+      if (s->f == NULL)
 	{
-	  verbose_msg (1, _("%s: Opening as syntax file."), s->fn);
-	  s->f = open_file (s->fn, "r");
+	  msg (VM (1), _("%s: Opening as syntax file."), s->fn);
+	  s->f = fn_open (s->fn, "r");
 
 	  if (s->f == NULL)
 	    {
@@ -318,23 +323,24 @@ getl_read_line (void)
 	    }
 	}
 
-      getl_buf_len = getline (&getl_buf, &getl_buf_size, s->f);
-      if (getl_buf_len == -1)
+      if (!ds_getline (&getl_buf, s->f))
 	{
 	  if (ferror (s->f))
 	    msg (ME, _("Reading `%s': %s."), s->fn, strerror (errno));
 	  getl_close_file ();
 	  continue;
 	}
+      if (ds_length (&getl_buf) > 0 && ds_end (&getl_buf)[-1] == '\n')
+	ds_truncate (&getl_buf, ds_length (&getl_buf) - 1);
 
-      /* Macro substitutions from DO REPEAT. */
-      if (DO_REPEAT_level)
-	perform_DO_REPEAT_substitutions ();
+      if (set_echo)
+	tab_output_text (TAB_LEFT | TAT_FIX, ds_value (&getl_buf));
 
       getl_head->ln++;
 
       /* Allows shebang invocation: `#! /usr/local/bin/pspp'. */
-      if (getl_buf[0] == '#' && getl_buf[1] == '!')
+      if (ds_value (&getl_buf)[0] == '#'
+	  && ds_value (&getl_buf)[1] == '!')
 	continue;
 
       return 1;
@@ -351,6 +357,51 @@ getl_read_line (void)
   return read_console ();
 }
 
+/* Closes the current file, whether it be a main file or included
+   file, then moves getl_head to the next file in the chain. */
+void
+getl_close_file (void)
+{
+  struct getl_script *s = getl_head;
+
+  if (!s)
+    return;
+  assert (getl_tail != NULL);
+
+  if (s->first_line)
+    {
+      struct getl_line_list *cur, *next;
+
+      s->fn = NULL; /* It will be freed below. */
+      for (cur = s->first_line; cur; cur = next)
+	{
+	  next = cur->next;
+	  free (cur->line);
+	  free (cur);
+	}
+
+      DO_REPEAT_level--;
+    }
+  
+  if (s->f && EOF == fn_close (s->fn, s->f))
+    msg (MW, _("Closing `%s': %s."), s->fn, strerror (errno));
+  free (s->fn);
+
+  if (s->included_from)
+    {
+      getl_head = s->included_from;
+      getl_head->includes = NULL;
+    }
+  else
+    {
+      getl_head = s->next;
+      if (NULL == getl_head)
+	getl_tail = NULL;
+    }
+  
+  free (s);
+}
+
 /* PORTME: Adapt to your local system's idea of the terminal. */
 #if HAVE_LIBREADLINE
 
@@ -360,25 +411,14 @@ getl_read_line (void)
 extern char *readline (char *);
 #endif /* no readline/readline.h */
 
-#if HAVE_LIBHISTORY
-#if HAVE_READLINE_HISTORY_H
-#include <readline/history.h>
-#else /* no readline/history.h */
-extern void add_history (char *);
-extern void using_history (void);
-extern int read_history (char *);
-extern void stifle_history (int);
-#endif /* no readline/history.h */
-#endif /* -lhistory */
-
 static int
 read_console (void)
 {
   char *line;
   char *prompt;
 
-  error_count = warning_count = 0;
-  error_already_flagged = 0;
+  err_error_count = err_warning_count = 0;
+  err_already_flagged = 0;
 
 #if HAVE_LIBHISTORY
   if (!history_file)
@@ -397,15 +437,19 @@ read_console (void)
     case GETL_PRPT_STANDARD:
       prompt = set_prompt;
       break;
+
     case GETL_PRPT_CONTINUATION:
       prompt = set_cprompt;
       break;
+
     case GETL_PRPT_DATA:
       prompt = set_dprompt;
       break;
+
     default:
       assert (0);
     }
+
   line = readline (prompt);
   if (!line)
     return 0;
@@ -415,10 +459,8 @@ read_console (void)
     add_history (line);
 #endif
 
-  free (getl_buf);
-  getl_buf = line;
-  getl_buf_len = strlen (getl_buf);
-  getl_buf_size = getl_buf_len + 1;
+  ds_clear (&getl_buf);
+  ds_concat (&getl_buf, line);
 
   return 1;
 }
@@ -426,64 +468,21 @@ read_console (void)
 static int
 read_console (void)
 {
-  error_count = warning_count = 0;
-  error_already_flagged = 0;
+  err_error_count = err_warning_count = 0;
+  err_already_flagged = 0;
 
   fputs (getl_prompt ? set_cprompt : set_prompt, stdout);
-  getl_buf_len = getline (&getl_buf, &getl_buf_size, stdin);
-  if (getl_buf_len != -1)
+  ds_clear (&getl_buf);
+  if (ds_getline (&getl_buf, stdin))
     return 1;
+
   if (ferror (stdin))
     msg (FE, "stdin: fgets(): %s.", strerror (errno));
+
   return 0;
 }
 #endif /* no -lreadline */
 
-/* Closes the current file, whether it be a main file or included
-   file, then moves getl_head to the next file in the chain. */
-void
-getl_close_file (void)
-{
-  getl_script *s = getl_head;
-
-  if (!s)
-    return;
-  assert (getl_tail);
-
-  if (s->first_line)
-    {
-      getl_line_list *cur, *next;
-
-      s->fn = NULL; /* It will be freed below. */
-      for (cur = s->first_line; cur; cur = next)
-	{
-	  next = cur->next;
-	  free (cur->line);
-	  free (cur);
-	}
-
-      DO_REPEAT_level--;
-    }
-  
-  if (s->f && EOF == close_file (s->f, s->fn))
-    msg (MW, _("Closing `%s': %s."), s->fn, strerror (errno));
-  free (s->fn);
-
-  if (s->included_from)
-    {
-      getl_head = s->included_from;
-      getl_head->includes = NULL;
-    }
-  else
-    {
-      getl_head = s->next;
-      if (NULL == getl_head)
-	getl_tail = NULL;
-    }
-  
-  free (s);
-}
-
 /* Closes all files. */
 void
 getl_close_all (void)
@@ -502,8 +501,19 @@ getl_perform_delayed_reset (void)
     {
       getl_head->separate = 0;
       discard_variables ();
-      reset_eof ();
+      lex_reset_eof ();
       return 1;
     }
   return 0;
 }
+
+/* Puts the current file and line number in *FN and *LN, respectively,
+   or NULL and -1 if none. */
+void
+getl_location (const char **fn, int *ln)
+{
+  if (fn != NULL)
+    *fn = getl_head ? getl_head->fn : NULL;
+  if (ln != NULL)
+    *ln = getl_head ? getl_head->ln : -1;
+}
