@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -24,15 +24,13 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <stdarg.h>
+#include "alloc.h"
 #include "error.h"
-#include "str.h"
-#include "common.h"
-#include "file-handle.h"
 #include "filename.h"
 #include "font.h"
 #include "hash.h"
-#include "misc.h"
-#include "arena.h"
+#include "pool.h"
+#include "str.h"
 #include "version.h"
 
 int font_number_to_index (int);
@@ -42,10 +40,10 @@ int space_index;
 static int font_msg (int, const char *,...)
      __attribute__ ((format (printf, 2, 3)));
 static void scan_badchars (char *, int);
-static void dup_char_metric (font_desc * font, int dest, int src);
-static void add_char_metric (font_desc * font, char_metrics *metrics,
+static void dup_char_metric (struct font_desc * font, int dest, int src);
+static void add_char_metric (struct font_desc * font, struct char_metrics *metrics,
 			     int code);
-static void add_kern (font_desc * font, int ch1, int ch2, int adjust);
+static void add_kern (struct font_desc * font, int ch1, int ch2, int adjust);
 
 /* Typical whitespace characters for tokenizing. */
 static const char whitespace[] = " \t\n\r\v";
@@ -72,14 +70,14 @@ groff_init (void)
    binary format in memory.  Installs the binary format in the global
    font table.  See groff_font(8) for a description of the font
    description format supported.  Returns nonzero on success. */
-font_desc *
+struct font_desc *
 groff_read_font (const char *fn)
 {
-  char_metrics *metrics;
+  struct char_metrics *metrics;
 
-  /* Arena created for font, font being created, font file. */
-  arena *font_arena = NULL;
-  font_desc *font = NULL;
+  /* Pool created for font, font being created, font file. */
+  struct pool *font_pool = NULL;
+  struct font_desc *font = NULL;
   FILE *f = NULL;
 
   /* Current line, size of line buffer, length of line. */
@@ -99,19 +97,26 @@ groff_read_font (const char *fn)
   /* Index for previous line. */
   int prev_index = -1;
 
+  /* Current location in file, used for error reporting. */
+  struct file_locator where;
+
 #if unix
-  fn = tilde_expand (fn);
+  fn = fn_tilde_expand (fn);
 #endif
 
-  verbose_msg (1, _("%s: Opening Groff font file..."), fn);
+  msg (VM (1), _("%s: Opening Groff font file..."), fn);
+
+  where.filename = fn;
+  where.line_number = 1;
+  err_push_file_locator (&where);
+
   f = fopen (fn, "r");
   if (!f)
     goto file_lossage;
-  push_cust (fn);
 
-  arena_create (&font_arena);
-  font = arena_alloc (&font_arena, sizeof (font_desc));
-  font->owner = font_arena;
+  font_pool = pool_create ();
+  font = pool_alloc (font_pool, sizeof *font);
+  font->owner = font_pool;
   font->name = NULL;
   font->internal_name = NULL;
   font->encoding = NULL;
@@ -131,7 +136,6 @@ groff_read_font (const char *fn)
   font->kern_max_used = 0;
 
   /* Parses first section of font file. */
-  cust_ln = 1;
   for (;;)
     {
       /* Location of '#' in line. */
@@ -150,7 +154,7 @@ groff_read_font (const char *fn)
       if (!key)
 	goto next_iteration;
 
-      if (streq (key, "internalname"))
+      if (!strcmp (key, "internalname"))
 	{
 	  font->internal_name = strtok_r (NULL, whitespace, &sp);
 	  if (font->internal_name == NULL)
@@ -158,9 +162,9 @@ groff_read_font (const char *fn)
 	      font_msg (SE, _("Missing font name."));
 	      goto lose;
 	    }
-	  font->internal_name = arena_strdup (&font_arena, font->internal_name);
+	  font->internal_name = pool_strdup (font_pool, font->internal_name);
 	}
-      else if (streq (key, "encoding"))
+      else if (!strcmp (key, "encoding"))
 	{
 	  font->encoding = strtok_r (NULL, whitespace, &sp);
 	  if (font->encoding == NULL)
@@ -168,9 +172,9 @@ groff_read_font (const char *fn)
 	      font_msg (SE, _("Missing encoding filename."));
 	      goto lose;
 	    }
-	  font->encoding = arena_strdup (&font_arena, font->encoding);
+	  font->encoding = pool_strdup (font_pool, font->encoding);
 	}
-      else if (streq (key, "spacewidth"))
+      else if (!strcmp (key, "spacewidth"))
 	{
 	  char *n = strtok_r (NULL, whitespace, &sp);
 	  char *tail;
@@ -182,7 +186,7 @@ groff_read_font (const char *fn)
 	      goto lose;
 	    }
 	}
-      else if (streq (key, "slant"))
+      else if (!strcmp (key, "slant"))
 	{
 	  char *n = strtok_r (NULL, whitespace, &sp);
 	  char *tail;
@@ -194,24 +198,24 @@ groff_read_font (const char *fn)
 	      goto lose;
 	    }
 	}
-      else if (streq (key, "ligatures"))
+      else if (!strcmp (key, "ligatures"))
 	{
 	  char *lig;
 
-	  while (1)
+	  for (;;)
 	    {
 	      lig = strtok_r (NULL, whitespace, &sp);
-	      if (!lig || streq (lig, "0"))
+	      if (!lig || !strcmp (lig, "0"))
 		break;
-	      else if (streq (lig, "ff"))
+	      else if (!strcmp (lig, "ff"))
 		font->ligatures |= LIG_ff;
-	      else if (streq (lig, "ffi"))
+	      else if (!strcmp (lig, "ffi"))
 		font->ligatures |= LIG_ffi;
-	      else if (streq (lig, "ffl"))
+	      else if (!strcmp (lig, "ffl"))
 		font->ligatures |= LIG_ffl;
-	      else if (streq (lig, "fi"))
+	      else if (!strcmp (lig, "fi"))
 		font->ligatures |= LIG_fi;
-	      else if (streq (lig, "fl"))
+	      else if (!strcmp (lig, "fl"))
 		font->ligatures |= LIG_fl;
 	      else
 		{
@@ -220,12 +224,12 @@ groff_read_font (const char *fn)
 		}
 	    }
 	}
-      else if (streq (key, "special"))
+      else if (!strcmp (key, "special"))
 	font->special = 1;
-      else if (streq (key, "charset") || streq (key, "kernpairs"))
+      else if (!strcmp (key, "charset") || !strcmp (key, "kernpairs"))
 	break;
 
-      cust_ln++;
+      where.line_number++;
     }
   if (ferror (f))
     goto file_lossage;
@@ -237,14 +241,14 @@ groff_read_font (const char *fn)
       if (!key)
 	goto next_iteration;
 
-      if (streq (key, "charset"))
+      if (!strcmp (key, "charset"))
 	charset = 1;
-      else if (streq (key, "kernpairs"))
+      else if (!strcmp (key, "kernpairs"))
 	charset = 0;
       else if (charset)
 	{
-	  char_metrics *metrics = arena_alloc (&font_arena,
-					       sizeof (char_metrics));
+	  struct char_metrics *metrics = pool_alloc (font_pool,
+						     sizeof *metrics);
 	  char *m, *type, *code, *tail;
 
 	  m = strtok_r (NULL, whitespace, &sp);
@@ -254,20 +258,20 @@ groff_read_font (const char *fn)
 			      "set."));
 	      goto lose;
 	    }
-	  if (streq (m, "\""))
+	  if (!strcmp (m, "\""))
 	    {
 	      if (!prev_index)
 		{
 		  font_msg (SE, _("Can't use ditto mark for first character."));
 		  goto lose;
 		}
-	      if (streq (key, "---"))
+	      if (!strcmp (key, "---"))
 		{
 		  font_msg (SE, _("Can't ditto into an unnamed character."));
 		  goto lose;
 		}
 	      dup_char_metric (font, font_char_name_to_index (key), prev_index);
-	      cust_ln++;
+	      where.line_number++;
 	      goto next_iteration;
 	    }
 
@@ -302,7 +306,7 @@ groff_read_font (const char *fn)
 	      goto lose;
 	    }
 
-	  if (!streq (key, "---"))
+	  if (strcmp (key, "---"))
 	    prev_index = font_char_name_to_index (key);
 	  else
 	    prev_index = font_number_to_index (metrics->code);
@@ -338,7 +342,7 @@ groff_read_font (const char *fn)
 	}
 
     next_iteration:
-      cust_ln++;
+      where.line_number++;
 
       len = getline (&line, &size, f);
     }
@@ -355,7 +359,6 @@ groff_read_font (const char *fn)
 #if unix
   free ((char *) fn);
 #endif
-  pop_cust ();
 
   /* Get font ascent and descent. */
   metrics = font_get_char_metrics (font, font_char_name_to_index ("d"));
@@ -363,9 +366,11 @@ groff_read_font (const char *fn)
   metrics = font_get_char_metrics (font, font_char_name_to_index ("p"));
   font->descent = metrics ? metrics->depth : 0;
 
-  verbose_msg (2, _("Font read successfully with internal name %s."),
-	       font->internal_name==NULL ? "<none>" : font->internal_name);
+  msg (VM (2), _("Font read successfully with internal name %s."),
+       font->internal_name == NULL ? "<none>" : font->internal_name);
   
+  err_pop_file_locator (&where);
+
   return font;
 
   /* Come here on a file error. */
@@ -375,13 +380,13 @@ file_lossage:
   /* Come here on any error. */
 lose:
   fclose (f);
-  arena_destroy (&font_arena);
+  pool_destroy (font_pool);
 #if unix
   free ((char *) fn);
 #endif
-  pop_cust ();
+  err_pop_file_locator (&where);
 
-  verbose_msg (1, _("Error reading font."));
+  msg (VM (1), _("Error reading font."));
   return NULL;
 }
 
@@ -392,7 +397,7 @@ font_msg (int class, const char *format,...)
   va_list args;
 
   va_start (args, format);
-  vmsg (class, format, args, _("installation error: Groff font error: "));
+  tmsg (class, format, args, _("installation error: Groff font error: "));
   va_end (args);
 
   return 0;
@@ -425,12 +430,11 @@ scan_badchars (char *line, int len)
 /* Character name hashing. */
 
 /* Associates a character index with a character name. */
-typedef struct
+struct index_hash
   {
     char *name;
     int index;
-  }
-index_hash;
+  };
 
 /* Character index hash table. */
 static struct
@@ -440,8 +444,8 @@ static struct
     int used;			/* Number of full entries. */
     int max_used;		/* # used entries where we enlarge & rehash. */
     int next_index;		/* Next index to allocate. */
-    index_hash *tab;		/* Hash table proper. */
-    arena *ar;			/* Arena for names. */
+    struct index_hash *tab;	/* Hash table proper. */
+    struct pool *ar;		/* Pool for names. */
   }
 hash;
 
@@ -472,15 +476,15 @@ font_char_name_to_index (const char *name)
       hash.used = 0;
       hash.max_used = hash.size / 2;
       hash.next_index = 256;
-      hash.tab = xmalloc (sizeof (index_hash) * hash.size);
-      arena_create (&hash.ar);
+      hash.tab = xmalloc (sizeof *hash.tab * hash.size);
+      hash.ar = pool_create ();
       for (i = 0; i < hash.size; i++)
 	hash.tab[i].name = NULL;
     }
 
   for (i = hashpjw (name) % hash.size; hash.tab[i].name;)
     {
-      if (streq (hash.tab[i].name, name))
+      if (!strcmp (hash.tab[i].name, name))
 	return hash.tab[i].index;
       if (++i >= hash.size)
 	i = 0;
@@ -489,13 +493,13 @@ font_char_name_to_index (const char *name)
   hash.used++;
   if (hash.used >= hash.max_used)
     {
-      index_hash *old_tab = hash.tab;
+      struct index_hash *old_tab = hash.tab;
       int old_size = hash.size;
       int i, j;
 
       hash.size = *hash.size_p++;
       hash.max_used = hash.size / 2;
-      hash.tab = xmalloc (sizeof (index_hash) * hash.size);
+      hash.tab = xmalloc (sizeof *hash.tab * hash.size);
       for (i = 0; i < hash.size; i++)
 	hash.tab[i].name = NULL;
       for (i = 0; i < old_size; i++)
@@ -509,7 +513,7 @@ font_char_name_to_index (const char *name)
       free (old_tab);
     }
 
-  hash.tab[i].name = arena_strdup (&hash.ar, name);
+  hash.tab[i].name = pool_strdup (hash.ar, name);
   hash.tab[i].index = hash.next_index;
   return hash.next_index++;
 }
@@ -533,7 +537,7 @@ font_number_to_index (int x)
 /* Ensures room for at least MIN_SIZE metric indexes in deref of
    FONT. */
 static void
-check_deref_space (font_desc *font, int min_size)
+check_deref_space (struct font_desc *font, int min_size)
 {
   if (min_size >= font->deref_size)
     {
@@ -542,8 +546,8 @@ check_deref_space (font_desc *font, int min_size)
       font->deref_size = min_size + 16;
       if (font->deref_size < 256)
 	font->deref_size = 256;
-      font->deref = arena_realloc (&font->owner, font->deref,
-				   sizeof (short) * font->deref_size);
+      font->deref = pool_realloc (font->owner, font->deref,
+				  sizeof *font->deref * font->deref_size);
       for (; i < font->deref_size; i++)
 	font->deref[i] = -1;
     }
@@ -551,14 +555,14 @@ check_deref_space (font_desc *font, int min_size)
 
 /* Inserts METRICS for character with code CODE into FONT. */
 static void
-add_char_metric (font_desc *font, char_metrics *metrics, int code)
+add_char_metric (struct font_desc *font, struct char_metrics *metrics, int code)
 {
   check_deref_space (font, code);
   if (font->metric_used >= font->metric_size)
     {
       font->metric_size += 64;
-      font->metric = arena_realloc (&font->owner, font->metric,
-			       sizeof (char_metrics *) * font->metric_size);
+      font->metric = pool_realloc (font->owner, font->metric,
+				   sizeof *font->metric * font->metric_size);
     }
   font->metric[font->metric_used] = metrics;
   font->deref[code] = font->metric_used++;
@@ -567,7 +571,7 @@ add_char_metric (font_desc *font, char_metrics *metrics, int code)
 /* Copies metric in FONT from character with code SRC to character
    with code DEST. */
 static void
-dup_char_metric (font_desc *font, int dest, int src)
+dup_char_metric (struct font_desc *font, int dest, int src)
 {
   check_deref_space (font, dest);
   assert (font->deref[src] != -1);
@@ -583,20 +587,20 @@ dup_char_metric (font_desc *font, int dest, int src)
 /* Adds an ADJUST-size kern to FONT between characters with codes CH1
    and CH2. */
 static void
-add_kern (font_desc *font, int ch1, int ch2, int adjust)
+add_kern (struct font_desc *font, int ch1, int ch2, int adjust)
 {
   int i;
 
   if (font->kern_used >= font->kern_max_used)
     {
-      kern_pair *old_kern = font->kern;
+      struct kern_pair *old_kern = font->kern;
       int old_kern_size = font->kern_size;
       int j;
 
       font->kern_size = *font->kern_size_p++;
       font->kern_max_used = font->kern_size / 2;
-      font->kern = arena_malloc (&font->owner,
-				 sizeof (kern_pair) * font->kern_size);
+      font->kern = pool_malloc (font->owner,
+				sizeof *font->kern * font->kern_size);
       for (i = 0; i < font->kern_size; i++)
 	font->kern[i].ch1 = -1;
 
@@ -612,7 +616,7 @@ add_kern (font_desc *font, int ch1, int ch2, int adjust)
 	  font->kern[j] = old_kern[i];
 	}
       if (old_kern)
-	arena_free (old_kern);
+	pool_free (font->owner, old_kern);
     }
 
   for (i = hash_kern (ch1, ch2) % font->kern_size; font->kern[i].ch1 != -1;)
@@ -645,17 +649,17 @@ find_font_file (const char *dev, const char *name)
      4. config_path
    */
   if ((path = getenv ("STAT_GROFF_FONT_PATH")) != NULL
-      && (filename = search_path (basename, path, NULL)) != NULL)
+      && (filename = fn_search_path (basename, path, NULL)) != NULL)
     goto win;
 
   if ((path = getenv ("GROFF_FONT_PATH")) != NULL
-      && (filename = search_path (basename, path, NULL)) != NULL)
+      && (filename = fn_search_path (basename, path, NULL)) != NULL)
     goto win;
 
-  if ((filename = search_path (basename, groff_font_path, NULL)) != NULL)
+  if ((filename = fn_search_path (basename, groff_font_path, NULL)) != NULL)
     goto win;
 
-  if ((filename = search_path (basename, config_path, NULL)) != NULL)
+  if ((filename = fn_search_path (basename, config_path, NULL)) != NULL)
     goto win;
 
   msg (IE, _("Groff font error: Cannot find \"%s\"."), basename);
@@ -667,11 +671,11 @@ win:
 
 /* Finds a font for device DEV with name NAME, reads it with
    groff_read_font(), and returns the resultant font. */
-font_desc *
+struct font_desc *
 groff_find_font (const char *dev, const char *name)
 {
   char *filename = find_font_file (dev, name);
-  font_desc *fd;
+  struct font_desc *fd;
 
   if (!filename)
     return NULL;
@@ -684,7 +688,7 @@ groff_find_font (const char *dev, const char *name)
    output driver *DRIVER, which must be previously allocated.  Returns
    nonzero on success. */
 int
-groff_read_DESC (const char *dev_name, groff_device_info * dev)
+groff_read_DESC (const char *dev_name, struct groff_device_info * dev)
 {
   char *filename;		/* Full name of DESC file. */
   FILE *f;			/* DESC file. */
@@ -702,6 +706,7 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
 				   can fit in driver->sizes. */
 
   char *sp;			/* Tokenization string pointer. */
+  struct file_locator where;
 
   int i;
 
@@ -718,36 +723,39 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
   if (!filename)
     return 0;
 
-  verbose_msg (1, _("%s: Opening Groff description file..."), filename);
+  where.filename = filename;
+  where.line_number = 0;
+  err_push_file_locator (&where);
+
+  msg (VM (1), _("%s: Opening Groff description file..."), filename);
   f = fopen (filename, "r");
   if (!f)
     goto file_lossage;
 
-  push_cust (filename);
   while ((line_len = getline (&line, &line_size, f)) != -1)
     {
-      cust_ln++;
+      where.line_number++;
 
       token = strtok_r (line, whitespace, &sp);
       if (!token)
 	continue;
 
-      if (streq (token, "sizes"))
+      if (!strcmp (token, "sizes"))
 	{
 	  if (found & 0x10000)
 	    font_msg (SW, _("Multiple `sizes' declarations."));
-	  while (1)
+	  for (;;)
 	    {
 	      char *tail;
 	      int lower, upper;
 
-	      while (1)
+	      for (;;)
 		{
 		  token = strtok_r (NULL, whitespace, &sp);
 		  if (token)
 		    break;
 
-		  cust_ln++;
+		  where.line_number++;
 		  if ((line_len = getline (&line, &line_size, f)) != -1)
 		    {
 		      if (ferror (f))
@@ -758,7 +766,7 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
 		    }
 		}
 
-	      if (streq (token, "0"))
+	      if (!strcmp (token, "0"))
 		break;
 
 	      errno = 0;
@@ -791,7 +799,8 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
 	      if (dev->n_sizes + 2 >= m_sizes)
 		{
 		  m_sizes += 1;
-		  dev->sizes = xrealloc (dev->sizes, m_sizes * sizeof (int[2]));
+		  dev->sizes = xrealloc (dev->sizes,
+					 m_sizes * sizeof *dev->sizes);
 		}
 	      dev->sizes[dev->n_sizes++][0] = lower;
 	      dev->sizes[dev->n_sizes][1] = upper;
@@ -799,7 +808,7 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
 	      found |= 0x10000;
 	    }
 	}
-      else if (streq (token, "family"))
+      else if (!strcmp (token, "family"))
 	{
 	  token = strtok_r (NULL, whitespace, &sp);
 	  if (!token)
@@ -814,7 +823,7 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
 	    }
 	  dev->family = xstrdup (token);
 	}
-      else if (streq (token, "charset"))
+      else if (!strcmp (token, "charset"))
 	break;
       else
 	{
@@ -824,7 +833,7 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
 	  int value;
 
 	  for (cp = id; *cp; cp++)
-	    if (streq (token, *cp))
+	    if (!strcmp (token, *cp))
 	      break;
 	  if (*cp == NULL)
 	    continue;		/* completely ignore unrecognized lines */
@@ -891,9 +900,9 @@ groff_read_DESC (const char *dev_name, groff_device_info * dev)
   dev->sizes[dev->n_sizes][0] = 0;
   dev->sizes[dev->n_sizes][1] = 0;
 
-  verbose_msg (2, _("Description file read successfully."));
+  msg (VM (2), _("Description file read successfully."));
   
-  pop_cust ();
+  err_pop_file_locator (&where);
   free (filename);
   free (line);
   return 1;
@@ -920,8 +929,10 @@ lossage:
       dev->font_name[i] = NULL;
     }
 #endif
+
+  err_pop_file_locator (&where);
   
-  verbose_msg (1, _("Error reading description file."));
+  msg (VM (1), _("Error reading description file."));
   
   return 0;
 }
@@ -929,8 +940,8 @@ lossage:
 /* Finds character with index CH (as returned by name_to_index() or
    number_to_index()) in font FONT and returns the associated metrics.
    Nonexistent characters have width 0. */
-char_metrics *
-font_get_char_metrics (const font_desc *font, int ch)
+struct char_metrics *
+font_get_char_metrics (const struct font_desc *font, int ch)
 {
   short index;
 
@@ -947,7 +958,7 @@ font_get_char_metrics (const font_desc *font, int ch)
 /* Finds kernpair consisting of CH1 and CH2, in that order, in font
    FONT and returns the associated kerning adjustment. */
 int
-font_get_kern_adjust (const font_desc *font, int ch1, int ch2)
+font_get_kern_adjust (const struct font_desc *font, int ch1, int ch2)
 {
   unsigned i;
 
@@ -965,20 +976,20 @@ font_get_kern_adjust (const font_desc *font, int ch1, int ch2)
 
 /* Returns a twelve-point fixed-pitch font that can be used as a
    last-resort fallback. */
-font_desc *
+struct font_desc *
 default_font (void)
 {
-  arena *font_arena;
-  static font_desc *font;
+  struct pool *font_pool;
+  static struct font_desc *font;
 
   if (font)
     return font;
-  arena_create (&font_arena);
-  font = arena_alloc (&font_arena, sizeof (font_desc));
-  font->owner = font_arena;
+  font_pool = pool_create ();
+  font = pool_alloc (font_pool, sizeof *font);
+  font->owner = font_pool;
   font->name = NULL;
-  font->internal_name = arena_strdup (&font_arena, _("<<fallback>>"));
-  font->encoding = arena_strdup (&font_arena, "text.enc");
+  font->internal_name = pool_strdup (font_pool, _("<<fallback>>"));
+  font->encoding = pool_strdup (font_pool, "text.enc");
   font->space_width = 12000;
   font->slant = 0.0;
   font->ligatures = 0;
