@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -23,16 +23,15 @@
 #include "filename.h"
 
 /* HTML output driver extension record. */
-typedef struct
+struct html_driver_ext
   {
     /* User parameters. */
     char *prologue_fn;		/* Prologue's filename relative to font dir. */
 
     /* Internal state. */
-    file_ext file;		/* Output file. */
+    struct file_ext file;	/* Output file. */
     int sequence_no;		/* Sequence number. */
-  }
-html_driver_ext;
+  };
 
 extern struct outp_class html_class;
 
