@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -38,17 +38,19 @@ char *alloca ();
 #include <assert.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include "arena.h"
-#include "file-handle.h"
+#include <float.h>
+#include "alloc.h"
+#include "command.h"
+#include "data-in.h"
 #include "dfm.h"
-#include "lexer.h"
-#include "lexerP.h"
-#include "var.h"
 #include "error.h"
-#include "format.h"
-#include "str.h"
-#include "vfm.h"
+#include "file-handle.h"
+#include "lexer.h"
 #include "misc.h"
+#include "pool.h"
+#include "str.h"
+#include "var.h"
+#include "vfm.h"
 
 #undef DEBUGGING
 /*#define DEBUGGING 1*/
@@ -114,7 +116,7 @@ static const char *content_names[PROX + 1] =
   };
 
 /* The data file to be read. */
-static file_handle *data_file;
+static struct file_handle *data_file;
 
 /* Format type. */
 static int fmt;			/* LIST or FREE. */
@@ -122,23 +124,23 @@ static int section;		/* LOWER or UPPER or FULL. */
 static int diag;		/* DIAGONAL or NODIAGONAL. */
 
 /* Arena used for all the MATRIX DATA allocations. */
-static arena *container;
+static struct pool *container;
 
 /* ROWTYPE_ specified explicitly in data? */
 static int explicit_rowtype;
 
 /* ROWTYPE_, VARNAME_ variables. */
-static variable *rowtype_, *varname_;
+static struct variable *rowtype_, *varname_;
 
 /* Is is per-factor data? */
 int is_per_factor[PROX + 1];
 
 /* Single SPLIT FILE variable. */
-static variable *single_split;
+static struct variable *single_split;
 
 /* Factor variables.  */
 static int n_factors;
-static variable **factors;
+static struct variable **factors;
 
 /* Number of cells, or -1 if none. */
 static int cells;
@@ -156,8 +158,8 @@ static int n_continuous;
 /* Index into default_dict.var of first continuous variables. */
 static int first_continuous;
 
-static int compare_variables_by_mxd_vartype (const void *_a,
-					     const void *_b);
+static int compare_variables_by_mxd_vartype (const void *pa,
+					     const void *pb);
 static void read_matrices_without_rowtype (void);
 static void read_matrices_with_rowtype (void);
 static int string_to_content_type (char *, int *);
@@ -171,10 +173,10 @@ cmd_matrix_data (void)
 {
   unsigned seen = 0;
   
-  match_id (MATRIX);
-  match_id (DATA);
-  
-  arena_create (&container);
+  lex_match_id ("MATRIX");
+  lex_match_id ("DATA");
+
+  container = pool_create ();
 
   discard_variables ();
 
@@ -190,9 +192,9 @@ cmd_matrix_data (void)
   n_contents = 0;
   while (token != '.')
     {
-      match_tok ('/');
+      lex_match ('/');
 
-      if (match_id (VARIABLES))
+      if (lex_match_id ("VARIABLES"))
 	{
 	  char **v;
 	  int nv;
@@ -204,7 +206,7 @@ cmd_matrix_data (void)
 	    }
 	  seen |= 1;
 	  
-	  match_tok ('=');
+	  lex_match ('=');
 	  if (!parse_DATA_LIST_vars (&v, &nv, PV_NO_DUPLICATE))
 	    goto lossage;
 	  
@@ -212,7 +214,7 @@ cmd_matrix_data (void)
 	    int i;
 
 	    for (i = 0; i < nv; i++)
-	      if (streq (v[i], "VARNAME_"))
+	      if (!strcmp (v[i], "VARNAME_"))
 		{
 		  msg (SE, _("VARNAME_ cannot be explicitly specified on "
 			     "VARIABLES."));
@@ -228,9 +230,9 @@ cmd_matrix_data (void)
 
 	    for (i = 0; i < nv; i++)
 	      {
-		variable *new_var;
+		struct variable *new_var;
 		
-		if (!streq (v[i], "ROWTYPE_"))
+		if (strcmp (v[i], "ROWTYPE_"))
 		  {
 		    new_var = force_create_variable (&default_dict, v[i],
 						     NUMERIC, 0);
@@ -251,43 +253,43 @@ cmd_matrix_data (void)
 	    rowtype_->p.mxd.subtype = 0;
 	  }
 	}
-      else if (match_id (FILE))
+      else if (lex_match_id ("FILE"))
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 	  data_file = fh_parse_file_handle ();
 	  if (!data_file)
 	    goto lossage;
 	}
-      else if (match_id (FORMAT))
+      else if (lex_match_id ("FORMAT"))
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 
-	  while (token == ID)
+	  while (token == T_ID)
 	    {
-	      if (match_id (LIST))
+	      if (lex_match_id ("LIST"))
 		fmt = LIST;
-	      else if (match_id (FREE))
+	      else if (lex_match_id ("FREE"))
 		fmt = FREE;
-	      else if (match_id (LOWER))
+	      else if (lex_match_id ("LOWER"))
 		section = LOWER;
-	      else if (match_id (UPPER))
+	      else if (lex_match_id ("UPPER"))
 		section = UPPER;
-	      else if (match_id (FULL))
+	      else if (lex_match_id ("FULL"))
 		section = FULL;
-	      else if (match_id (DIAGONAL))
+	      else if (lex_match_id ("DIAGONAL"))
 		diag = DIAGONAL;
-	      else if (match_id (NODIAGONAL))
+	      else if (lex_match_id ("NODIAGONAL"))
 		diag = NODIAGONAL;
 	      else 
 		{
-		  syntax_error (_("in FORMAT subcommand"));
+		  lex_error (_("in FORMAT subcommand"));
 		  goto lossage;
 		}
 	    }
 	}
-      else if (match_id (SPLIT))
+      else if (lex_match_id ("SPLIT"))
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 
 	  if (seen & 2)
 	    {
@@ -296,44 +298,43 @@ cmd_matrix_data (void)
 	    }
 	  seen |= 2;
 	  
-	  if (token != ID)
+	  if (token != T_ID)
 	    {
-	      syntax_error (_("in SPLIT subcommand"));
+	      lex_error (_("in SPLIT subcommand"));
 	      goto lossage;
 	    }
 	  
-	  if (!is_varname (tokstr)
-	      && (lookahead () == '.' || lookahead () == '/'))
+	  if (!is_varname (tokid)
+	      && (lex_look_ahead () == '.' || lex_look_ahead () == '/'))
 	    {
-	      if (streq (tokstr, "ROWTYPE_") || streq (tokstr, "VARNAME_"))
+	      if (!strcmp (tokid, "ROWTYPE_") || !strcmp (tokid, "VARNAME_"))
 		{
 		  msg (SE, _("Split variable may not be named ROWTYPE_ "
 			     "or VARNAME_."));
 		  goto lossage;
 		}
 
-	      single_split = force_create_variable (&default_dict, tokstr,
+	      single_split = force_create_variable (&default_dict, tokid,
 						    NUMERIC, 0);
-	      get_token ();
+	      lex_get ();
 
 	      single_split->p.mxd.vartype = MXD_CONTINUOUS;
 
 	      default_dict.n_splits = 1;
-	      default_dict.splits = xmalloc (2 * sizeof (variable *));
+	      default_dict.splits = xmalloc (2 * sizeof *default_dict.splits);
 	      default_dict.splits[0] = single_split;
 	      default_dict.splits[1] = NULL;
 	    }
 	  else
 	    {
-	      variable **v;
+	      struct variable **v;
 	      int n;
 
 	      if (!parse_variables (NULL, &v, &n, PV_NO_DUPLICATE))
 		goto lossage;
 
 	      default_dict.n_splits = n;
-	      default_dict.splits = v = xrealloc (v, (sizeof (variable *)
-						      * (n + 1)));
+	      default_dict.splits = v = xrealloc (v, sizeof *v * (n + 1));
 	      v[n] = NULL;
 	    }
 	  
@@ -345,7 +346,7 @@ cmd_matrix_data (void)
 		if (default_dict.splits[i]->p.mxd.vartype != MXD_CONTINUOUS)
 		  {
 		    msg (SE, _("Split variable %s is already another type."),
-			 tokstr);
+			 tokid);
 		    goto lossage;
 		  }
 		default_dict.splits[i]->p.mxd.vartype = MXD_SPLIT;
@@ -353,9 +354,9 @@ cmd_matrix_data (void)
 	      }
 	  }
 	}
-      else if (match_id (FACTORS))
+      else if (lex_match_id ("FACTORS"))
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 	  
 	  if (seen & 4)
 	    {
@@ -375,7 +376,7 @@ cmd_matrix_data (void)
 		if (factors[i]->p.mxd.vartype != MXD_CONTINUOUS)
 		  {
 		    msg (SE, _("Factor variable %s is already another type."),
-			 tokstr);
+			 tokid);
 		    goto lossage;
 		  }
 		factors[i]->p.mxd.vartype = MXD_FACTOR;
@@ -383,9 +384,9 @@ cmd_matrix_data (void)
 	      }
 	  }
 	}
-      else if (match_id (CELLS))
+      else if (lex_match_id ("CELLS"))
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 	  
 	  if (cells != -1)
 	    {
@@ -393,18 +394,18 @@ cmd_matrix_data (void)
 	      goto lossage;
 	    }
 
-	  if (token != NUM || tokint == NOT_LONG || tokint < 1)
+	  if (!lex_integer_p () || lex_integer () < 1)
 	    {
-	      syntax_error (_("expecting positive integer"));
+	      lex_error (_("expecting positive integer"));
 	      goto lossage;
 	    }
 
-	  cells = tokint;
-	  get_token ();
+	  cells = lex_integer ();
+	  lex_get ();
 	}
-      else if (match_id (N))
+      else if (lex_match_id ("N"))
 	{
-	  match_tok ('=');
+	  lex_match ('=');
 
 	  if (pop_n != -1)
 	    {
@@ -412,16 +413,16 @@ cmd_matrix_data (void)
 	      goto lossage;
 	    }
 
-	  if (token != NUM || tokint == NOT_LONG || tokint < 1)
+	  if (!lex_integer_p () || lex_integer () < 1)
 	    {
-	      syntax_error (_("expecting positive integer"));
+	      lex_error (_("expecting positive integer"));
 	      goto lossage;
 	    }
 
-	  pop_n = tokint;
-	  get_token ();
+	  pop_n = lex_integer ();
+	  lex_get ();
 	}
-      else if (match_id (CONTENTS))
+      else if (lex_match_id ("CONTENTS"))
 	{
 	  int inside_parens = 0;
 	  unsigned collide = 0;
@@ -434,7 +435,7 @@ cmd_matrix_data (void)
 	    }
 	  seen |= 8;
 
-	  match_tok ('=');
+	  lex_match ('=');
 	  
 	  {
 	    int i;
@@ -445,7 +446,7 @@ cmd_matrix_data (void)
 
 	  for (;;)
 	    {
-	      if (match_tok ('('))
+	      if (lex_match ('('))
 		{
 		  if (inside_parens)
 		    {
@@ -455,7 +456,7 @@ cmd_matrix_data (void)
 		  inside_parens = 1;
 		  item = LPAREN;
 		}
-	      else if (match_tok (')'))
+	      else if (lex_match (')'))
 		{
 		  if (!inside_parens)
 		    {
@@ -475,20 +476,20 @@ cmd_matrix_data (void)
 		  int content_type;
 		  int collide_index;
 		  
-		  if (token != ID)
+		  if (token != T_ID)
 		    {
-		      syntax_error (_("in CONTENTS subcommand"));
+		      lex_error (_("in CONTENTS subcommand"));
 		      goto lossage;
 		    }
 
-		  content_type = string_to_content_type (tokstr,
+		  content_type = string_to_content_type (tokid,
 							 &collide_index);
 		  if (content_type == -1)
 		    {
-		      syntax_error (_("in CONTENTS subcommand"));
+		      lex_error (_("in CONTENTS subcommand"));
 		      goto lossage;
 		    }
-		  get_token ();
+		  lex_get ();
 
 		  if (collide & (1 << collide_index))
 		    {
@@ -516,14 +517,14 @@ cmd_matrix_data (void)
 	}
       else 
 	{
-	  syntax_error (NULL);
+	  lex_error (NULL);
 	  goto lossage;
 	}
     }
   
   if (token != '.')
     {
-      syntax_error (_("expecting end of command"));
+      lex_error (_("expecting end of command"));
       goto lossage;
     }
   
@@ -571,7 +572,7 @@ cmd_matrix_data (void)
   {
     int i;
     
-    qsort (default_dict.var, default_dict.nvar, sizeof (variable *),
+    qsort (default_dict.var, default_dict.nvar, sizeof *default_dict.var,
 	   compare_variables_by_mxd_vartype);
 
     for (i = 0; i < default_dict.nvar; i++)
@@ -580,7 +581,7 @@ cmd_matrix_data (void)
 
   /* Set formats. */
   {
-    static const fmt_spec fmt_tab[MXD_COUNT] =
+    static const struct fmt_spec fmt_tab[MXD_COUNT] =
       {
 	{FMT_F, 4, 0},
         {FMT_A, 8, 0},
@@ -594,7 +595,7 @@ cmd_matrix_data (void)
     first_continuous = -1;
     for (i = 0; i < default_dict.nvar; i++)
       {
-	variable *v = default_dict.var[i];
+	struct variable *v = default_dict.var[i];
 	int type = v->p.mxd.vartype;
 	
 	assert (type >= 0 && type < MXD_COUNT);
@@ -622,15 +623,15 @@ cmd_matrix_data (void)
   else
     read_matrices_without_rowtype ();
 
-  arena_destroy (&container);
+  pool_destroy (container);
 
-  return 1;
+  return CMD_SUCCESS;
 
 lossage:
   discard_variables ();
   free (factors);
-  arena_destroy (&container);
-  return 0;
+  pool_destroy (container);
+  return CMD_FAILURE;
 }
 
 /* Look up string S as a content-type name and return the
@@ -668,7 +669,7 @@ string_to_content_type (char *s, int *collide)
     };
 
   for (tp = tab; tp->value != -1; tp++)
-    if (streq (s, tp->string))
+    if (!strcmp (s, tp->string))
       {
 	if (collide)
 	  *collide = tp->collide;
@@ -681,10 +682,10 @@ string_to_content_type (char *s, int *collide)
 /* Compare two variables using p.mxd.vartype and p.mxd.subtype
    fields. */
 static int
-compare_variables_by_mxd_vartype (const void *_a, const void *_b)
+compare_variables_by_mxd_vartype (const void *pa, const void *pb)
 {
-  matrix_data_proc *a = &(*((variable **) _a))->p.mxd;
-  matrix_data_proc *b = &(*((variable **) _b))->p.mxd;
+  struct matrix_data_proc *a = &(*((struct variable **) pa))->p.mxd;
+  struct matrix_data_proc *b = &(*((struct variable **) pb))->p.mxd;
   
   return (a->vartype != b->vartype
 	  ? a->vartype - b->vartype
@@ -979,14 +980,17 @@ static int
 
       if (is_num)
 	{
-	  fmt_spec f;
+	  struct data_in di;
 
-	  mtoken = MNUM;
-	  f.type = FMT_F;
-	  f.w = mtoklen;
-	  f.d = 0;
-	  if (!parse_string_as_format (mtokstr, mtoklen, &f, first_column,
-				       (value *) &mtokval))
+	  di.s = mtokstr;
+	  di.e = mtokstr + mtoklen;
+	  di.v = (union value *) &mtokval;
+	  di.f1 = first_column;
+	  di.format.type = FMT_F;
+	  di.format.w = mtoklen;
+	  di.format.d = 0;
+
+	  if (!data_in (&di))
 	    return 0;
 	}
       else
@@ -1054,8 +1058,8 @@ read_matrices_without_rowtype (void)
     cells = 1;
   
   mtoken = MNULL;
-  split_values = xmalloc (sizeof (double) * default_dict.n_splits);
-  nr_factor_values = xmalloc (sizeof (double) * n_factors * cells);
+  split_values = xmalloc (sizeof *split_values * default_dict.n_splits);
+  nr_factor_values = xmalloc (sizeof *nr_factor_values * n_factors * cells);
   max_cell_index = 0;
 
   matrix_data_source.read = (void (*)(void)) matrix_data_read_without_rowtype;
@@ -1203,7 +1207,7 @@ nr_read_data_lines (int per_factor, int cell, int content, int compare)
 	      return 0;
 	    if (mtoken != MNUM)
 	      {
-		msg (SE, _("Syntax error expecting value for %s %s."),
+		msg (SE, _("expecting value for %s %s"),
 		     default_dict.var[j]->name, context ());
 		return 0;
 	      }
@@ -1232,7 +1236,7 @@ matrix_data_read_without_rowtype (void)
   {
     int *cp;
 
-    nr_data = arena_alloc (&container, (PROX + 1) * sizeof (double **));
+    nr_data = pool_alloc (container, (PROX + 1) * sizeof *nr_data);
     
     {
       int i;
@@ -1255,12 +1259,12 @@ matrix_data_read_without_rowtype (void)
 	    int n_vectors = per_factor ? cells : 1;
 	    int i;
 	    
-	    nr_data[*cp] = arena_alloc (&container,
-					n_vectors * sizeof (double *));
+	    nr_data[*cp] = pool_alloc (container,
+				       n_vectors * sizeof **nr_data);
 	    
 	    for (i = 0; i < n_vectors; i++)
-	      nr_data[*cp][i] = arena_alloc (&container,
-					     n_entries * sizeof (double));
+	      nr_data[*cp][i] = pool_alloc (container,
+					    n_entries * sizeof ***nr_data);
 	  }
 	}
   }
@@ -1352,8 +1356,11 @@ nr_read_splits (int compare)
 	if (!mget_token ())
 	  return 0;
 	if (mtoken != MNUM)
-	  return msg (SE, _("Syntax error expecting SPLIT FILE value %s."),
-		      context ());
+	  {
+	    msg (SE, _("Syntax error expecting SPLIT FILE value %s."),
+		 context ());
+	    return 0;
+	  }
 
 	if (!compare)
 	  split_values[i] = mtokval;
@@ -1397,8 +1404,11 @@ nr_read_factors (int cell)
 	if (!mget_token ())
 	  return 0;
 	if (mtoken != MNUM)
-	  return msg (SE, _("Syntax error expecting factor value %s."),
-		      context ());
+	  {
+	    msg (SE, _("Syntax error expecting factor value %s."),
+		 context ());
+	    return 0;
+	  }
 	
 	if (!compare)
 	  nr_factor_values[i + n_factors * cell] = mtokval;
@@ -1423,8 +1433,8 @@ dump_cell_content (int content, double *cp)
   int type = content_type[content];
 
   {
-    strbarepadcpy (temp_case->data[rowtype_->fv].s,
-		   content_names[content], 8, ' ');
+    st_bare_pad_copy (temp_case->data[rowtype_->fv].s,
+		      content_names[content], 8);
     
     if (type != 1)
       memset (&temp_case->data[varname_->fv].s, ' ', 8);
@@ -1448,10 +1458,9 @@ dump_cell_content (int content, double *cp)
 	    cp++;
 	  }
 	if (type == 1)
-	  strbarepadcpy (temp_case->data[varname_->fv].s,
-			 default_dict.var[first_continuous
-					 + i]->name,
-			 8, ' ');
+	  st_bare_pad_copy (temp_case->data[varname_->fv].s,
+			    default_dict.var[first_continuous + i]->name,
+			    8);
 	debug_printf (("\n"));
 	write_case ();
       }
@@ -1523,20 +1532,19 @@ nr_output_data (void)
 static int wr_content;
 
 /* All the data for one set of factor values. */
-typedef struct factor_data
+struct factor_data
   {
     double *factors;
     int n_rows[PROX + 1];
     double *data[PROX + 1];
     struct factor_data *next;
-  }
-factor_data;
+  };
 
 /* All the data, period. */
-factor_data *wr_data;
+struct factor_data *wr_data;
 
 /* Current factor. */
-factor_data *wr_current;
+struct factor_data *wr_current;
 
 static int wr_read_splits (void);
 static int wr_output_data (void);
@@ -1600,7 +1608,7 @@ wr_read_splits (void)
   else
     {
       compare = 0;
-      split_values = xmalloc (sizeof (double) * default_dict.n_splits);
+      split_values = xmalloc (sizeof *split_values * default_dict.n_splits);
     }
   
   {
@@ -1612,8 +1620,11 @@ wr_read_splits (void)
 	if (!mget_token ())
 	  return 0;
 	if (mtoken != MNUM)
-	  return msg (SE, _("Syntax error %s expecting SPLIT FILE value."),
-		      context ());
+	  {
+	    msg (SE, _("Syntax error %s expecting SPLIT FILE value."),
+		 context ());
+	    return 0;
+	  }
 
 	if (compare && split_values[i] != mtokval && !different)
 	  {
@@ -1632,10 +1643,10 @@ wr_read_splits (void)
 /* Return strcmp()-type comparison of the n_factors factors at _A and
    _B.  Sort missing values toward the end. */
 static int
-compare_factors (const void *_a, const void *_b)
+compare_factors (const void *pa, const void *pb)
 {
-  const double *a = (*(factor_data **) _a)->factors;
-  const double *b = (*(factor_data **) _b)->factors;
+  const double *a = (*(struct factor_data **) pa)->factors;
+  const double *b = (*(struct factor_data **) pb)->factors;
   int i;
 
   for (i = 0; i < n_factors; i++, a++, b++)
@@ -1668,16 +1679,16 @@ wr_output_data (void)
 
   /* Sort the wr_data list. */
   {
-    factor_data **factors;
-    factor_data *iter;
+    struct factor_data **factors;
+    struct factor_data *iter;
     int i;
 
-    factors = xmalloc (sizeof (factor_data *) * cells);
+    factors = xmalloc (sizeof *factors * cells);
 
     for (i = 0, iter = wr_data; iter; iter = iter->next, i++)
       factors[i] = iter;
 
-    qsort (factors, cells, sizeof (factor_data *), compare_factors);
+    qsort (factors, cells, sizeof *factors, compare_factors);
 
     wr_data = factors[0];
     for (i = 0; i < cells - 1; i++)
@@ -1689,7 +1700,7 @@ wr_output_data (void)
 
   /* Write out records for every set of factor values. */
   {
-    factor_data *iter;
+    struct factor_data *iter;
     
     for (iter = wr_data; iter; iter = iter->next)
       {
@@ -1738,9 +1749,10 @@ wr_output_data (void)
       }
   }
   
-  arena_destroy (&container);
+  pool_destroy (container);
+  container = pool_create ();
+  
   wr_data = wr_current = NULL;
-  arena_create (&container);
   
   return 1;
 }
@@ -1750,11 +1762,15 @@ static int
 wr_read_rowtype (void)
 {
   if (wr_content != -1)
-    return msg (SE, _("Multiply specified ROWTYPE_ %s."),
-		context ());
+    {
+      msg (SE, _("Multiply specified ROWTYPE_ %s."), context ());
+      return 0;
+    }
   if (mtoken != MSTR)
-    return msg (SE, _("Syntax error %s expecting ROWTYPE_ string."),
-		context ());
+    {
+      msg (SE, _("Syntax error %s expecting ROWTYPE_ string."), context ());
+      return 0;
+    }
   
   {
     char s[16];
@@ -1770,7 +1786,10 @@ wr_read_rowtype (void)
   }
 
   if (wr_content == -1)
-    return msg (SE, _("Syntax error %s."), context ());
+    {
+      msg (SE, _("Syntax error %s."), context ());
+      return 0;
+    }
 
   return 1;
 }
@@ -1780,7 +1799,7 @@ wr_read_rowtype (void)
 static int 
 wr_read_factors (void)
 {
-  double *factor_values = local_alloc (sizeof (double) * n_factors);
+  double *factor_values = local_alloc (sizeof *factor_values * n_factors);
 
   wr_content = -1;
   {
@@ -1823,16 +1842,14 @@ wr_read_factors (void)
       
       for (i = 0; i < n_factors; i++)
 	if (factor_values[i] != wr_current->factors[i])
-	  {
-	    goto cache_miss;
-	  }
+	  goto cache_miss;
       goto winnage;
     }
 
   /* Linear search through the list. */
 cache_miss:
   {
-    factor_data *iter;
+    struct factor_data *iter;
 
     for (iter = wr_data; iter; iter = iter->next)
       {
@@ -1851,9 +1868,9 @@ cache_miss:
 
   /* Not found.  Make a new item. */
   {
-    factor_data *new = arena_alloc (&container, sizeof (factor_data));
+    struct factor_data *new = pool_alloc (container, sizeof *new);
 
-    new->factors = arena_alloc (&container, sizeof (double) * n_factors);
+    new->factors = pool_alloc (container, sizeof *new->factors * n_factors);
     
     {
       int i;
@@ -1890,7 +1907,7 @@ lossage:
 static int 
 wr_read_indeps (void)
 {
-  factor_data *c = wr_current;
+  struct factor_data *c = wr_current;
   const int type = content_type[wr_content];
   const int n_rows = c->n_rows[wr_content];
   double *cp;
@@ -1903,8 +1920,8 @@ wr_read_indeps (void)
       if (type == 1)
 	n_items *= n_continuous;
       
-      c->data[wr_content] = arena_alloc (&container,
-					 sizeof (double) * n_items);
+      c->data[wr_content] = pool_alloc (container,
+					sizeof **c->data * n_items);
     }
 
   cp = &c->data[wr_content][n_rows * n_continuous];
@@ -1990,7 +2007,7 @@ wr_read_indeps (void)
 
 /* Matrix source. */
 
-case_stream matrix_data_source = 
+struct case_stream matrix_data_source = 
   {
     NULL,
     NULL,
