@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -22,12 +22,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
-#include <string.h>
 #include <ctype.h>
+#include "alloc.h"
 #include "approx.h"
-#include "common.h"
 #include "error.h"
 #include "filename.h"
+#include "lexer.h"
 #include "misc.h"
 #include "output.h"
 #include "settings.h"
@@ -37,7 +37,7 @@
    drivername:classname:devicetype:options, where devicetype is zero
    or more of screen, printer, listing? */
 
-/* FIXME: Have the reentrancy problems been solved?. */
+/* FIXME: Have the reentrancy problems been solved? */
 
 /* Where the output driver name came from. */
 enum
@@ -47,28 +47,26 @@ enum
   };
 
 /* Names the output drivers to be used. */
-typedef struct outp_names
+struct outp_names
   {
     char *name;			/* Name of the output driver. */
     int source;			/* OUTP_S_* */
     struct outp_names *next, *prev;
-  }
-outp_names;
+  };
 
 /* Defines an init file macro. */
-typedef struct outp_defn
+struct outp_defn
   {
     char *key;
     char *value;
     struct outp_defn *next, *prev;
-  }
-outp_defn;
+  };
 
-static outp_defn *outp_macros;
-static outp_names *outp_configure_vec;
+static struct outp_defn *outp_macros;
+static struct outp_names *outp_configure_vec;
 
-outp_driver_class_list *outp_class_list;
-outp_driver *outp_driver_list;
+struct outp_driver_class_list *outp_class_list;
+struct outp_driver *outp_driver_list;
 
 char *outp_title;
 char *outp_subtitle;
@@ -77,7 +75,7 @@ char *outp_subtitle;
    disabled. */
 int disabled_devices;
 
-static void destroy_driver (outp_driver *);
+static void destroy_driver (struct outp_driver *);
 static void configure_driver (char *);
 
 #if GLOBAL_DEBUGGING
@@ -89,9 +87,9 @@ static int iterating_driver_list;
 
 /* Add a class to the class list. */
 static void
-add_class (outp_class *class)
+add_class (struct outp_class *class)
 {
-  outp_driver_class_list *new_list = xmalloc (sizeof (outp_driver_class_list));
+  struct outp_driver_class_list *new_list = xmalloc (sizeof *new_list);
 
   new_list->class = class;
   new_list->ref_count = 0;
@@ -110,10 +108,10 @@ add_class (outp_class *class)
 
 /* Finds the outp_names in outp_configure_vec with name between BP and
    EP exclusive. */
-static outp_names *
+static struct outp_names *
 search_names (char *bp, char *ep)
 {
-  outp_names *n;
+  struct outp_names *n;
 
   for (n = outp_configure_vec; n; n = n->next)
     if ((int) strlen (n->name) == ep - bp && !memcmp (n->name, bp, ep - bp))
@@ -123,7 +121,7 @@ search_names (char *bp, char *ep)
 
 /* Deletes outp_names NAME from outp_configure_vec. */
 static void
-delete_name (outp_names * n)
+delete_name (struct outp_names * n)
 {
   free (n->name);
   if (n->prev)
@@ -140,7 +138,7 @@ delete_name (outp_names * n)
 static void
 add_name (char *bp, char *ep, int source)
 {
-  outp_names *n = xmalloc (sizeof (outp_names));
+  struct outp_names *n = xmalloc (sizeof *n);
   n->name = xmalloc (ep - bp + 1);
   memcpy (n->name, bp, ep - bp);
   n->name[ep - bp] = 0;
@@ -157,7 +155,7 @@ add_name (char *bp, char *ep, int source)
 static void
 check_configure_vec (void)
 {
-  outp_names *n;
+  struct outp_names *n;
 
   for (n = outp_configure_vec; n; n = n->next)
     if (n->source == OUTP_S_COMMAND_LINE)
@@ -173,13 +171,13 @@ check_configure_vec (void)
 static void
 expand_name (char *bp, char *ep)
 {
-  outp_names *n = search_names (bp, ep);
+  struct outp_names *n = search_names (bp, ep);
   if (!n)
     return;
   delete_name (n);
 
   bp = ep + 1;
-  while (1)
+  for (;;)
     {
       while (isspace ((unsigned char) *bp))
 	bp++;
@@ -200,17 +198,17 @@ const char *
 find_defn_value (const char *key)
 {
   static char buf[INT_DIGITS + 1];
-  outp_defn *d;
+  struct outp_defn *d;
 
   for (d = outp_macros; d; d = d->next)
-    if (streq (key, d->key))
+    if (!strcmp (key, d->key))
       return d->value;
-  if (streq (key, "viewwidth"))
+  if (!strcmp (key, "viewwidth"))
     {
       sprintf (buf, "%d", set_viewwidth);
       return buf;
     }
-  else if (streq (key, "viewlength"))
+  else if (!strcmp (key, "viewlength"))
     {
       sprintf (buf, "%d", set_viewlength);
       return buf;
@@ -223,13 +221,13 @@ find_defn_value (const char *key)
 int
 outp_init (void)
 {
-  extern outp_class ascii_class;
+  extern struct outp_class ascii_class;
 #if !NO_POSTSCRIPT
-  extern outp_class postscript_class;
-  extern outp_class epsf_class;
+  extern struct outp_class postscript_class;
+  extern struct outp_class epsf_class;
 #endif
 #if !NO_HTML
-  extern outp_class html_class;
+  extern struct outp_class html_class;
 #endif
 
   char def[] = "default";
@@ -252,7 +250,7 @@ outp_init (void)
 static void
 delete_macros (void)
 {
-  outp_defn *d, *next;
+  struct outp_defn *d, *next;
 
   for (d = outp_macros; d; d = next)
     {
@@ -272,16 +270,23 @@ outp_read_devices (void)
   char *init_fn;
 
   FILE *f = NULL;
-  char *buf = NULL;
-  size_t size = 0;
+  struct string line;
+  struct file_locator where;
 
 #if GLOBAL_DEBUGGING
   if (iterating_driver_list)
     reentrancy ();
 #endif
-  init_fn = search_path (getenv_default ("STAT_OUTPUT_INIT_FILE", "devices"),
-			 getenv_default ("STAT_OUTPUT_INIT_PATH", config_path),
-			 NULL);
+
+  init_fn = fn_search_path (fn_getenv_default ("STAT_OUTPUT_INIT_FILE",
+					       "devices"),
+			    fn_getenv_default ("STAT_OUTPUT_INIT_PATH",
+					       config_path),
+			    NULL);
+  where.filename = init_fn;
+  where.line_number = 0;
+  err_push_file_locator (&where);
+
   if (init_fn == NULL)
     {
       msg (IE, _("Cannot find output initialization file.  Use `-vv' to view "
@@ -289,27 +294,26 @@ outp_read_devices (void)
       goto exit;
     }
 
-  verbose_msg (1, _("%s: Opening device description file..."), init_fn);
+  msg (VM (1), _("%s: Opening device description file..."), init_fn);
   f = fopen (init_fn, "r");
-  if (!f)
+  if (f == NULL)
     {
       msg (IE, _("Opening %s: %s."), init_fn, strerror (errno));
       goto exit;
     }
 
-  push_cust (init_fn);
-  while (1)
+  ds_init (NULL, &line, 128);
+  for (;;)
     {
-      int len = get_config_line (&buf, &size, f);
       char *cp;
 
-      if (-1 == len)
+      if (!ds_get_config_line (f, &line, &where))
 	{
 	  if (ferror (f))
 	    msg (ME, _("Reading %s: %s."), init_fn, strerror (errno));
 	  break;
 	}
-      for (cp = buf; isspace ((unsigned char) *cp); cp++);
+      for (cp = ds_value (&line); isspace ((unsigned char) *cp); cp++);
       if (!strncmp ("define", cp, 6) && isspace ((unsigned char) cp[6]))
 	outp_configure_macro (&cp[7]);
       else if (*cp)
@@ -320,7 +324,7 @@ outp_read_devices (void)
 	    expand_name (cp, ep);
 	  else if (*ep == ':')
 	    {
-	      outp_names *n = search_names (cp, ep);
+	      struct outp_names *n = search_names (cp, ep);
 	      if (n)
 		{
 		  configure_driver (cp);
@@ -331,24 +335,24 @@ outp_read_devices (void)
 	    msg (IS, _("Syntax error."));
 	}
     }
-  pop_cust ();
   result = 1;
 
   check_configure_vec ();
 
 exit:
+  err_pop_file_locator (&where);
   if (f && -1 == fclose (f))
     msg (MW, _("Closing %s: %s."), init_fn, strerror (errno));
   free (init_fn);
-  free (buf);
+  ds_destroy (&line);
   delete_macros ();
   if (outp_driver_list == NULL)
     msg (MW, _("No output drivers are active."));
 
   if (result)
-    verbose_msg (2, _("Device definition file read successfully."));
+    msg (VM (2), _("Device definition file read successfully."));
   else
-    verbose_msg (1, _("Error reading device definition file."));
+    msg (VM (1), _("Error reading device definition file."));
   return result;
 }
 
@@ -356,7 +360,7 @@ exit:
 void
 outp_configure_clear (void)
 {
-  outp_names *n, *next;
+  struct outp_names *n, *next;
 
   for (n = outp_configure_vec; n; n = next)
     {
@@ -382,7 +386,7 @@ outp_configure_add (char *bp)
 void
 outp_configure_macro (char *bp)
 {
-  outp_defn *d;
+  struct outp_defn *d;
   char *ep;
 
   while (isspace ((unsigned char) *bp))
@@ -391,7 +395,7 @@ outp_configure_macro (char *bp)
   while (*ep && !isspace ((unsigned char) *ep) && *ep != '=')
     ep++;
 
-  d = xmalloc (sizeof (outp_defn));
+  d = xmalloc (sizeof *d);
   d->key = xmalloc (ep - bp + 1);
   memcpy (d->key, bp, ep - bp);
   d->key[ep - bp] = 0;
@@ -408,7 +412,7 @@ outp_configure_macro (char *bp)
     ep++;
   while (isspace ((unsigned char) *ep))
     ep++;
-  d->value = interp_vars (ep, find_defn_value);
+  d->value = fn_interp_vars (ep, find_defn_value);
   d->next = outp_macros;
   d->prev = NULL;
   if (outp_macros)
@@ -419,9 +423,9 @@ outp_configure_macro (char *bp)
 /* Destroys all the drivers in driver list *DL and sets *DL to
    NULL. */
 void
-destroy_list (outp_driver ** dl)
+destroy_list (struct outp_driver ** dl)
 {
-  outp_driver *d, *next;
+  struct outp_driver *d, *next;
 
   for (d = *dl; d; d = next)
     {
@@ -450,7 +454,7 @@ void
 outp_list_classes (void)
 {
   int width = set_viewwidth;
-  outp_driver_class_list *c;
+  struct outp_driver_class_list *c;
 
   printf (_("Driver classes:\n\t"));
   width -= 8;
@@ -468,23 +472,10 @@ outp_list_classes (void)
   putc('\n', stdout);
 }
 
-static int op_token;		/* `=', 'a', '{', 0. */
-static a_string op_tokstr;
-static int op_tokstr_size;
+static int op_token;		/* `=', 'a', 0. */
+static struct string op_tokstr;
 static char *prog;
 
-/* Ensures that op_tokstr.s is big enough for at least MIN_SIZE
-   bytes. */
-static inline void
-expand_op_tokstr (int min_size)
-{
-  if (op_tokstr_size < min_size)
-    {
-      op_tokstr_size = min_size * 2;
-      op_tokstr.s = xrealloc (op_tokstr.s, op_tokstr_size);
-    }
-}
-
 /* Parses a token from prog into op_token, op_tokstr.  Sets op_token
    to '=' on an equals sign, to 'a' on a string or identifier token,
    or to 0 at end of line.  Returns the new op_token. */
@@ -500,64 +491,68 @@ tokener (void)
   while (isspace ((unsigned char) *prog))
     prog++;
   if (!*prog)
-    return (op_token = 0);
-  else if (*prog == '=')
-    return (op_token = *prog++);
+    {
+      op_token = 0;
+      return 0;
+    }
+
+  if (*prog == '=')
+    op_token = *prog++;
   else
     {
-      op_tokstr.l = 0;
+      ds_clear (&op_tokstr);
 
-      expand_op_tokstr (16);
       if (*prog == '\'' || *prog == '"')
 	{
 	  int quote = *prog++;
 
 	  while (*prog && *prog != quote)
 	    {
-	      expand_op_tokstr (op_tokstr.l + 2);
 	      if (*prog != '\\')
-		op_tokstr.s[op_tokstr.l++] = *prog++;
+		ds_putchar (&op_tokstr, *prog++);
 	      else
 		{
+		  int c;
+		  
 		  prog++;
-		  assert (*prog);	/* How could a line end in `\'? */
+		  assert ((int) *prog);	/* How could a line end in `\'? */
 		  switch (*prog++)
 		    {
 		    case '\'':
-		      op_tokstr.s[op_tokstr.l++] = '\'';
+		      c = '\'';
 		      break;
 		    case '"':
-		      op_tokstr.s[op_tokstr.l++] = '"';
+		      c = '"';
 		      break;
 		    case '?':
-		      op_tokstr.s[op_tokstr.l++] = '?';
+		      c = '?';
 		      break;
 		    case '\\':
-		      op_tokstr.s[op_tokstr.l++] = '\\';
+		      c = '\\';
 		      break;
 		    case '}':
-		      op_tokstr.s[op_tokstr.l++] = '}';
+		      c = '}';
 		      break;
 		    case 'a':
-		      op_tokstr.s[op_tokstr.l++] = '\a';
+		      c = '\a';
 		      break;
 		    case 'b':
-		      op_tokstr.s[op_tokstr.l++] = '\b';
+		      c = '\b';
 		      break;
 		    case 'f':
-		      op_tokstr.s[op_tokstr.l++] = '\f';
+		      c = '\f';
 		      break;
 		    case 'n':
-		      op_tokstr.s[op_tokstr.l++] = '\n';
+		      c = '\n';
 		      break;
 		    case 'r':
-		      op_tokstr.s[op_tokstr.l++] = '\r';
+		      c = '\r';
 		      break;
 		    case 't':
-		      op_tokstr.s[op_tokstr.l++] = '\t';
+		      c = '\t';
 		      break;
 		    case 'v':
-		      op_tokstr.s[op_tokstr.l++] = '\v';
+		      c = '\v';
 		      break;
 		    case '0':
 		    case '1':
@@ -568,16 +563,15 @@ tokener (void)
 		    case '6':
 		    case '7':
 		      {
-			int c = prog[-1] - '0';
+			c = prog[-1] - '0';
 			while (*prog >= '0' && *prog <= '7')
 			  c = c * 8 + *prog++ - '0';
-			op_tokstr.s[op_tokstr.l++] = c;
 		      }
 		      break;
 		    case 'x':
 		    case 'X':
 		      {
-			int c = 0;
+			c = 0;
 			while (isxdigit ((unsigned char) *prog))
 			  {
 			    c *= 16;
@@ -588,80 +582,77 @@ tokener (void)
 				    - 'a' + 10);
 			    prog++;
 			  }
-			op_tokstr.s[op_tokstr.l++] = c;
 		      }
 		      break;
 		    default:
 		      msg (IS, _("Syntax error in string constant."));
 		    }
+		  ds_putchar (&op_tokstr, (unsigned char) c);
 		}
 	    }
 	  prog++;
 	}
       else
 	while (*prog && !isspace ((unsigned char) *prog) && *prog != '=')
-	  {
-	    expand_op_tokstr (op_tokstr.l + 2);
-	    op_tokstr.s[op_tokstr.l++] = *prog++;
-	  }
-      op_tokstr.s[op_tokstr.l] = 0;
-      return (op_token = 'a');
+	  ds_putchar (&op_tokstr, *prog++);
+      op_token = 'a';
     }
+
+  return 1;
 }
 
 /* Applies the user-specified options in string S to output driver D
    (at configuration time). */
 static void
-parse_options (char *s, outp_driver * d)
+parse_options (char *s, struct outp_driver * d)
 {
   prog = s;
   op_token = -1;
 
+  ds_init (NULL, &op_tokstr, 64);
   while (tokener ())
     {
       char key[65];
-      outp_value val;
 
       if (op_token != 'a')
 	{
 	  msg (IS, _("Syntax error in options."));
-	  return;
+	  break;
 	}
 
-      if (op_tokstr.l > 64)
-	op_tokstr.l = 64;
-      memcpy (key, op_tokstr.s, op_tokstr.l);
-      key[op_tokstr.l] = 0;
+      ds_truncate (&op_tokstr, 64);
+      strcpy (key, ds_value (&op_tokstr));
 
-      if (tokener () != '=')
+      tokener ();
+      if (op_token != '=')
 	{
 	  msg (IS, _("Syntax error in options (`=' expected)."));
-	  return;
+	  break;
 	}
+
       tokener ();
-      if (op_token != 'a' && op_token != '{')
+      if (op_token != 'a')
 	{
 	  msg (IS, _("Syntax error in options (value expected after `=')."));
-	  return;
+	  break;
 	}
-      val.type = op_token;
-      val.string = op_tokstr;
-      d->class->option (d, key, &val);
+      d->class->option (d, key, &op_tokstr);
     }
+  ds_destroy (&op_tokstr);
 }
 
 /* Find the driver in outp_driver_list with name NAME. */
-static outp_driver *
+static struct outp_driver *
 find_driver (char *name)
 {
-  outp_driver *d;
+  struct outp_driver *d;
 
 #if GLOBAL_DEBUGGING
   if (iterating_driver_list)
     reentrancy ();
 #endif
   for (d = outp_driver_list; d; d = d->next)
-    if (streq (d->name, name))
+    if (!strcmp (d->name, name))
       return d;
   return NULL;
 }
@@ -705,10 +696,10 @@ static void
 configure_driver (char *s)
 {
   char *token, *cp;
-  outp_driver *d = NULL, *iter;
-  outp_driver_class_list *c = NULL;
+  struct outp_driver *d = NULL, *iter;
+  struct outp_driver_class_list *c = NULL;
 
-  s = interp_vars (s, find_defn_value);
+  s = fn_interp_vars (s, find_defn_value);
 
   /* Driver name. */
   token = colon_tokenize (s, &cp);
@@ -718,7 +709,7 @@ configure_driver (char *s)
       goto error;
     }
 
-  d = xmalloc (sizeof (outp_driver));
+  d = xmalloc (sizeof *d);
 
   d->class = NULL;
   d->name = xstrdup (token);
@@ -740,7 +731,7 @@ configure_driver (char *s)
     }
 
   for (c = outp_class_list; c; c = c->next)
-    if (streq (c->class->name, token))
+    if (!strcmp (c->class->name, token))
       break;
   if (!c)
     {
@@ -772,11 +763,11 @@ configure_driver (char *s)
       for (type = strtok_r (token, " \t\r\v", &sp); type;
 	   type = strtok_r (NULL, " \t\r\v", &sp))
 	{
-	  if (streq (type, "listing"))
+	  if (!strcmp (type, "listing"))
 	    d->device |= OUTP_DEV_LISTING;
-	  else if (streq (type, "screen"))
+	  else if (!strcmp (type, "screen"))
 	    d->device |= OUTP_DEV_SCREEN;
-	  else if (streq (type, "printer"))
+	  else if (!strcmp (type, "printer"))
 	    d->device |= OUTP_DEV_PRINTER;
 	  else
 	    {
@@ -819,13 +810,13 @@ exit:
 
 /* Destroys output driver D. */
 static void
-destroy_driver (outp_driver *d)
+destroy_driver (struct outp_driver *d)
 {
   if (d->page_open)
     d->class->close_page (d);
   if (d->class)
     {
-      outp_driver_class_list *c;
+      struct outp_driver_class_list *c;
 
       if (d->driver_open)
 	d->class->close_driver (d);
@@ -833,7 +824,7 @@ destroy_driver (outp_driver *d)
       for (c = outp_class_list; c; c = c->next)
 	if (c->class == d->class)
 	  break;
-      assert (c);
+      assert (c != NULL);
       
       c->ref_count--;
       if (c->ref_count == 0)
@@ -857,8 +848,8 @@ destroy_driver (outp_driver *d)
 static int
 option_cmp (const void *a, const void *b)
 {
-  const outp_option *o1 = a;
-  const outp_option *o2 = b;
+  const struct outp_option *o1 = a;
+  const struct outp_option *o2 = b;
   return strcmp (o1->keyword, o2->keyword);
 }
 
@@ -866,11 +857,11 @@ option_cmp (const void *a, const void *b)
    information structure INFO.  Returns category code or 0 on failure;
    if category code is negative then stores subcategory in *SUBCAT. */
 int
-outp_match_keyword (const char *s, outp_option *tab,
-		    outp_option_info *info, int *subcat)
+outp_match_keyword (const char *s, struct outp_option *tab,
+		    struct outp_option_info *info, int *subcat)
 {
   char *cp;
-  outp_option *oip;
+  struct outp_option *oip;
 
   /* Form hash table. */
   if (NULL == info->initial)
@@ -878,13 +869,13 @@ outp_match_keyword (const char *s, outp_option *tab,
       /* Count items. */
       int count, i;
       char s[256], *cp;
-      outp_option *ptr[255], **oip;
+      struct outp_option *ptr[255], **oip;
 
       for (count = 0; tab[count].keyword[0]; count++)
 	;
 
       /* Sort items. */
-      qsort (tab, count, sizeof (outp_option), option_cmp);
+      qsort (tab, count, sizeof *tab, option_cmp);
 
       cp = s;
       oip = ptr;
@@ -899,8 +890,8 @@ outp_match_keyword (const char *s, outp_option *tab,
       *++cp = 0;
 
       info->initial = xstrdup (s);
-      info->options = xmalloc (sizeof (outp_option *) * (cp - s));
-      memcpy (info->options, ptr, sizeof (outp_option *) * (cp - s));
+      info->options = xmalloc (sizeof *info->options * (cp - s));
+      memcpy (info->options, ptr, sizeof *info->options * (cp - s));
     }
 
   cp = info->initial;
@@ -920,7 +911,7 @@ outp_match_keyword (const char *s, outp_option *tab,
 #if 0
       printf ("- %s\n", oip->keyword);
 #endif
-      if (streq (s, oip->keyword))
+      if (!strcmp (s, oip->keyword))
 	{
 	  if (oip->cat < 0)
 	    *subcat = oip->subcat;
@@ -945,26 +936,26 @@ int
 outp_evaluate_dimension (char *dimen, char **tail)
 {
   char *s = dimen;
-  char *_tail;
+  char *ptail;
   double value;
 
-  value = strtod (s, &_tail);
-  if (_tail == s)
+  value = strtod (s, &ptail);
+  if (ptail == s)
     goto lossage;
-  if (*_tail == '-')
+  if (*ptail == '-')
     {
       double b, c;
-      s = &_tail[1];
-      b = strtod (s, &_tail);
-      if (b <= 0.0 || _tail == s)
+      s = &ptail[1];
+      b = strtod (s, &ptail);
+      if (b <= 0.0 || ptail == s)
 	goto lossage;
-      if (*_tail != '/')
+      if (*ptail != '/')
 	goto lossage;
-      s = &_tail[1];
-      c = strtod (s, &_tail);
-      if (c <= 0.0 || _tail == s)
+      s = &ptail[1];
+      c = strtod (s, &ptail);
+      if (c <= 0.0 || ptail == s)
 	goto lossage;
-      s = _tail;
+      s = ptail;
       if (approx_eq (c, 0.0))
 	goto lossage;
       if (value > 0)
@@ -972,18 +963,18 @@ outp_evaluate_dimension (char *dimen, char **tail)
       else
 	value -= b / c;
     }
-  else if (*_tail == '/')
+  else if (*ptail == '/')
     {
       double b;
-      s = &_tail[1];
-      b = strtod (s, &_tail);
-      if (approx_le (b, 0.0) || _tail == s)
+      s = &ptail[1];
+      b = strtod (s, &ptail);
+      if (approx_le (b, 0.0) || ptail == s)
 	goto lossage;
-      s = _tail;
+      s = ptail;
       value /= b;
     }
   else
-    s = _tail;
+    s = ptail;
   if (*s == 0 || isspace ((unsigned char) *s))
     {
       if (value < 50.0)
@@ -1033,13 +1024,13 @@ outp_evaluate_dimension (char *dimen, char **tail)
 	    *tail = NULL;
 	    return 0;
 	  }
-      _tail += 2;
+      ptail += 2;
       value *= factor;
     }
   if (approx_lt (value, 0.0))
     goto lossage;
   if (tail)
-    *tail = _tail;
+    *tail = ptail;
   return value + 0.5;
 
 lossage:
@@ -1069,14 +1060,21 @@ internal_get_paper_size (char *size, int *h, int *v)
   else if (*tail == 'b' && tail[1] == 'y')
     tail += 2;
   else
-    return msg (SE, _("`x' expected in paper size `%s'."), size);
+    {
+      msg (SE, _("`x' expected in paper size `%s'."), size);
+      return 0;
+    }
   *v = outp_evaluate_dimension (tail, &tail);
   if (tail == NULL)
     return 0;
   while (isspace ((unsigned char) *tail))
     tail++;
   if (*tail)
-    return msg (SE, _("Trailing garbage `%s' on paper size `%s'."), tail, size);
+    {
+      msg (SE, _("Trailing garbage `%s' on paper size `%s'."), tail, size);
+      return 0;
+    }
+  
   return 1;
 }
 
@@ -1089,22 +1087,21 @@ internal_get_paper_size (char *size, int *h, int *v)
 int
 outp_get_paper_size (char *size, int *h, int *v)
 {
-  typedef struct
-  {
-    char *name;
-    int use;
-    int h, v;
-  }
-  paper_size;
+  struct paper_size
+    {
+      char *name;
+      int use;
+      int h, v;
+    };
 
-  static paper_size cache[4];
+  static struct paper_size cache[4];
   static int use;
 
   FILE *f;
   char *pprsz_fn;
 
-  char *buf = NULL;
-  size_t buf_size = 0;
+  struct string line;
+  struct file_locator where;
 
   int free_it = 0;
   int result = 0;
@@ -1122,7 +1119,11 @@ outp_get_paper_size (char *size, int *h, int *v)
   while (isspace ((unsigned char) *ep) && ep >= size)
     ep--;
   if (ep == size)
-    return msg (SE, _("Paper size name must not be empty."));
+    {
+      msg (SE, _("Paper size name must not be empty."));
+      return 0;
+    }
+  
   ep++;
   if (*ep)
     *ep = 0;
@@ -1137,19 +1138,23 @@ outp_get_paper_size (char *size, int *h, int *v)
 	return 1;
       }
 
-  pprsz_fn = search_path (getenv_default ("STAT_OUTPUT_PAPERSIZE_FILE",
-					  "papersize"),
-			  getenv_default ("STAT_OUTPUT_INIT_PATH",
-					  config_path),
-			  NULL);
+  pprsz_fn = fn_search_path (fn_getenv_default ("STAT_OUTPUT_PAPERSIZE_FILE",
+						"papersize"),
+			     fn_getenv_default ("STAT_OUTPUT_INIT_PATH",
+						config_path),
+			     NULL);
+
+  where.filename = pprsz_fn;
+  where.line_number = 0;
+  err_push_file_locator (&where);
+
   if (pprsz_fn == NULL)
     {
       msg (IE, _("Cannot find `papersize' configuration file."));
       goto exit;
     }
 
-  verbose_msg (1, _("%s: Opening paper size definition file..."),
-	       pprsz_fn);
+  msg (VM (1), _("%s: Opening paper size definition file..."), pprsz_fn);
   f = fopen (pprsz_fn, "r");
   if (!f)
     {
@@ -1157,26 +1162,25 @@ outp_get_paper_size (char *size, int *h, int *v)
       goto exit;
     }
 
-  push_cust (pprsz_fn);
-  while (1)
+  ds_init (NULL, &line, 128);
+  for (;;)
     {
-      int len = get_config_line (&buf, &buf_size, f);
       char *cp, *bp, *ep;
 
-      if (-1 == len)
+      if (!ds_get_config_line (f, &line, &where))
 	{
 	  if (ferror (f))
 	    msg (ME, _("Reading %s: %s."), pprsz_fn, strerror (errno));
 	  break;
 	}
-      for (cp = buf; isspace ((unsigned char) *cp); cp++);
+      for (cp = ds_value (&line); isspace ((unsigned char) *cp); cp++);
       if (*cp == 0)
 	continue;
       if (*cp != '"')
-	goto syntax_error;
+	goto lex_error;
       for (bp = ep = cp + 1; *ep && *ep != '"'; ep++);
       if (!*ep)
-	goto syntax_error;
+	goto lex_error;
       *ep = 0;
       if (0 != strcasecmp (bp, size))
 	continue;
@@ -1192,7 +1196,7 @@ outp_get_paper_size (char *size, int *h, int *v)
       size = &ep[1];
       break;
 
-    syntax_error:
+    lex_error:
       msg (IE, _("Syntax error in paper size definition."));
     }
 
@@ -1216,14 +1220,15 @@ outp_get_paper_size (char *size, int *h, int *v)
     }
 
 exit:
-  free (buf);
+  err_pop_file_locator (&where);
+  ds_destroy (&line);
   if (free_it)
     free (size);
 
   if (result)
-    verbose_msg (2, _("Paper size definition file read successfully."));
+    msg (VM (2), _("Paper size definition file read successfully."));
   else
-    verbose_msg (1, _("Error reading paper size definition file."));
+    msg (VM (1), _("Error reading paper size definition file."));
   
   return result;
 }
@@ -1232,11 +1237,11 @@ exit:
    none.  Otherwise D must be the last driver returned by this
    function, in which case the next enabled driver is returned or NULL
    if that was the last. */
-outp_driver *
-outp_drivers (outp_driver *d)
+struct outp_driver *
+outp_drivers (struct outp_driver *d)
 {
 #if GLOBAL_DEBUGGING
-  outp_driver *orig_d = d;
+  struct outp_driver *orig_d = d;
 #endif
 
   for (;;)
@@ -1280,42 +1285,9 @@ outp_enable_device (int enable, int device)
     disabled_devices |= device;
 }
 
-#if 0
-/* This code has been removed from tokener() until such time as I can remember
-   how it was meant to work. */
-if (*prog == '{')
-  {
-    int inner_quotes = 0;
-
-    prog++;
-    while (*prog && (*prog != '}' || inner_quotes))
-      {
-	if (inner_quotes)
-	  {
-	    if (*prog == inner_quotes)
-	      inner_quotes = 0;
-	  }
-	else
-	  {
-	    if (*prog == '"' || *prog == '\'')
-	      inner_quotes = *prog;
-	  }
-
-	expand_op_tokstr (op_tokstr_len + 1);
-	if (*prog == '\\')
-	  if (prog[1] == '"' || prog[1] == '\'')
-	    prog++;
-	op_tokstr.s[op_tokstr.l++] = *prog++;
-      }
-
-    op_tokstr.s[op_tokstr.l] = 0;
-    return (op_token = '{');
-  }
-#endif
-
 /* Ejects the paper on device D, if the page is not blank. */
 int
-outp_eject_page (outp_driver *d)
+outp_eject_page (struct outp_driver *d)
 {
   if (d->page_open == 0)
     return 1;
@@ -1340,13 +1312,12 @@ outp_eject_page (outp_driver *d)
 /* Returns the width of string S, in device units, when output on
    device D. */
 int
-outp_string_width (outp_driver *d, const char *s)
+outp_string_width (struct outp_driver *d, const char *s)
 {
-  outp_text text;
+  struct outp_text text;
 
   text.options = OUTP_T_JUST_LEFT;
-  text.s.s = (char *) s;
-  text.s.l = strlen (s);
+  ls_init (&text.s, (char *) s, strlen (s));
   d->class->text_metrics (d, &text);
 
   return text.h;
