@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -36,17 +36,18 @@ char *alloca ();
 #endif
 
 #include <assert.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
 #include <math.h>
-#include "common.h"
+#include "alloc.h"
 #include "avl.h"
-#include "error.h"
 #include "file-handle.h"
 #include "format.h"
 #include "getline.h"
+#include "magic.h"
 #include "misc.h"
 #include "pfm.h"
 #include "str.h"
@@ -57,11 +58,11 @@ char *alloca ();
 #include "debug-print.h"
 
 /* pfm's file_handle extension. */
-typedef struct
+struct pfm_fhuser_ext
   {
     FILE *file;			/* Actual file. */
 
-    dictionary *dict;		/* File's dictionary. */
+    struct dictionary *dict;	/* File's dictionary. */
     int weight_index;		/* 0-based index of weight variable, or -1. */
 
     unsigned char *trans;	/* 256-byte character set translation table. */
@@ -73,41 +74,53 @@ typedef struct
     unsigned char buf[83];	/* Input buffer. */
     unsigned char *bp;		/* Buffer pointer. */
     int cc;			/* Current character. */
-  }
-pfm_fhuser_ext;
+  };
 
-static fh_ext_class pfm_r_class;
+static struct fh_ext_class pfm_r_class;
 
 static int 
-corrupt_msg (file_handle *h, const char *format,...)
+corrupt_msg (struct file_handle *h, const char *format,...)
   __attribute__ ((format (printf, 2, 3)));
 
 /* Displays a corruption error. */
 static int
-corrupt_msg (file_handle *h, const char *format,...)
+corrupt_msg (struct file_handle *h, const char *format, ...)
 {
-  pfm_fhuser_ext *ext = h->ext;
-  va_list args;
-  char *banner;
+  struct pfm_fhuser_ext *ext = h->ext;
+  char buf[1024];
   
-  banner = local_alloc (strlen (h->fn) + 80);
-  sprintf (banner, _("portable file %s corrupt at offset %ld: "),
-	   h->fn, ftell (ext->file) - (82 - (long) (ext->bp - ext->buf)));
+  {
+    va_list args;
+
+    va_start (args, format);
+    vsnprintf (buf, 1024, format, args);
+    va_end (args);
+  }
   
-  va_start (args, format);
-  vmsg (ME, format, args, banner);
-  va_end (args);
+  {
+    char *title;
+    struct error e;
 
-  local_free (banner);
+    e.class = ME;
+    getl_location (&e.where.filename, &e.where.line_number);
+    e.title = title = local_alloc (strlen (h->fn) + 80);
+    sprintf (title, _("portable file %s corrupt at offset %ld: "),
+	     h->fn, ftell (ext->file) - (82 - (long) (ext->bp - ext->buf)));
+    e.text = buf;
 
+    err_vmsg (&e);
+
+    local_free (title);
+  }
+  
   return 0;
 }
 
 /* Closes a portable file after we're done with it. */
 static void
-pfm_close (file_handle * h)
+pfm_close (struct file_handle * h)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
 
   if (EOF == fclose (ext->file))
     msg (ME, _("%s: Closing portable file: %s."), h->fn, strerror (errno));
@@ -129,9 +142,9 @@ pfm_close (file_handle * h)
 /* Read an 80-character line into handle H's buffer.  Return
    success. */
 static int
-fill_buf (file_handle *h)
+fill_buf (struct file_handle *h)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
 
   if (80 != fread (ext->buf, 1, 80, ext->file))
     lose ((h, _("Unexpected end of file.")));
@@ -167,9 +180,9 @@ fill_buf (file_handle *h)
 
 /* Read a single character into cur_char.  Return success; */
 static int
-read_char (file_handle *h)
+read_char (struct file_handle *h)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
 
   if (ext->bp >= &ext->buf[80] && !fill_buf (h))
     return 0;
@@ -183,9 +196,9 @@ read_char (file_handle *h)
 /* Skip a single character if present, and return whether it was
    skipped. */
 static inline int
-skip_char (file_handle *h, int c)
+skip_char (struct file_handle *h, int c)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
   
   if (ext->cc == c)
     {
@@ -200,20 +213,20 @@ skip_char (file_handle *h, int c)
    skipped. */
 #define match(C) skip_char (h, C)
 
-static int read_header (file_handle *h);
-static int read_version_data (file_handle *h, pfm_read_info *inf);
-static int read_variables (file_handle *h);
-static int read_value_label (file_handle *h);
-void dump_dictionary (dictionary *dict);
+static int read_header (struct file_handle *h);
+static int read_version_data (struct file_handle *h, struct pfm_read_info *inf);
+static int read_variables (struct file_handle *h);
+static int read_value_label (struct file_handle *h);
+void dump_dictionary (struct dictionary *dict);
 
 /* Reads the dictionary from file with handle H, and returns it in a
    dictionary structure.  This dictionary may be modified in order to
    rename, reorder, and delete variables, etc. */
-dictionary *
-pfm_read_dictionary (file_handle *h, pfm_read_info *inf)
+struct dictionary *
+pfm_read_dictionary (struct file_handle *h, struct pfm_read_info *inf)
 {
   /* The file handle extension record. */
-  pfm_fhuser_ext *ext;
+  struct pfm_fhuser_ext *ext;
 
   /* Check whether the file is already open. */
   if (h->class == &pfm_r_class)
@@ -229,17 +242,17 @@ pfm_read_dictionary (file_handle *h, pfm_read_info *inf)
       return NULL;
     }
 
-  verbose_msg (1, _("%s: Opening portable-file handle %s for reading."),
-	       fh_handle_filename (h), fh_handle_name (h));
+  msg (VM (1), _("%s: Opening portable-file handle %s for reading."),
+       fh_handle_filename (h), fh_handle_name (h));
 
   /* Open the physical disk file. */
-  ext = xmalloc (sizeof (pfm_fhuser_ext));
+  ext = xmalloc (sizeof (struct pfm_fhuser_ext));
   ext->file = fopen (h->norm_fn, "rb");
   if (ext->file == NULL)
     {
       msg (ME, _("An error occurred while opening \"%s\" for reading "
 	   "as a portable file: %s."), h->fn, strerror (errno));
-      cond_fail ();
+      err_cond_fail ();
       free (ext);
       return NULL;
     }
@@ -273,7 +286,7 @@ pfm_read_dictionary (file_handle *h, pfm_read_info *inf)
   if (!match (79 /* F */))
     lose ((h, _("Data record expected.")));
 
-  verbose_msg (2, _("Read portable-file dictionary successfully."));
+  msg (VM (2), _("Read portable-file dictionary successfully."));
 
 #if DEBUGGING
   dump_dictionary (ext->dict);
@@ -282,7 +295,7 @@ pfm_read_dictionary (file_handle *h, pfm_read_info *inf)
 
  lossage:
   /* Come here on unsuccessful completion. */
-  verbose_msg (1, _("Error reading portable-file dictionary."));
+  msg (VM (1), _("Error reading portable-file dictionary."));
   
   fclose (ext->file);
   if (ext && ext->dict)
@@ -296,9 +309,9 @@ pfm_read_dictionary (file_handle *h, pfm_read_info *inf)
 /* Read a floating point value and return its value, or
    second_lowest_value on error. */
 static double
-read_float (file_handle *h)
+read_float (struct file_handle *h)
 {
-  pfm_fhuser_ext *ext = h->ext;			      
+  struct pfm_fhuser_ext *ext = h->ext;			      
   double num = 0.;
   int got_dot = 0;
   int got_digit = 0;
@@ -410,7 +423,7 @@ read_float (file_handle *h)
   
 /* Read an integer and return its value, or NOT_INT on failure. */
 int
-read_int (file_handle *h)
+read_int (struct file_handle *h)
 {
   double f = read_float (h);
 
@@ -428,9 +441,9 @@ read_int (file_handle *h)
    failure.  The buffer can be deallocated by calling with a NULL
    argument. */
 static unsigned char *
-read_string (file_handle *h)
+read_string (struct file_handle *h)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
   static char *buf;
   int n;
   
@@ -468,9 +481,9 @@ read_string (file_handle *h)
 
 /* Reads the 464-byte file header. */
 int
-read_header (file_handle *h)
+read_header (struct file_handle *h)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
 
   /* For now at least, just ignore the vanity splash strings. */
   {
@@ -529,9 +542,9 @@ read_header (file_handle *h)
 /* Reads the version and date info record, as well as product and
    subproduct identification records if present. */
 int
-read_version_data (file_handle *h, pfm_read_info *inf)
+read_version_data (struct file_handle *h, struct pfm_read_info *inf)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
 
   /* Version. */
   if (!match (74 /* A */))
@@ -619,10 +632,11 @@ read_version_data (file_handle *h, pfm_read_info *inf)
 }
 
 static int
-convert_format (file_handle *h, int fmt[3], fmt_spec *v, variable *vv)
+convert_format (struct file_handle *h, int fmt[3], struct fmt_spec *v,
+		struct variable *vv)
 {
   if (fmt[0] < 0
-      || (size_t) fmt[0] >= sizeof (translate_fmt) / sizeof (*translate_fmt))
+      || (size_t) fmt[0] >= sizeof translate_fmt / sizeof *translate_fmt)
     lose ((h, _("%s: Bad format specifier byte %d."), vv->name, fmt[0]));
 
   v->type = translate_fmt[fmt[0]];
@@ -663,13 +677,13 @@ asciify (char *s)
     *s = spss2ascii[(unsigned char) *s];
 }
 
-static int parse_value (file_handle *, value *, variable *);
+static int parse_value (struct file_handle *, union value *, struct variable *);
 
 /* Read information on all the variables.  */
 static int
-read_variables (file_handle *h)
+read_variables (struct file_handle *h)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
   int i;
   
   if (!match (68 /* 4 */))
@@ -678,7 +692,7 @@ read_variables (file_handle *h)
   ext->nvars = read_int (h);
   if (ext->nvars <= 0 || ext->nvars == NOT_INT)
     lose ((h, _("Invalid number of variables %d."), ext->nvars));
-  ext->vars = xmalloc (sizeof (int) * ext->nvars);
+  ext->vars = xmalloc (sizeof *ext->vars * ext->nvars);
 
   /* Purpose of this value is unknown.  It is typically 161. */
   {
@@ -707,7 +721,7 @@ read_variables (file_handle *h)
       int width;
       unsigned char *name;
       int fmt[6];
-      variable *v;
+      struct variable *v;
       int j;
 
       if (!match (71 /* 7 */))
@@ -854,7 +868,7 @@ read_variables (file_handle *h)
 
 /* Parse a value for variable VV into value V.  Returns success. */
 static int
-parse_value (file_handle *h, value *v, variable *vv)
+parse_value (struct file_handle *h, union value *v, struct variable *vv)
 {
   if (vv->type == ALPHA)
     {
@@ -884,13 +898,13 @@ parse_value (file_handle *h, value *v, variable *vv)
 
 /* Parse a value label record and return success. */
 static int
-read_value_label (file_handle *h)
+read_value_label (struct file_handle *h)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
 
   /* Variables. */
   int nv;
-  variable **v;
+  struct variable **v;
 
   /* Labels. */
   int n_labels;
@@ -901,7 +915,7 @@ read_value_label (file_handle *h)
   if (nv == NOT_INT)
     return 0;
 
-  v = xmalloc (sizeof (variable *) * nv);
+  v = xmalloc (sizeof *v * nv);
   for (i = 0; i < nv; i++)
     {
       char *name = read_string (h);
@@ -925,9 +939,9 @@ read_value_label (file_handle *h)
 
   for (i = 0; i < n_labels; i++)
     {
-      value val;
+      union value val;
       char *label;
-      value_label *vl;
+      struct value_label *vl;
 
       int j;
       
@@ -940,7 +954,7 @@ read_value_label (file_handle *h)
       asciify (label);
 
       /* Create a label. */
-      vl = xmalloc (sizeof (value_label));
+      vl = xmalloc (sizeof *vl);
       vl->v = val;
       vl->s = xstrdup (label);
       vl->ref_count = nv;
@@ -948,8 +962,8 @@ read_value_label (file_handle *h)
       /* Assign the value_label's to each variable. */
       for (j = 0; j < nv; j++)
 	{
-	  variable *var = v[j];
-	  value_label *old;
+	  struct variable *var = v[j];
+	  struct value_label *old;
 
 	  /* Create AVL tree if necessary. */
 	  if (!var->val_lab)
@@ -983,11 +997,11 @@ read_value_label (file_handle *h)
    which must have the get.fv elements appropriately set.  Returns
    nonzero only if successful. */
 int
-pfm_read_case (file_handle *h, value *perm, dictionary *dict)
+pfm_read_case (struct file_handle *h, union value *perm, struct dictionary *dict)
 {
-  pfm_fhuser_ext *ext = h->ext;
+  struct pfm_fhuser_ext *ext = h->ext;
 
-  value *temp, *tp;
+  union value *temp, *tp;
   int i;
 
   /* Check for end of file. */
@@ -997,7 +1011,7 @@ pfm_read_case (file_handle *h, value *perm, dictionary *dict)
   /* The first concern is to obtain a full case relative to the data
      file.  (Cases in the data file have no particular relationship to
      cases in the active file.) */
-  tp = temp = local_alloc (sizeof (value) * ext->case_size);
+  tp = temp = local_alloc (sizeof *tp * ext->case_size);
   for (tp = temp, i = 0; i < ext->nvars; i++)
     if (ext->vars[i] == 0)
       {
@@ -1013,7 +1027,7 @@ pfm_read_case (file_handle *h, value *perm, dictionary *dict)
 	  goto unexpected_eof;
 	asciify (s);
 	  
-	strbarepadcpy (tp->s, s, ext->vars[i], ' ');
+	st_bare_pad_copy (tp->s, s, ext->vars[i]);
 	tp += DIV_RND_UP (ext->vars[i], MAX_SHORT_STRING);
       }
 
@@ -1021,7 +1035,7 @@ pfm_read_case (file_handle *h, value *perm, dictionary *dict)
      format. */
   for (i = 0; i < dict->nvar; i++)
     {
-      variable *v = dict->var[i];
+      struct variable *v = dict->var[i];
 
       if (v->get.fv == -1)
 	continue;
@@ -1043,7 +1057,7 @@ pfm_read_case (file_handle *h, value *perm, dictionary *dict)
   return 0;
 }
 
-static fh_ext_class pfm_r_class =
+static struct fh_ext_class pfm_r_class =
 {
   5,
   N_("reading as a portable file"),
