@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -37,16 +37,16 @@ char *alloca ();
 
 #include <assert.h>
 #include <stdlib.h>
-#include "common.h"
+#include "alloc.h"
+#include "command.h"
 #include "dfm.h"
 #include "error.h"
 #include "expr.h"
 #include "file-handle.h"
+#include "lexer.h"
 #include "misc.h"
 #include "som.h"
 #include "tab.h"
-#include "lexer.h"
-#include "lexerP.h"
 #include "var.h"
 
 #undef DEBUGGING
@@ -64,10 +64,9 @@ enum
   };
 
 /* Describes how to output one field. */
-typedef struct prt_out_spec prt_out_spec;
 struct prt_out_spec
   {
-    prt_out_spec *next;
+    struct prt_out_spec *next;
     int type;			/* PRT_* constant. */
     int fc;			/* 0-based first column. */
     union
@@ -75,14 +74,13 @@ struct prt_out_spec
 	char *c;		/* PRT_CONST: Associated string. */
 	struct
 	  {
-	    variable *v;	/* PRT_VAR: Associated variable. */
-	    fmt_spec f;		/* PRT_VAR: Output spec. */
+	    struct variable *v;	/* PRT_VAR: Associated variable. */
+	    struct fmt_spec f;	/* PRT_VAR: Output spec. */
 	  }
 	v;
       }
     u;
-  }
-/* prt_out_spec */ ;
+  };
 
 /* Enums for use with print_trns's `options' field. */
 enum
@@ -94,37 +92,36 @@ enum
   };
 
 /* PRINT, PRINT EJECT, WRITE private data structure. */
-typedef struct
+struct print_trns
   {
-    trns_header h;
+    struct trns_header h;
     struct file_handle *handle;	/* Output file, NULL=listing file. */
     int options;		/* PRT_* bitmapped field. */
-    prt_out_spec *spec;		/* Output specifications. */
+    struct prt_out_spec *spec;	/* Output specifications. */
     int max_width;		/* Maximum line width including null. */
 #if !PAGED_STACK
     char *line;			/* Buffer for sticking lines in. */
 #endif
-  }
-print_trns;
+  };
 
 /* PRT_PRINT or PRT_WRITE. */
 int which_cmd;
 
 /* Holds information on parsing the data file. */
-static print_trns prt;
+static struct print_trns prt;
 
 /* Last prt_out_spec in the chain.  Used for building the linked-list. */
-static prt_out_spec *next;
+static struct prt_out_spec *next;
 
 /* Number of records. */
 static int nrec;
 
 static int internal_cmd_print (int flags);
-static int print_trns_proc (any_trns *, ccase *);
-static void print_trns_free (any_trns *);
+static int print_trns_proc (struct trns_header *, struct ccase *);
+static void print_trns_free (struct trns_header *);
 static int parse_specs (void);
 static void dump_table (void);
-static void append_var_spec (prt_out_spec * spec);
+static void append_var_spec (struct prt_out_spec *spec);
 static void alloc_line (void);
 
 #if DEBUGGING
@@ -137,34 +134,24 @@ void debug_print (void);
 int
 cmd_print (void)
 {
-  int code;
-  match_id (PRINT);
-  code = internal_cmd_print (PRT_PRINT);
-  if (!code)
-    print_trns_free ((any_trns *) & prt);
-  return code;
+  lex_match_id ("PRINT");
+  return internal_cmd_print (PRT_PRINT);
 }
 
 /* Parses PRINT EJECT command. */
 int
 cmd_print_eject (void)
 {
-  int code = internal_cmd_print (PRT_PRINT | PRT_EJECT);
-  match_id (EJECT);
-  if (!code)
-    print_trns_free ((any_trns *) & prt);
-  return code;
+  lex_match_id ("EJECT");
+  return internal_cmd_print (PRT_PRINT | PRT_EJECT);
 }
 
 /* Parses WRITE command. */
 int
 cmd_write (void)
 {
-  int code = internal_cmd_print (PRT_WRITE);
-  match_id (WRITE);
-  if (!code)
-    print_trns_free ((any_trns *) & prt);
-  return code;
+  lex_match_id ("WRITE");
+  return internal_cmd_print (PRT_WRITE);
 }
 
 /* Parses the output commands.  F is PRT_PRINT, PRT_WRITE, or
@@ -176,7 +163,7 @@ internal_cmd_print (int f)
   int table = 0;
 
   /* malloc()'d transformation. */
-  print_trns *trns;
+  struct print_trns *trns;
 
   /* Fill in prt to facilitate error-handling. */
   prt.h.proc = print_trns_proc;
@@ -193,43 +180,41 @@ internal_cmd_print (int f)
   which_cmd = f & PRT_CMD_MASK;
 
   /* Parse the command options. */
-  while (!match_tok ('/'))
+  while (!lex_match ('/'))
     {
-      if (match_id (OUTFILE))
+      if (lex_match_id ("OUTFILE"))
 	{
-	  match_tok ('=');
-	  if (token == ID)
-	    prt.handle = fh_get_handle_by_name (tokstr);
-	  else if (token == STRING)
-	    prt.handle = fh_get_handle_by_filename (tokstr);
-	  else
-	    return msg (SE, _("A file name or handle was expected in the "
-			"OUTFILE subcommand."));
+	  lex_match ('=');
+
+	  prt.handle = fh_parse_file_handle ();
 	  if (!prt.handle)
-	    return 0;
-	  get_token ();
+	    goto lossage;
 	}
-      else if (match_id (RECORDS))
+      else if (lex_match_id ("RECORDS"))
 	{
-	  match_tok ('=');
-	  match_tok ('(');
-	  force_int ();
-	  nrec = tokint;
-	  get_token ();
-	  match_tok (')');
+	  lex_match ('=');
+	  lex_match ('(');
+	  if (!lex_force_int ())
+	    goto lossage;
+	  nrec = lex_integer ();
+	  lex_get ();
+	  lex_match (')');
 	}
-      else if (match_id (TABLE))
+      else if (lex_match_id ("TABLE"))
 	table = 1;
-      else if (match_id (NOTABLE))
+      else if (lex_match_id ("NOTABLE"))
 	table = 0;
       else
-	return syntax_error (_("while expecting a valid subcommand"));
+	{
+	  lex_error (_("expecting a valid subcommand"));
+	  goto lossage;
+	}
     }
 
   /* Parse variables and strings. */
   if (!parse_specs ())
-    return 0;
-
+    goto lossage;
+  
   /* Output the variable table if requested. */
   if (table)
     dump_table ();
@@ -239,49 +224,52 @@ internal_cmd_print (int f)
   alloc_line ();
 
   /* Put the transformation in the queue. */
-  trns = xmalloc (sizeof (print_trns));
-  memcpy (trns, &prt, sizeof (print_trns));
-  add_transformation ((any_trns *) trns);
+  trns = xmalloc (sizeof *trns);
+  memcpy (trns, &prt, sizeof *trns);
+  add_transformation ((struct trns_header *) trns);
 
 #if DEBUGGING
   debug_print ();
 #endif
 
-  return 1;
+  return CMD_SUCCESS;
+
+ lossage:
+  print_trns_free ((struct trns_header *) & prt);
+  return CMD_FAILURE;
 }
 
 /* Appends the field output specification SPEC to the list maintained
    in prt. */
 static void
-append_var_spec (prt_out_spec * spec)
+append_var_spec (struct prt_out_spec *spec)
 {
   if (next == 0)
-    prt.spec = next = xmalloc (sizeof (prt_out_spec));
+    prt.spec = next = xmalloc (sizeof *spec);
   else
-    next = next->next = xmalloc (sizeof (prt_out_spec));
+    next = next->next = xmalloc (sizeof *spec);
 
-  memcpy (next, spec, sizeof (prt_out_spec));
+  memcpy (next, spec, sizeof *spec);
   next->next = NULL;
 }
 
 /* Field parsing.  Mostly stolen from data-list.c. */
 
 /* Used for chaining together fortran-like format specifiers. */
-typedef struct fmt_list
+struct fmt_list
 {
   struct fmt_list *next;
   int count;
-  fmt_spec f;
+  struct fmt_spec f;
   struct fmt_list *down;
-}
-fmt_list;
+};
 
 /* Used as "local" variables among the fixed-format parsing funcs.  If
    it were guaranteed that PSPP were going to be compiled by gcc,
    I'd make all these functions a single set of nested functions. */
 static struct
   {
-    variable **v;		/* variable list */
+    struct variable **v;		/* variable list */
     int nv;			/* number of variables in list */
     int cv;			/* number of variables from list used up so far
 				   by the FORTRAN-like format specifiers */
@@ -289,7 +277,7 @@ static struct
     int recno;			/* current 1-based record number */
     int sc;			/* 1-based starting column for next variable */
 
-    prt_out_spec spec;		/* next format spec to append to list */
+    struct prt_out_spec spec;		/* next format spec to append to list */
     int fc, lc;			/* first, last 1-based column number of current
 				   var */
 
@@ -299,7 +287,7 @@ static struct
 fx;
 
 static int fixed_parse_compatible (void);
-static fmt_list *fixed_parse_fortran (void);
+static struct fmt_list *fixed_parse_fortran (void);
 
 static int parse_string_argument (void);
 static int parse_variable_argument (void);
@@ -318,22 +306,26 @@ parse_specs (void)
 
   while (token != '.')
     {
-      while (match_tok ('/'))
+      while (lex_match ('/'))
 	{
 	  int prev_recno = fx.recno;
 
 	  fx.recno++;
-	  if (token == NUM)
+	  if (token == T_NUM)
 	    {
-	      force_int ();
-	      if (tokint < fx.recno)
-		return msg (SE, _("The record number specified, %ld, is "
-			    "before the previous record, %d.  Data "
-			    "fields must be listed in order of "
-			    "increasing record number."),
-			    tokint, fx.recno - 1);
-	      fx.recno = tokint;
-	      get_token ();
+	      if (!lex_force_int ())
+		return 0;
+	      if (lex_integer () < fx.recno)
+		{
+		  msg (SE, _("The record number specified, %ld, is "
+			     "before the previous record, %d.  Data "
+			     "fields must be listed in order of "
+			     "increasing record number."),
+		       lex_integer (), fx.recno - 1);
+		  return 0;
+		}
+	      fx.recno = lex_integer ();
+	      lex_get ();
 	    }
 
 	  fx.spec.type = PRT_NEWLINE;
@@ -343,7 +335,7 @@ parse_specs (void)
 	  fx.sc = 1;
 	}
 
-      if (token == STRING)
+      if (token == T_STRING)
 	code = parse_string_argument ();
       else
 	code = parse_variable_argument ();
@@ -356,10 +348,18 @@ parse_specs (void)
   if (!nrec)
     nrec = fx.recno;
   else if (fx.recno > nrec)
-    return msg (SE, _("Variables are specified on records that "
-		"should not exist according to RECORDS subcommand."));
+    {
+      msg (SE, _("Variables are specified on records that "
+		 "should not exist according to RECORDS subcommand."));
+      return 0;
+    }
+      
   if (token != '.')
-    return syntax_error (_("expecting end of command"));
+    {
+      lex_error (_("expecting end of command"));
+      return 0;
+    }
+  
   return 1;
 }
 
@@ -369,11 +369,11 @@ parse_string_argument (void)
 {
   fx.spec.type = PRT_CONST;
   fx.spec.fc = fx.sc - 1;
-  fx.spec.u.c = xstrdup (tokstr);
-  get_token ();
+  fx.spec.u.c = xstrdup (ds_value (&tokstr));
+  lex_get ();
 
   /* Parse the included column range. */
-  if (token == NUM)
+  if (token == T_NUM)
     {
       /* Width of column range in characters. */
       int c_len;
@@ -384,38 +384,38 @@ parse_string_argument (void)
       /* 1-based index of last column in range. */
       int lc;
 
-      if (tokint == NOT_LONG || tokint <= 0)
+      if (!lex_integer_p () || lex_integer () <= 0)
 	{
 	  msg (SE, _("%g is not a valid column location."), tokval);
 	  goto fail;
 	}
-      fx.spec.fc = tokint - 1;
+      fx.spec.fc = lex_integer () - 1;
 
-      get_token ();
-      convert_negative_to_dash ();
-      if (match_tok ('-'))
+      lex_get ();
+      lex_negative_to_dash ();
+      if (lex_match ('-'))
 	{
-	  if (token != NUM)
+	  if (!lex_integer_p ())
 	    {
 	      msg (SE, _("Column location expected following `%d-'."),
 		   fx.spec.fc + 1);
 	      goto fail;
 	    }
-	  if (token == NOT_LONG || tokint <= 0)
+	  if (lex_integer () <= 0)
 	    {
 	      msg (SE, _("%g is not a valid column location."), tokval);
 	      goto fail;
 	    }
-	  if (tokint < fx.spec.fc + 1)
+	  if (lex_integer () < fx.spec.fc + 1)
 	    {
 	      msg (SE, _("%d-%ld is not a valid column range.  The second "
 		   "column must be greater than or equal to the first."),
-		   fx.spec.fc + 1, tokint);
+		   fx.spec.fc + 1, lex_integer ());
 	      goto fail;
 	    }
-	  lc = tokint - 1;
+	  lc = lex_integer () - 1;
 
-	  get_token ();
+	  lex_get ();
 	}
       else
 	/* If only a starting location is specified then the field is
@@ -458,7 +458,7 @@ parse_variable_argument (void)
   if (!parse_variables (NULL, &fx.v, &fx.nv, PV_DUPLICATE))
     return 0;
 
-  if (token == NUM)
+  if (token == T_NUM)
     {
       if (!fixed_parse_compatible ())
 	goto fail;
@@ -475,7 +475,7 @@ parse_variable_argument (void)
       /* User wants dictionary format specifiers. */
       int i;
 
-      match_tok ('*');
+      lex_match ('*');
       for (i = 0; i < fx.nv; i++)
 	{
 	  /* Variable. */
@@ -513,40 +513,52 @@ fixed_parse_compatible (void)
   type = fx.v[0]->type;
   for (i = 1; i < fx.nv; i++)
     if (type != fx.v[i]->type)
-      return msg (SE, _("%s is not of the same type as %s.  To specify "
-		  "variables of different types in the same variable "
-		  "list, use a FORTRAN-like format specifier."),
-		  fx.v[i]->name, fx.v[0]->name);
+      {
+	msg (SE, _("%s is not of the same type as %s.  To specify "
+		   "variables of different types in the same variable "
+		   "list, use a FORTRAN-like format specifier."),
+	     fx.v[i]->name, fx.v[0]->name);
+	return 0;
+      }
 
-  force_int ();
-  fx.fc = tokint - 1;
+  if (!lex_force_int ())
+    return 0;
+  fx.fc = lex_integer () - 1;
   if (fx.fc < 0)
-    return msg (SE, _("Column positions for fields must "
-		"be positive."));
-  get_token ();
-
-  convert_negative_to_dash ();
-  if (match_tok ('-'))
     {
-      force_int ();
-      fx.lc = tokint - 1;
+      msg (SE, _("Column positions for fields must be positive."));
+      return 0;
+    }
+  lex_get ();
+
+  lex_negative_to_dash ();
+  if (lex_match ('-'))
+    {
+      if (!lex_force_int ())
+	return 0;
+      fx.lc = lex_integer () - 1;
       if (fx.lc < 0)
-	return msg (SE, _("Column positions for fields must "
-		    "be positive."));
+	{
+	  msg (SE, _("Column positions for fields must be positive."));
+	  return 0;
+	}
       else if (fx.lc < fx.fc)
-	return msg (SE, _("The ending column for a field must be "
-		    "greater than the starting column."));
-      get_token ();
+	{
+	  msg (SE, _("The ending column for a field must not "
+		     "be less than the starting column."));
+	  return 0;
+	}
+      lex_get ();
     }
   else
     fx.lc = fx.fc;
 
   fx.spec.u.v.f.w = fx.lc - fx.fc + 1;
-  if (match_tok ('('))
+  if (lex_match ('('))
     {
-      fmt_desc *fdp;
+      struct fmt_desc *fdp;
 
-      if (token == ID)
+      if (token == T_ID)
 	{
 	  const char *cp;
 
@@ -554,34 +566,45 @@ fixed_parse_compatible (void)
 	  if (fx.spec.u.v.f.type == -1)
 	    return 0;
 	  if (*cp)
-	    return msg (SE, _("A format specifier on this line "
-			"has extra characters on the end."));
-	  get_token ();
-	  match_tok (',');
+	    {
+	      msg (SE, _("A format specifier on this line "
+			 "has extra characters on the end."));
+	      return 0;
+	    }
+	  lex_get ();
+	  lex_match (',');
 	}
       else
 	fx.spec.u.v.f.type = FMT_F;
 
-      if (token == NUM)
+      if (token == T_NUM)
 	{
-	  force_int ();
-	  if (tokint < 1)
-	    return msg (SE, _("The value for number of decimal places "
-			"must be at least 1."));
-	  fx.spec.u.v.f.d = tokint;
-	  get_token ();
+	  if (!lex_force_int ())
+	    return 0;
+	  if (lex_integer () < 1)
+	    {
+	      msg (SE, _("The value for number of decimal places "
+			 "must be at least 1."));
+	      return 0;
+	    }
+	  fx.spec.u.v.f.d = lex_integer ();
+	  lex_get ();
 	}
       else
 	fx.spec.u.v.f.d = 0;
 
       fdp = &formats[fx.spec.u.v.f.type];
       if (fdp->n_args < 2 && fx.spec.u.v.f.d)
-	return msg (SE, _("Input format %s doesn't accept decimal "
-		    "places."), fdp->name);
+	{
+	  msg (SE, _("Input format %s doesn't accept decimal places."),
+	       fdp->name);
+	  return 0;
+	}
       if (fx.spec.u.v.f.d > 16)
 	fx.spec.u.v.f.d = 16;
 
-      force_match (')');
+      if (!lex_force_match (')'))
+	return 0;
     }
   else
     {
@@ -592,17 +615,23 @@ fixed_parse_compatible (void)
   fx.sc = fx.lc + 1;
 
   if ((fx.lc - fx.fc + 1) % fx.nv)
-    return msg (SE, _("The %d columns %d-%d can't be evenly divided into %d "
-		"fields."), fx.lc - fx.fc + 1, fx.fc + 1, fx.lc + 1, fx.nv);
+    {
+      msg (SE, _("The %d columns %d-%d can't be evenly divided into %d "
+		 "fields."), fx.lc - fx.fc + 1, fx.fc + 1, fx.lc + 1, fx.nv);
+      return 0;
+    }
 
   dividend = (fx.lc - fx.fc + 1) / fx.nv;
   fx.spec.u.v.f.w = dividend;
   if (!check_output_specifier (&fx.spec.u.v.f))
     return 0;
   if ((type == ALPHA) ^ (formats[fx.spec.u.v.f.type].cat & FCAT_STRING))
-    return msg (SE, _("%s variables cannot be displayed with format %s."),
-		type == ALPHA ? _("String") : _("Numeric"),
-		fmt_to_string (&fx.spec.u.v.f));
+    {
+      msg (SE, _("%s variables cannot be displayed with format %s."),
+	   type == ALPHA ? _("String") : _("Numeric"),
+	   fmt_to_string (&fx.spec.u.v.f));
+      return 0;
+    }
 
   /* Check that, for string variables, the user didn't specify a width
      longer than an actual string width. */
@@ -629,9 +658,9 @@ fixed_parse_compatible (void)
 
 /* Destroy a format list and, optionally, all its sublists. */
 static void
-destroy_fmt_list (fmt_list * f, int recurse)
+destroy_fmt_list (struct fmt_list * f, int recurse)
 {
-  fmt_list *next;
+  struct fmt_list *next;
 
   for (; f; f = next)
     {
@@ -646,7 +675,7 @@ destroy_fmt_list (fmt_list * f, int recurse)
    FORTRAN-like format specifications, like 4(F10,2X)) into the
    structure prt. */
 static int
-dump_fmt_list (fmt_list * f)
+dump_fmt_list (struct fmt_list * f)
 {
   int i;
 
@@ -672,18 +701,24 @@ dump_fmt_list (fmt_list * f)
 	  }
 	else
 	  {
-	    variable *v;
+	    struct variable *v;
 
 	    if (fx.cv >= fx.nv)
-	      return msg (SE, _("The number of format "
-			  "specifictions exceeds the number of variable "
-			  "names given."));
+	      {
+		msg (SE, _("The number of format "
+			   "specifications exceeds the number of variable "
+			   "names given."));
+		return 0;
+	      }
 
 	    v = fx.v[fx.cv++];
 	    if ((v->type == ALPHA) ^ (formats[f->f.type].cat & FCAT_STRING))
-	      return msg (SE, _("Display format %s may not be used with a "
-			  "%s variable."), fmt_to_string (&f->f),
-			  v->type == ALPHA ? _("string") : _("numeric"));
+	      {
+		msg (SE, _("Display format %s may not be used with a "
+			   "%s variable."), fmt_to_string (&f->f),
+		     v->type == ALPHA ? _("string") : _("numeric"));
+		return 0;
+	      }
 	    if (!check_string_specifier (&f->f, v->width))
 	      return 0;
 
@@ -702,29 +737,26 @@ dump_fmt_list (fmt_list * f)
    itself to parse nested levels of parentheses.  Returns to its
    original caller NULL, to indicate error, non-NULL, but nothing
    useful, to indicate success (it returns a free()'d block). */
-static fmt_list *
+static struct fmt_list *
 fixed_parse_fortran (void)
 {
-  fmt_list *head;
-  fmt_list *fl = NULL;
+  struct fmt_list *head;
+  struct fmt_list *fl = NULL;
 
-  get_token ();			/* skip opening parenthesis */
+  lex_get ();			/* skip opening parenthesis */
   while (token != ')')
     {
       if (fl)
-	fl = fl->next = xmalloc (sizeof (fmt_list));
+	fl = fl->next = xmalloc (sizeof *fl);
       else
-	head = fl = xmalloc (sizeof (fmt_list));
+	head = fl = xmalloc (sizeof *fl);
 
-      if (token == NUM)
+      if (token == T_NUM)
 	{
-	  if (tokint == NOT_LONG)
-	    {
-	      msg (SE, _("Integer expected for repeat count."));
-	      goto fail;
-	    }
-	  fl->count = tokint;
-	  get_token ();
+	  if (!lex_integer_p ())
+	    goto fail;
+	  fl->count = lex_integer ();
+	  lex_get ();
 	}
       else
 	fl->count = 1;
@@ -738,16 +770,16 @@ fixed_parse_fortran (void)
 	  if (!fl->down)
 	    goto fail;
 	}
-      else if (match_tok ('/'))
+      else if (lex_match ('/'))
 	fl->f.type = FMT_NEWREC;
       else if (!parse_format_specifier (&fl->f, 1)
 	       || !check_output_specifier (&fl->f))
 	goto fail;
 
-      match_tok (',');
+      lex_match (',');
     }
   fl->next = NULL;
-  get_token ();
+  lex_get ();
 
   if (fx.level)
     return head;
@@ -775,9 +807,9 @@ fail:
 static void
 dump_table (void)
 {
-  prt_out_spec *spec;
+  struct prt_out_spec *spec;
   const char *filename;
-  tab_table *t;
+  struct tab_table *t;
   int recno;
   int nspec;
 
@@ -785,7 +817,7 @@ dump_table (void)
     if (spec->type == PRT_CONST || spec->type == PRT_VAR)
       nspec++;
   t = tab_create (4, nspec + 1, 0);
-  tab_columns (t, SOM_COL_DOWN, 1);
+  tab_columns (t, TAB_COL_DOWN, 1);
   tab_box (t, TAL_1, TAL_1, TAL_0, TAL_1, 0, 0, 3, nspec);
   tab_hline (t, TAL_2, 0, 3, 1);
   tab_headers (t, 0, 0, 1, 0);
@@ -859,7 +891,7 @@ alloc_line (void)
   int pot_w;			/* Potential w. */
 
   /* Iterator. */
-  prt_out_spec *i;
+  struct prt_out_spec *i;
 
   for (i = prt.spec; i; i = i->next)
     {
@@ -894,13 +926,13 @@ alloc_line (void)
 
 /* Performs the transformation inside print_trns T on case C. */
 static int
-print_trns_proc (any_trns * trns, ccase * c)
+print_trns_proc (struct trns_header * trns, struct ccase * c)
 {
   /* Transformation. */
-  print_trns *t = (print_trns *) trns;
+  struct print_trns *t = (struct print_trns *) trns;
 
   /* Iterator. */
-  prt_out_spec *i;
+  struct prt_out_spec *i;
 
   /* Line buffer. */
 #if PAGED_STACK
@@ -961,13 +993,12 @@ print_trns_proc (any_trns * trns, ccase * c)
 
       case PRT_VAR:
 	if (i->u.v.v->type == NUMERIC)
-	  convert_format_to_string (&buf[i->fc], &i->u.v.f,
-				    &c->data[i->u.v.v->fv]);
+	  data_out (&buf[i->fc], &i->u.v.f, &c->data[i->u.v.v->fv]);
 	else
 	  {
-	    value t;
+	    union value t;
 	    t.c = c->data[i->u.v.v->fv].s;
-	    convert_format_to_string (&buf[i->fc], &i->u.v.f, &t);
+	    data_out (&buf[i->fc], &i->u.v.f, &t);
 	  }
 	len = i->fc + i->u.v.f.w;
 	break;
@@ -987,11 +1018,11 @@ print_trns_proc (any_trns * trns, ccase * c)
 
 /* Frees all the data inside print_trns T.  Does not free T. */
 static void
-print_trns_free (any_trns * t)
+print_trns_free (struct trns_header * t)
 {
-  prt_out_spec *i, *n;
+  struct prt_out_spec *i, *n;
 
-  for (i = ((print_trns *) t)->spec; i; i = n)
+  for (i = ((struct print_trns *) t)->spec; i; i = n)
     {
       switch (i->type)
 	{
@@ -1011,69 +1042,69 @@ print_trns_free (any_trns * t)
       free (i);
     }
 #if !PAGED_STACK
-  free (((print_trns *) t)->line);
+  free (((struct print_trns *) t)->line);
 #endif
 }
 
 /* PRINT SPACE. */
 
 /* PRINT SPACE transformation. */
-typedef struct
+struct print_space_trns
 {
-  trns_header h;
+  struct trns_header h;
 
-  file_handle *handle;		/* Output file, NULL=listing file. */
-  struct expression *e;	/* Number of lines; NULL=1. */
+  struct file_handle *handle;	/* Output file, NULL=listing file. */
+  struct expression *e;		/* Number of lines; NULL=1. */
 }
 print_space_trns;
 
-static int print_space_trns_proc (any_trns *, ccase *);
-static void print_space_trns_free (any_trns *);
+static int print_space_trns_proc (struct trns_header *, struct ccase *);
+static void print_space_trns_free (struct trns_header *);
 
 int
 cmd_print_space (void)
 {
-  print_space_trns *t;
-  file_handle *handle;
-  expression *e;
+  struct print_space_trns *t;
+  struct file_handle *handle;
+  struct expression *e;
 
-  match_id (SPACE);
-  if (match_id (OUTFILE))
+  lex_match_id ("SPACE");
+  if (lex_match_id ("OUTFILE"))
     {
-      match_tok ('=');
+      lex_match ('=');
 
-      if (token == ID)
-	handle = fh_get_handle_by_name (tokstr);
-      else if (token == STRING)
-	handle = fh_get_handle_by_filename (tokstr);
+      if (token == T_ID)
+	handle = fh_get_handle_by_name (tokid);
+      else if (token == T_STRING)
+	handle = fh_get_handle_by_filename (tokid);
       else
-	return msg (SE, _("A file name or handle was expected in the "
-		    "OUTFILE subcommand."));
+	{
+	  msg (SE, _("A file name or handle was expected in the "
+		     "OUTFILE subcommand."));
+	  return CMD_FAILURE;
+	}
+      
       if (!handle)
-	return 0;
-      get_token ();
+	return CMD_FAILURE;
+      lex_get ();
     }
   else
     handle = NULL;
 
   if (token != '.')
     {
-      e = parse_expression (PXP_NONE);
-      if (e->type != EX_NUMERIC)
-	{
-	  free_expression (e);
-	  return msg (SE, _("The argument must be a numeric expression."));
-	}
+      e = expr_parse (PXP_NUMERIC);
       if (token != '.')
 	{
-	  free_expression (e);
-	  return syntax_error (_("expecting end of command"));
+	  expr_free (e);
+	  lex_error (_("expecting end of command"));
+	  return CMD_FAILURE;
 	}
     }
   else
     e = NULL;
 
-  t = xmalloc (sizeof (print_space_trns));
+  t = xmalloc (sizeof *t);
   t->h.proc = print_space_trns_proc;
   if (e)
     t->h.free = print_space_trns_free;
@@ -1082,21 +1113,21 @@ cmd_print_space (void)
   t->handle = handle;
   t->e = e;
 
-  add_transformation ((any_trns *) t);
-  return 1;
+  add_transformation ((struct trns_header *) t);
+  return CMD_SUCCESS;
 }
 
 static int
-print_space_trns_proc (any_trns * trns, ccase * c)
+print_space_trns_proc (struct trns_header * trns, struct ccase * c)
 {
-  print_space_trns *t = (print_space_trns *) trns;
+  struct print_space_trns *t = (struct print_space_trns *) trns;
   int n;
 
   if (t->e)
     {
-      value v;
+      union value v;
 
-      evaluate_expression (t->e, c, &v);
+      expr_evaluate (t->e, c, &v);
       n = v.f;
       if (n < 0)
 	{
@@ -1132,9 +1163,9 @@ print_space_trns_proc (any_trns * trns, ccase * c)
 }
 
 static void
-print_space_trns_free (any_trns * trns)
+print_space_trns_free (struct trns_header * trns)
 {
-  free_expression (((print_space_trns *) trns)->e);
+  expr_free (((struct print_space_trns *) trns)->e);
 }
 
 /* Debugging code. */
@@ -1143,7 +1174,7 @@ print_space_trns_free (any_trns * trns)
 void
 debug_print (void)
 {
-  prt_out_spec *p;
+  struct prt_out_spec *p;
 
   if (prt.handle == NULL)
     {
