@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -21,14 +21,15 @@
 #include <assert.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include "alloc.h"
 #include "approx.h"
-#include "common.h"
-#include "error.h"
-#include "misc.h"
-#include "str.h"
-#include "lexer.h"
-#include "var.h"
 #include "cases.h"
+#include "command.h"
+#include "error.h"
+#include "lexer.h"
+#include "magic.h"
+#include "str.h"
+#include "var.h"
 
 #undef DEBUGGING
 /*#define DEBUGGING 1 */
@@ -50,41 +51,38 @@ enum
 
 /* Describes how to recode a single value or range of values into a
    single value.  */
-typedef struct coding coding;
 struct coding
   {
     int type;			/* RCD_* */
-    value f1, f2;		/* Describe value or range as src.  Long
+    union value f1, f2;		/* Describe value or range as src.  Long
 				   strings are stored in `c'. */
-    value t;			/* Describes value as dest. Long strings in `c'. */
+    union value t;		/* Describes value as dest. Long strings in `c'. */
   };
 
 /* Describes how to recode a single variable. */
-typedef struct rcd_var rcd_var;
 struct rcd_var
   {
-    rcd_var *next;
+    struct rcd_var *next;
 
     unsigned flags;		/* RCD_SRC_* | RCD_DEST_* | RCD_MISC_* */
 
-    variable *src;		/* Source variable. */
-    variable *dest;		/* Destination variable. */
+    struct variable *src;	/* Source variable. */
+    struct variable *dest;	/* Destination variable. */
     char dest_name[9];		/* Name of dest variable if we're creating it. */
 
     int has_sysmis;		/* Do we recode for SYSMIS? */
-    value sysmis;		/* Coding for SYSMIS (if src is numeric). */
+    union value sysmis;		/* Coding for SYSMIS (if src is numeric). */
 
-    coding *map;		/* Coding for other values. */
+    struct coding *map;		/* Coding for other values. */
     int nmap, mmap;		/* Length of map, max capacity of map. */
   };
 
 /* RECODE transformation. */
-typedef struct
+struct recode_trns
   {
-    trns_header h;
-    rcd_var *codings;
-  }
-recode_trns;
+    struct trns_header h;
+    struct rcd_var *codings;
+  };
 
 /* What we're recoding from (`src'==`source'). */
 #define RCD_SRC_ERROR		0000u	/* Bad value for src. */
@@ -106,11 +104,11 @@ recode_trns;
 #define RCD_MISC_MISSING	0100u	/* Encountered MISSING or SYSMIS in
 					   this input spec. */
 
-static int parse_dest_spec (rcd_var * rcd, value * v, int *max_dst_width);
-static int parse_src_spec (rcd_var * rcd, int type, int max_src_width);
-static int internal_cmd_recode (void);
-static int recode_trns_proc (any_trns *, ccase *);
-static void recode_trns_free (any_trns *);
+static int parse_dest_spec (struct rcd_var * rcd, union value *v,
+			    size_t *max_dst_width);
+static int parse_src_spec (struct rcd_var * rcd, int type, size_t max_src_width);
+static int recode_trns_proc (struct trns_header *, struct ccase *);
+static void recode_trns_free (struct trns_header *);
 static double convert_to_double (char *, int);
 
 #if DEBUGGING
@@ -120,60 +118,43 @@ static void debug_print (rcd_var * head);
 /* Parser. */
 
 /* First transformation in the list.  rcd is in this list. */
-static rcd_var *head;
+static struct rcd_var *head;
 
 /* Variables in the current part of the recoding. */
-variable **v;
+struct variable **v;
 int nv;
 
-/* Calls internal_cmd_recode(), which does the real work, then cleans
-   up after it. */
+/* Parses the RECODE transformation. */
 int
 cmd_recode (void)
-{
-  int code = internal_cmd_recode ();
-  free (v);
-  if (!code)
-    {
-      recode_trns t;
-
-      t.codings = head;
-      recode_trns_free ((any_trns *) & t);
-    }
-  return code;
-}
-
-/* Parses the RECODE transformation. */
-static int
-internal_cmd_recode (void)
 {
   int i;
 
   /* Transformation that we're constructing. */
-  static rcd_var *rcd;
+  struct rcd_var *rcd;
 
   /* Type of the src variables. */
   int type;
 
   /* Length of longest src string. */
-  int max_src_width;
+  size_t max_src_width;
 
   /* Length of longest dest string. */
-  int max_dst_width;
+  size_t max_dst_width;
 
   /* For stepping through, constructing the linked list of
      recodings. */
-  rcd_var *iter;
+  struct rcd_var *iter;
 
   /* The real transformation, just a wrapper for a list of
      rcd_var's. */
-  recode_trns *trns;
+  struct recode_trns *trns;
 
-  match_id (RECODE);
+  lex_match_id ("RECODE");
 
   /* Parses each specification between slashes. */
-  head = rcd = xmalloc (sizeof (rcd_var));
-  while (1)
+  head = rcd = xmalloc (sizeof *rcd);
+  for (;;)
     {
       /* Whether we've already encountered a specification for SYSMIS. */
       int had_sysmis = 0;
@@ -187,7 +168,7 @@ internal_cmd_recode (void)
 
       /* Parse variable names. */
       if (!parse_variables (NULL, &v, &nv, PV_SAME_TYPE))
-	return 0;
+	goto lossage;
 
       /* Ensure all variables are same type; find length of longest
          source variable. */
@@ -196,7 +177,7 @@ internal_cmd_recode (void)
 
       if (type == ALPHA)
 	for (i = 0; i < nv; i++)
-	  if (v[i]->width > max_src_width)
+	  if (v[i]->width > (int) max_src_width)
 	    max_src_width = v[i]->width;
 
       /* Set up flags. */
@@ -208,14 +189,15 @@ internal_cmd_recode (void)
 
       /* Parse each coding in parentheses. */
       max_dst_width = 0;
-      force_match ('(');
-      while (1)
+      if (!lex_force_match ('('))
+	goto lossage;
+      for (;;) 
 	{
 	  /* Get the input value (before the `='). */
 	  int mark = rcd->nmap;
 	  int code = parse_src_spec (rcd, type, max_src_width);
 	  if (!code)
-	    return 0;
+	    goto lossage;
 
 	  /* ELSE is the same as any other input spec except that it
 	     precludes later sysmis specifications. */
@@ -229,12 +211,12 @@ internal_cmd_recode (void)
 	     specification.  */
 	  if (code == 1)
 	    {
-	      value output;
+	      union value output;
 
 	      /* Get the output value (after the `='). */
-	      get_token ();	/* Skip `='. */
+	      lex_get ();	/* Skip `='. */
 	      if (!parse_dest_spec (rcd, &output, &max_dst_width))
-		return 0;
+		goto lossage;
 
 	      /* Set the value for SYSMIS if requested and if we don't
 	         already have one. */
@@ -263,8 +245,8 @@ internal_cmd_recode (void)
 		  free (output.c);
 		}
 	    }
-	  get_token ();		/* Skip `)'. */
-	  if (!match_tok ('('))
+	  lex_get ();		/* Skip `)'. */
+	  if (!lex_match ('('))
 	    break;
 	}
 
@@ -279,7 +261,7 @@ internal_cmd_recode (void)
       iter = rcd;
       for (i = 1; i < nv; i++)
 	{
-	  iter = iter->next = xmalloc (sizeof (rcd_var));
+	  iter = iter->next = xmalloc (sizeof *iter);
 	  iter->next = NULL;
 	  iter->flags = rcd->flags | RCD_MISC_DUPLICATE;
 	  iter->src = v[i];
@@ -290,7 +272,7 @@ internal_cmd_recode (void)
 	  iter->map = rcd->map;
 	}
 
-      if (match_id (INTO))
+      if (lex_match_id ("INTO"))
 	{
 	  char **names;
 	  int nnames;
@@ -298,23 +280,24 @@ internal_cmd_recode (void)
 	  int success = 0;
 
 	  if (!parse_mixed_vars (&names, &nnames, PV_NONE))
-	    return 0;
+	    goto lossage;
 
 	  if (nnames != nv)
 	    {
 	      for (i = 0; i < nnames; i++)
 		free (names[i]);
 	      free (names);
-	      return msg (SE, _("%d variable(s) cannot be recoded into "
-			  "%d variable(s).  Specify the same number "
-			  "of variables as input and output variables."),
-			  nv, nnames);
+	      msg (SE, _("%d variable(s) cannot be recoded into "
+			 "%d variable(s).  Specify the same number "
+			 "of variables as input and output variables."),
+		   nv, nnames);
+	      goto lossage;
 	    }
 
 	  if ((rcd->flags & RCD_DEST_MASK) == RCD_DEST_STRING)
 	    for (i = 0, iter = rcd; i < nv; i++, iter = iter->next)
 	      {
-		variable *v = find_variable (names[i]);
+		struct variable *v = find_variable (names[i]);
 
 		if (!v)
 		  {
@@ -333,14 +316,14 @@ internal_cmd_recode (void)
 			 "variables are string variables."), v->name);
 		    goto INTO_fail;
 		  }
-		if (v->width > max_dst_width)
+		if (v->width > (int) max_dst_width)
 		  max_dst_width = v->width;
 		iter->dest = v;
 	      }
 	  else
 	    for (i = 0, iter = rcd; i < nv; i++, iter = iter->next)
 	      {
-		variable *v = find_variable (names[i]);
+		struct variable *v = find_variable (names[i]);
 
 		if (v)
 		  {
@@ -366,7 +349,7 @@ internal_cmd_recode (void)
 	    free (names[i]);
 	  free (names);
 	  if (!success)
-	    return 0;
+	    goto lossage;
 	}
       else
 	{
@@ -375,27 +358,34 @@ internal_cmd_recode (void)
 
 	  if ((rcd->flags & RCD_SRC_MASK) == RCD_SRC_NUMERIC
 	      && (rcd->flags & RCD_DEST_MASK) != RCD_DEST_NUMERIC)
-	    return msg (SE, _("INTO must be used when the input values are "
-			"numeric and output values are string."));
+	    {
+	      msg (SE, _("INTO must be used when the input values are "
+			 "numeric and output values are string."));
+	      goto lossage;
+	    }
+	  
 	  if ((rcd->flags & RCD_SRC_MASK) != RCD_SRC_NUMERIC
 	      && (rcd->flags & RCD_DEST_MASK) == RCD_DEST_NUMERIC)
-	    return msg (SE, _("INTO must be used when the input values are "
-			"string and output values are numeric."));
+	    {
+	      msg (SE, _("INTO must be used when the input values are "
+			 "string and output values are numeric."));
+	      goto lossage;
+	    }
 	}
 
       if ((rcd->flags & RCD_DEST_MASK) == RCD_DEST_STRING)
 	{
-	  coding *cp;
+	  struct coding *cp;
 
 	  for (cp = rcd->map; cp->type != RCD_END; cp++)
 	    if (cp->t.c)
 	      {
-		if ((int) strlen (cp->t.c) < max_dst_width)
+		if (strlen (cp->t.c) < max_dst_width)
 		  {
 		    /* The NULL is only really necessary for the
 		       debugging code. */
 		    char *repl = xmalloc (max_dst_width + 1);
-		    strpadcpy (repl, cp->t.c, max_dst_width + 1, ' ');
+		    st_pad_copy (repl, cp->t.c, max_dst_width + 1);
 		    free (cp->t.c);
 		    cp->t.c = repl;
 		  }
@@ -407,16 +397,20 @@ internal_cmd_recode (void)
 	  
 	}
 
-      if (!match_tok ('/'))
+      if (!lex_match ('/'))
 	break;
       while (rcd->next)
 	rcd = rcd->next;
-      rcd = rcd->next = xmalloc (sizeof (rcd_var));
+      rcd = rcd->next = xmalloc (sizeof *rcd);
 
       free (v);
     }
+
   if (token != '.')
-    return syntax_error (_("expecting end of command"));
+    {
+      lex_error (_("expecting end of command"));
+      goto lossage;
+    }
 
   for (rcd = head; rcd; rcd = rcd->next)
     if (rcd->dest_name[0])
@@ -428,51 +422,65 @@ internal_cmd_recode (void)
 	    /* This can occur if a destname is duplicated.  We could
 	       give an error at parse time but I don't care enough. */
 	    rcd->dest = find_variable (rcd->dest_name);
-	    assert (rcd->dest);
+	    assert (rcd->dest != NULL);
 	  }
 	else
 	  envector (rcd->dest);
       }
 
-  trns = xmalloc (sizeof (recode_trns));
+  trns = xmalloc (sizeof *trns);
   trns->h.proc = recode_trns_proc;
   trns->h.free = recode_trns_free;
   trns->codings = head;
-  add_transformation ((any_trns *) trns);
+  add_transformation ((struct trns_header *) trns);
 
 #if DEBUGGING
   debug_print (head);
 #endif
 
-  return 1;
+  return CMD_SUCCESS;
+
+ lossage:
+  {
+    struct recode_trns t;
+
+    t.codings = head;
+    recode_trns_free ((struct trns_header *) &t);
+    return CMD_FAILURE;
+  }
 }
 
 static int
-parse_dest_spec (rcd_var * rcd, value * v, int *max_dst_width)
+parse_dest_spec (struct rcd_var * rcd, union value * v, size_t *max_dst_width)
 {
   int flags;
 
   v->c = NULL;
 
-  if (token == NUM || (token == ID && id_match ("SYSMIS", tokstr)))
+  if (token == T_NUM)
     {
-      v->f = token == NUM ? tokval : SYSMIS;
-      get_token ();
+      v->f = tokval;
+      lex_get ();
       flags = RCD_DEST_NUMERIC;
     }
-  else if (token == STRING)
+  else if (lex_match_id ("SYSMIS"))
     {
-      int max = *max_dst_width;
-      int toklen = strlen (tokstr);
+      v->f = SYSMIS;
+      flags = RCD_DEST_NUMERIC;
+    }
+  else if (token == T_STRING)
+    {
+      size_t max = *max_dst_width;
+      size_t toklen = ds_length (&tokstr);
       if (toklen > max)
 	max = toklen;
       v->c = xmalloc (max + 1);
-      strpadcpy (v->c, tokstr, max + 1, ' ');
+      st_pad_copy (v->c, ds_value (&tokstr), max + 1);
       flags = RCD_DEST_STRING;
       *max_dst_width = max;
-      get_token ();
+      lex_get ();
     }
-  else if (match_id (COPY))
+  else if (lex_match_id ("COPY"))
     {
       if ((rcd->flags & RCD_SRC_MASK) == RCD_SRC_NUMERIC)
 	{
@@ -494,10 +502,12 @@ parse_dest_spec (rcd_var * rcd, value * v, int *max_dst_width)
 	   || ((rcd->flags & RCD_DEST_MASK) == RCD_DEST_STRING
 	       && flags != RCD_DEST_STRING))
 #endif
-    else
-    if ((rcd->flags & RCD_DEST_MASK) ^ flags)
-      return msg (SE, _("Inconsistent output types.  The output values "
-		  "must be all numeric or all string."));
+    else if ((rcd->flags & RCD_DEST_MASK) ^ flags)
+      {
+	msg (SE, _("Inconsistent output types.  The output values "
+		   "must be all numeric or all string."));
+	return 0;
+      }
 
   return 1;
 }
@@ -507,21 +517,21 @@ parse_dest_spec (rcd_var * rcd, value * v, int *max_dst_width)
    but with CONVERT as the keyword; 3 for success but with ELSE as the
    keyword. */
 static int
-parse_src_spec (rcd_var * rcd, int type, int max_src_width)
+parse_src_spec (struct rcd_var * rcd, int type, size_t max_src_width)
 {
-  coding *c;
+  struct coding *c;
 
-  while (1)
+  for (;;)
     {
       if (rcd->nmap >= rcd->mmap - 1)
 	{
 	  rcd->mmap += 16;
-	  rcd->map = xrealloc (rcd->map, rcd->mmap * sizeof (coding));
+	  rcd->map = xrealloc (rcd->map, rcd->mmap * sizeof *rcd->map);
 	}
 
       c = &rcd->map[rcd->nmap];
       c->f1.c = c->f2.c = NULL;
-      if (match_id (ELSE))
+      if (lex_match_id ("ELSE"))
 	{
 	  c->type = RCD_ELSE;
 	  rcd->nmap++;
@@ -529,69 +539,85 @@ parse_src_spec (rcd_var * rcd, int type, int max_src_width)
 	}
       else if (type == NUMERIC)
 	{
-	  if (token == ID)
+	  if (token == T_ID)
 	    {
-	      if (match_id (LO) || match_id (LOWEST))
+	      if (lex_match_id ("LO") || lex_match_id ("LOWEST"))
 		{
-		  force_match_id (THRU);
-		  if (match_id (HI) || match_id (HIGHEST))
+		  if (!lex_force_match_id ("THRU"))
+		    return 0;
+		  if (lex_match_id ("HI") || lex_match_id ("HIGHEST"))
 		    c->type = RCD_ELSE;
-		  else if (token == NUM)
+		  else if (token == T_NUM)
 		    {
 		      c->type = RCD_LOW;
 		      c->f1.f = tokval;
-		      get_token ();
+		      lex_get ();
 		    }
 		  else
-		    return syntax_error (_("following LO THRU"));
+		    {
+		      lex_error (_("following LO THRU"));
+		      return 0;
+		    }
 		}
-	      else if (match_id (MISSING))
+	      else if (lex_match_id ("MISSING"))
 		{
 		  c->type = RCD_USER;
 		  rcd->flags |= RCD_MISC_MISSING;
 		}
-	      else if (match_id (SYSMIS))
+	      else if (lex_match_id ("SYSMIS"))
 		{
 		  c->type = RCD_END;
 		  rcd->flags |= RCD_MISC_MISSING;
 		}
 	      else
-		return syntax_error (_("in source value"));
+		{
+		  lex_error (_("in source value"));
+		  return 0;
+		}
 	    }
-	  else if (token == NUM)
+	  else if (token == T_NUM)
 	    {
 	      c->f1.f = tokval;
-	      get_token ();
-	      if (match_id (THRU))
+	      lex_get ();
+	      if (lex_match_id ("THRU"))
 		{
-		  if (match_id (HI) || match_id (HIGHEST))
+		  if (lex_match_id ("HI") || lex_match_id ("HIGHEST"))
 		    c->type = RCD_HIGH;
-		  else if (token == NUM)
+		  else if (token == T_NUM)
 		    {
 		      c->type = RCD_RANGE;
 		      c->f2.f = tokval;
-		      get_token ();
+		      lex_get ();
 		    }
 		  else
-		    return syntax_error (NULL);
+		    {
+		      lex_error (NULL);
+		      return 0;
+		    }
 		}
 	      else
 		c->type = RCD_SINGLE;
 	    }
 	  else
-	    return syntax_error ("in source value");
+	    {
+	      lex_error (_("in source value"));
+	      return 0;
+	    }
 	}
       else
-	/* type==ALPHA */
 	{
-	  if (match_id (CONVERT))
+	  assert (type == ALPHA);
+	  if (lex_match_id ("CONVERT"))
 	    {
 	      if ((rcd->flags & RCD_DEST_MASK) == RCD_DEST_ERROR)
 		rcd->flags |= RCD_DEST_NUMERIC;
 	      else if ((rcd->flags & RCD_DEST_MASK) != RCD_DEST_NUMERIC)
-		return msg (SE, _("Keyword CONVERT may only be used with "
-			    "string input values and numeric output "
-			    "values."));
+		{
+		  msg (SE, _("Keyword CONVERT may only be used with "
+			     "string input values and numeric output "
+			     "values."));
+		  return 0;
+		}
 
 	      c->type = RCD_CONVERT;
 	      rcd->nmap++;
@@ -604,17 +630,18 @@ parse_src_spec (rcd_var * rcd, int type, int max_src_width)
 	         than necessary based on the DEBUGGING `#define' is just
 	         *inviting* bugs. */
 	      c->type = RCD_SINGLE;
-	      force_string ();
+	      if (!lex_force_string ())
+		return 0;
 	      c->f1.c = xmalloc (max_src_width + 1);
-	      strpadcpy (c->f1.c, tokstr, max_src_width + 1, ' ');
-	      get_token ();
+	      st_pad_copy (c->f1.c, ds_value (&tokstr), max_src_width + 1);
+	      lex_get ();
 	    }
 	}
 
       if (c->type != RCD_END)
 	rcd->nmap++;
 
-      match_tok (',');
+      lex_match (',');
       if (token == '=')
 	break;
     }
@@ -624,12 +651,12 @@ parse_src_spec (rcd_var * rcd, int type, int max_src_width)
 /* Data transformation. */
 
 static void
-recode_trns_free (any_trns * t)
+recode_trns_free (struct trns_header * t)
 {
   int i;
-  rcd_var *head, *next;
+  struct rcd_var *head, *next;
 
-  head = ((recode_trns *) t)->codings;
+  head = ((struct recode_trns *) t)->codings;
   while (head)
     {
       if (head->map && !(head->flags & RCD_MISC_DUPLICATE))
@@ -666,11 +693,11 @@ recode_trns_free (any_trns * t)
     }
 }
 
-static inline coding *
-find_src_numeric (rcd_var * v, ccase * c)
+static inline struct coding *
+find_src_numeric (struct rcd_var * v, struct ccase * c)
 {
   double cmp = c->data[v->src->fv].f;
-  coding *cp;
+  struct coding *cp;
 
   if (cmp == SYSMIS)
     {
@@ -717,12 +744,12 @@ find_src_numeric (rcd_var * v, ccase * c)
       }
 }
 
-static inline coding *
-find_src_string (rcd_var * v, ccase * c)
+static inline struct coding *
+find_src_string (struct rcd_var * v, struct ccase * c)
 {
   char *cmp = c->data[v->src->fv].s;
   int w = v->src->width;
-  coding *cp;
+  struct coding *cp;
 
   for (cp = v->map;; cp++)
     switch (cp->type)
@@ -751,12 +778,12 @@ find_src_string (rcd_var * v, ccase * c)
 }
 
 static int
-recode_trns_proc (any_trns * t, ccase * c)
+recode_trns_proc (struct trns_header * t, struct ccase * c)
 {
-  rcd_var *v;
-  coding *cp;
+  struct rcd_var *v;
+  struct coding *cp;
 
-  for (v = ((recode_trns *) t)->codings; v; v = v->next)
+  for (v = ((struct recode_trns *) t)->codings; v; v = v->next)
     {
       switch (v->flags & RCD_SRC_MASK)
 	{
@@ -783,8 +810,9 @@ recode_trns_proc (any_trns * t, ccase * c)
 	{
 	  char *val = cp->t.c;
 	  if (val == NULL)
-	    strbarepadlencpy (c->data[v->dest->fv].s, c->data[v->src->fv].c,
-			      v->dest->width, ' ', v->src->width);
+	    st_bare_pad_len_copy (c->data[v->dest->fv].s,
+				  c->data[v->src->fv].c,
+				  v->dest->width, v->src->width);
 	  else
 	    memcpy (c->data[v->dest->fv].s, cp->t.c, v->dest->width);
 	}
@@ -797,7 +825,7 @@ recode_trns_proc (any_trns * t, ccase * c)
 
 #if DEBUGGING
 static void
-dump_dest (rcd_var * v, value * c)
+dump_dest (struct rcd_var * v, union value * c)
 {
   if ((v->flags & RCD_DEST_MASK) == RCD_DEST_NUMERIC)
     if (c->f == SYSMIS)
@@ -813,10 +841,10 @@ dump_dest (rcd_var * v, value * c)
 }
 
 static void
-debug_print (rcd_var * head)
+debug_print (struct rcd_var * head)
 {
-  rcd_var *iter, *start;
-  coding *c;
+  struct rcd_var *iter, *start;
+  struct coding *c;
 
   printf ("RECODE\n");
   for (iter = head; iter; iter = iter->next)
@@ -886,7 +914,7 @@ debug_print (rcd_var * head)
 	  printf (")");
 	}
       printf ("\n    INTO");
-      while (1)
+      for (;;)
 	{
 	  printf (" %s",
 		start->dest_name[0] ? start->dest_name : start->dest->name);
