@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -39,12 +39,14 @@ char *alloca ();
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
-#include "common.h"
+#include <float.h>
+#include "alloc.h"
 #include "avl.h"
 #include "error.h"
 #include "file-handle.h"
 #include "format.h"
 #include "getline.h"
+#include "magic.h"
 #include "misc.h"
 #include "sfm.h"
 #include "sfmP.h"
@@ -61,12 +63,12 @@ char *alloca ();
    desk.  */
 
 /* sfm's file_handle extension. */
-typedef struct
+struct sfm_fhuser_ext
   {
     FILE *file;			/* Actual file. */
     int opened;			/* Reference count. */
 
-    dictionary *dict;		/* File's dictionary. */
+    struct dictionary *dict;	/* File's dictionary. */
 
     int reverse_endian;		/* 1=file has endianness opposite us. */
     int case_size;		/* Number of `values's per case. */
@@ -89,17 +91,39 @@ typedef struct
     unsigned char x[sizeof (flt64)];
     /* Current instruction octet. */
     unsigned char *y;		/* Location in current instruction octet. */
-  }
-sfm_fhuser_ext;
+  };
 
-static fh_ext_class sfm_r_class;
+static struct fh_ext_class sfm_r_class;
 
 #if GLOBAL_DEBUGGING
-void dump_dictionary (dictionary * dict);
+void dump_dictionary (struct dictionary * dict);
 #endif
 
 /* Utilities. */
 
+/* bswap_int32(): Reverse the byte order of 32-bit integer *X. */
+#if __linux__
+#include <asm/byteorder.h>
+static inline void
+bswap_int32 (int32 * x)
+{
+  *x = ntohl (*x);
+}
+#else /* not Linux */
+static inline void
+bswap_int32 (int32 * x)
+{
+  unsigned char *y = (char *) x;
+  unsigned char t;
+  t = y[0];
+  y[0] = y[3];
+  y[3] = t;
+  t = y[1];
+  y[1] = y[2];
+  y[2] = t;
+}
+#endif /* not Linux */
+
 /* Reverse the byte order of 64-bit floating point *X. */
 static inline void
 bswap_flt64 (flt64 * x)
@@ -130,32 +154,49 @@ bswap_flt64 (flt64 * x)
       size_t x;
 
       for (x = 0; x < sizeof (flt64) / 2; x++)
-	t = y[x], y[x] = y[sizeof (flt64) - x], y[sizeof (flt64) - x] = t;
+	{
+	  t = y[x];
+	  y[x] = y[sizeof (flt64) - x];
+	  y[sizeof (flt64) - x] = t;
+	}
     }
 }
 
-static int 
+static void
 corrupt_msg (int class, const char *format,...)
   __attribute__ ((format (printf, 2, 3)));
 
 /* Displays a corrupt sysfile error. */
-static int
+static void
 corrupt_msg (int class, const char *format,...)
 {
-  va_list args;
+  char buf[1024];
+  
+  {
+    va_list args;
 
-  va_start (args, format);
-  vmsg (class, format, args, _("corrupt system file: "));
-  va_end (args);
+    va_start (args, format);
+    vsnprintf (buf, 1024, format, args);
+    va_end (args);
+  }
+  
+  {
+    struct error e;
 
-  return 0;
+    e.class = class;
+    getl_location (&e.where.filename, &e.where.line_number);
+    e.title = _("corrupt system file: ");
+    e.text = buf;
+
+    err_vmsg (&e);
+  }
 }
 
 /* Closes a system file after we're done with it. */
 static void
-sfm_close (file_handle * h)
+sfm_close (struct file_handle * h)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   ext->opened--;
   assert (ext->opened == 0);
@@ -167,9 +208,9 @@ sfm_close (file_handle * h)
 
 /* Closes a system file if we're done with it. */
 void
-sfm_maybe_close (file_handle *h)
+sfm_maybe_close (struct file_handle *h)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   if (ext->opened == 1)
     fh_close_handle (h);
@@ -179,17 +220,17 @@ sfm_maybe_close (file_handle *h)
 
 /* Dictionary reader. */
 
-static void *bufread (file_handle * handle, void *buf, size_t nbytes,
+static void *bufread (struct file_handle * handle, void *buf, size_t nbytes,
 		      size_t minalloc);
 
-static int read_header (file_handle * h, sfm_read_info * inf);
-static int parse_format_spec (file_handle * h, int32 s,
-			      fmt_spec * v, variable *vv);
-static int read_value_labels (file_handle * h, variable ** var_by_index);
-static int read_variables (file_handle * h, variable *** var_by_index);
-static int read_machine_int32_info (file_handle * h, int size, int count);
-static int read_machine_flt64_info (file_handle * h, int size, int count);
-static int read_documents (file_handle * h);
+static int read_header (struct file_handle * h, struct sfm_read_info * inf);
+static int parse_format_spec (struct file_handle * h, int32 s,
+			      struct fmt_spec * v, struct variable *vv);
+static int read_value_labels (struct file_handle * h, struct variable ** var_by_index);
+static int read_variables (struct file_handle * h, struct variable *** var_by_index);
+static int read_machine_int32_info (struct file_handle * h, int size, int count);
+static int read_machine_flt64_info (struct file_handle * h, int size, int count);
+static int read_documents (struct file_handle * h);
 
 /* Displays the message X with corrupt_msg, then jumps to the lossage
    label. */
@@ -214,15 +255,15 @@ static int read_documents (file_handle * h);
 /* Reads the dictionary from file with handle H, and returns it in a
    dictionary structure.  This dictionary may be modified in order to
    rename, reorder, and delete variables, etc.	*/
-dictionary *
-sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
+struct dictionary *
+sfm_read_dictionary (struct file_handle * h, struct sfm_read_info * inf)
 {
   /* The file handle extension record. */
-  sfm_fhuser_ext *ext;
+  struct sfm_fhuser_ext *ext;
 
   /* Allows for quick reference to variables according to indexes
      relative to position within a case. */
-  variable **var_by_index = NULL;
+  struct variable **var_by_index = NULL;
 
   /* Check whether the file is already open. */
   if (h->class == &sfm_r_class)
@@ -238,17 +279,17 @@ sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
       return NULL;
     }
 
-  verbose_msg (1, _("%s: Opening system-file handle %s for reading."),
-	       fh_handle_filename (h), fh_handle_name (h));
+  msg (VM (1), _("%s: Opening system-file handle %s for reading."),
+       fh_handle_filename (h), fh_handle_name (h));
   
   /* Open the physical disk file. */
-  ext = xmalloc (sizeof (sfm_fhuser_ext));
+  ext = xmalloc (sizeof (struct sfm_fhuser_ext));
   ext->file = fopen (h->norm_fn, "rb");
   if (ext->file == NULL)
     {
       msg (ME, _("An error occurred while opening \"%s\" for reading "
 	   "as a system file: %s."), h->fn, strerror (errno));
-      cond_fail ();
+      err_cond_fail ();
       free (ext);
       return NULL;
     }
@@ -258,7 +299,7 @@ sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
   h->ext = ext;
   ext->dict = NULL;
   ext->buf = ext->ptr = ext->end = NULL;
-  ext->y = &ext->x[sizeof (ext->x)];
+  ext->y = ext->x + sizeof ext->x;
   ext->opened = 1;
 
   /* Default special constants. */
@@ -277,7 +318,7 @@ sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
   /* Handle weighting. */
   if (ext->weight_index != -1)
     {
-      variable *wv = var_by_index[ext->weight_index];
+      struct variable *wv = var_by_index[ext->weight_index];
 
       if (wv == NULL)
 	lose ((ME, _("%s: Weighting variable may not be a continuation of "
@@ -296,7 +337,7 @@ sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
     {
       int32 rec_type;
 
-      assertive_bufread (h, &rec_type, sizeof (rec_type), 0);
+      assertive_bufread (h, &rec_type, sizeof rec_type, 0);
       if (ext->reverse_endian)
 	bswap_int32 (&rec_type);
 
@@ -329,7 +370,7 @@ sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
 
 	    int skip = 0;
 
-	    assertive_bufread (h, &data, sizeof (data), 0);
+	    assertive_bufread (h, &data, sizeof data, 0);
 	    if (ext->reverse_endian)
 	      {
 		bswap_int32 (&data.subtype);
@@ -382,7 +423,7 @@ sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
 	  {
 	    int32 filler;
 
-	    assertive_bufread (h, &filler, sizeof (filler), 0);
+	    assertive_bufread (h, &filler, sizeof filler, 0);
 	    goto break_out_of_loop;
 	  }
 
@@ -393,7 +434,7 @@ sfm_read_dictionary (file_handle * h, sfm_read_info * inf)
 
 break_out_of_loop:
   /* Come here on successful completion. */
-  verbose_msg (2, _("Read system-file dictionary successfully."));
+  msg (VM (2), _("Read system-file dictionary successfully."));
     
 #if DEBUGGING
   dump_dictionary (ext->dict);
@@ -403,7 +444,7 @@ break_out_of_loop:
 
 lossage:
   /* Come here on unsuccessful completion. */
-  verbose_msg (1, _("Error reading system-file header."));
+  msg (VM (1), _("Error reading system-file header."));
   
   free (var_by_index);
   fclose (ext->file);
@@ -417,9 +458,9 @@ lossage:
 
 /* Read record type 7, subtype 3. */
 static int
-read_machine_int32_info (file_handle * h, int size, int count)
+read_machine_int32_info (struct file_handle * h, int size, int count)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   int32 data[8];
   int file_endian;
@@ -431,7 +472,7 @@ read_machine_int32_info (file_handle * h, int size, int count)
 	   "subtype 3.	Expected size %d, count 8."),
 	   h->fn, size, count, sizeof (int32)));
 
-  assertive_bufread (h, data, sizeof (data), 0);
+  assertive_bufread (h, data, sizeof data, 0);
   if (ext->reverse_endian)
     for (i = 0; i < 8; i++)
       bswap_int32 (&data[i]);
@@ -482,9 +523,9 @@ lossage:
 
 /* Read record type 7, subtype 4. */
 static int
-read_machine_flt64_info (file_handle * h, int size, int count)
+read_machine_flt64_info (struct file_handle * h, int size, int count)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   flt64 data[3];
 
@@ -495,7 +536,7 @@ read_machine_flt64_info (file_handle * h, int size, int count)
 	   "subtype 4.	Expected size %d, count 8."),
 	   h->fn, size, count, sizeof (flt64)));
 
-  assertive_bufread (h, data, sizeof (data), 0);
+  assertive_bufread (h, data, sizeof data, 0);
   if (ext->reverse_endian)
     for (i = 0; i < 3; i++)
       bswap_flt64 (&data[i]);
@@ -522,17 +563,17 @@ lossage:
 }
 
 static int
-read_header (file_handle * h, sfm_read_info * inf)
+read_header (struct file_handle * h, struct sfm_read_info * inf)
 {
-  sfm_fhuser_ext *ext = h->ext;	/* File extension strcut. */
-  sysfile_header hdr;		/* Disk buffer. */
-  dictionary *dict;		/* File dictionary. */
-  char prod_name[sizeof (hdr.prod_name) + 1];	/* Buffer for product name. */
+  struct sfm_fhuser_ext *ext = h->ext;	/* File extension strcut. */
+  struct sysfile_header hdr;		/* Disk buffer. */
+  struct dictionary *dict;		/* File dictionary. */
+  char prod_name[sizeof hdr.prod_name + 1];	/* Buffer for product name. */
   int skip_amt;			/* Amount of product name to omit. */
   int i;
 
   /* Create the dictionary. */
-  dict = ext->dict = xmalloc (sizeof (dictionary));
+  dict = ext->dict = xmalloc (sizeof *dict);
   dict->var = NULL;
   dict->var_by_name = NULL;
   dict->nvar = 0;
@@ -548,14 +589,14 @@ read_header (file_handle * h, sfm_read_info * inf)
   dict->documents = NULL;
 
   /* Read header, check magic. */
-  assertive_bufread (h, &hdr, sizeof (sysfile_header), 0);
+  assertive_bufread (h, &hdr, sizeof hdr, 0);
   if (0 != strncmp ("$FL2", hdr.rec_type, 4))
     lose ((ME, _("%s: Bad magic.  Proper system files begin with "
 		 "the four characters `$FL2'. This file will not be read."),
 	   h->fn));
 
   /* Check eye-catcher string. */
-  memcpy (prod_name, hdr.prod_name, sizeof (hdr.prod_name));
+  memcpy (prod_name, hdr.prod_name, sizeof hdr.prod_name);
   for (i = 0; i < 60; i++)
     if (!isprint ((unsigned char) prod_name[i]))
       prod_name[i] = ' ';
@@ -608,9 +649,9 @@ read_header (file_handle * h, sfm_read_info * inf)
   /* Copy basic info and verify correctness. */
   ext->case_size = hdr.case_size;
   if (hdr.case_size <= 0 || ext->case_size > (INT_MAX
-					      / (int) sizeof (value) / 2))
+					      / (int) sizeof (union value) / 2))
     lose ((ME, _("%s: Number of elements per case (%d) is not between 1 "
-	   "and %d."), h->fn, hdr.case_size, INT_MAX / sizeof (value) / 2));
+	   "and %d."), h->fn, hdr.case_size, INT_MAX / sizeof (union value) / 2));
 
   ext->compressed = hdr.compressed;
 
@@ -636,7 +677,7 @@ read_header (file_handle * h, sfm_read_info * inf)
     int i;
 
     dict->label = NULL;
-    for (i = sizeof (hdr.file_label) - 1; i >= 0; i--)
+    for (i = sizeof hdr.file_label - 1; i >= 0; i--)
       if (!isspace ((unsigned char) hdr.file_label[i])
 	  && hdr.file_label[i] != 0)
 	{
@@ -688,29 +729,29 @@ lossage:
    file.  Also, the get.* elements are set to appropriate values to
    allow the file to be read.  */
 static int
-read_variables (file_handle * h, variable *** var_by_index)
+read_variables (struct file_handle * h, struct variable *** var_by_index)
 {
   int i;
 
-  sfm_fhuser_ext *ext = h->ext;	/* File extension record. */
-  dictionary *dict = ext->dict;	/* Dictionary being constructed. */
-  sysfile_variable sv;		/* Disk buffer. */
+  struct sfm_fhuser_ext *ext = h->ext;	/* File extension record. */
+  struct dictionary *dict = ext->dict;	/* Dictionary being constructed. */
+  struct sysfile_variable sv;		/* Disk buffer. */
   int long_string_count = 0;	/* # of long string continuation
 				   records still expected. */
   int next_value = 0;		/* Index to next `value' structure. */
 
   /* Allocate variables. */
-  dict->var = xmalloc (sizeof (variable *) * ext->case_size);
-  *var_by_index = xmalloc (sizeof (variable *) * ext->case_size);
+  dict->var = xmalloc (sizeof *dict->var * ext->case_size);
+  *var_by_index = xmalloc (sizeof **var_by_index * ext->case_size);
 
   /* Read in the entry for each variable and use the info to
      initialize the dictionary. */
   for (i = 0; i < ext->case_size; i++)
     {
-      variable *vv;
+      struct variable *vv;
       int j;
 
-      assertive_bufread (h, &sv, sizeof (sysfile_variable), 0);
+      assertive_bufread (h, &sv, sizeof sv, 0);
 
       if (ext->reverse_endian)
 	{
@@ -756,8 +797,7 @@ read_variables (file_handle * h, variable *** var_by_index)
 		     "-3, -2, 0, 1, 2, or 3."), h->fn, i));
 
       /* Construct internal variable structure, initialize critical bits. */
-      vv = (*var_by_index)[i] = dict->var[dict->nvar++]
-	= xmalloc (sizeof (variable));
+      vv = (*var_by_index)[i] = dict->var[dict->nvar++] = xmalloc (sizeof *vv);
       vv->index = dict->nvar - 1;
       vv->foo = -1;
       vv->label = NULL;
@@ -828,7 +868,7 @@ read_variables (file_handle * h, variable *** var_by_index)
 	  int32 len;
 
 	  /* Read length of label. */
-	  assertive_bufread (h, &len, sizeof (int32), 0);
+	  assertive_bufread (h, &len, sizeof len, 0);
 	  if (ext->reverse_endian)
 	    bswap_int32 (&len);
 
@@ -853,7 +893,7 @@ read_variables (file_handle * h, variable *** var_by_index)
 	    lose ((ME, _("%s: Long string variable %s may not have missing "
 		   "values."), h->fn, vv->name));
 
-	  assertive_bufread (h, mv, sizeof (flt64) * abs (sv.n_missing_values), 0);
+	  assertive_bufread (h, mv, sizeof *mv * abs (sv.n_missing_values), 0);
 
 	  if (ext->reverse_endian && vv->type == NUMERIC)
 	    for (j = 0; j < abs (sv.n_missing_values); j++)
@@ -916,7 +956,7 @@ read_variables (file_handle * h, variable *** var_by_index)
   if (next_value != ext->case_size)
     lose ((ME, _("%s: System file header indicates %d variable positions but "
 	   "%d were read from file."), h->fn, ext->case_size, next_value));
-  dict->var = xrealloc (dict->var, sizeof (variable *) * dict->nvar);
+  dict->var = xrealloc (dict->var, sizeof *dict->var * dict->nvar);
 
   /* Construct AVL tree of dictionary in order to speed up later
      processing and to check for duplicate varnames. */
@@ -946,11 +986,10 @@ lossage:
 /* Translates the format spec from sysfile format to internal
    format. */
 static int
-parse_format_spec (file_handle * h, int32 s, fmt_spec * v,
-		   variable *vv)
+parse_format_spec (struct file_handle *h, int32 s, struct fmt_spec *v, struct variable *vv)
 {
   if ((size_t) ((s >> 16) & 0xff)
-      >= sizeof (translate_fmt) / sizeof (*translate_fmt))
+      >= sizeof translate_fmt / sizeof *translate_fmt)
     lose ((ME, _("%s: Bad format specifier byte (%d)."),
 	   h->fn, (s >> 16) & 0xff));
   
@@ -978,15 +1017,15 @@ lossage:
 /* Reads value labels from sysfile H and inserts them into the
    associated dictionary. */
 int
-read_value_labels (file_handle * h, variable ** var_by_index)
+read_value_labels (struct file_handle * h, struct variable ** var_by_index)
 {
-  sfm_fhuser_ext *ext = h->ext;	/* File extension record. */
+  struct sfm_fhuser_ext *ext = h->ext;	/* File extension record. */
 
   flt64 *raw_label = NULL;	/* Array of raw label values. */
-  value_label **cooked_label = NULL;	/* Array of cooked labels. */
+  struct value_label **cooked_label = NULL;	/* Array of cooked labels. */
   int32 n_labels;		/* Number of labels. */
 
-  variable **var = NULL;	/* Associated variables. */
+  struct variable **var = NULL;	/* Associated variables. */
   int32 n_vars;			/* Number of associated variables. */
 
   int i;
@@ -996,13 +1035,13 @@ read_value_labels (file_handle * h, variable ** var_by_index)
      don't know yet whether it is of numeric or string type. */
 
   /* Read number of labels. */
-  assertive_bufread (h, &n_labels, sizeof (n_labels), 0);
+  assertive_bufread (h, &n_labels, sizeof n_labels, 0);
   if (ext->reverse_endian)
     bswap_int32 (&n_labels);
 
   /* Allocate memory. */
-  raw_label = xmalloc (sizeof (flt64) * n_labels);
-  cooked_label = xmalloc (sizeof (value_label *) * n_labels);
+  raw_label = xmalloc (sizeof *raw_label * n_labels);
+  cooked_label = xmalloc (sizeof *cooked_label * n_labels);
   for (i = 0; i < n_labels; i++)
     cooked_label[i] = NULL;
 
@@ -1015,12 +1054,12 @@ read_value_labels (file_handle * h, variable ** var_by_index)
       int rem;
 
       /* Read value, label length. */
-      assertive_bufread (h, &value, sizeof (value), 0);
-      assertive_bufread (h, &label_len, sizeof (unsigned char), 0);
-      memcpy (&raw_label[i], &value, sizeof (value));
+      assertive_bufread (h, &value, sizeof value, 0);
+      assertive_bufread (h, &label_len, 1, 0);
+      memcpy (&raw_label[i], &value, sizeof value);
 
       /* Read label. */
-      cooked_label[i] = xmalloc (sizeof (value_label));
+      cooked_label[i] = xmalloc (sizeof **cooked_label);
       cooked_label[i]->s = xmalloc (label_len + 1);
       assertive_bufread (h, cooked_label[i]->s, label_len, 0);
       cooked_label[i]->s[label_len] = 0;
@@ -1038,7 +1077,7 @@ read_value_labels (file_handle * h, variable ** var_by_index)
   {
     int32 rec_type;
     
-    assertive_bufread (h, &rec_type, sizeof (rec_type), 0);
+    assertive_bufread (h, &rec_type, sizeof rec_type, 0);
     if (ext->reverse_endian)
       bswap_int32 (&rec_type);
     
@@ -1049,7 +1088,7 @@ read_value_labels (file_handle * h, variable ** var_by_index)
 
   /* Read number of variables associated with value label from type 4
      record. */
-  assertive_bufread (h, &n_vars, sizeof (n_vars), 0);
+  assertive_bufread (h, &n_vars, sizeof n_vars, 0);
   if (ext->reverse_endian)
     bswap_int32 (&n_vars);
   if (n_vars < 1 || n_vars > ext->dict->nvar)
@@ -1058,16 +1097,16 @@ read_value_labels (file_handle * h, variable ** var_by_index)
 	   h->fn, n_vars, ext->dict->nvar));
 
   /* Allocate storage. */
-  var = xmalloc (sizeof (variable *) * n_vars);
+  var = xmalloc (sizeof *var * n_vars);
 
   /* Read the list of variables. */
   for (i = 0; i < n_vars; i++)
     {
       int32 var_index;
-      variable *v;
+      struct variable *v;
 
       /* Read variable index, check range. */
-      assertive_bufread (h, &var_index, sizeof (var_index), 0);
+      assertive_bufread (h, &var_index, sizeof var_index, 0);
       if (ext->reverse_endian)
 	bswap_int32 (&var_index);
       if (var_index < 1 || var_index > ext->case_size)
@@ -1120,7 +1159,7 @@ read_value_labels (file_handle * h, variable ** var_by_index)
   /* Assign the value_label's to each variable. */
   for (i = 0; i < n_vars; i++)
     {
-      variable *v = var[i];
+      struct variable *v = var[i];
       int j;
 
       /* Create AVL tree if necessary. */
@@ -1130,7 +1169,7 @@ read_value_labels (file_handle * h, variable ** var_by_index)
       /* Add each label to the variable. */
       for (j = 0; j < n_labels; j++)
 	{
-	  value_label *old = avl_replace (v->val_lab, cooked_label[j]);
+	  struct value_label *old = avl_replace (v->val_lab, cooked_label[j]);
 	  if (old == NULL)
 	    continue;
 
@@ -1169,9 +1208,9 @@ lossage:
    MINALLOC bytes.  Returns a pointer to the buffer on success, NULL
    on failure. */
 static void *
-bufread (file_handle * h, void *buf, size_t nbytes, size_t minalloc)
+bufread (struct file_handle * h, void *buf, size_t nbytes, size_t minalloc)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   if (buf == NULL)
     buf = xmalloc (max (nbytes, minalloc));
@@ -1190,17 +1229,17 @@ bufread (file_handle * h, void *buf, size_t nbytes, size_t minalloc)
    the documents and n_documents fields in the associated
    dictionary. */
 static int
-read_documents (file_handle * h)
+read_documents (struct file_handle * h)
 {
-  sfm_fhuser_ext *ext = h->ext;
-  dictionary *dict = ext->dict;
+  struct sfm_fhuser_ext *ext = h->ext;
+  struct dictionary *dict = ext->dict;
   int32 n_lines;
 
   if (dict->documents != NULL)
     lose ((ME, _("%s: System file contains multiple type 6 (document) records."),
 	   h->fn));
 
-  assertive_bufread (h, &n_lines, sizeof (n_lines), 0);
+  assertive_bufread (h, &n_lines, sizeof n_lines, 0);
   dict->n_documents = n_lines;
   if (dict->n_documents <= 0)
     lose ((ME, _("%s: Number of document lines (%ld) must be greater than 0."),
@@ -1220,7 +1259,7 @@ lossage:
 #include "debug-print.h"
 /* Displays dictionary DICT on stdout. */
 void
-dump_dictionary (dictionary * dict)
+dump_dictionary (struct dictionary * dict)
 {
   int i;
 
@@ -1228,7 +1267,7 @@ dump_dictionary (dictionary * dict)
   for (i = 0; i < dict->nvar; i++)
     {
       char print[32];
-      variable *v = dict->var[i];
+      struct variable *v = dict->var[i];
       int n, j;
 
       debug_printf (("	 var %s", v->name));
@@ -1304,14 +1343,14 @@ dump_dictionary (dictionary * dict)
    appropriately.  Returns nonzero only if both no errors occur and
    data was read. */
 static int
-buffer_input (file_handle * h)
+buffer_input (struct file_handle * h)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
   size_t amt;
 
   if (ext->buf == NULL)
-    ext->buf = xmalloc (sizeof (flt64) * 128);
-  amt = fread (ext->buf, sizeof (flt64), 128, ext->file);
+    ext->buf = xmalloc (sizeof *ext->buf * 128);
+  amt = fread (ext->buf, sizeof *ext->buf, 128, ext->file);
   if (ferror (ext->file))
     {
       msg (ME, _("%s: Error reading file: %s."), h->fn, strerror (errno));
@@ -1331,11 +1370,11 @@ buffer_input (file_handle * h)
    byte gives a value for that byte or indicates that the value can be
    found following the instructions. */
 static int
-read_compressed_data (file_handle * h, flt64 * temp)
+read_compressed_data (struct file_handle * h, flt64 * temp)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
-  const unsigned char *p_end = &ext->x[sizeof (flt64)];
+  const unsigned char *p_end = ext->x + sizeof (flt64);
   unsigned char *p = ext->y;
 
   const flt64 *temp_beg = temp;
@@ -1364,13 +1403,13 @@ read_compressed_data (file_handle * h, flt64 * temp)
 		  lose ((ME, _("%s: Unexpected end of file."), h->fn));
 		  goto lossage;
 		}
-	    memcpy (temp++, ext->ptr++, sizeof (flt64));
+	    memcpy (temp++, ext->ptr++, sizeof *temp);
 	    if (temp >= temp_end)
 	      goto winnage;
 	    break;
 	  case 254:
 	    /* Code 254 indicates a string that is all blanks. */
-	    memset (temp++, ' ', sizeof (flt64));
+	    memset (temp++, ' ', sizeof *temp);
 	    if (temp >= temp_end)
 	      goto winnage;
 	    break;
@@ -1405,7 +1444,7 @@ read_compressed_data (file_handle * h, flt64 * temp)
 	      lose ((ME, _("%s: Unexpected end of file."), h->fn));
 	    goto lossage;
 	  }
-      memcpy (ext->x, ext->ptr++, sizeof (flt64));
+      memcpy (ext->x, ext->ptr++, sizeof *temp);
       p = ext->x;
     }
 
@@ -1430,9 +1469,9 @@ lossage:
    which must have the get.* elements appropriately set.  Returns
    nonzero only if successful.  */
 int
-sfm_read_case (file_handle * h, value * perm, dictionary * dict)
+sfm_read_case (struct file_handle * h, union value * perm, struct dictionary * dict)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   size_t nbytes;
   flt64 *temp;
@@ -1446,7 +1485,7 @@ sfm_read_case (file_handle * h, value * perm, dictionary * dict)
   /* The first concern is to obtain a full case relative to the data
      file.  (Cases in the data file have no particular relationship to
      cases in the active file.) */
-  nbytes = sizeof (flt64) * ext->case_size;
+  nbytes = sizeof *temp * ext->case_size;
   temp = local_alloc (nbytes);
 
   if (ext->compressed == 0)
@@ -1469,7 +1508,7 @@ sfm_read_case (file_handle * h, value * perm, dictionary * dict)
      format. */
   for (i = 0; i < dict->nvar; i++)
     {
-      variable *v = dict->var[i];
+      struct variable *v = dict->var[i];
 
       if (v->get.fv == -1)
 	continue;
@@ -1493,7 +1532,7 @@ lossage:
   return 0;
 }
 
-static fh_ext_class sfm_r_class =
+static struct fh_ext_class sfm_r_class =
 {
   3,
   N_("reading as a system file"),
