@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -43,12 +43,13 @@ char *alloca ();
 #if HAVE_UNISTD_H
 #include <unistd.h>	/* Required by SunOS4. */
 #endif
-#include "common.h"
+#include "alloc.h"
 #include "approx.h"
 #include "avl.h"
 #include "error.h"
 #include "file-handle.h"
 #include "getline.h"
+#include "magic.h"
 #include "misc.h"
 #include "sfm.h"
 #include "sfmP.h"
@@ -71,7 +72,7 @@ char *alloca ();
 #define COMPRESSION_BIAS 100
 
 /* sfm writer file_handle extension. */
-typedef struct
+struct sfm_fhuser_ext
   {
     FILE *file;			/* Actual file. */
 
@@ -84,28 +85,27 @@ typedef struct
     int n_cases;		/* Number of cases written so far. */
 
     char *elem_type; 		/* ALPHA or NUMERIC for each flt64 element. */
-  }
-sfm_fhuser_ext;
+  };
 
-static fh_ext_class sfm_w_class;
+static struct fh_ext_class sfm_w_class;
 
 static char *append_string_max (char *, const char *, const char *);
-static int write_header (sfm_write_info *inf);
-static int bufwrite (file_handle *h, const void *buf, size_t nbytes);
-static int write_variable (sfm_write_info *inf, variable *v);
-static int write_value_labels (sfm_write_info *inf, variable * s, int index);
-static int write_rec_7_34 (sfm_write_info *inf);
-static int write_documents (sfm_write_info *inf);
+static int write_header (struct sfm_write_info *inf);
+static int bufwrite (struct file_handle *h, const void *buf, size_t nbytes);
+static int write_variable (struct sfm_write_info *inf, struct variable *v);
+static int write_value_labels (struct sfm_write_info *inf, struct variable * s, int index);
+static int write_rec_7_34 (struct sfm_write_info *inf);
+static int write_documents (struct sfm_write_info *inf);
 
 /* Writes the dictionary INF->dict to system file INF->h.  The system
    file is compressed if INF->compress is nonzero.  INF->case_size is
    set to the number of flt64 elements in a single case.  Returns
    nonzero only if successful. */
 int
-sfm_write_dictionary (sfm_write_info *inf)
+sfm_write_dictionary (struct sfm_write_info *inf)
 {
-  dictionary *d = inf->dict;
-  sfm_fhuser_ext *ext;
+  struct dictionary *d = inf->dict;
+  struct sfm_fhuser_ext *ext;
   int i;
   int index;
 
@@ -116,19 +116,19 @@ sfm_write_dictionary (sfm_write_info *inf)
       return 0;
     }
 
-  verbose_msg (1, _("%s: Opening system-file handle %s for writing."),
-	       fh_handle_filename (inf->h), fh_handle_name (inf->h));
+  msg (VM (1), _("%s: Opening system-file handle %s for writing."),
+       fh_handle_filename (inf->h), fh_handle_name (inf->h));
   
   /* Open the physical disk file. */
   inf->h->class = &sfm_w_class;
-  inf->h->ext = ext = xmalloc (sizeof (sfm_fhuser_ext));
+  inf->h->ext = ext = xmalloc (sizeof (struct sfm_fhuser_ext));
   ext->file = fopen (inf->h->norm_fn, "wb");
   ext->elem_type = NULL;
   if (ext->file == NULL)
     {
       msg (ME, _("An error occurred while opening \"%s\" for writing "
 	   "as a system file: %s."), inf->h->fn, strerror (errno));
-      cond_fail ();
+      err_cond_fail ();
       free (ext);
       return 0;
     }
@@ -150,7 +150,7 @@ sfm_write_dictionary (sfm_write_info *inf)
   /* Write out value labels. */
   for (index = i = 0; i < d->nvar; i++)
     {
-      variable *v = d->var[i];
+      struct variable *v = d->var[i];
 
       if (!write_value_labels (inf, v, index))
 	goto lossage;
@@ -175,16 +175,16 @@ sfm_write_dictionary (sfm_write_info *inf)
     rec_999.rec_type = 999;
     rec_999.filler = 0;
 
-    if (!bufwrite (inf->h, &rec_999, sizeof (rec_999)))
+    if (!bufwrite (inf->h, &rec_999, sizeof rec_999))
       goto lossage;
   }
 
-  verbose_msg (2, _("Wrote system-file header successfully."));
+  msg (VM (2), _("Wrote system-file header successfully."));
   
   return 1;
 
 lossage:
-  verbose_msg (1, _("Error writing system-file header."));
+  msg (VM (1), _("Error writing system-file header."));
   fclose (ext->file);
   inf->h->class = NULL;
   inf->h->ext = NULL;
@@ -207,11 +207,11 @@ rerange (int x)
 /* Write the sysfile_header header to the system file represented by
    INF. */
 static int
-write_header (sfm_write_info *inf)
+write_header (struct sfm_write_info *inf)
 {
-  dictionary *d = inf->dict;
-  sfm_fhuser_ext *ext = inf->h->ext;
-  sysfile_header hdr;
+  struct dictionary *d = inf->dict;
+  struct sfm_fhuser_ext *ext = inf->h->ext;
+  struct sysfile_header hdr;
   char *p;
   int i;
 
@@ -230,16 +230,16 @@ write_header (sfm_write_info *inf)
   hdr.case_size = 0;
   for (i = 0; i < d->nvar; i++)
     {
-      variable *v = d->var[i];
+      struct variable *v = d->var[i];
       hdr.case_size += (v->type == NUMERIC ? 1
 			: DIV_RND_UP (v->width, sizeof (flt64)));
     }
   inf->case_size = hdr.case_size;
 
-  p = ext->elem_type = xmalloc (sizeof (unsigned char) * inf->case_size);
+  p = ext->elem_type = xmalloc (inf->case_size);
   for (i = 0; i < d->nvar; i++)
     {
-      variable *v = d->var[i];
+      struct variable *v = d->var[i];
       int count = (v->type == NUMERIC ? 1
                    : DIV_RND_UP (v->width, sizeof (flt64)));
       while (count--)
@@ -255,7 +255,7 @@ write_header (sfm_write_info *inf)
 
       for (i = 0; i < d->weight_index; i++)
 	{
-	  variable *v = d->var[i];
+	  struct variable *v = d->var[i];
 	  recalc_weight_index += (v->type == NUMERIC ? 1
 				  : DIV_RND_UP (v->width, sizeof (flt64)));
 	}
@@ -289,16 +289,16 @@ write_header (sfm_write_info *inf)
       char buf[10];
 
       sprintf (buf, "%02d %s %02d", day, month_name[mon - 1], year);
-      memcpy (hdr.creation_date, buf, sizeof (hdr.creation_date));
+      memcpy (hdr.creation_date, buf, sizeof hdr.creation_date);
       sprintf (buf, "%02d:%02d:%02d", hour - 1, min - 1, sec - 1);
-      memcpy (hdr.creation_time, buf, sizeof (hdr.creation_time));
+      memcpy (hdr.creation_time, buf, sizeof hdr.creation_time);
     }
 
-  strbarepadcpy (hdr.file_label, d->label ? d->label : "",
-		 sizeof (hdr.file_label), ' ');
-  memset (hdr.padding, 0, sizeof (hdr.padding));
+  st_bare_pad_copy (hdr.file_label, d->label ? d->label : "",
+		    sizeof hdr.file_label);
+  memset (hdr.padding, 0, sizeof hdr.padding);
 
-  if (!bufwrite (inf->h, &hdr, sizeof (hdr)))
+  if (!bufwrite (inf->h, &hdr, sizeof hdr))
     return 0;
   return 1;
 }
@@ -306,7 +306,7 @@ write_header (sfm_write_info *inf)
 /* Translates format spec from internal form in SRC to system file
    format in DEST. */
 static inline void
-write_format_spec (fmt_spec *src, int32 *dest)
+write_format_spec (struct fmt_spec *src, int32 *dest)
 {
   *dest = (formats[src->type].spss << 16) | (src->w << 8) | src->d;
 }
@@ -314,9 +314,9 @@ write_format_spec (fmt_spec *src, int32 *dest)
 /* Write the variable record(s) for primary variable P and secondary
    variable S to the system file represented by INF. */
 static int
-write_variable (sfm_write_info *inf, variable *v)
+write_variable (struct sfm_write_info *inf, struct variable *v)
 {
-  sysfile_variable sv;
+  struct sysfile_variable sv;
 
   /* Missing values. */
   flt64 m[3];			/* Missing value values. */
@@ -379,7 +379,7 @@ write_variable (sfm_write_info *inf, variable *v)
   write_format_spec (&v->write, &sv.write);
   memcpy (sv.name, v->name, strlen (v->name));
   memset (&sv.name[strlen (v->name)], ' ', 8 - strlen (v->name));
-  if (!bufwrite (inf->h, &sv, sizeof (sv)))
+  if (!bufwrite (inf->h, &sv, sizeof sv))
     return 0;
 
   if (v->label)
@@ -394,7 +394,7 @@ write_variable (sfm_write_info *inf, variable *v)
       int ext_len;
 
       l.label_len = min (strlen (v->label), 120);
-      ext_len = ROUND_UP (l.label_len, sizeof (int32));
+      ext_len = ROUND_UP (l.label_len, sizeof l.label_len);
       memcpy (l.label, v->label, l.label_len);
       memset (&l.label[l.label_len], ' ', ext_len - l.label_len);
 
@@ -402,7 +402,7 @@ write_variable (sfm_write_info *inf, variable *v)
 	  return 0;
     }
 
-  if (nm && !bufwrite (inf->h, m, sizeof (flt64) * nm))
+  if (nm && !bufwrite (inf->h, m, sizeof *m * nm))
     return 0;
 
   if (v->type == ALPHA && v->width > (int) sizeof (flt64))
@@ -413,13 +413,13 @@ write_variable (sfm_write_info *inf, variable *v)
       sv.type = -1;
       sv.has_var_label = 0;
       sv.n_missing_values = 0;
-      memset (&sv.print, 0, sizeof (sv.print));
-      memset (&sv.write, 0, sizeof (sv.write));
-      memset (&sv.name, 0, sizeof (sv.name));
+      memset (&sv.print, 0, sizeof sv.print);
+      memset (&sv.write, 0, sizeof sv.write);
+      memset (&sv.name, 0, sizeof sv.name);
 
       pad_count = DIV_RND_UP (v->width, (int) sizeof (flt64)) - 1;
       for (i = 0; i < pad_count; i++)
-	if (!bufwrite (inf->h, &sv, sizeof (sv)))
+	if (!bufwrite (inf->h, &sv, sizeof sv))
 	  return 0;
     }
 
@@ -430,28 +430,26 @@ write_variable (sfm_write_info *inf, variable *v)
    index INDEX to the system file associated with INF.  Returns
    nonzero only if successful. */
 static int
-write_value_labels (sfm_write_info * inf, variable *v, int index)
+write_value_labels (struct sfm_write_info * inf, struct variable *v, int index)
 {
-  typedef struct
+  struct value_label_rec
     {
       int32 rec_type P;
       int32 n_labels P;
       flt64 labels[1] P;
-    }
-  value_label_rec;
+    };
 
-  typedef struct
+  struct variable_index_rec
     {
       int32 rec_type P;
       int32 n_vars P;
       int32 vars[1] P;
-    }
-  variable_index_rec;
+    };
 
   avl_traverser i;
-  value_label_rec *vlr;
-  variable_index_rec vir;
-  value_label *vl;
+  struct value_label_rec *vlr;
+  struct variable_index_rec vir;
+  struct value_label *vl;
   size_t vlr_size;
   flt64 *loc;
   avl_traverser_init (i);
@@ -460,7 +458,7 @@ write_value_labels (sfm_write_info * inf, variable *v, int index)
     return 1;
 
   /* Pass 1: Count bytes. */
-  vlr_size = (sizeof (value_label_rec)
+  vlr_size = (sizeof (struct value_label_rec)
 	      + sizeof (flt64) * (avl_count (v->val_lab) - 1));
   while (NULL != (vl = avl_traverse (v->val_lab, &i)))
     vlr_size += ROUND_UP (strlen (vl->s) + 1, sizeof (flt64));
@@ -492,7 +490,7 @@ write_value_labels (sfm_write_info * inf, variable *v, int index)
   vir.rec_type = 4;
   vir.n_vars = 1;
   vir.vars[0] = index + 1;
-  if (!bufwrite (inf->h, &vir, sizeof (vir)))
+  if (!bufwrite (inf->h, &vir, sizeof vir))
     return 0;
 
   return 1;
@@ -500,9 +498,9 @@ write_value_labels (sfm_write_info * inf, variable *v, int index)
 
 /* Writes record type 6, document record. */
 static int
-write_documents (sfm_write_info * inf)
+write_documents (struct sfm_write_info * inf)
 {
-  dictionary *d = inf->dict;
+  struct dictionary *d = inf->dict;
   struct
   {
     int32 rec_type P;		/* Always 6. */
@@ -512,7 +510,7 @@ write_documents (sfm_write_info * inf)
 
   rec_6.rec_type = 6;
   rec_6.n_lines = d->n_documents;
-  if (!bufwrite (inf->h, &rec_6, sizeof (rec_6)))
+  if (!bufwrite (inf->h, &rec_6, sizeof rec_6))
     return 0;
   if (!bufwrite (inf->h, d->documents, 80 * d->n_documents))
     return 0;
@@ -522,21 +520,21 @@ write_documents (sfm_write_info * inf)
 
 /* Writes record type 7, subtypes 3 and 4. */
 static int
-write_rec_7_34 (sfm_write_info * inf)
+write_rec_7_34 (struct sfm_write_info * inf)
 {
   struct
-  {
-    int32 rec_type_3 P;
-    int32 subtype_3 P;
-    int32 data_type_3 P;
-    int32 n_elem_3 P;
-    int32 elem_3[8] P;
-    int32 rec_type_4 P;
-    int32 subtype_4 P;
-    int32 data_type_4 P;
-    int32 n_elem_4 P;
-    flt64 elem_4[3] P;
-  }
+    {
+      int32 rec_type_3 P;
+      int32 subtype_3 P;
+      int32 data_type_3 P;
+      int32 n_elem_3 P;
+      int32 elem_3[8] P;
+      int32 rec_type_4 P;
+      int32 subtype_4 P;
+      int32 data_type_4 P;
+      int32 n_elem_4 P;
+      flt64 elem_4[3] P;
+    }
   rec_7;
 
   /* Components of the version number, from major to minor. */
@@ -592,7 +590,7 @@ write_rec_7_34 (sfm_write_info * inf)
   rec_7.elem_4[1] = FLT64_MAX;
   rec_7.elem_4[2] = second_lowest_flt64;
 
-  if (!bufwrite (inf->h, &rec_7, sizeof (rec_7)))
+  if (!bufwrite (inf->h, &rec_7, sizeof rec_7))
     return 0;
   return 1;
 }
@@ -600,14 +598,16 @@ write_rec_7_34 (sfm_write_info * inf)
 /* Write NBYTES starting at BUF to the system file represented by
    H. */
 static int
-bufwrite (file_handle * h, const void *buf, size_t nbytes)
+bufwrite (struct file_handle * h, const void *buf, size_t nbytes)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   assert (buf);
   if (1 != fwrite (buf, nbytes, 1, ext->file))
-    return msg (ME, _("%s: Writing system file: %s."),
-		h->fn, strerror (errno));
+    {
+      msg (ME, _("%s: Writing system file: %s."), h->fn, strerror (errno));
+      return 0;
+    }
   return 1;
 }
 
@@ -626,16 +626,16 @@ append_string_max (char *dest, const char *src, const char *end)
    element.  If there's not room, pads out the current instruction
    octet with zero and dumps out the buffer. */
 static inline int
-ensure_buf_space (file_handle * h)
+ensure_buf_space (struct file_handle *h)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   if (ext->ptr >= ext->end)
     {
       memset (ext->x, 0, ext->y - ext->x);
       ext->x = ext->y;
       ext->ptr = ext->buf;
-      if (!bufwrite (h, ext->buf, sizeof (flt64) * 128))
+      if (!bufwrite (h, ext->buf, sizeof *ext->buf * 128))
 	return 0;
     }
   return 1;
@@ -644,20 +644,20 @@ ensure_buf_space (file_handle * h)
 /* Writes case ELEM consisting of N_ELEM flt64 elements to the system
    file represented by H.  Return success. */
 int
-sfm_write_case (file_handle * h, const flt64 * elem, int n_elem)
+sfm_write_case (struct file_handle * h, const flt64 *elem, int n_elem)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
   const flt64 *end_elem = &elem[n_elem];
   char *elem_type = ext->elem_type;
 
   ext->n_cases++;
 
   if (ext->compressed == 0)
-    return bufwrite (h, elem, sizeof (flt64) * n_elem);
+    return bufwrite (h, elem, sizeof *elem * n_elem);
 
   if (ext->buf == NULL)
     {
-      ext->buf = xmalloc (sizeof (flt64) * 128);
+      ext->buf = xmalloc (sizeof *ext->buf * 128);
       ext->ptr = ext->buf;
       ext->end = &ext->buf[128];
       ext->x = (unsigned char *) (ext->ptr++);
@@ -715,28 +715,29 @@ sfm_write_case (file_handle * h, const flt64 * elem, int n_elem)
 
 /* Closes a system file after we're done with it. */
 static void
-sfm_close (file_handle * h)
+sfm_close (struct file_handle * h)
 {
-  sfm_fhuser_ext *ext = h->ext;
+  struct sfm_fhuser_ext *ext = h->ext;
 
   if (ext->buf != NULL && ext->ptr > ext->buf)
     {
       memset (ext->x, 0, ext->y - ext->x);
-      bufwrite (h, ext->buf, (ext->ptr - ext->buf) * sizeof (flt64));
+      bufwrite (h, ext->buf, (ext->ptr - ext->buf) * sizeof *ext->buf);
     }
 
   /* Attempt to seek back to the beginning in order to write the
      number of cases.  If that's not possible (i.e., we're writing to
      a tty or a pipe), then it's not a big deal because we wrote the
      code that indicates an unknown number of cases. */
-  if (0 == fseek (ext->file, offsetof (sysfile_header, ncases), SEEK_SET))
+  if (0 == fseek (ext->file, offsetof (struct sysfile_header, ncases),
+		  SEEK_SET))
     {
       int32 n_cases = ext->n_cases;
 
       /* I don't really care about the return value: it doesn't matter
          whether this data is written.  This is the only situation in
          which you will see me fail to check a return value. */
-      fwrite (&n_cases, sizeof (n_cases), 1, ext->file);
+      fwrite (&n_cases, sizeof n_cases, 1, ext->file);
     }
 
   if (EOF == fclose (ext->file))
@@ -747,7 +748,7 @@ sfm_close (file_handle * h)
   free (ext);
 }
 
-static fh_ext_class sfm_w_class =
+static struct fh_ext_class sfm_w_class =
 {
   4,
   N_("writing as a system file"),
