@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -22,19 +22,19 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include "alloc.h"
 #include "approx.h"
+#include "command.h"
 #include "error.h"
 #include "expr.h"
 #include "heap.h"
+#include "lexer.h"
 #include "misc.h"
-#include "stat.h"
+#include "sort.h"
 #include "str.h"
 #include "var.h"
-#include "lexer.h"
-#include "vfmP.h"
-#include "sort.h"
 #include "vfm.h"
-#include "sort.h"
+#include "vfmP.h"
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
@@ -44,16 +44,20 @@
 #include <sys/types.h>
 #endif
 
+#if HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 #undef DEBUGGING
 /*#define DEBUGGING 1*/
 #include "debug-print.h"
 
 /* Variables to sort. */
-variable **v_sort;
+struct variable **v_sort;
 int nv_sort;
 
 /* Used when internal-sorting to a separate file. */
-static case_list **separate_case_tab;
+static struct case_list **separate_case_tab;
 
 /* Exported by qsort.c. */
 void blp_quicksort (void *pbase, size_t total_elems, size_t size,
@@ -72,14 +76,12 @@ int
 cmd_sort_cases (void)
 {
   /* First, just parse the command. */
-  match_id (SORT);
-  match_id (CASES);
-  match_tok (BY);
+  lex_match_id ("SORT");
+  lex_match_id ("CASES");
+  lex_match (T_BY);
 
   if (!parse_sort_variables ())
-    return 0;
-  if (token != '.')
-    return syntax_error (NULL);
+    return CMD_FAILURE;
       
   cancel_temporary ();
 
@@ -98,7 +100,10 @@ cmd_sort_cases (void)
   {
     int success = sort_cases (0);
     free (v_sort);
-    return success;
+    if (success)
+      return lex_end_of_command ();
+    else
+      return CMD_FAILURE;
   }
 }
 
@@ -116,19 +121,21 @@ parse_sort_variables (void)
       if (!parse_variables (&default_dict, &v_sort, &nv_sort,
 			    PV_NO_DUPLICATE | PV_APPEND | PV_NO_SCRATCH))
 	return 0;
-      if (match_tok ('('))
+      if (lex_match ('('))
 	{
-	  if (match_id (D) || match_id (DOWN))
+	  if (lex_match_id ("D") || lex_match_id ("DOWN"))
 	    order = SRT_DESCEND;
-	  else if (!match_id (A) && !match_id (UP))
+	  else if (!lex_match_id ("A") && !lex_match_id ("UP"))
 	    {
 	      free (v_sort);
-	      return msg (SE, _("`A' or `D' expected inside parentheses."));
+	      msg (SE, _("`A' or `D' expected inside parentheses."));
+	      return 0;
 	    }
-	  if (!match_tok (')'))
+	  if (!lex_match (')'))
 	    {
 	      free (v_sort);
-	      return msg (SE, _("`)' expected."));
+	      msg (SE, _("`)' expected."));
+	      return 0;
 	    }
 	}
       for (; prev_nv_sort < nv_sort; prev_nv_sort++)
@@ -157,7 +164,7 @@ sort_cases (int separate)
     procedure (NULL, NULL, NULL);
   
   /* SORT CASES cancels PROCESS IF. */
-  free_expression (process_if_expr);
+  expr_free (process_if_expr);
   process_if_expr = NULL;
 
   if (do_internal_sort (separate))
@@ -178,7 +185,7 @@ do_internal_sort (int separate)
 	procedure (NULL, NULL, NULL);
       if (vfm_source == &vfm_memory_stream)
 	{
-	  case_list **case_tab = malloc (sizeof (case_list *)
+	  struct case_list **case_tab = malloc (sizeof *case_tab
 					 * (vfm_source_info.ncases + 1));
 	  if (vfm_source_info.ncases == 0)
 	    {
@@ -187,13 +194,13 @@ do_internal_sort (int separate)
 	    }
 	  if (case_tab != NULL)
 	    {
-	      case_list *clp = memory_source_cases;
-	      case_list **ctp = case_tab;
+	      struct case_list *clp = memory_source_cases;
+	      struct case_list **ctp = case_tab;
 	      int i;
 
 	      for (; clp; clp = clp->next)
 		*ctp++ = clp;
-	      qsort (case_tab, vfm_source_info.ncases, sizeof (case_list *),
+	      qsort (case_tab, vfm_source_info.ncases, sizeof *case_tab,
 		     compare_case_lists);
 
 	      if (!separate)
@@ -218,11 +225,11 @@ do_internal_sort (int separate)
 /* Compares the NV_SORT variables in V_SORT[] between the `case_list's
    at _A and _B, and returns a strcmp()-type result. */
 static int
-compare_case_lists (const void *_a, const void *_b)
+compare_case_lists (const void *pa, const void *pb)
 {
-  case_list *a = *(case_list **) _a;
-  case_list *b = *(case_list **) _b;
-  variable *v;
+  struct case_list *a = *(struct case_list **) pa;
+  struct case_list *b = *(struct case_list **) pb;
+  struct variable *v;
   int result = 0;
   int i;
 
@@ -284,18 +291,17 @@ compare_case_lists (const void *_a, const void *_b)
 #define MIN_BUFFER_SIZE_RECS	16
 
 /* Structure for replacement selection tree. */
-typedef struct repl_sel_tree
+struct repl_sel_tree
   {
     struct repl_sel_tree *loser;/* Loser associated w/this internal node. */
     int rn;			/* Run number of `loser'. */
     struct repl_sel_tree *fe;	/* Internal node above this external node. */
     struct repl_sel_tree *fi;	/* Internal node above this internal node. */
-    value record[1];		/* The case proper. */
-  }
-repl_sel_tree;
+    union value record[1];	/* The case proper. */
+  };
 
 /* Static variables used for sorting. */
-static repl_sel_tree **x;	/* Buffers. */
+static struct repl_sel_tree **x; /* Buffers. */
 static int x_max;		/* Size of buffers, in records. */
 static int records_per_buffer;	/* Number of records in each buffer. */
 
@@ -321,7 +327,7 @@ static char *tmp_extname;	/* Temporary file extension name. */
    that we need to know which runs are the shortest at any given time.
    Priority queues as implemented by heap.c are a natural for this
    task (probably because I wrote the code specifically for it). */
-static heap *huffman_queue;	/* Huffman priority queue. */
+static struct heap *huffman_queue;	/* Huffman priority queue. */
 
 /* Prototypes for helper functions. */
 static void sort_stream_write (void);
@@ -373,7 +379,8 @@ do_external_sort (int separate)
   x = NULL;
   tmp_basename = NULL;
 
-  if (!heap_create (&huffman_queue, 512))
+  huffman_queue = heap_create (512);
+  if (huffman_queue == NULL)
     return 0;
 
   if (!allocate_cases ())
@@ -392,7 +399,7 @@ do_external_sort (int separate)
   /* Despite the name, flow of control comes here regardless of
      whether or not the sort is successful. */
 lossage:
-  heap_destroy (&huffman_queue);
+  heap_destroy (huffman_queue);
 
   if (x)
     {
@@ -452,8 +459,9 @@ allocate_file_handles (void)
   if (-1 == mkdir (buf, S_IRWXU))
     {
       free (buf);
-      return msg (SE, _("%s: Cannot create temporary directory: %s."),
-		  buf, strerror (errno));
+      msg (SE, _("%s: Cannot create temporary directory: %s."),
+	   buf, strerror (errno));
+      return 0;
     }
   *cp++ = DIR_SEPARATOR;
 
@@ -486,24 +494,24 @@ static int
 allocate_cases (void)
 {
   /* This is the size of one case. */
-  const int case_size = (sizeof (repl_sel_tree)
-			 + sizeof (value) * (default_dict.nval - 1)
-			 + sizeof (repl_sel_tree *));
+  const int case_size = (sizeof (struct repl_sel_tree)
+			 + sizeof (union value) * (default_dict.nval - 1)
+			 + sizeof (struct repl_sel_tree *));
 
   x = NULL;
 
-  /* Allocate as many cases as we can, arbitrarily assuming a space of
-     four void pointers for malloc()'s internal bookkeeping. */
+  /* Allocate as many cases as we can, assuming a space of four
+     void pointers for malloc()'s internal bookkeeping. */
   x_max = MAX_WORKSPACE / (case_size + 4 * sizeof (void *));
-  x = malloc (sizeof (repl_sel_tree *) * x_max);
+  x = malloc (sizeof (struct repl_sel_tree *) * x_max);
   if (x != NULL)
     {
       int i;
 
       for (i = 0; i < x_max; i++)
 	{
-	  x[i] = malloc (sizeof (repl_sel_tree)
-			 + sizeof (value) * (default_dict.nval - 1));
+	  x[i] = malloc (sizeof (struct repl_sel_tree)
+			 + sizeof (union value) * (default_dict.nval - 1));
 	  if (x[i] == NULL)
 	    break;
 	}
@@ -539,30 +547,31 @@ allocate_cases (void)
 /* Replacement selection. */
 
 static int rmax, rc, rq;
-static repl_sel_tree *q;
-static value *lastkey;
+static struct repl_sel_tree *q;
+static union value *lastkey;
 static int run_no, file_index;
 static int deferred_abort;
 static int run_length;
 
-static int compare_record (value *, value *);
+static int compare_record (union value *, union value *);
 
 static inline void
-output_record (value *v)
+output_record (union value *v)
 {
-  value *src_case;
+  union value *src_case;
   
   if (deferred_abort)
     return;
 
   if (compaction_necessary)
     {
-      compact_case (compaction_case, (ccase *) v);
-      src_case = (value *) compaction_case;
+      compact_case (compaction_case, (struct ccase *) v);
+      src_case = (union value *) compaction_case;
     }
-  else src_case = (value *) v;
+  else
+    src_case = (union value *) v;
 
-  if ((int) fwrite (src_case, sizeof (value), compaction_nval,
+  if ((int) fwrite (src_case, sizeof *src_case, compaction_nval,
 		    handle[file_index])
       != compaction_nval)
     {
@@ -580,14 +589,15 @@ static int
 close_handle (int i)
 {
   int result = fclose (handle[i]);
-  verbose_msg (2, _("SORT: Closing handle %d."), i);
+  msg (VM (2), _("SORT: Closing handle %d."), i);
   
   handle[i] = NULL;
   if (EOF == result)
     {
       sprintf (tmp_extname, "%08x", i);
-      return msg (SE, _("%s: Error closing temporary file: %s."),
-		  tmp_basename, strerror (errno));
+      msg (SE, _("%s: Error closing temporary file: %s."),
+	   tmp_basename, strerror (errno));
+      return 0;
     }
   return 1;
 }
@@ -607,8 +617,8 @@ static int
 open_handle_w (int handle_no, int run_no)
 {
   sprintf (tmp_extname, "%08x", run_no);
-  verbose_msg (1, _("SORT: %s: Opening for writing as run %d."),
-	       tmp_basename, run_no);
+  msg (VM (1), _("SORT: %s: Opening for writing as run %d."),
+       tmp_basename, run_no);
 
   /* The `x' modifier causes the GNU C library to insist on creating a
      new file--if the file already exists, an error is signaled.  The
@@ -623,13 +633,17 @@ open_handle_r (int handle_no, int run_no)
   FILE *f;
 
   sprintf (tmp_extname, "%08x", run_no);
-  verbose_msg (1, _("SORT: %s: Opening for writing as run %d."),
-	       tmp_basename, run_no);
+  msg (VM (1), _("SORT: %s: Opening for writing as run %d."),
+       tmp_basename, run_no);
   f = handle[handle_no] = fopen (tmp_basename, "rb");
 
   if (f == NULL)
-    return msg (SE, _("%s: Error opening temporary file for reading: %s."),
-		tmp_basename, strerror (errno));
+    {
+      msg (SE, _("%s: Error opening temporary file for reading: %s."),
+	   tmp_basename, strerror (errno));
+      return 0;
+    }
+  
   return 1;
 }
 
@@ -696,7 +710,7 @@ end_run (void)
   /* Advance to next run. */
   run_no++;
   if (run_no)
-    heap_insert (&huffman_queue, run_no - 1, run_length);
+    heap_insert (huffman_queue, run_no - 1, run_length);
 }
 
 /* Performs 5.4.1R. */
@@ -717,13 +731,13 @@ write_initial_runs (int separate)
 
     for (j = 0; j < x_max; j++)
       {
-	repl_sel_tree *J = x[j];
+	struct repl_sel_tree *J = x[j];
 
 	J->loser = J;
 	J->rn = 0;
 	J->fe = x[(x_max + j) / 2];
 	J->fi = x[j / 2];
-	memset (J->record, 0, default_dict.nval * sizeof (value));
+	memset (J->record, 0, default_dict.nval * sizeof (union value));
       }
   }
 
@@ -739,7 +753,7 @@ write_initial_runs (int separate)
   /* Final iterations of steps R4, R5, R6, R7, R2, R3, ... */
   for (;;)
     {
-      repl_sel_tree *t;
+      struct repl_sel_tree *t;
 
       /* R4. */
       rq = rmax + 1;
@@ -755,7 +769,7 @@ write_initial_runs (int separate)
 	      || (t->rn == rq
 		  && compare_record (t->loser->record, q->record) < 0))
 	    {
-	      repl_sel_tree *temp_tree;
+	      struct repl_sel_tree *temp_tree;
 	      int temp_int;
 
 	      temp_tree = t->loser;
@@ -788,7 +802,7 @@ write_initial_runs (int separate)
 	{
 	  output_record (q->record);
 	  lastkey = x[x_max]->record;
-	  memcpy (lastkey, q->record, sizeof (value) * vfm_sink_info.nval);
+	  memcpy (lastkey, q->record, sizeof (union value) * vfm_sink_info.nval);
 	}
     }
   assert (run_no == rmax);
@@ -823,13 +837,13 @@ write_initial_runs (int separate)
 /* Compares the NV_SORT variables in V_SORT[] between the `value's at
    A and B, and returns a strcmp()-type result. */
 static int
-compare_record (value * a, value * b)
+compare_record (union value * a, union value * b)
 {
   int i;
   int result = 0;
-  variable *v;
+  struct variable *v;
 
-  assert (a);
+  assert (a != NULL);
   if (b == NULL)		/* Sort NULLs after everything else. */
     return -1;
 
@@ -910,10 +924,10 @@ merge (void)
   order = MAX_MERGE_ORDER;
   if (x_max / order < MIN_BUFFER_SIZE_RECS)
     order = x_max / MIN_BUFFER_SIZE_RECS;
-  else if (x_max / order * sizeof (value) * default_dict.nval
+  else if (x_max / order * sizeof (union value) * default_dict.nval
 	   < MIN_BUFFER_SIZE_BYTES)
     order = x_max / (MIN_BUFFER_SIZE_BYTES
-		     / (sizeof (value) * (default_dict.nval - 1)));
+		     / (sizeof (union value) * (default_dict.nval - 1)));
 
   /* Make sure the order of merge is bounded. */
   if (order < 2)
@@ -933,7 +947,7 @@ merge (void)
     assert (n_dummy_runs >= 0);
     while (n_dummy_runs--)
       {
-	heap_insert (&huffman_queue, -2, 0);
+	heap_insert (huffman_queue, -2, 0);
 	rmax++;
       }
   }
@@ -955,7 +969,7 @@ merge (void)
       debug_printf ((_("merging runs")));
       for (i = order - 1; i >= 0; i--)
 	{
-	  run_index[i] = heap_delete (&huffman_queue, &run_length[i]);
+	  run_index[i] = heap_delete (huffman_queue, &run_length[i]);
 	  assert (run_index[i] != -1);
 	  total_run_length += run_length[i];
 	  debug_printf ((" %d(%d)", run_index[i], run_length[i]));
@@ -966,7 +980,7 @@ merge (void)
 	{
 	  int index;
 
-	  while (-1 != (index = heap_delete (&huffman_queue, NULL)))
+	  while (-1 != (index = heap_delete (huffman_queue, NULL)))
 	    {
 	      sprintf (tmp_extname, "%08x", index);
 	      if (remove (tmp_basename) != 0)
@@ -977,8 +991,11 @@ merge (void)
 	  return 0;
 	}
 
-      if (!heap_insert (&huffman_queue, run_no++, total_run_length))
-	return msg (SE, _("Out of memory expanding Huffman priority queue."));
+      if (!heap_insert (huffman_queue, run_no++, total_run_length))
+	{
+	  msg (SE, _("Out of memory expanding Huffman priority queue."));
+	  return 0;
+	}
 
       rmax -= order - 1;
     }
@@ -987,7 +1004,7 @@ merge (void)
      all that merging.  This represents the entire sorted active file.
      So we could find a total case count by deleting this element from
      the queue. */
-  assert (heap_size (&huffman_queue) == 1);
+  assert (heap_size (huffman_queue) == 1);
 
   return 1;
 }
@@ -1050,7 +1067,7 @@ merge_once (int run_index[], int run_length[], int n_runs)
 
 	  buffered[i] = min (records_per_buffer, run_length[i]);
 	  for (j = 0; j < buffered[i]; j++)
-	    if ((int) fread (x[j + ofs]->record, sizeof (value),
+	    if ((int) fread (x[j + ofs]->record, sizeof (union value),
 			     default_dict.nval, handle[i])
 		!= default_dict.nval)
 	      {
@@ -1079,7 +1096,7 @@ merge_once (int run_index[], int run_length[], int n_runs)
 			    x[buffer_ptr[i]]->record) > 0)
 	  min = i;
 
-      if ((int) fwrite (x[buffer_ptr[min]]->record, sizeof (value),
+      if ((int) fwrite (x[buffer_ptr[min]]->record, sizeof (union value),
 			default_dict.nval, handle[N_INPUT_BUFFERS])
 	  != default_dict.nval)
 	{
@@ -1108,7 +1125,7 @@ merge_once (int run_index[], int run_length[], int n_runs)
 
 	      buffered[min] = min (records_per_buffer, run_length[min]);
 	      for (j = 0; j < buffered[min]; j++)
-		if ((int) fread (x[j + ofs]->record, sizeof (value),
+		if ((int) fread (x[j + ofs]->record, sizeof (union value),
 				 default_dict.nval, handle[min])
 		    != default_dict.nval)
 		  {
@@ -1167,9 +1184,10 @@ merge_once (int run_index[], int run_length[], int n_runs)
     if (fclose (f) == EOF)
       {
 	sprintf (tmp_extname, "%08x", run_no);
-	return msg (SE, _("%s: Error closing temporary file in merge: "
-			  "%s."),
-		    tmp_basename, strerror (errno));
+	msg (SE, _("%s: Error closing temporary file in merge: "
+		   "%s."),
+	     tmp_basename, strerror (errno));
+	return 0;
       }
   }
 
@@ -1219,9 +1237,9 @@ read_sort_output (int (*write_case) (void))
 
   if (separate_case_tab)
     {
-      ccase *save_temp_case = temp_case;
-      
-      case_list **p;
+      struct ccase *save_temp_case = temp_case;
+      struct case_list **p;
+
       for (p = separate_case_tab; *p; p++)
 	{
 	  temp_case = &(*p)->c;
@@ -1239,7 +1257,7 @@ read_sort_output (int (*write_case) (void))
 	{
 	  msg (ME, _("%s: Cannot open sort result file: %s."), tmp_basename,
 	       strerror (errno));
-	  failure ();
+	  err_failure ();
 	  return;
 	}
 
@@ -1253,7 +1271,7 @@ read_sort_output (int (*write_case) (void))
 	      else
 		msg (ME, _("%s: Unexpected end of sort result file: %s."),
 		     tmp_basename, strerror (errno));
-	      failure ();
+	      err_failure ();
 	      break;
 	    }
 
@@ -1277,7 +1295,7 @@ read_sort_output (int (*write_case) (void))
 /* Alternate interface to sort_stream_write used for external sorting
    when SEPARATE is true. */
 static int
-write_separate (ccase *c)
+write_separate (struct ccase *c)
 {
   assert (c == temp_case);
 
@@ -1291,7 +1309,7 @@ write_separate (ccase *c)
 static void
 sort_stream_write (void)
 {
-  repl_sel_tree *t;
+  struct repl_sel_tree *t;
 
   /* R4. */
   memcpy (q->record, temp_case->data, vfm_sink_info.case_size);
@@ -1307,9 +1325,9 @@ sort_stream_write (void)
     {
       /* R6. */
       if (t->rn < rq
-       || (t->rn == rq && compare_record (t->loser->record, q->record) < 0))
+	  || (t->rn == rq && compare_record (t->loser->record, q->record) < 0))
 	{
-	  repl_sel_tree *temp_tree;
+	  struct repl_sel_tree *temp_tree;
 	  int temp_int;
 
 	  temp_tree = t->loser;
@@ -1355,7 +1373,7 @@ sort_stream_mode (void)
   vfm_sink = NULL;
 }
 
-case_stream sort_stream =
+struct case_stream sort_stream =
   {
     NULL,
     sort_stream_read,
