@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -25,7 +25,7 @@ int sort_cases (int separate);
 void read_sort_output (int (*write_case)(void));
 
 /* Variables to sort. */
-extern variable **v_sort;
+extern struct variable **v_sort;
 extern int nv_sort;
 
 #endif /* !sort_h */
