@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -40,22 +40,22 @@ char *alloca ();
 #include <stdarg.h>
 #include <limits.h>
 #include <stdlib.h>
-#include "error.h"
-#include "str.h"
+#include "alloc.h"
+#include "command.h"
+#include "format.h"
+#include "magic.h"
 #include "misc.h"
-#include "arena.h"
 #include "output.h"
+#include "pool.h"
 #include "som.h"
 #include "tab.h"
 #include "var.h"
-#include "misc.h"
-#include "command.h"
 
 #undef DEBUGGING
 /*#define DEBUGGING 1 */
 #include "debug-print.h"
 
-extern som_table_class tab_table_class;
+extern struct som_table_class tab_table_class;
 
 #if DEBUGGING
 #define DEFFIRST(NAME, LABEL) LABEL,
@@ -71,55 +71,52 @@ static const char *tab_names[] =
 /* Creates a table with NC columns and NR rows.  If REALLOCABLE is
    nonzero then the table's size can be increased later; otherwise,
    its size can only be reduced. */
-tab_table *
+struct tab_table *
 tab_create (int nc, int nr, int reallocable)
 {
-  void *(*alloc_func) (arena **, size_t);
+  void *(*alloc_func) (struct pool *, size_t);
 
-  tab_table *t;
+  struct tab_table *t;
   
   {
-    arena *container;
-    
-    arena_create (&container);
-    t = arena_alloc (&container, sizeof (tab_table));
+    struct pool *container = pool_create ();
+    t = pool_alloc (container, sizeof *t);
     t->container = container;
   }
   
-  t->col_style = SOM_COL_NONE;
+  t->col_style = TAB_COL_NONE;
   t->col_group = 0;
-  t->title.l = 0;
-  t->title.s = NULL;
+  ls_null (&t->title);
   t->flags = SOMF_NONE;
   t->nr = nr;
   t->nc = t->cf = nc;
   t->l = t->r = t->t = t->b = 0;
 
-  alloc_func = reallocable ? arena_malloc : arena_alloc;
+  alloc_func = reallocable ? pool_malloc : pool_alloc;
 #if GLOBAL_DEBUGGING
   t->reallocable = reallocable;
 #endif
 
-  t->cc = alloc_func (&t->container, nr * nc * sizeof (a_string));
-  t->ct = alloc_func (&t->container, nr * nc);
+  t->cc = alloc_func (t->container, nr * nc * sizeof *t->cc);
+  t->ct = alloc_func (t->container, nr * nc);
   memset (t->ct, TAB_EMPTY, nc * nr);
 
-  t->rh = alloc_func (&t->container, nc * (nr + 1));
+  t->rh = alloc_func (t->container, nc * (nr + 1));
   memset (t->rh, 0, nc * (nr + 1));
 
-  t->hrh = alloc_func (&t->container, sizeof (int) * (nr + 1));
-  memset (t->hrh, 0, sizeof (int) * (nr + 1));
+  t->hrh = alloc_func (t->container, sizeof *t->hrh * (nr + 1));
+  memset (t->hrh, 0, sizeof *t->hrh * (nr + 1));
 
-  t->trh = alloc_func (&t->container, nr + 1);
+  t->trh = alloc_func (t->container, nr + 1);
   memset (t->trh, 0, nr + 1);
 
-  t->rv = alloc_func (&t->container, (nc + 1) * nr);
+  t->rv = alloc_func (t->container, (nc + 1) * nr);
   memset (t->rv, 0, (nc + 1) * nr);
 
-  t->wrv = alloc_func (&t->container, sizeof (int) * (nc + 1));
-  memset (t->wrv, 0, sizeof (int) * (nc + 1));
+  t->wrv = alloc_func (t->container, sizeof *t->wrv * (nc + 1));
+  memset (t->wrv, 0, sizeof *t->wrv * (nc + 1));
 
-  t->trv = alloc_func (&t->container, nc + 1);
+  t->trv = alloc_func (t->container, nc + 1);
   memset (t->trv, 0, nc + 1);
 
   t->dim = NULL;
@@ -131,17 +128,17 @@ tab_create (int nc, int nr, int reallocable)
 
 /* Destroys table T. */
 void
-tab_destroy (tab_table *t)
+tab_destroy (struct tab_table *t)
 {
   assert (t != NULL);
-  arena_destroy (&t->container);
+  pool_destroy (t->container);
 }
 
 /* Sets the width and height of a table, in columns and rows,
    respectively.  Use only to reduce the size of a table, since it
    does not change the amount of allocated memory. */
 void
-tab_resize (tab_table *t, int nc, int nr)
+tab_resize (struct tab_table *t, int nc, int nr)
 {
   assert (t != NULL);
   if (nc != -1)
@@ -163,7 +160,7 @@ tab_resize (tab_table *t, int nc, int nr)
    in space and time.  Avoid doing such.  FIXME: In fact, transferring
    of rules isn't even implemented yet. */
 void
-tab_realloc (tab_table *t, int nc, int nr)
+tab_realloc (struct tab_table *t, int nc, int nr)
 {
   int ro, co;
   
@@ -188,34 +185,34 @@ tab_realloc (tab_table *t, int nc, int nr)
       int mr1 = min (nr, t->nr);
       int mc1 = min (nc, t->nc);
       
-      a_string *new_cc;
+      struct len_string *new_cc;
       unsigned char *new_ct;
       int r;
 
-      new_cc = arena_malloc (&t->container, nr * nc * sizeof (a_string));
-      new_ct = arena_malloc (&t->container, nr * nc);
+      new_cc = pool_malloc (t->container, nr * nc * sizeof *new_cc);
+      new_ct = pool_malloc (t->container, nr * nc);
       for (r = 0; r < mr1; r++)
 	{
-	  memcpy (&new_cc[r * nc], &t->cc[r * t->nc], mc1 * sizeof (a_string));
+	  memcpy (&new_cc[r * nc], &t->cc[r * t->nc], mc1 * sizeof *t->cc);
 	  memcpy (&new_ct[r * nc], &t->ct[r * t->nc], mc1);
 	  memset (&new_ct[r * nc + t->nc], TAB_EMPTY, nc - t->nc);
 	}
-      arena_free (t->cc);
-      arena_free (t->ct);
+      pool_free (t->container, t->cc);
+      pool_free (t->container, t->ct);
       t->cc = new_cc;
       t->ct = new_ct;
       t->cf = nc;
     }
   else if (nr != t->nr)
     {
-      t->cc = arena_realloc (&t->container, t->cc,
-			     nr * nc * sizeof (a_string));
-      t->ct = arena_realloc (&t->container, t->ct, nr * nc);
+      t->cc = pool_realloc (t->container, t->cc, nr * nc * sizeof *t->cc);
+      t->ct = pool_realloc (t->container, t->ct, nr * nc);
 
-      t->rh = arena_realloc (&t->container, t->rh, nc * (nr + 1));
-      t->rv = arena_realloc (&t->container, t->rv, (nc + 1) * nr);
-      t->trh = arena_realloc (&t->container, t->trh, nr + 1);
-      t->hrh = arena_realloc (&t->container, t->hrh, sizeof (int) * (nr + 1));
+      t->rh = pool_realloc (t->container, t->rh, nc * (nr + 1));
+      t->rv = pool_realloc (t->container, t->rv, (nc + 1) * nr);
+      t->trh = pool_realloc (t->container, t->trh, nr + 1);
+      t->hrh = pool_realloc (t->container, t->hrh,
+			     sizeof *t->hrh * (nr + 1));
       
       if (nr > t->nr)
 	{
@@ -239,7 +236,7 @@ tab_realloc (tab_table *t, int nc, int nr)
    are repeated when a table is broken across multiple columns or
    multiple pages. */
 void
-tab_headers (tab_table *table, int l, int r, int t, int b)
+tab_headers (struct tab_table *table, int l, int r, int t, int b)
 {
   assert (table != NULL);
   table->l = l;
@@ -251,10 +248,10 @@ tab_headers (tab_table *table, int l, int r, int t, int b)
 /* Set up table T so that, when it is an appropriate size, it will be
    displayed across the page in columns.
 
-   STYLE is a SOM_COL_* constant.  GROUP is the number of rows to take
+   STYLE is a TAB_COL_* constant.  GROUP is the number of rows to take
    as a unit. */
 void
-tab_columns (tab_table *t, int style, int group)
+tab_columns (struct tab_table *t, int style, int group)
 {
   assert (t != NULL);
   t->col_style = style;
@@ -266,7 +263,7 @@ tab_columns (tab_table *t, int style, int group)
 /* Draws a vertical line to the left of cells at horizontal position X
    from Y1 to Y2 inclusive in style STYLE, if style is not -1. */
 void
-tab_vline (tab_table *t, int style, int x, int y1, int y2)
+tab_vline (struct tab_table *t, int style, int x, int y1, int y2)
 {
   int y;
 
@@ -302,7 +299,7 @@ tab_vline (tab_table *t, int style, int x, int y1, int y2)
 /* Draws a horizontal line above cells at vertical position Y from X1
    to X2 inclusive in style STYLE, if style is not -1. */
 void
-tab_hline (tab_table * t, int style, int x1, int x2, int y)
+tab_hline (struct tab_table * t, int style, int x1, int x2, int y)
 {
   int x;
 
@@ -342,7 +339,7 @@ tab_hline (tab_table * t, int style, int x1, int x2, int y)
    drawing those lines.  This is distinct from 0, which draws a null
    line. */
 void
-tab_box (tab_table *t, int f_h, int f_v, int i_h, int i_v,
+tab_box (struct tab_table *t, int f_h, int f_v, int i_h, int i_v,
 	 int x1, int y1, int x2, int y2)
 {
   assert (t != NULL);
@@ -426,10 +423,10 @@ tab_box (tab_table *t, int f_h, int f_v, int i_h, int i_v,
 }
 
 /* Formats text TEXT and arguments ARGS as indicated in OPT and sets
-   the resultant string into S in TABLE's arena. */
+   the resultant string into S in TABLE's pool. */
 static void
-text_format (tab_table *table, int opt, const char *text, va_list args,
-	     a_string *s)
+text_format (struct tab_table *table, int opt, const char *text, va_list args,
+	     struct len_string *s)
 {
   int len;
   
@@ -444,24 +441,8 @@ text_format (tab_table *table, int opt, const char *text, va_list args,
     }
   else
     len = strlen (text);
-  
-  if ((opt & TAT_TITLE) == TAT_TITLE)
-    {
-      s->s = arena_alloc (&table->container, len + 5);
-      s->l = len + 5;
 
-      s->s[0] = 0;
-      STORE_2 (&s->s[1], 5);
-      s->s[3] = TAG_FONT_BY_POSITION;
-      s->s[4] = OUTP_F_BI;
-      memcpy (&s->s[5], text, len);
-    }
-  else
-    {
-      s->s = arena_alloc (&table->container, len);
-      s->l = len;
-      memcpy (s->s, text, len);
-    }
+  ls_create_buffer (table->container, s, text, len);
   
   if (opt & TAT_PRINTF)
     local_free (text);
@@ -470,7 +451,7 @@ text_format (tab_table *table, int opt, const char *text, va_list args,
 /* Set the title of table T to TITLE, which is formatted with printf
    if FORMAT is nonzero. */
 void
-tab_title (tab_table *t, int format, const char *title, ...)
+tab_title (struct tab_table *t, int format, const char *title, ...)
 {
   va_list args;
 
@@ -482,7 +463,7 @@ tab_title (tab_table *t, int format, const char *title, ...)
 
 /* Set DIM_FUNC as the dimension function for table T. */
 void
-tab_dim (tab_table *t, tab_dim_func *dim_func)
+tab_dim (struct tab_table *t, tab_dim_func *dim_func)
 {
   assert (t != NULL && t->dim == NULL);
   t->dim = dim_func;
@@ -493,7 +474,7 @@ tab_dim (tab_table *t, tab_dim_func *dim_func)
    wrapping.  The width will be no larger than the page width minus
    left and right rule widths. */
 int
-tab_natural_width (tab_table *t, outp_driver *d, int c)
+tab_natural_width (struct tab_table *t, struct outp_driver *d, int c)
 {
   int width;
 
@@ -503,16 +484,15 @@ tab_natural_width (tab_table *t, outp_driver *d, int c)
 
     for (width = r = 0; r < t->nr; r++)
       {
-	outp_text text;
+	struct outp_text text;
 	unsigned char opt = t->ct[c + r * t->cf];
 		
 	if (opt & (TAB_JOIN | TAB_EMPTY))
 	  continue;
 
 	text.s = t->cc[c + r * t->cf];
-	assert (text.s.s != NULL);
-	text.options = ((opt & TAB_RICH ? OUTP_T_FANCY : OUTP_T_NONE)
-			| OUTP_T_JUST_LEFT);
+	assert (!ls_null_p (&text.s));
+	text.options = OUTP_T_JUST_LEFT;
 
 	d->class->text_metrics (d, &text);
 	if (text.h > width)
@@ -542,7 +522,7 @@ tab_natural_width (tab_table *t, outp_driver *d, int c)
    is, the minimum height necessary to display the information in the
    cell at the widths set for each column. */
 int
-tab_natural_height (tab_table *t, outp_driver *d, int r)
+tab_natural_height (struct tab_table *t, struct outp_driver *d, int r)
 {
   int height;
 
@@ -553,7 +533,7 @@ tab_natural_height (tab_table *t, outp_driver *d, int r)
     
     for (height = d->font_height, c = 0; c < t->nc; c++)
       {
-	outp_text text;
+	struct outp_text text;
 	unsigned char opt = t->ct[c + r * t->cf];
 
 	assert (t->w[c] != NOT_INT);
@@ -561,9 +541,8 @@ tab_natural_height (tab_table *t, outp_driver *d, int r)
 	  continue;
 
 	text.s = t->cc[c + r * t->cf];
-	assert (text.s.s != NULL);
-	text.options = ((opt & TAB_RICH ? OUTP_T_FANCY : OUTP_T_NONE)
-			| OUTP_T_HORZ | OUTP_T_JUST_LEFT);
+	assert (!ls_null_p (&text.s));
+	text.options = OUTP_T_HORZ | OUTP_T_JUST_LEFT;
 	text.h = t->w[c];
 	d->class->text_metrics (d, &text);
 
@@ -578,7 +557,7 @@ tab_natural_height (tab_table *t, outp_driver *d, int r)
 /* Callback function to set all columns and rows to their natural
    dimensions.  Not really meant to be called directly.  */
 void
-tab_natural_dimensions (tab_table *t, outp_driver *d)
+tab_natural_dimensions (struct tab_table *t, struct outp_driver *d)
 {
   int i;
 
@@ -597,11 +576,11 @@ tab_natural_dimensions (tab_table *t, outp_driver *d)
 /* Sets cell (C,R) in TABLE, with options OPT, to have a value taken
    from V, displayed with format spec F. */
 void
-tab_value (tab_table *table, int c, int r, unsigned char opt,
-	   const value *v, const fmt_spec *f)
+tab_value (struct tab_table *table, int c, int r, unsigned char opt,
+	   const union value *v, const struct fmt_spec *f)
 {
   char *contents;
-  value temp_val;
+  union value temp_val;
 
   assert (table != NULL && v != NULL && f != NULL);
 #if GLOBAL_DEBUGGING
@@ -618,9 +597,8 @@ tab_value (tab_table *table, int c, int r, unsigned char opt,
     }
 #endif
 
-  contents = arena_alloc (&table->container, f->w);
-  table->cc[c + r * table->cf].l = f->w;
-  table->cc[c + r * table->cf].s = contents;
+  contents = pool_alloc (table->container, f->w);
+  ls_init (&table->cc[c + r * table->cf], contents, f->w);
   table->ct[c + r * table->cf] = opt;
   
   if (formats[f->type].cat & FCAT_STRING)
@@ -628,19 +606,19 @@ tab_value (tab_table *table, int c, int r, unsigned char opt,
       temp_val.c = (char *) v->s;
       v = &temp_val;
     }
-  convert_format_to_string (contents, f, v);
+  data_out (contents, f, v);
 }
 
 /* Sets cell (C,R) in TABLE, with options OPT, to have value VAL
    with NDEC decimal places. */
 void
-tab_float (tab_table *table, int c, int r, unsigned char opt,
+tab_float (struct tab_table *table, int c, int r, unsigned char opt,
 	   double val, int w, int d)
 {
   char *contents;
   char buf[40], *cp;
   
-  fmt_spec f;
+  struct fmt_spec f;
 
   assert (table != NULL && w <= 40);
   
@@ -662,15 +640,14 @@ tab_float (tab_table *table, int c, int r, unsigned char opt,
     }
 #endif
 
-  convert_format_to_string (buf, &f, (value *) &val);
+  data_out (buf, &f, (union value *) &val);
   cp = buf;
   while (isspace ((unsigned char) *cp) && cp < &buf[w])
     cp++;
   f.w = w - (cp - buf);
 
-  contents = arena_alloc (&table->container, f.w);
-  table->cc[c + r * table->cf].l = f.w;
-  table->cc[c + r * table->cf].s = contents;
+  contents = pool_alloc (table->container, f.w);
+  ls_init (&table->cc[c + r * table->cf], contents, f.w);
   table->ct[c + r * table->cf] = opt;
   memcpy (contents, cp, f.w);
 }
@@ -678,7 +655,7 @@ tab_float (tab_table *table, int c, int r, unsigned char opt,
 /* Sets cell (C,R) in TABLE, with options OPT, to have text value
    TEXT. */
 void
-tab_text (tab_table *table, int c, int r, unsigned opt, const char *text, ...)
+tab_text (struct tab_table *table, int c, int r, unsigned opt, const char *text, ...)
 {
   va_list args;
 
@@ -706,10 +683,10 @@ tab_text (tab_table *table, int c, int r, unsigned opt, const char *text, ...)
 /* Joins cells (X1,X2)-(Y1,Y2) inclusive in TABLE, and sets them with
    options OPT to have text value TEXT. */
 void
-tab_joint_text (tab_table *table, int x1, int y1, int x2, int y2,
+tab_joint_text (struct tab_table *table, int x1, int y1, int x2, int y2,
 		unsigned opt, const char *text, ...)
 {
-  tab_joined_cell *j;
+  struct tab_joined_cell *j;
 
   assert (table != NULL && text != NULL);
 #if GLOBAL_DEBUGGING
@@ -729,7 +706,7 @@ tab_joint_text (tab_table *table, int x1, int y1, int x2, int y2,
     }
 #endif
   
-  j = arena_alloc (&table->container, sizeof (tab_joined_cell));
+  j = pool_alloc (table->container, sizeof *j);
   j->hit = 0;
   j->x1 = x1 + table->col_ofs;
   j->y1 = y1 + table->row_ofs;
@@ -747,7 +724,7 @@ tab_joint_text (tab_table *table, int x1, int y1, int x2, int y2,
   opt |= TAB_JOIN;
   
   {
-    a_string *cc = &table->cc[x1 + y1 * table->cf];
+    struct len_string *cc = &table->cc[x1 + y1 * table->cf];
     unsigned char *ct = &table->ct[x1 + y1 * table->cf];
     const int ofs = table->cf - (x2 - x1);
 
@@ -759,8 +736,7 @@ tab_joint_text (tab_table *table, int x1, int y1, int x2, int y2,
 	
 	for (x = x1; x < x2; x++)
 	  {
-	    cc->l = 0;
-	    (cc++)->s = (char *) j;
+	    ls_init (cc++, (char *) j, 0);
 	    *ct++ = opt;
 	  }
 	
@@ -772,7 +748,8 @@ tab_joint_text (tab_table *table, int x1, int y1, int x2, int y2,
 
 /* Sets cell (C,R) in TABLE, with options OPT, to contents STRING. */
 void
-tab_raw (tab_table *table, int c, int r, unsigned opt, a_string *string)
+tab_raw (struct tab_table *table, int c, int r, unsigned opt,
+	 struct len_string *string)
 {
   assert (table != NULL && string != NULL);
   
@@ -799,7 +776,7 @@ tab_raw (tab_table *table, int c, int r, unsigned opt, a_string *string)
 /* Sets the widths of all the columns and heights of all the rows in
    table T for driver D. */
 static void
-nowrap_dim (tab_table *t, outp_driver *d)
+nowrap_dim (struct tab_table *t, struct outp_driver *d)
 {
   t->w[0] = tab_natural_width (t, d, 0);
   t->h[0] = d->font_height;
@@ -808,7 +785,7 @@ nowrap_dim (tab_table *t, outp_driver *d)
 /* Sets the widths of all the columns and heights of all the rows in
    table T for driver D. */
 static void
-wrap_dim (tab_table *t, outp_driver *d)
+wrap_dim (struct tab_table *t, struct outp_driver *d)
 {
   t->w[0] = tab_natural_width (t, d, 0);
   t->h[0] = tab_natural_height (t, d, 0);
@@ -820,7 +797,7 @@ wrap_dim (tab_table *t, outp_driver *d)
 void
 tab_output_text (int options, const char *buf, ...)
 {
-  tab_table *t = tab_create (1, 1, 0);
+  struct tab_table *t = tab_create (1, 1, 0);
 
   assert (buf != NULL);
   if (options & TAT_PRINTF)
@@ -833,6 +810,19 @@ tab_output_text (int options, const char *buf, ...)
       buf = temp_buf;
       va_end (args);
     }
+  
+  if (options & TAT_FIX)
+    {
+      struct outp_driver *d;
+
+      for (d = outp_drivers (NULL); d; d = outp_drivers (d))
+	{
+	  if (!d->page_open)
+	    d->class->open_page (d);
+	  
+	  d->class->text_set_font_by_name (d, "FIXED");
+	}
+    }
 
   tab_text (t, 0, 0, options &~ TAT_PRINTF, buf);
   tab_flags (t, SOMF_NO_TITLE | SOMF_NO_SPACING);
@@ -842,13 +832,21 @@ tab_output_text (int options, const char *buf, ...)
     tab_dim (t, wrap_dim);
   tab_submit (t);
 
+  if (options & TAT_FIX)
+    {
+      struct outp_driver *d;
+
+      for (d = outp_drivers (NULL); d; d = outp_drivers (d))
+	d->class->text_set_font_by_name (d, "PROP");
+    }
+  
   if (options & TAT_PRINTF)
     local_free (buf);
 }
 
 /* Set table flags to FLAGS. */
 void
-tab_flags (tab_table *t, unsigned flags)
+tab_flags (struct tab_table *t, unsigned flags)
 {
   assert (t != NULL);
   t->flags = flags;
@@ -856,9 +854,9 @@ tab_flags (tab_table *t, unsigned flags)
 
 /* Easy, type-safe way to submit a tab table to som. */
 void
-tab_submit (tab_table *t)
+tab_submit (struct tab_table *t)
 {
-  som_table s;
+  struct som_table s;
 
   assert (t != NULL);
   s.class = &tab_table_class;
@@ -872,7 +870,7 @@ tab_submit (tab_table *t)
 /* Set table row and column offsets for all functions that affect
    cells or rules. */
 void
-tab_offset (tab_table *t, int col, int row)
+tab_offset (struct tab_table *t, int col, int row)
 {
   int diff = 0;
 
@@ -902,7 +900,7 @@ tab_offset (tab_table *t, int col, int row)
 /* Increment the row offset by one. If the table is too small,
    increase its size. */
 void
-tab_next_row (tab_table *t)
+tab_next_row (struct tab_table *t)
 {
   assert (t != NULL);
   t->cc += t->cf;
@@ -911,26 +909,26 @@ tab_next_row (tab_table *t)
     tab_realloc (t, -1, t->nr * 4 / 3);
 }
 
-static tab_table *t;
-static outp_driver *d;
+static struct tab_table *t;
+static struct outp_driver *d;
 int tab_hit;
 
 /* Set the current table to TABLE. */
 static void
-tabi_table (som_table *table)
+tabi_table (struct som_table *table)
 {
   assert (table != NULL);
   t = table->ext;
   tab_offset (t, 0, 0);
   
   assert (t->w == NULL && t->h == NULL);
-  t->w = arena_alloc (&t->container, sizeof (int) * t->nc);
-  t->h = arena_alloc (&t->container, sizeof (int) * t->nr);
+  t->w = pool_alloc (t->container, sizeof *t->w * t->nc);
+  t->h = pool_alloc (t->container, sizeof *t->h * t->nr);
 }
 
 /* Set the current output device to DRIVER. */
 static void
-tabi_driver (outp_driver *driver)
+tabi_driver (struct outp_driver *driver)
 {
   int i;
 
@@ -950,7 +948,7 @@ tabi_driver (outp_driver *driver)
     t->w[i] = -1;
 #endif
 
-  assert (t->dim);
+  assert (t->dim != NULL);
   t->dim (t, d);
 
 #if GLOBAL_DEBUGGING
@@ -1144,19 +1142,18 @@ tabi_title (int x, int y)
   if (cur_proc)
     cp = spprintf (cp, " %s", cur_proc);
   cp = stpcpy (cp, ".  ");
-  if (t->title.s)
+  if (!ls_empty_p (&t->title))
     {
-      memcpy (cp, t->title.s, t->title.l);
-      cp += t->title.l;
+      memcpy (cp, ls_value (&t->title), ls_length (&t->title));
+      cp += ls_length (&t->title);
     }
   *cp = 0;
   
   {
-    outp_text text;
+    struct outp_text text;
 
-    text.options = OUTP_T_JUST_LEFT | OUTP_T_FANCY | OUTP_T_HORZ | OUTP_T_VERT;
-    text.s.s = buf;
-    text.s.l = cp - buf;
+    text.options = OUTP_T_JUST_LEFT | OUTP_T_HORZ | OUTP_T_VERT;
+    ls_init (&text.s, buf, cp - buf);
     text.h = d->width;
     text.v = d->font_height;
     text.x = 0;
@@ -1213,7 +1210,7 @@ tabi_render (int x1, int y1, int x2, int y2)
 		  );
 }
 
-som_table_class tab_table_class =
+struct som_table_class tab_table_class =
   {
     tabi_table,
     tabi_driver,
@@ -1249,7 +1246,7 @@ som_table_class tab_table_class =
 
    FIXME: Doesn't use r1?  Huh?  */
 static int
-render_strip (int x, int y, int r, int c1, int c2, unused int r1, int r2)
+render_strip (int x, int y, int r, int c1, int c2, int r1 unused, int r2)
 {
   int x_origin = x;
 
@@ -1267,8 +1264,8 @@ render_strip (int x, int y, int r, int c1, int c2, unused int r1, int r2)
 
 	      if (style != TAL_0)
 		{
-		  color clr = {0, 0, 0, 0};
-		  rect rct;
+		  const struct color clr = {0, 0, 0, 0};
+		  struct rect rct;
 
 		  rct.x1 = x;
 		  rct.y1 = y;
@@ -1278,9 +1275,9 @@ render_strip (int x, int y, int r, int c1, int c2, unused int r1, int r2)
 		}
 	      x += t->w[c / 2];
 	    } else {
-	      color clr = {0, 0, 0, 0};
-	      rect rct;
-	      outp_styles s;
+	      const struct color clr = {0, 0, 0, 0};
+	      struct rect rct;
+	      struct outp_styles s;
 
 	      rct.x1 = x;
 	      rct.y1 = y;
@@ -1309,15 +1306,14 @@ render_strip (int x, int y, int r, int c1, int c2, unused int r1, int r2)
 
 	      if (!(t->ct[index] & TAB_JOIN))
 		{
-		  outp_text text;
+		  struct outp_text text;
 
-		  text.options = ((t->ct[index]
-				   & (OUTP_T_JUST_MASK | OUTP_T_FANCY))
+		  text.options = ((t->ct[index] & OUTP_T_JUST_MASK)
 				  | OUTP_T_HORZ | OUTP_T_VERT);
 		  if ((t->ct[index] & TAB_EMPTY) == 0)
 		    {
 		      text.s = t->cc[index];
-		      assert (text.s.s != NULL);
+		      assert (!ls_null_p (&text.s));
 		      text.h = t->w[c / 2];
 		      text.v = t->h[r / 2];
 		      text.x = x;
@@ -1325,7 +1321,8 @@ render_strip (int x, int y, int r, int c1, int c2, unused int r1, int r2)
 		      d->class->text_draw (d, &text);
 		    }
 		} else {
-		  tab_joined_cell *j = (tab_joined_cell *) t->cc[index].s;
+		  struct tab_joined_cell *j =
+		    (struct tab_joined_cell *) ls_value (&t->cc[index]);
 
 		  if (j->hit != tab_hit)
 		    {
@@ -1334,10 +1331,9 @@ render_strip (int x, int y, int r, int c1, int c2, unused int r1, int r2)
 		      if (j->x1 == c / 2 && j->y1 == r / 2
 			  && j->x2 <= c2 && j->y2 <= r2)
 			{
-			  outp_text text;
+			  struct outp_text text;
 
-			  text.options = ((t->ct[index]
-					   & (OUTP_T_JUST_MASK | OUTP_T_FANCY))
+			  text.options = ((t->ct[index] & OUTP_T_JUST_MASK)
 					  | OUTP_T_HORZ | OUTP_T_VERT);
 			  text.s = j->contents;
 			  text.x = x;
@@ -1368,8 +1364,8 @@ render_strip (int x, int y, int r, int c1, int c2, unused int r1, int r2)
 
 	      if (style != TAL_0)
 		{
-		  color clr = {0, 0, 0, 0};
-		  rect rct;
+		  const struct color clr = {0, 0, 0, 0};
+		  struct rect rct;
 
 		  rct.x1 = x;
 		  rct.y1 = y;
