@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -21,20 +21,21 @@
 #include <assert.h>
 #include <stddef.h>
 #include <stdlib.h>
-#include "common.h"
-#include "error.h"
+#include "alloc.h"
 #include "avl.h"
-#include "str.h"
-#include "lexer.h"
-#include "var.h"
+#include "command.h"
 #include "do-ifP.h"
+#include "error.h"
+#include "lexer.h"
+#include "str.h"
+#include "var.h"
 
 #undef DEBUGGING
 /*#define DEBUGGING 1*/
 #include "debug-print.h"
 
 int temporary;
-dictionary *temp_dict;
+struct dictionary *temp_dict;
 int temp_trns;
 
 #if 0
@@ -56,21 +57,13 @@ display_tree (char *s, avl_tree *tree)
 int
 cmd_temporary (void)
 {
-  /* Return value. */
-  int code = 1;
-
-  match_id (TEMPORARY);
-  if (token != '.')
-    {
-      return syntax_error (_("expecting end of command"));
-      code = -2;
-    }
+  lex_match_id ("TEMPORARY");
 
   /* TEMPORARY is not allowed inside DO IF or LOOP. */
   if (ctl_stack)
     {
       msg (SE, _("This command is not valid inside DO IF or LOOP."));
-      return 0;
+      return CMD_FAILURE;
     }
 
   /* TEMPORARY can only appear once! */
@@ -78,7 +71,7 @@ cmd_temporary (void)
     {
       msg (SE, _("This command may only appear once between "
 	   "procedures and procedure-like commands."));
-      return 0;
+      return CMD_FAILURE;
     }
 
   /* Everything is temporary, even if we think it'll last forever.
@@ -91,12 +84,12 @@ cmd_temporary (void)
     temp_trns = n_trns;
   debug_printf (("TEMPORARY: temp_trns=%d\n", temp_trns));
 
-  return code;
+  return lex_end_of_command ();
 }
 
 /* Copies a variable structure. */
 void
-copy_variable (variable *dest, const variable *src)
+copy_variable (struct variable *dest, const struct variable *src)
 {
   int i, n;
 
@@ -140,10 +133,10 @@ copy_variable (variable *dest, const variable *src)
 
 /* Returns a newly created empty dictionary.  The file label and
    documents are copied from default_dict if COPY is nonzero. */
-dictionary *
+struct dictionary *
 new_dictionary (int copy)
 {
-  dictionary *d = xmalloc (sizeof (dictionary));
+  struct dictionary *d = xmalloc (sizeof *d);
   
   d->var = NULL;
   d->var_by_name = avl_create (NULL, cmp_variable, NULL);
@@ -187,14 +180,16 @@ new_dictionary (int copy)
     
 /* Copies the current dictionary info into a newly allocated
    dictionary structure, which is returned. */
-dictionary *
+struct dictionary *
 save_dictionary (void)
 {
   /* Dictionary being created. */
-  dictionary *d = xmalloc (sizeof (dictionary));
+  struct dictionary *d;
 
   int i;
 
+  d = xmalloc (sizeof *d);
+
   /* First the easy stuff. */
   *d = default_dict;
   d->label = default_dict.label ? xstrdup (default_dict.label) : NULL;
@@ -208,10 +203,10 @@ save_dictionary (void)
 
   /* Then the variables. */
   d->var_by_name = avl_create (NULL, cmp_variable, NULL);
-  d->var = xmalloc (default_dict.nvar * sizeof (variable *));
+  d->var = xmalloc (default_dict.nvar * sizeof *d->var);
   for (i = 0; i < default_dict.nvar; i++)
     {
-      d->var[i] = xmalloc (sizeof (variable));
+      d->var[i] = xmalloc (sizeof *d->var[i]);
       copy_variable (d->var[i], default_dict.var[i]);
       strcpy (d->var[i]->name, default_dict.var[i]->name);
       d->var[i]->index = i;
@@ -224,7 +219,7 @@ save_dictionary (void)
       int i;
 
       d->n_splits = default_dict.n_splits;
-      d->splits = xmalloc ((default_dict.n_splits + 1) * sizeof (variable *));
+      d->splits = xmalloc ((default_dict.n_splits + 1) * sizeof *d->splits);
       for (i = 0; i < default_dict.n_splits; i++)
 	d->splits[i] = d->var[default_dict.splits[i]->index];
       d->splits[default_dict.n_splits] = NULL;
@@ -241,7 +236,7 @@ save_dictionary (void)
 /* Copies dictionary D into the active file dictionary.  Deletes
    dictionary D. */
 void
-restore_dictionary (dictionary * d)
+restore_dictionary (struct dictionary * d)
 {
   int i;
 
@@ -265,13 +260,14 @@ restore_dictionary (dictionary * d)
 
   /* 2. Copy dictionary D into the active file dictionary. */
 #if __CHECKER__
-  strncpy (d->weight_var, d->weight_var, 9);
-  strncpy (d->filter_var, d->filter_var, 9);
-  memset (&((char *) d)[(offsetof (dictionary, filter_var)
-			 + sizeof (d->filter_var))],
-	  '*',
-	  (sizeof (dictionary) - offsetof (dictionary, filter_var)
-	   - sizeof (d->filter_var)));
+  {
+    size_t offset;
+    
+    offset = offsetof (struct dictionary, filter_var) + sizeof d->filter_var;
+    strncpy (d->weight_var, d->weight_var, 9);
+    strncpy (d->filter_var, d->filter_var, 9);
+    memset (&((char *) d)[offset], '*', sizeof *d - offset);
+  }
 #endif
   default_dict = *d;
   if (!default_dict.var_by_name)
@@ -288,7 +284,7 @@ restore_dictionary (dictionary * d)
 
 /* Destroys dictionary D. */
 void
-free_dictionary (dictionary * d)
+free_dictionary (struct dictionary * d)
 {
   int i;
 
@@ -301,7 +297,7 @@ free_dictionary (dictionary * d)
 
   for (i = 0; i < d->nvar; i++)
     {
-      variable *v = d->var[i];
+      struct variable *v = d->var[i];
 
       if (v->val_lab)
 	{
