@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -20,7 +20,8 @@
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "common.h"
+#include "alloc.h"
+#include "command.h"
 #include "error.h"
 #include "lexer.h"
 #include "str.h"
@@ -37,46 +38,46 @@ static void debug_print (void);
 int
 cmd_variable_labels (void)
 {
-  variable **v;
+  struct variable **v;
   int nv;
 
   int i;
 
-  match_id (VARIABLE);
-  match_id (LABELS);
-  match_tok ('/');
+  lex_match_id ("VARIABLE");
+  lex_match_id ("LABELS");
+  lex_match ('/');
   do
     {
       parse_variables (NULL, &v, &nv, PV_NONE);
 
-      if (token != STRING)
+      if (token != T_STRING)
 	{
 	  msg (SE, _("String expected for variable label."));
 	  free (v);
-	  return -1;
+	  return CMD_PART_SUCCESS_MAYBE;
 	}
-      if (strlen (tokstr) > 120)
+      if (ds_length (&tokstr) > 120)
 	{
 	  msg (SW, _("Truncating variable label to 120 characters."));
-	  tokstr[120] = 0;
+	  ds_truncate (&tokstr, 120);
 	}
       for (i = 0; i < nv; i++)
 	{
 	  if (v[i]->label)
 	    free (v[i]->label);
-	  v[i]->label = xstrdup (tokstr);
+	  v[i]->label = xstrdup (ds_value (&tokstr));
 	}
 
-      get_token ();
+      lex_get ();
       while (token == '/')
-	get_token ();
+	lex_get ();
       free (v);
     }
   while (token != '.');
 #if DEBUGGING
   debug_print ();
 #endif
-  return 1;
+  return CMD_SUCCESS;
 }
 
 #if DEBUGGING
