@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -21,18 +21,17 @@
 #define vector_h 1
 
 /* Represents a vector as created by the VECTOR transformation. */
-typedef struct vector
+struct vector
   {
     int index;			/* Index into vec[]. */
     char name[9];		/* Name. */
     struct variable **v;	/* Vector of variables. */
     int nv;			/* Number of variables. */
-  }
-vector;
+  };
 
-extern vector *vec;
+extern struct vector *vec;
 extern int nvec;
 
-vector *find_vector (const char *name);
+struct vector *find_vector (const char *name);
 
 #endif /* !vector_h */
