@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@ extern const char bare_version[];
 /* "GNU PSPP A.B.C" */
 extern const char version[];
 
-/* "GNU PSPP version A.B (date), Copyright (C) 1997 Free Software
+/* "GNU PSPP version A.B (date), Copyright (C) XXXX Free Software
    Foundation, Inc." */
 extern const char stat_version[];
 
@@ -45,4 +45,7 @@ extern const char include_path[];
 /* Font path. */
 extern const char groff_font_path[];
 
+/* Locale directory. */
+extern const char locale_dir[];
+
 #endif /* !version_h */
