@@ -1,5 +1,5 @@
 /* PSPP - computes sample statistics.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
 
    This program is free software; you can redistribute it and/or
@@ -42,52 +42,53 @@ char *alloca ();
 #if HAVE_UNISTD_H
 #include <unistd.h>	/* Required by SunOS4. */
 #endif
-#include "som.h"
-#include "tab.h"
-#include "str.h"
-#include "common.h"
+#include "alloc.h"
+#include "approx.h"
+#include "do-ifP.h"
 #include "error.h"
 #include "expr.h"
-#include "approx.h"
 #include "misc.h"
+#include "random.h"
+#include "som.h"
+#include "str.h"
+#include "tab.h"
 #include "var.h"
-#include "do-ifP.h"
+#include "vector.h"
 #include "vfm.h"
 #include "vfmP.h"
-#include "vector.h"
 
 /*
    Virtual File Manager (vfm):
 
-   vfm is used to process data files in stat.  It uses the model that
-   data is read from one stream (the data source), then written to
-   another (the data sink).  The data source is then deleted and the
-   data sink becomes the data source for the next procedure. */
+   vfm is used to process data files.  It uses the model that data is
+   read from one stream (the data source), then written to another
+   (the data sink).  The data source is then deleted and the data sink
+   becomes the data source for the next procedure. */
 
 #undef DEBUGGING
 /*#define DEBUGGING 1 */
 #include "debug-print.h"
 
 /* This is used to read from the active file. */
-case_stream *vfm_source;
+struct case_stream *vfm_source;
 
 /* `value' indexes to initialize to particular values for certain cases. */
-long_vec reinit_sysmis;		/* SYSMIS for every case. */
-long_vec reinit_blanks;		/* Blanks for every case. */
-long_vec init_zero;		/* Zero for first case only. */
-long_vec init_blanks;		/* Blanks for first case only. */
+struct long_vec reinit_sysmis;		/* SYSMIS for every case. */
+struct long_vec reinit_blanks;		/* Blanks for every case. */
+struct long_vec init_zero;		/* Zero for first case only. */
+struct long_vec init_blanks;		/* Blanks for first case only. */
 
 /* This is used to write to the replacement active file. */
-case_stream *vfm_sink;
+struct case_stream *vfm_sink;
 
 /* Information about the data source. */
-stream_info vfm_source_info;
+struct stream_info vfm_source_info;
 
 /* Information about the data sink. */
-stream_info vfm_sink_info;
+struct stream_info vfm_sink_info;
 
 /* Filter variable and  `value' index. */
-static variable *filter_var;
+static struct variable *filter_var;
 static int filter_index;
 
 #define FILTERED							\
@@ -107,7 +108,7 @@ int compaction_nval;
 
 /* Temporary case buffer with enough room for `compaction_nval'
    `value's. */
-ccase *compaction_case;
+struct ccase *compaction_case;
 
 /* Within a session, when paging is turned on, it is never turned back
    off.  This policy might be too aggressive. */
@@ -117,8 +118,8 @@ static int paging = 0;
 time_t last_vfm_invocation;
 
 /* Functions called during procedure processing. */
-static int (*proc_func) (ccase *);	/* Called for each case. */
-static int (*virt_proc_func) (ccase *);		/* Called by SPLIT_FILE_procfunc. */
+static int (*proc_func) (struct ccase *);	/* Called for each case. */
+static int (*virt_proc_func) (struct ccase *);	/* From SPLIT_FILE_procfunc. */
 static void (*begin_func) (void);	/* Called at beginning of a series. */
 static void (*virt_begin_func) (void);	/* Called by SPLIT_FILE_procfunc. */
 static void (*end_func) (void);	/* Called after end of a series. */
@@ -131,13 +132,11 @@ static int case_count;
 int n_lag;			/* Number of cases to lag. */
 static int lag_count;		/* Number of cases in lag_queue so far. */
 static int lag_head;		/* Index where next case will be added. */
-static ccase **lag_queue;	/* Array of n_lag ccase * elements. */
-
-
+static struct ccase **lag_queue; /* Array of n_lag ccase * elements. */
 
 static void open_active_file (void);
 static void close_active_file (void);
-static int SPLIT_FILE_procfunc (ccase *);
+static int SPLIT_FILE_procfunc (struct ccase *);
 static void finish_compaction (void);
 static void lag_case (void);
 static int procedure_write_case (void);
@@ -154,7 +153,7 @@ static int procedure_write_case (void);
    called after each series. */
 void
 procedure (void (*beginfunc) (void),
-	   int (*procfunc) (ccase *curcase),
+	   int (*procfunc) (struct ccase *curcase),
 	   void (*endfunc) (void))
 {
   end_func = endfunc;
@@ -196,7 +195,7 @@ static int not_canceled;
    process_active_file() ignores TEMPORARY, SPLIT FILE, and N. */
 void
 process_active_file (void (*beginfunc) (void),
-		     int (*casefunc) (ccase *curcase),
+		     int (*casefunc) (struct ccase *curcase),
 		     void (*endfunc) (void))
 {
   proc_func = casefunc;
@@ -249,7 +248,7 @@ process_active_file_write_case (void)
   if (not_canceled
       && !FILTERED
       && (process_if_expr == NULL ||
-	  evaluate_expression (process_if_expr, temp_case, NULL) == 1.0))
+	  expr_evaluate (process_if_expr, temp_case, NULL) == 1.0))
     not_canceled = proc_func (temp_case);
   
   case_count++;
@@ -301,8 +300,8 @@ prepare_for_writing (void)
 
   vfm_sink_info.ncases = 0;
   vfm_sink_info.nval = default_dict.nval;
-  vfm_sink_info.case_size = (sizeof (ccase)
-			     + (default_dict.nval - 1) * sizeof (value));
+  vfm_sink_info.case_size = (sizeof (struct ccase)
+			     + (default_dict.nval - 1) * sizeof (union value));
   
   if (vfm_sink == NULL)
     {
@@ -357,8 +356,8 @@ make_temp_case (void)
   temp_case = xmalloc (vfm_sink_info.case_size);
 
   if (compaction_necessary)
-    compaction_case = xmalloc (sizeof (ccase)
-			       + sizeof (value) * (compaction_nval - 1));
+    compaction_case = xmalloc (sizeof (struct ccase)
+			       + sizeof (union value) * (compaction_nval - 1));
   
 #if __CHECKER__
   /* Initialize the unused trailing parts of string variables to avoid
@@ -368,7 +367,7 @@ make_temp_case (void)
     
     for (i = 0; i < default_dict.nvar; i++)
       {
-	variable *v = default_dict.var[i];
+	struct variable *v = default_dict.var[i];
       
 	if (v->type == ALPHA && v->width % 8 != 0)
 	  memcpy (&temp_case->data[v->fv + v->nv - 1]
@@ -449,7 +448,7 @@ setup_filter (void)
   
   if (default_dict.filter_var[0])
     {
-      variable *fv = find_variable (default_dict.filter_var);
+      struct variable *fv = find_variable (default_dict.filter_var);
       
       if (fv == NULL || fv->type == ALPHA)
 	default_dict.filter_var[0] = 0;
@@ -472,9 +471,9 @@ setup_lag (void)
 
   lag_count = 0;
   lag_head = 0;
-  lag_queue = xmalloc (n_lag * sizeof (ccase *));
+  lag_queue = xmalloc (n_lag * sizeof *lag_queue);
   for (i = 0; i < n_lag; i++)
-    lag_queue[i] = xmalloc (temp_dict->nval * sizeof (value));
+    lag_queue[i] = xmalloc (temp_dict->nval * sizeof **lag_queue);
 }
 
 /* There is a lot of potential confusion in the vfm and related
@@ -586,8 +585,8 @@ close_active_file (void)
   vfm_source = vfm_sink;
   vfm_source_info.ncases = vfm_sink_info.ncases;
   vfm_source_info.nval = compaction_nval;
-  vfm_source_info.case_size = (sizeof (ccase)
-			       + (compaction_nval - 1) * sizeof (value));
+  vfm_source_info.case_size = (sizeof (struct ccase)
+			       + (compaction_nval - 1) * sizeof (union value));
   if (vfm_source->mode)
     vfm_source->mode ();
 
@@ -607,7 +606,7 @@ close_active_file (void)
   compaction_case = NULL;
 
   /* Cancel PROCESS IF. */
-  free_expression (process_if_expr);
+  expr_free (process_if_expr);
   process_if_expr = NULL;
 
   /* Cancel FILTER if temporary. */
@@ -656,7 +655,7 @@ disk_stream_init (void)
       msg (ME, _("An error occurred attempting to create a temporary "
 		 "file for use as the active file: %s."),
 	   strerror (errno));
-      failure ();
+      err_failure ();
     }
 }
 
@@ -674,7 +673,7 @@ disk_stream_read (void)
 	  msg (ME, _("An error occurred while attempting to read from "
 	       "a temporary file created for the active file: %s."),
 	       strerror (errno));
-	  failure ();
+	  err_failure ();
 	  return;
 	}
 
@@ -687,22 +686,22 @@ disk_stream_read (void)
 static void
 disk_stream_write (void)
 {
-  value *src_case;
+  union value *src_case;
 
   if (compaction_necessary)
     {
       compact_case (compaction_case, temp_case);
-      src_case = (value *) compaction_case;
+      src_case = (union value *) compaction_case;
     }
-  else src_case = (value *) temp_case;
+  else src_case = (union value *) temp_case;
 
-  if (fwrite (src_case, sizeof (value) * compaction_nval, 1,
+  if (fwrite (src_case, sizeof *src_case * compaction_nval, 1,
 	      disk_sink_file) != 1)
     {
       msg (ME, _("An error occurred while attempting to write to a "
 		 "temporary file used as the active file: %s."),
 	   strerror (errno));
-      failure ();
+      err_failure ();
     }
 }
 
@@ -716,7 +715,7 @@ disk_stream_mode (void)
       msg (ME, _("An error occurred while attempting to rewind a "
 		 "temporary file used as the active file: %s."),
 	   strerror (errno));
-      failure ();
+      err_failure ();
     }
   
   /* Sink --> source variables. */
@@ -746,7 +745,7 @@ disk_stream_destroy_sink (void)
 }
 
 /* Disk stream. */
-case_stream vfm_disk_stream = 
+struct case_stream vfm_disk_stream = 
   {
     disk_stream_init,
     disk_stream_read,
@@ -760,11 +759,11 @@ case_stream vfm_disk_stream =
 /* Memory case stream. */
 
 /* List of cases stored in the stream. */
-case_list *memory_source_cases;
-case_list *memory_sink_cases;
+struct case_list *memory_source_cases;
+struct case_list *memory_sink_cases;
 
 /* Current case. */
-case_list *memory_sink_iter;
+struct case_list *memory_sink_iter;
 
 /* Maximum number of cases. */
 int memory_sink_max_cases;
@@ -777,7 +776,7 @@ memory_stream_init (void)
   memory_sink_iter = NULL;
   
   assert (compaction_nval);
-  memory_sink_max_cases = MAX_WORKSPACE / (sizeof (value) * compaction_nval);
+  memory_sink_max_cases = MAX_WORKSPACE / (sizeof (union value) * compaction_nval);
 }
 
 /* Reads the case stream from memory and passes it to write_case(). */
@@ -789,7 +788,7 @@ memory_stream_read (void)
       memcpy (temp_case, &memory_source_cases->c, vfm_source_info.case_size);
       
       {
-	case_list *current = memory_source_cases;
+	struct case_list *current = memory_source_cases;
 	memory_source_cases = memory_source_cases->next;
 	free (current);
       }
@@ -803,8 +802,9 @@ memory_stream_read (void)
 static void
 memory_stream_write (void)
 {
-  case_list *new_case = malloc (sizeof (case_list)
-				+ (compaction_nval - 1) * sizeof (value));
+  struct case_list *new_case = malloc (sizeof (struct case_list)
+				       + ((compaction_nval - 1)
+					  * sizeof (union value)));
 
   /* If we've got memory to spare then add it to the linked list. */
   if (vfm_sink_info.ncases <= memory_sink_max_cases && new_case != NULL)
@@ -812,7 +812,7 @@ memory_stream_write (void)
       if (compaction_necessary)
 	compact_case (&new_case->c, temp_case);
       else
-	memcpy (&new_case->c, temp_case, sizeof (value) * compaction_nval);
+	memcpy (&new_case->c, temp_case, sizeof (union value) * compaction_nval);
 
       /* Append case to linked list. */
       if (memory_sink_cases)
@@ -823,7 +823,7 @@ memory_stream_write (void)
   else
     {
       /* Out of memory.  Write the active file to disk. */
-      case_list *cur, *next;
+      struct case_list *cur, *next;
 
       /* Notify the user. */
       if (!new_case)
@@ -833,10 +833,10 @@ memory_stream_write (void)
 	msg (MW, _("Workspace limit of %d KB (%d cases at %d bytes each) "
 		   "overflowed.  Paging active file to disk."),
 	     MAX_WORKSPACE / 1024, memory_sink_max_cases,
-	     compaction_nval * sizeof (value));
+	     compaction_nval * sizeof (union value));
 
       free (new_case);
-      
+
       /* Switch to a disk sink. */
       vfm_sink = &vfm_disk_stream;
       vfm_sink->init ();
@@ -851,14 +851,14 @@ memory_stream_write (void)
       for (cur = memory_sink_cases; cur; cur = next)
 	{
 	  next = cur->next;
-	  if (fwrite (cur->c.data, sizeof (value) * compaction_nval, 1,
+	  if (fwrite (cur->c.data, sizeof (union value) * compaction_nval, 1,
 		      disk_sink_file) != 1)
 	    {
 	      msg (ME, _("An error occurred while attempting to "
 			 "write to a temporary file created as the "
 			 "active file, while paging to disk: %s."),
 		   strerror (errno));
-	      failure ();
+	      err_failure ();
 	    }
 	  free (cur);
 	}
@@ -883,19 +883,19 @@ page_to_disk (void)
       /* Write the cases to disk and destroy them.  We can't call
          vfm->sink->write() because of compaction. */
       {
-	case_list *cur, *next;
+	struct case_list *cur, *next;
 	
 	for (cur = memory_source_cases; cur; cur = next)
 	  {
 	    next = cur->next;
-	    if (fwrite (cur->c.data, sizeof (value) * compaction_nval, 1,
+	    if (fwrite (cur->c.data, sizeof *cur->c.data * compaction_nval, 1,
 			disk_sink_file) != 1)
 	      {
 		msg (ME, _("An error occurred while attempting to "
 			   "write to a temporary file created as the "
 			   "active file, while paging to disk: %s."),
 		     strerror (errno));
-		failure ();
+		err_failure ();
 	      }
 	    free (cur);
 	  }
@@ -925,7 +925,7 @@ memory_stream_mode (void)
 static void
 memory_stream_destroy_source (void)
 {
-  case_list *cur, *next;
+  struct case_list *cur, *next;
   
   for (cur = memory_source_cases; cur; cur = next)
     {
@@ -939,7 +939,7 @@ memory_stream_destroy_source (void)
 static void
 memory_stream_destroy_sink (void)
 {
-  case_list *cur, *next;
+  struct case_list *cur, *next;
   
   for (cur = memory_sink_cases; cur; cur = next)
     {
@@ -950,7 +950,7 @@ memory_stream_destroy_sink (void)
 }
   
 /* Memory stream. */
-case_stream vfm_memory_stream = 
+struct case_stream vfm_memory_stream = 
   {
     memory_stream_init,
     memory_stream_read,
@@ -970,14 +970,14 @@ lag_case (void)
 {
   if (lag_count < n_lag)
     lag_count++;
-  memcpy (lag_queue[lag_head], temp_case, sizeof (value) * temp_dict->nval);
+  memcpy (lag_queue[lag_head], temp_case, sizeof (union value) * temp_dict->nval);
   if (++lag_head >= n_lag)
     lag_head = 0;
 }
 
 /* Returns a pointer to the lagged case from N_BEFORE cases before the
    current one, or NULL if there haven't been that many cases yet. */
-ccase *
+struct ccase *
 lagged_case (int n_before)
 {
   assert (n_before <= n_lag);
@@ -1062,7 +1062,7 @@ procedure_write_case (void)
   if (proc_func != NULL
       && !FILTERED
       && (process_if_expr == NULL ||
-	  evaluate_expression (process_if_expr, temp_case, NULL) == 1.0))
+	  expr_evaluate (process_if_expr, temp_case, NULL) == 1.0))
     proc_func (temp_case);
 
   case_count++;
@@ -1087,12 +1087,12 @@ done:
 /* Appends TRNS to t_trns[], the list of all transformations to be
    performed on data as it is read from the active file. */
 void
-add_transformation (any_trns * trns)
+add_transformation (struct trns_header * trns)
 {
   if (n_trns >= m_trns)
     {
       m_trns += 16;
-      t_trns = xrealloc (t_trns, sizeof (any_trns *) * m_trns);
+      t_trns = xrealloc (t_trns, sizeof *t_trns * m_trns);
     }
   t_trns[n_trns] = trns;
   trns->index = n_trns++;
@@ -1120,10 +1120,10 @@ cancel_transformations (void)
 
 /* Dumps out the values of all the split variables for the case C. */
 static void
-dump_splits (ccase *c)
+dump_splits (struct ccase *c)
 {
-  variable **iter;
-  tab_table *t;
+  struct variable **iter;
+  struct tab_table *t;
   int i;
 
   t = tab_create (3, default_dict.n_splits + 1, 0);
@@ -1135,7 +1135,7 @@ dump_splits (ccase *c)
   tab_text (t, 2, 0, TAB_LEFT, _("Label"));
   for (iter = default_dict.splits, i = 0; *iter; iter++, i++)
     {
-      variable *v = *iter;
+      struct variable *v = *iter;
       char temp_buf[80];
       char *val_lab;
 
@@ -1143,10 +1143,10 @@ dump_splits (ccase *c)
       tab_text (t, 0, i + 1, TAB_LEFT | TAT_PRINTF, "%s", v->name);
       
       {
-	value val = c->data[v->fv];
+	union value val = c->data[v->fv];
 	if (v->type == ALPHA)
 	  val.c = c->data[v->fv].s;
-	convert_format_to_string (temp_buf, &v->print, &val);
+	data_out (temp_buf, &v->print, &val);
       }
       
       temp_buf[v->print.w] = 0;
@@ -1164,10 +1164,10 @@ dump_splits (ccase *c)
    SPLIT FILE is active.  This function forms a wrapper around that
    procfunc by dividing the input into series. */
 static int
-SPLIT_FILE_procfunc (ccase *c)
+SPLIT_FILE_procfunc (struct ccase *c)
 {
-  static ccase *prev_case;
-  variable **iter;
+  static struct ccase *prev_case;
+  struct variable **iter;
 
   /* The first case always begins a new series.  We also need to
      preserve the values of the case for later comparison. */
@@ -1189,7 +1189,7 @@ SPLIT_FILE_procfunc (ccase *c)
      the previous case. */
   for (iter = default_dict.splits; *iter; iter++)
     {
-      variable *v = *iter;
+      struct variable *v = *iter;
       
       switch (v->type)
 	{
@@ -1224,7 +1224,7 @@ not_equal:
 
 /* Copies case SRC to case DEST, compacting it in the process. */
 void
-compact_case (ccase *dest, const ccase *src)
+compact_case (struct ccase *dest, const struct ccase *src)
 {
   int i;
   int nval = 0;
@@ -1234,7 +1234,7 @@ compact_case (ccase *dest, const ccase *src)
   if (temporary == 2)
     {
       if (dest != compaction_case)
-	memcpy (dest, compaction_case, sizeof (value) * compaction_nval);
+	memcpy (dest, compaction_case, sizeof (union value) * compaction_nval);
       return;
     }
 
@@ -1242,7 +1242,7 @@ compact_case (ccase *dest, const ccase *src)
      DEST. */
   for (i = 0; i < default_dict.nvar; i++)
     {
-      variable *v = default_dict.var[i];
+      struct variable *v = default_dict.var[i];
       
       if (v->name[0] == '#')
 	continue;
@@ -1251,9 +1251,9 @@ compact_case (ccase *dest, const ccase *src)
 	dest->data[nval++] = src->data[v->fv];
       else
 	{
-	  int w = DIV_RND_UP (v->width, sizeof (value));
+	  int w = DIV_RND_UP (v->width, sizeof (union value));
 	  
-	  memcpy (&dest->data[nval], &src->data[v->fv], w * sizeof (value));
+	  memcpy (&dest->data[nval], &src->data[v->fv], w * sizeof (union value));
 	  nval += w;
 	}
     }
@@ -1269,7 +1269,7 @@ finish_compaction (void)
 
   for (i = 0; i < default_dict.nvar; i++)
     {
-      variable *v = default_dict.var[i];
+      struct variable *v = default_dict.var[i];
 
       if (v->name[0] == '#')
 	{
@@ -1282,14 +1282,14 @@ finish_compaction (void)
       if (v->type == NUMERIC)
 	nval++;
       else
-	nval += DIV_RND_UP (v->width, sizeof (value));
+	nval += DIV_RND_UP (v->width, sizeof (union value));
       
       default_dict.var[copy_index++] = v;
     }
   if (copy_index != default_dict.nvar)
     {
-      default_dict.var = xrealloc(default_dict.var,
-				  sizeof (variable *) * copy_index);
+      default_dict.var = xrealloc (default_dict.var,
+				   sizeof *default_dict.var * copy_index);
       default_dict.nvar = copy_index;
     }
 }
